function [slip_data] = get_normal(xnod,icone,slip,key)
%
% Calculo de la normal a cada nodo perteneciente al borde SLIP
%
%  if ndm==2
%
%      [slip_data] = get_normal(xnod,icone,slip,[key])
%
%    no key   ---> slip = [ele face]
%    key = 0  ---> slip = [ele face]
%    key = 1  ---> slip = [node1 node2]
%
%  elseif ndm==3
%
%    [slip_data] = get_normal(xnod,icone)
%
%    xnod de [numnp,3] nodos del volumne
%    icone de [numel_surf,nen_surf] conectividad de la superficie
%
%-----------------------------------------------------------------

if nargin<4, key=0; end

[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);

if ndm==2,
   nslip = length(slip(:,1));    
    if key==0,
       if nen==4, node_face = [1;2;3;4;1];end
      if nen==3, node_face = [1;2;3;1]  ;end
      eles= slip(:,1);
      face= slip(:,2);
      nod1= getcol(icone(eles,:),node_face(face));
      nod2= getcol(icone(eles,:),node_face(face+1));
   else
      nod1 = slip(:,1);
      nod2 = slip(:,2);         
   end

   L_ele  = [xnod(nod2,1)-xnod(nod1,1) , xnod(nod2,2)-xnod(nod1,2)];
   tg_ele = L_ele;
   L_ele  = sqrt(sum(L_ele'.^2))';

   tg_ele = tg_ele ./ (L_ele*ones(1,2));
   no_ele = [-tg_ele(:,2) tg_ele(:,1)];
   normal = zeros(numnp,ndm);
   L_nod  = zeros(numnp,1);

   for k=1:nslip,
     normal(nod1(k),:) = normal(nod1(k),:)+L_ele(k)*no_ele(k,:);
     normal(nod2(k),:) = normal(nod2(k),:)+L_ele(k)*no_ele(k,:);
     L_nod(nod1(k)) = L_nod(nod1(k))+L_ele(k);
	 L_nod(nod2(k)) = L_nod(nod2(k))+L_ele(k);
   end

   nn = find(L_nod>0);
   nodes_slip = nn;
   normal(nn,:) = normal(nn,:)./(L_nod(nn)*ones(1,ndm));
   normal = normal(nn,:);
   vaux = sqrt(sum(normal'.^2))';
   normal = normal./(vaux*[1,1]);
   slip_data = [nodes_slip , normal ] ;
   
 else
    
    % calculo de la normal en cada nodo del borde en la malla de superficie
    [numel,nen]=size(icone);
    if nen==3,  
      nodes = icone;
		dx12 = xnod(nodes(:,2),:)-xnod(nodes(:,1),:);
		dx13 = xnod(nodes(:,3),:)-xnod(nodes(:,1),:);
		norf = pvec(dx12,dx13);
		norn = zeros(numnp,3);
		area = zeros(numnp,1);
		norn(nodes(:,1),:)=norn(nodes(:,1),:)+norf;
		norn(nodes(:,2),:)=norn(nodes(:,2),:)+norf;
		norn(nodes(:,3),:)=norn(nodes(:,3),:)+norf;
		areaf = sqrt(sum(norf'.^2))';
		area(nodes(:,1),1)=area(nodes(:,1),1)+areaf;
		area(nodes(:,2),1)=area(nodes(:,2),1)+areaf;
		area(nodes(:,3),1)=area(nodes(:,3),1)+areaf;
		nn=find(area>0);
		slip_data = nn;
		norn(nn,:)=norn(nn,:)./(area(nn)*ones(1,3));
		vaux = sqrt(sum(norn'.^2))';
		norn(nn,:)=norn(nn,:)./(vaux(nn)*ones(1,3));
		slip_data = [slip_data , norn(nn,:)];
    else
       error(' IMPLEMENTACION 3D NO DISPONIBLE AUN ')
    end    
 end

