function [Cd] = drag(xnod,icone,R_sphere,list_ele,u,tau,u_inf,ro_inf);
%
% routine to compute the drag force coefficient on a sphere 
% centered at the origin of coordinates
%
%      [Cd] = drag(xnod,icone,R_sphere,list_ele,u,tau,u_inf,ro_inf)
%
% Inputs:
%   xnod,icone   ---> coordinates and connectivities (whole mesh)
%   R_sphere     ---> sphere radius
%   list_ele     ---> elements over the sphere
%               [ element  face ]
%   u            ---> nodal solution
%   tau          ---> nodal traction
%   u_inf        ---> velocity vector at free stream [ux  uy  uz]
%   ro_inf       ---> density at free stream

node_face = [ 1 2 3 4 ; 
		  1 2 5 6 ;
		  2 3 6 7 ;
		  3 4 7 8 ;
		  1 4 5 8 ;
		  5 6 7 8 ];

normal = xnod(:,:) / R_sphere;

[nn,mm]=size(list_ele);

for l=1:nn
 iele = list_ele(l,1);
 iface = list_ele(l,2);
 nod_face = node_face(iface,:);
 inod = icone(iele,nod_face);
 normal_node = -normal(inod,:);
 pressu = u(inod,5)*ones(1,3);
 pressu = pressu.*normal_node;
 stress = tau(inod,1:3);
 force = pressu+stress;
 force = sum(force)/4;
 ledge1 = (xnod(inod(2),1)-xnod(inod(1),1))^2 + ...
	    (xnod(inod(2),2)-xnod(inod(1),2))^2 + ...
	    (xnod(inod(2),3)-xnod(inod(1),3))^2 ;
 ledge2 = (xnod(inod(3),1)-xnod(inod(2),1))^2 + ...
	    (xnod(inod(3),2)-xnod(inod(2),2))^2 + ...
	    (xnod(inod(3),3)-xnod(inod(2),3))^2 ;
 ledge3 = (xnod(inod(4),1)-xnod(inod(3),1))^2 + ...
	    (xnod(inod(4),2)-xnod(inod(3),2))^2 + ...
	    (xnod(inod(4),3)-xnod(inod(3),3))^2 ;
 ledge4 = (xnod(inod(1),1)-xnod(inod(4),1))^2 + ...
	    (xnod(inod(1),2)-xnod(inod(4),2))^2 + ...
	    (xnod(inod(1),3)-xnod(inod(4),3))^2 ;
 ledge1 = sqrt(ledge1);
 ledge2 = sqrt(ledge2);
 ledge3 = sqrt(ledge3);
 ledge4 = sqrt(ledge4);
 area  = 0.5*(ledge1+ledge3)*0.5*(ledge2+ledge4);
 force = force*area;
 drag = [drag ; force];
end

drag_total = sum(drag);

e_u_inf = u_inf ./ (norm(u_inf)*ones(1,3));
Cd = drag_total*e_u_inf';

Cd = Cd/(0.5*ro_inf*(norm(u_inf)^2)*pi*R_sphere*R_sphere);
