function iconew = htot(icone,Nx,Ny,Nz)
%
%    hexahedral  ------->   tetrahedral
% program to generate a tetrahedral mesh starting
% with an hexahedral one.
%
% This program works ONLY with structured mesh
% It is based on the splitting of an hexahedral
% into five tetrahedrals.
% This procedure is repeated over all the elements
% according to an splitting orientation that alternates
% between 0 and 1, like a chess board in 3D.
%
%----------------------------------------------------------
%         iconew = htot(icone,Nx,Ny,Nz)
%----------------------------------------------------------
% 
% Input data:
%            [Nx,Ny,Nz] = number of elements in [x,y,z]
%                 icone = hexahedral mesh connectivity
% Output data:    
%                iconew = tetrahedral mesh connectivity
%

%   cara hexahedro    nodos         tetraedro        cara
%-------------------------------------------------------------
%   SPLIT_0
%-------------------------------------------------------------
%       1            1-2-3-4          1               4
%                                     2               4
%       2            1-2-5-6          1               1
%                                     3               1
%       3            2-3-6-7          1               2
%                                     4               3
%       4            3-4-7-8          4               1
%                                     2               2
%       5            1-4-5-8          2               3
%                                     3               3
%       6            5-6-7-8          3               2
%                                     4               2
%-------------------------------------------------------------
%   SPLIT_1
%-------------------------------------------------------------
%       1            1-2-3-4          1               1
%                                     2               2
%       2            1-2-5-6          1               4
%                                     3               3
%       3            2-3-6-7          3               2
%                                     2               1
%       4            3-4-7-8          2               2
%                                     4               2
%       5            1-4-5-8          1               3
%                                     4               1
%       6            5-6-7-8          3               4
%                                     4               4

%    tetrahedral faces :
%  
%      faces           nodes  
%        1             1 2 4
%        2             2 3 4 
%        3             1 4 3
%        4             1 3 2
%
 
mapface_0 = [1 4 2 4 ; 1 1 3 1 ; 1 2 4 3 ; 4 1 2 2 ; 2 3 3 3 ; 3 2 4 2 ];
mapface_1 = [1 1 2 2 ; 1 4 3 3 ; 3 2 2 1 ; 2 2 4 2 ; 1 3 4 1 ; 3 4 4 4 ];

iconew =[];
[numel,nen]=size(icone);
itypo=zeros(numel,1);

split_0 = [1 2 3 6 ; 1 3 4 8 ; 1 6 8 5 ; 3 8 6 7 ; 1 3 8 6 ];
split_1 = [1 2 4 5 ; 2 3 4 7 ; 5 7 6 2 ; 5 8 7 4 ; 5 7 2 4 ];
for k=1:Nz,
  disp([' ' num2str(k) ' de ' num2str(Nz)]);
 iz = rem(k,2);
 for j=1:Ny
  disp([' ' num2str(j) ' de ' num2str(Ny)]);
  iy = rem(j,2);
  for i=1:Nx,
   ix= rem(i,2);
   is = rem(ix+iy+iz,2);   % flag to assign the splitting orientation
   iele = (k-1)*Nx*Ny+(j-1)*Nx+i;
   itypo(iele) = is;
   if is==0,
     iconew = [iconew ; [ icone(iele,split_0(1,:)) ; icone(iele,split_0(2,:)) ; 
                          icone(iele,split_0(3,:)) ; icone(iele,split_0(4,:)) ;
                          icone(iele,split_0(5,:))                              ]  ];
   else
     iconew = [iconew ; [ icone(iele,split_1(1,:)) ; icone(iele,split_1(2,:)) ; 
                          icone(iele,split_1(3,:)) ; icone(iele,split_1(4,:)) ;
                          icone(iele,split_1(5,:))                              ]  ];
   end  
  end
 end
end

