function zprof=naca4ds(digits,n)
% zprof=naca4ds(digits,n)
%
%    Returns a grid of 2*n points over profiles of the 
%       NACA four-digit family
%       Ex: z=naca4ds(0012,64) to construct a 128 points grid
%            counterclockwise for the NACA0012 profile


% Construir perfiles de la familia NACA de 4 digitos
%teta=(0:n)'/n*pi;
teta=sqrt((0:n)'/n)*pi;
x=(1+cos(teta))/2;

% maximum camber
remaind=digits;
div=fix(remaind/1000);
remaind=rem(remaind,1000);
maxcamb=div/100;

%location of maximum camber
div=fix(remaind/100);
remaind=rem(remaind,100);
xmc=div/10;
if maxcamb==0, xmc=0.3; end

%thickness
t=remaind/100;

%Linea media antes del punto de maxima curvatura
z=-(x/xmc-1).^2+1;
z=z.*(x<xmc);

%Linea media despues del punto de maxima curvatura
z=z+(1-((x-xmc)./(1-xmc)).^2).*(x>=xmc);

z=z*maxcamb;

%Pendiente de la linea media

%Antes del punto de maxima curvatura
dzdx=2*(1-x/xmc)/xmc.*(x<xmc);

%Despues del punto de maxima curvatura
dzdx=dzdx-2*(x-xmc)/(1-xmc)^2 .*(x>=xmc);

dzdx=dzdx*maxcamb;

%Espesor del perfil
delta  = (5*t)*(0.2969*(x.^0.5) - 0.126*x - 0.3516*(x.^2) ...
                          + 0.2848*(x.^3) - 0.1041*(x.^4) ) ;

zup=x+i*z+sign(-dzdx+i*1).*delta;
zdown=x+i*z-sign(-dzdx+i*1).*delta;

zprof=[zup;zdown(n:-1:2)];

plotc(zprof)
plota(x+i*z,'-g')
xlabel('x')
ylabel('y')
title(['NACA' num2str(digits) '   profile'])
%reqmet
