function [xh_grid,yh_grid,uh_grid,Nx,Ny]=view_sol(xnod,icone,u);
%
%    [xh_grid,yh_grid,uh_grid,Nx,Ny]=view_sol(xnod,icone,u);
%
% routine to view the solution of an scalar field 
% on an unstructured mesh projected on an strutured one .
%
% Inputs :
%--------------
%      xnod
%     icone
%       sol

toler = 1e-10;
zerom = -toler;
onep = 1+toler;

% generate an L-shape mesh for testing this program
%-----------------------------------------------------
% Lx = 2; Ly=2; hx=1; hy=1; nx = 10; ny=10;
%
%[xnod,icone]=mesh_L(Lx,Ly,hx,hy,nx,ny);

% generate a solution for testing this program
%--------------------------------------------------------
umin = min(u);
if umin < 0 , umin = 1.5*umin ; else umin=0.7*umin ; end

% compute the bounding box of the whole domain
%------------------------------------------------------
[x_box,hele,Nx,Ny,xelemin,xelemax,yelemin,yelemax] = ...
bou_box(xnod,icone);

% we divide the set of cuadrangular elements in the mesh
% in triangular elements.
[numel,nen]=size(icone);
ico_new=[];
 for k=1:numel
  ico_new=[ico_new ; [icone(k,1:3) ]];
 end
 for k=1:numel
  ico_new=[ico_new ; [icone(k,[3 4 1])]];
 end
icone=ico_new;
xelemin = [xelemin;xelemin];
yelemin = [yelemin;yelemin];
xelemax = [xelemax;xelemax];
yelemax = [yelemax;yelemax];

% generation of the projection grid
%--------------------------------------
x_g = (x_box(1,1):hele:x_box(2,1))';
y_g = (x_box(1,2):hele:x_box(3,2))';
x_grid=[];
 for k=1:Ny+1,
  x_grid = [x_grid ; [ x_g ones(Nx+1,1)*y_g(k)] ];
 end

% compute in which element on the original
% mesh is placed each node of the fine grid.
%
% Loop over the elements in the original mesh
%     built bounded box (bbe) for the element
%      ---> (xelemin;elemax;yelemin;yelemax) is the bbe
%     determine the set of nodes on the fine grid belonging to bbe   put a flag assigning to each node in the fine grid an element
%                in the original mesh
% end
%

[numel,nen]=size(icone);
[numnp,ndm]=size(xnod);
[numnp_fine,ndm]=size(x_grid);

node_to_elem = zeros(numnp_fine,1);
u_grid = umin*ones(numnp_fine,1);

for iele=1:numel
  % integer coordinates of the corners in the
  % bounded box of this element with IP's and JP's nodes on
  % the fine grid.
  %
  %      3 ----------------- 4
  %       |                 |
  %       |                 |
  %       |                 |
  %      1 ----------------- 2
  %
  IP1 = 1 + fix((xelemin(iele)-x_box(1,1))/hele);
  JP1 = 1 + fix((yelemin(iele)-x_box(1,2))/hele);
  IP2 = 2 + fix((xelemax(iele)-x_box(1,1))/hele);
  if IP2 > Nx+1, IP2=Nx+1; end
  JP2 = JP1;
  IP3 = IP1;
  JP3 = 2 + fix((yelemax(iele)-x_box(1,2))/hele);
  if JP3 > Ny+1, JP3=Ny+1;end
  IP4 = IP2;
  JP4 = JP3;

% set of nodes on fine grid that belongs to the bbe

 setnodes = [];
 for k=JP1:JP3,
   setnodes=[setnodes;(k-1)*(Nx+1)+(IP1:IP2)' ];
 end

% coordinates of the chosen nodes

 x_set = x_grid(setnodes,:);

 x_t = xnod(icone(iele,1:3),1:2);

%  transformation matrix between the original and master domain
  A = [x_t(2,1)-x_t(1,1) x_t(3,1)-x_t(1,1) ; 
       x_t(2,2)-x_t(1,2) x_t(3,2)-x_t(1,2) ];

  A = inv(A);

  b = (x_set(:,1) - x_t(1,1));
  b = [b (x_set(:,2) - x_t(1,2))];
% natural coordinates of the fine grid nodes in the bbe
  psi_set = (A*b')';

% choose the nodes that belong to the element

  iflag = ( ...
  psi_set(:,1)>=zerom & psi_set(:,1)<= onep & ...
  psi_set(:,2)>=zerom & psi_set(:,2)<=(psi_set(:,1)+toler) );  

  nn = length(iflag);
%  inodes=[];
%  for kk=1:nn,
%   inodes = [inodes;setnodes(kk)];
%  end

%  nn = length(inodes);
%  node_to_elem(inodes,1) = ones(nn,1)*iele;

% compute the scalar field in each node belonging to the element
%-----------------------------------------------------------------

   uele = u(icone(iele,1:3));

   u_grid(setnodes,1)= ...
   (ones(nn,1)-psi_set(:,1)-psi_set(:,2))*uele(1) + ...
   psi_set(:,1)*uele(2) + psi_set(:,2)*uele(3) ;

end

xh_grid=[];
yh_grid=[];
uh_grid=[];
for k=1:Ny+1,
 in=(k-1)*(Nx+1)+(1:Nx+1);
 xh_grid=[xh_grid x_grid(in,1)];
 yh_grid=[yh_grid x_grid(in,2)];
 uh_grid=[uh_grid u_grid(in,1)];
end

mesh(x_grid(1:Nx+1,1),x_grid(1:Nx+1:(Nx+1)*(Ny+1),2),uh_grid'),pause

Nx = Nx+1;
Ny = Ny+1;
