function [ar,emax,emin,diangle] = ar3d(xnod,icone,ijob)
%
%   [ar,emax,emin,diangle] = ar3d(xnod,icone,ijob)
%
%   ar : aspect ratio
%   emin : minimal edge length element by element
%   emax : maximal edge length element by element
%
% diangle : dihedral angle distribution (if ijob==1)
%

if nargin<3, ijob=0; end

big_amount = 1e5;
big = 1e15;

ar=[]; 
[numel,nen]=size(icone);
[nnod,ndm]=size(xnod);

if nen==3,
    edges = [1,2 ; 2,3 ; 3,1];
elseif (nen==4)&(ndm==2),
    edges = [1,2 ; 2,3 ; 3,4 ; 4,1];
elseif (nen==4)&(ndm==3),
    edges = [1,2 ; 2,3 ; 3,1 ; 1,4 ; 2,4 ; 3,4];
elseif (nen==6)&(ndm==3), 
    edges = [1,2 ; 2,3 ; 3,1 ; 4,5 ; 5,6 ; 6,1 ; 1,4 ; 2,5 ; 3,6];    
elseif nen==8,
    edges = [1,2 ; 2,3 ; 3,4 ; 4,1 ; 
        5,6 ; 6,7 ; 7,8 ; 8,5 ; 
        1,5 ; 2,6 ; 3,7 ; 4,8 ];
else
    error(' Error in the type of element')
end

n_edges = length(edges(:,1));

edges_h = [];

flag_diangle = (ijob==1);

if size(icone,1)>big_amount
    nchunks = round(size(icone,1)/big_amount);
else
    nchunks = 1;
end

nele_chunk = round(size(icone,1)/nchunks);

emin = [];
emax = [];

for ich=1:nchunks
    eles_1 = (ich-1)*nele_chunk+1;
    eles_2 = min([(ich)*nele_chunk,size(icone,1)]);   
    
    dsth=[];
    eles = (eles_1:eles_2);
    disp([ num2str(ich) '/' num2str(nchunks)])
    for iedge=1:n_edges,
        nodo_1 = icone(eles,edges(iedge,1));
        nodo_2 = icone(eles,edges(iedge,2));
        dst = (xnod(nodo_2,:)-xnod(nodo_1,:));
        if flag_diangle,
            edges_h = [edges_h, dst];
        end
        dst = sqrt(sum(dst'.^2))';
        dsth =[dsth,dst];
    end
    
    emin = [emin;min(dsth')'];
    emax = [emax;max(dsth')'];
    
    clear dst nodo_1 nodo_2
    pack
end

nn = find(emin>0);
ar = big*ones(numel,1);
ar(nn) = emax(nn) ./ emin(nn);

if flag_diangle,
    if ndm==2,
        for iedge=1:n_edges,
            jedge=iedge+1;
            if jedge>n_edges, jedge=1; end
            vaux1 = -edges_h(:,(iedge-1)*ndm+(1:ndm));    
            vaux2 = edges_h(:,(jedge-1)*ndm+(1:ndm));    
            %    if ndm==2, vaux1(:,3)=0; vaux2(:,3)=0; end
            %    vaux  = pvec(vaux1,vaux2);
            vaux  = sum((vaux1.*vaux2)')';
            vmod1 = sqrt(sum(vaux1'.^2))';
            vmod2 = sqrt(sum(vaux2'.^2))';    
            %    vmod  = sqrt(sum(vaux'.^2))';  
            vmod = vaux;
            angle = acos(vmod./(vmod1.*vmod2));
            if iedge==1, 
                diangle = angle;
            else
                diangle = min([diangle';angle'])';
            end
        end
        
        
    elseif (nen==4)&(ndm==3)
        
        % defining the four plane normals
        % plane # 1 : edges 1 & 3
        iedge = 1;
        vaux1 = edges_h(:,(iedge-1)*3+(1:3));
        iedge = 3;    
        vaux2 = -edges_h(:,(iedge-1)*3+(1:3));
        nn = pvec(vaux1,vaux2);
        nn_mod = sqrt(sum(nn'.^2))';
        n1 = nn./(nn_mod*ones(1,3));
        
        % plane # 2 : edges 5 & 2
        iedge = 5;
        vaux1 = edges_h(:,(iedge-1)*3+(1:3));
        iedge = 2;    
        vaux2 = edges_h(:,(iedge-1)*3+(1:3));
        nn = pvec(vaux1,vaux2);
        nn_mod = sqrt(sum(nn'.^2))';
        n2 = nn./(nn_mod*ones(1,3));
        
        % plane # 3 : edges 3 & 4
        iedge = 3;
        vaux1 = -edges_h(:,(iedge-1)*3+(1:3));
        iedge = 4;    
        vaux2 = edges_h(:,(iedge-1)*3+(1:3));
        nn = pvec(vaux1,vaux2);
        nn_mod = sqrt(sum(nn'.^2))';
        n3 = nn./(nn_mod*ones(1,3));
        
        % plane # 4 : edges 4 & 1
        iedge = 4;
        vaux1 = edges_h(:,(iedge-1)*3+(1:3));
        iedge = 1;    
        vaux2 = edges_h(:,(iedge-1)*3+(1:3));
        nn = pvec(vaux1,vaux2);
        nn_mod = sqrt(sum(nn'.^2))';
        n4 = nn./(nn_mod*ones(1,3));
        
        clear vaux1 vaux2 nn nn_mod 
        pack
        
        % compute minimum dihedral angle
        diangle = zeros(length(n1),1);
        vaux = acos(sum((n1.*n2)')');     vauxb = acos(sum((-n1.*n2)')'); diangle(:,1) = min([abs(vaux)';abs(vauxb)'])';
        vaux = acos(sum((n1.*n3)')');     vauxb = acos(sum((-n1.*n3)')'); vauxa = min([abs(vaux)';abs(vauxb)'])';diangle=min([diangle';vauxa'])';
        vaux = acos(sum((n1.*n4)')');     vauxb = acos(sum((-n1.*n4)')'); vauxa = min([abs(vaux)';abs(vauxb)'])';diangle=min([diangle';vauxa'])';
        vaux = acos(sum((n2.*n3)')');     vauxb = acos(sum((-n2.*n3)')'); vauxa = min([abs(vaux)';abs(vauxb)'])';diangle=min([diangle';vauxa'])';
        vaux = acos(sum((n2.*n4)')');     vauxb = acos(sum((-n2.*n4)')'); vauxa = min([abs(vaux)';abs(vauxb)'])';diangle=min([diangle';vauxa'])';
        vaux = acos(sum((n3.*n4)')');     vauxb = acos(sum((-n3.*n4)')'); vauxa = min([abs(vaux)';abs(vauxb)'])';diangle=min([diangle';vauxa'])';
                
    else
        error(' Diangle : Not implemented yet for this type of element ')
    end
end
