%
%  generate cylindrical mesh
%

% generamos malla 2D

Hmin = 0;
Ht    = 25;
Hmax = Hmin+Ht;

problem_size = 100;

if problem_size==100
    Nh = 60;
    Nr = 20;
    Nt = 20;        
elseif problem_size==400
    Nh = 120;
    Nr = 22;
    Nt = 30;    
elseif problem_size==1000
    Nh = 240;
    Nr = 30;
    Nt = 37;
end    

R = 0.3;
Ri = 0.35*R;

if rem(45,90/Nt)~=0
    cc = fix(45/(90/Nt));
    Nt = 2*cc;
end

eta = (0:Nt)'/Nt;
psi = (0:Nr)'/Nr; 
if 0
    psi = 1-(1-psi).^2;
end
theta = eta*pi/2;

rnod = []; bfh=[]; icone=[]; 
ico1 = [(1:Nr)',(2:Nr+1)'];
for k=1:Nt+1
    theta_p = theta(k);
    if theta_p>pi/4,
        Yi   = Ri;
        Xi   = Ri*tan(pi/2-theta_p);        
    else
        Xi   = Ri;
        Yi   = Ri*tan(theta_p);
    end
    ri = sqrt(Xi.^2+Yi.^2);
    bfh = [bfh ; [ri,theta_p]];
    r    = psi*(R-ri)+ri;    
    rnod = [rnod;[r,ones(Nr+1,1)*theta(k)]];
    if k<=Nt, icone = [icone;[ico1+(k-1)*(Nr+1) , ico1(:,[2,1])+k*(Nr+1)]]; end
    %    keyboard;pause
    
end

figure(1);clf;plot(rnod(:,1),rnod(:,2),'.k')

xnod = [rnod(:,1).*cos(rnod(:,2)) , rnod(:,1).*sin(rnod(:,2))];

figure(2);clf;plot(xnod(:,1),xnod(:,2),'.k');axis equal;grid on

q1 = bfh(Nt/2+1:end,:);
q2 = bfh(1:Nt/2+1,:);
x1 = q1(:,1).*cos(q1(:,2));
x2 = q2(:,1).*sin(q2(:,2));

[xnod2,icone2]=qq3d([x1,0*x1],[0*x2,x2]);

figure(2);hold on;plot(xnod2(:,1),xnod2(:,2),'ro');

[numnp1,bas]=size(xnod);
xnod3 = [xnod;xnod2];
icone3 = [icone;icone2+numnp1];

figure(1);clf;pltmsh(xnod3,icone3);

[ar,emax,emin] = ar3d(xnod3,icone3);
tol = 0.1*min(emin);

[xnew,iconew,imap] = sew_mesh(xnod3,icone3,tol);

% generamos el segundo cuadrante
[numnp,bas]=size(xnew);
Rota = [cos(pi/2),-sin(pi/2);sin(pi/2),cos(pi/2)];
xnew2 = (Rota*xnew')';
xnew2 = [xnew;xnew2];
iconew2 = [iconew;iconew+numnp];

[xnew,iconew,imap] = sew_mesh(xnew2,iconew2,tol);

figure(2);clf;pltmsh(xnew,iconew);

if 1
% generamos la otra mitad
[numnp,bas]=size(xnew);
Rota = [cos(pi),-sin(pi);sin(pi),cos(pi)];
xnew2 = (Rota*xnew')';
xnew2 = [xnew;xnew2];
iconew2 = [iconew;iconew+numnp];

[xnew,iconew,imap] = sew_mesh(xnew2,iconew2,tol);

figure(3);clf;pltmsh(xnew,iconew);
end

% extrudo en la longitud
xnod = xnew; icone = iconew;
[numnp,bas] = size(xnod);
icone3d = [];
x3d = [xnod,ones(length(xnod(:,1)),1)*Hmin];
H = (0:Nh)'/Nh*Ht+Hmin;
for k=1:Nh
    x3d=[x3d;[xnod,ones(length(xnod(:,1)),1)*H(k+1)]];
    icone3d = [icone3d;[ numnp*(k-1)+icone,numnp*k+icone]];
end

% cambio ejes y numeraciones para que el eje de extrusion sea el eje Y
% lo necesito porque asi estan las fuentes puntuales acomodadas en el CAD
x3d = [x3d(:,1) , x3d(:,3) , -x3d(:,2)];
icone3d = icone3d(:,[1,4,3,2,5,8,7,6]);

%keyboard;pause