function [Drag,Lift] = Drag_Lift_body(xnod,icone,ifaceb,state_ifaceb,visco)
%
%      [Drag,Lift] = Drag_Lift_body(xnod,icone,ifaceb,state,visco)
%
%                Drag and Lift forces 
%
%   Input: 
%         state_ifaceb [node , state] at the boundary over a body
%         ifaceb       [elem , face ] at the boundary over a body
%   
%

% Datos dependiente de la aplicacion
idf_pressure = 3;
idf_u        = 1;
idf_v        = 2; 
ndf_sec      = 2; % cantidad de grados de libertad de variables secundarias
tol          = 1e-12;

[nnod,ndm]=size(xnod);
[nele,nen]=size(icone);
[neleb,bas]=size(ifaceb);
[bas,ndf]=size(state_ifaceb);

ndf = ndf-1;  % el primer grado de libertad de state_ifaceb es el nro de nodo

U = zeros(nnod,ndf);
nodes_bou = state_ifaceb(:,1);
U(nodes_bou,1:ndf) = state_ifaceb(:,2:ndf+1);

ceros    = zeros(nele,nen*ndf);
  
Rg   = zeros(nnod,2);
Mg   = zeros(nnod,1);

Rele = zeros(nele,2*nen);
Mele = zeros(nele,1*nen);
  
npg = 2;
xpg = sqrt(3)/3;
xpg = [-xpg xpg];
wpg = [1 1];

eles = ifaceb(:,1);
faces = ifaceb(:,2);

n1 = faces;
n2 = n1+1;
n2 = n2.*(n2<=nen)+(n2>nen);
n1 = getcol(icone(eles,:),n1);
n2 = getcol(icone(eles,:),n2);
dd = xnod(n2,:)-xnod(n1,:);
dl = sqrt(sum(dd'.^2))';
dd = dd./(dl*[1,1]);
nx = dd(:,2); ny = -dd(:,1);
nor = [nx,ny];

% Localizacion
%------------------------------------------------------------------
Ue    = ceros;
for inod=1:nen,
   for idf=1:ndf,
     nodes = icone(eles,inod);
     Ue(eles,(inod-1)*ndf+idf) = U(nodes,idf);
   end 
end

% Loop sobre los puntos de Gauss
for ipg=1:npg,

  Nb(1)  = 0.5*(1-xpg(ipg));
  Nb(2)  = 0.5*(1+xpg(ipg));
  xp(:,1) = xnod(n1,1)*Nb(1)+xnod(n2,1)*Nb(2);
  xp(:,2) = xnod(n1,2)*Nb(1)+xnod(n2,2)*Nb(2);
  [psi] = x2psi(xnod,icone(eles,:),xp);
  mask = (psi<tol);
  psi = psi.*(1-mask)+tol*mask;
  
  [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psi(:,1),psi(:,2));
  [dNdx,dNdy,N]  = shapefun(X2S,psi(:,1),psi(:,2),3); 
  Jaco           = dl/2;
  xsj            = wpg(ipg)*Jaco;
  
  Upg    = zeros(length(eles),ndf);
  dUdxpg = zeros(length(eles),ndf);
  dUdypg = zeros(length(eles),ndf);
  
  for inod=1:nen,
   for idf=1:ndf,
      Upg(:,idf)    =    Upg(:,idf) +    N(:,inod).* Ue(eles,(inod-1)*ndf+idf);
      dUdxpg(:,idf) = dUdxpg(:,idf) + dNdx(:,inod).* Ue(eles,(inod-1)*ndf+idf);
      dUdypg(:,idf) = dUdypg(:,idf) + dNdy(:,inod).* Ue(eles,(inod-1)*ndf+idf);            
   end   
   Mele(eles,inod) = Mele(eles,inod) + N(:,inod).*xsj;      
  end
    
  for inod=1:nen,        
     % pressure forces
     Fpx = Upg(:,idf_pressure).*nx;
     Fpy = Upg(:,idf_pressure).*ny;
     % viscous forces
     Fvx = visco*(dUdxpg(:,idf_u).*nx+dUdypg(:,idf_u).*ny);
     Fvy = visco*(dUdxpg(:,idf_v).*nx+dUdypg(:,idf_v).*ny);     
     Rele(eles,(inod-1)*ndf_sec+1) = Rele(eles,(inod-1)*ndf_sec+1) + N(:,inod).*(Fpx+Fvx).*xsj;
     Rele(eles,(inod-1)*ndf_sec+2) = Rele(eles,(inod-1)*ndf_sec+2) + N(:,inod).*(Fpy+Fvy).*xsj;     
  end % inod

 end % ipg
 
 for inod=1:nen,
   nodes = icone(:,inod);
   for idf=1:ndf_sec,
    Rg(nodes,idf) = ... 
        Rg(nodes,idf) + Rele(:,(inod-1)*ndf_sec+idf);      
   end
    Mg(nodes,1) = Mg(nodes,1) + Mele(:,inod);          
  end
  
  mask = (Mg(:,1)<tol);
  Mg(:,1)=Mg(:,1).*(1-mask)+mask;
  Drag = Rg(:,1)./Mg(:,1);
  Lift = Rg(:,2)./Mg(:,1);  
  
