function [xnew,iconew,phinew] = reftri(xnod,icone,N,phi)
%
%       [xnew,iconew,phinew] = reftri(xnod,icone,N,phi)
%
% Rutina para refinar homogeneamente una malla de elementos 
% triangulares en N x N subtriangulos 
%
% Input:  La malla original de triangulos 
%         Un vector incognita phi con NDF grados de libertad
%         La particion N
%          

% triangulo original
%xnod  = [1 -1 ; 2 0 ; 0 3 ; 2 2 ; -2 -4];
%phi   = [0 0 ; 1 -1 ; 2 -2 ; 3 3; -1 -1];
%icone = [1 2 3 ; 2 4 3 ; 1 3 5];
%N = 11;  % numero de nodos

[numel,nen]=size(icone);
[numnp,ndm]=size(xnod);


% Dado un triangulo original lo dividimos en N x N subtriangulos
% Numeracion de los nodos de la submalla triangular
% Dado un par (k,l) la numeracion es la siguiente
%
%  (k,l) -->  l*N - l*(l-1)/2 + k
% con k=1:N-l y l=1:N

% matriz y vector de nodos
xa=[];
for l=1:N,
 for k=1:N+1-l,
% for k=1:N-l,
   ia(l,k) = (l-1)*N - (l-1)*(l-2)/2 + k ;
   xa = [ xa ; [(k-1)/(N-1) (l-1)/(N-1)] ] ;
 end
end

% conectividades de la submalla triangular
ja = [];
for l=1:N-1,
% for k=1:N-l-1,
 for k=1:N-l,
   ja = [ja ; 
              [ia(l,k) ia(l,k+1) ia(l+1,k)] ;
              [ia(l,k+1) ia(l+1,k+1) ia(l+1,k)]  ];
 end
 k = N-l;
 ja = [ja ;  [ia(l,k) ia(l,k+1) ia(l+1,k)] ];
end

% mapeo al triangulo original

xnew=[];
iconew=[];
phinew=[];

for k=1:numel,
 n1=icone(k,1);n2=icone(k,2);n3=icone(k,3);  
 xnew  = [xnew ;  ... 
      xnod(n1,1) + ... 
   (xnod(n2,1)-xnod(n1,1))*xa(:,1)+(xnod(n3,1)-xnod(n1,1))*xa(:,2) ...
      xnod(n1,2) + ... 
   (xnod(n2,2)-xnod(n1,2))*xa(:,1)+(xnod(n3,2)-xnod(n1,2))*xa(:,2)];
 iconew = [iconew ; (k-1)*N*(N+1)/2+ja];
end

if nargin>3,
[numnp,ndf]=size(phi);
for k=1:numel,
 phi_idf=[];
 for idf=1:ndf, 
  n1=icone(k,1);n2=icone(k,2);n3=icone(k,3);  
  phi_idf = [ phi_idf ...  
       phi(n1,idf) + (phi(n2,idf)-phi(n1,idf)).*xa(:,1) + ...
                (phi(n3,idf)-phi(n1,idf))*xa(:,2) ];
 end
 phinew=[phinew;phi_idf];
end
end