%------------------------------------------------------------
%
%   Generador de malla ad-hoc
%
%   Flujo alrededor de un cilindro 2D
%
%------------------------------------------------------------
%------------------------------------------------------------
%-------------------- Datos
%------------------------------------------------------------
R = 1;
H = 21;
V = 21;
Hdown = 42;
ntheta = 41;
nradio = 71;
ndown = 70;
limit = 1e-6;
%------------------------------------------------------------
%------------------ Fin Datos

%------------------------------------------------------------
% Generacion de un octavo de malla alrededor de un cilindro
%------------------------------------------------------------
theta = (1:ntheta-2)'/(ntheta-1);
theta = theta*pi/4;

x1 = R * cos(theta+pi/4);
y1 = R * sin(theta+pi/4);

x2 = (ntheta-2:-1:1)'/(ntheta-1);
x2 = x2 * H ;
y2 = ones(size(x2)) * V ;

xoct = [];
for k=1:nradio,
  alpha = (k-1)/(nradio-1);
  alpha = alpha^2;
  x = (1-alpha)*x1+alpha*x2;
  y = (1-alpha)*y1+alpha*y2;
  xoct = [xoct ; [x y] ];
end

% Generacion de un cuarto de malla
%------------------------------------------------------------
xoct_mirror = [];
for k=1:nradio,
  nodes = (k-1)*(ntheta-2)+(ntheta-2:-1:1)';
  xoct_mirror = [ xoct_mirror ; [ xoct(nodes,2) xoct(nodes,1)] ];
end

% Agregar hilera de nodos en la interfase entre octavos
%------------------------------------------------------------

xa=[]; xb=[]; ya=[]; yb=[];

for k=1:nradio,
  alpha = (k-1)/(nradio-1);
  alpha = alpha^2;
  x  = (1-alpha)*(R*cos(pi/4))+alpha*H;
  y  = (1-alpha)*(R*sin(pi/4))+alpha*V;
  xa = [xa ; x ];
  ya = [ya ; y ];
  x  = (1-alpha)*(R*cos(pi/2))+alpha*0;
  y  = (1-alpha)*(R*sin(pi/2))+alpha*V;
  xb = [xb ; x ];
  yb = [yb ; y ];
end

xqua = [];
for k=1:nradio,
  xqua = [xqua ; [ xb(k) yb(k)]; xoct((k-1)*(ntheta-2)+(ntheta-2:-1:1),:) ; 
    [ xa(k) ya(k)]; xoct_mirror((k-1)*(ntheta-2)+(ntheta-2:-1:1),:) ];
end

nqua = length(xqua(:,1));

% Generacion de las conectividades del cuarto de malla
%------------------------------------------------------------

ico_qua = [];
nico = 2*(ntheta-1);
ico = [(1:nico-1)' (2:nico)' nico+(2:nico)' nico+(1:nico-1)' ];
ico = [ico ; [nico nqua+1 nqua+1+nico 2*nico] ];

for k=1:nradio-1,
  ico_qua = [ico_qua ; (k-1)*nico+ico ];
end

% Identificamos los nodos a cocer entre cuartos
%------------------------------------------------------------
nelequa = length(ico_qua(:,1));
nodo_inter = (nico:nico:nqua)';

% Generacion de la malla completa
%------------------------------------------------------------

xnod = xqua;
icone = ico_qua;

for k=1:3,
  rota = [ cos(k*pi/2) +sin(k*pi/2) ; -sin(k*pi/2) cos(k*pi/2) ];
  xnod = [xnod ; (rota*xqua')' ];
  if k==3,
    ico_qua(nico:nico:nelequa,2) = ico_qua(1:nico:nelequa,1)-k*nqua;
    ico_qua(nico:nico:nelequa,3) = ico_qua(1:nico:nelequa,4)-k*nqua;
  end
  icone = [icone ; k*nqua + ico_qua ];
end

% Agregado block downstream
%------------------------------------------------------------

nodes = find(abs(xnod(:,1)-H)<limit);
[yint,iii]=sort(xnod(nodes,2));
nodes = nodes(iii);
nint = length(yint);

dxdown = Hdown/ndown;
xdown = [];
for k=1:ndown,
  xdown = [ xdown ; [(H+k*dxdown)*ones(nint,1) yint]];
end
icodown=[];
ico = [ (2:nint)' (1:nint-1)'  nint+(1:nint-1)' nint+(2:nint)' ] - nint;

for k=1:ndown,
  icodown = [icodown ; ico+(k-1)*nint];
end

icodown = icodown+length(xnod(:,1));
xnod = [xnod;xdown];

icodown(1:nint-1,1) = nodes(2:nint);
icodown(1:nint-1,2) = nodes(1:nint-1);

icone = [icone;icodown];

% Boundary conditions
%------------------------------------------------------------
nodes_in  = find(abs(xnod(:,1)+H)<limit);
nodes_out = find(abs(xnod(:,1)-(H+Hdown))<limit);
mask = (abs(xnod(:,2)-V)<limit)&(xnod(:,1)>-H+limit)&(xnod(:,1)<(H+Hdown)-limit);
nodes_top = find(mask==1);
mask = (abs(xnod(:,2)+V)<limit)&(xnod(:,1)>-H+limit)&(xnod(:,1)<(H+Hdown)-limit);
nodes_bot = find(mask==1);
radio = sqrt(xnod(:,1).^2+xnod(:,2).^2);
nodes_cyl = find(abs(radio-R)<limit);

% PARA FEAP
%fixa=[];
%id = [0,0,1];
%fixa = [fixa ; [nodes_out  ones(length(nodes_out),1)*id ]];
%id = [0,1,0];
%fixa = [fixa ; [nodes_top  ones(length(nodes_top),1)*id ]];
%id = [0,1,0];
%fixa = [fixa ; [nodes_bot  ones(length(nodes_bot),1)*id ]];
%id = [1,1,0];
%fixa = [fixa ; [nodes_cyl ones(length(nodes_cyl),1)*id ]];
%id = [1,1,0];
%fixa = [fixa ; [nodes_in  ones(length(nodes_in),1)*id ]];

% PARA FEM.CPP
fixa = [];
nn = [nodes_in];
fixa = [fixa ; [ nn ones(length(nn),1) ones(length(nn),1) ]];
nn = [nodes_cyl];
fixa = [fixa ; [ nn ones(length(nn),1) zeros(length(nn),1) ]];
nn = [nodes_in;nodes_cyl;nodes_bot;nodes_top;nodes_out];
fixa = [fixa ; [ nn 2*ones(length(nn),1) zeros(length(nn),1) ]];
nn = [nodes_out];
fixa = [fixa ; [ nn 3*ones(length(nn),1) zeros(length(nn),1) ]];

% Guardamos la malla
% ---------------------

my_fprintf('XNOD','%10.5f%10.5f \n',xnod);
my_fprintf('ICONE','%10i%10i%10i%10i \n',icone);
my_fprintf('FIXA','%10i%10i%10.5f \n',fixa);