function [phi,Rg]=solver(phi,Rg,Kg,fixa,prop,prop_solver,ndf)
%
%    [phi,Rg]=solver(phi,Rg,Kg,fixa,prop,prop_solver,ndf)
%
%
%   Rutina SOLVER de resolucion del sistema de ecuaciones
%   
%   Type_solver = 0 ---> metodo directo con un parametro de
%                        parametro de relajacion "relax"
%                        grado de implicitud (1/2 Cranck Nicholson) "alpha" (use >0)
%               = 1 ---> Line search
%               = 2 ---> Explicito - Forward Euler
%               = 3 ---> GMRES   

type_solver = prop_solver(1);
relax       = prop_solver(2);
nu_max      = prop_solver(3);
alfa_min    = prop_solver(4);
alpha       = prop_solver(5);
ntime       = prop_solver(6);
nstep       = prop_solver(7);
scaling     = prop_solver(8);

if type_solver==0,
     
     if alpha==0, alpha=1; end
     alpha = prop(14);
     free = find(fixa==0);
     Rg = Rg(free);
     Kg = Kg(free,free);
if scaling,
% SCALING
     scalev = sum(abs(Kg'))';
	 Rg   = Rg ./ scalev;
	 Kg   = diag(scalev)\Kg;
% FIN SCALING
end


     dphi = Kg\Rg;
     dphi = dphi/alpha;
     phiv = m2v(phi);
     phiv(free) = phiv(free)+ relax*dphi;
     dphi = relax*dphi;
     phi  = v2m(phiv,ndf);

  elseif type_solver==1,
      
     linesear

  elseif type_solver==2,

% solver Forward Euler explicito 

     free = find(fixa==0);
     Rg = Rg(free);
%     Kg = diag(Kg(free,free));
     Kg = Kg(free);     
     dphi = Rg./Kg;
     phiv = m2v(phi);
     phiv(free) = phiv(free)+ dphi;
     phi  = v2m(phiv,ndf);

  elseif type_solver==3,

     free = find(fixa==0);
     Rg = Rg(free);
     Kg = Kg(free,free);
     
     if scaling==1
     % scaling
     scalev = sum(abs(Kg'))';
     Rg   = Rg ./ scalev;
     Kg   = diag(scalev)\Kg;
     % end scaling
     end
  
     TOL = 1e-5;
     MAXIT = 25;
     RESTART = 25;
     %M1=[]; M2=[]; 
     %M1 = diag(diag(Kg));
     %M2 = eye(size(Kg));
     [M1,M2] = luinc(Kg,TOL); 
     phiv = m2v(phi);     
     X0 = phiv(free);
     [dphi,FLAG,RELRES] = GMRES(Kg,Rg,RESTART,TOL,MAXIT,M1,M2,X0);
%     disp([' GMRES --- flag & |R| ' num2str(FLAG) '  &  ' num2str(RELRES) ])
     phiv(free) = phiv(free)+ dphi;
     phi  = v2m(phiv,ndf);

  else
	 error(' Bad choice of Solver type [0,1,2] ')
  end
  
  if 0,  % aun no se si se puede quitar esto 
     nofree = find(fixa==1);
     phiv = m2v(phi);
     phiv(nofree) = forc(nofree);
     phi  = v2m(phiv,ndf);
  end
  
