function [Kele,Rele] = bc_thermal_contact(xnod,icone,T,thermal_contact_array,Kele,Rele,ijob,axisim)
%
%   [Kele,Rele] = bc_thermal_contact(xnod,icone,T,thermal_contact_array,Kele,Rele,ijob,axisim)
%
%   Thermal contact between 2 layers of nodes
%
%   thermal_contact_array : [ele_master face_master ele_slave face_slave contact_coefficient]
%

teco = thermal_contact_array;

[numel,nen] = size(icone);
[nteco,bas] = size(teco);
[numnp,ndm] = size(xnod);

if ndm~=2, error(' bc_thermal_contact ONLY for 2D problems ');end

npg = 2;

if nteco~=0,
   
  pgauss1d;
  nodes_master = teco(:,1:2);
  nodes_slave  = teco(:,3:4);   
  coef_teco    = teco(:,5);  

  lteco  = sqrt((xnod(nodes_master(:,1),1) - xnod(nodes_master(:,2),1)).^2 + ... 
                (xnod(nodes_master(:,1),2) - xnod(nodes_master(:,2),2)).^2);             
  Jaco   = lteco/2;

  re = [xnod(nodes_master(:,1),1) xnod(nodes_master(:,2),1)];

  Te_master = [T(nodes_master(:,1)) T(nodes_master(:,2))];
  Te_slave  = [T(nodes_slave (:,2)) T(nodes_slave (:,1))];  
  
  inod_slave = [4,3];
  
  for ipg=1:2,
     
   N = ones(nteco,1)*[(1-psipg(ipg))/2 , (1+psipg(ipg))/2 ];
   
% Localizacion , scatter   
   Tpg_master   = zeros(nteco,1);
   Tpg_slave    = zeros(nteco,1);   
   radpg        = zeros(nteco,1);
   
   for inod=1:2,    
    radpg        = radpg +  N(:,inod).*re(:,inod);
    Tpg_master   = Tpg_master +  N(:,inod).*Te_master(:,inod);
    Tpg_slave    = Tpg_slave  +  N(:,inod).*Te_slave (:,inod);    
   end 
        
   xsj = wpg(ipg)*Jaco;
   if axisim,
     xsj = xsj.*radpg;
   end
   
%   disp('bc_thermal_contact')
%   keyboard;pause
%   for iteco=1:nteco,
       for jnod=1:2,
           jnod_slave = inod_slave(jnod);
           vaux = N(:,jnod).*coef_teco.*(Tpg_master-Tpg_slave).*xsj;
           Rele(:,jnod,1)       = Rele(:,jnod,1) - vaux;                 
           Rele(:,jnod_slave,1) = Rele(:,jnod_slave,1) + vaux;
           for jjnod=1:2, 
               jjnod_slave = inod_slave(jjnod);
               vaux = N(:,jnod).*coef_teco.*N(:,jjnod).*xsj;
               Kele(:,jnod,1,jjnod,1) = Kele(:,jnod,1,jjnod,1)+ vaux;
               Kele(:,jnod_slave,1,jjnod_slave,1) = Kele(:,jnod_slave,1,jjnod_slave,1)+ vaux; 
               Kele(:,jnod,1,jjnod_slave,1) = Kele(:,jnod,1,jjnod_slave,1)- vaux;
               Kele(:,jnod_slave,1,jjnod,1) = Kele(:,jnod_slave,1,jjnod,1)- vaux;
           end % for iinod
       end  % for inod
%   end % for iteco
   
end % puntos de Gauss del contorno

end % if nteco

