%
%   Ejemplo Generacion de malla para una valvula
%  

%  Dividimos la valvula en partes:
%  [1] Cabeza - guia
%  [2] Cuello
%  [3] Cabeza - asiento

% Datos
D_head       = 40;   % diametro de la cabeza
D_stem       = 10;   % diametro del vastago
L_valve      = 90;   % longitud total de la valvula
R_neck       =  5;   % Radio del cuello
D_seat_in    = 30;   % diametro interior del asiento
D_seat_out   = 36;   % diametro exterior del asiento
H_seat_in    =  2;   % altura asiento parte interior
H_seat_out   =  4;   % altura asiento parte exterior
H_head       =  4;   % altura cabeza de valvula
L_guide      = 10;   % longitud de la guia
L_guide_up   = 10;   % longitud por sobre la guia
L_guide_down = 10;   % longitud por debajo de la guia
Npoints      = 12;   % numero de puntos que definen la geometria
h_default    = L_valve/60; % longitud de los elementos por default

Hs = h_default*ones(Npoints,1);
Hs(12) = 0.4;
Hs(5:9)  = 0.4;

% generamos el CAD
% puntos
P1 = [ D_stem/2 , 0];
P2 = P1 + [0 , L_guide_up];
P3 = P2 + [0 , L_guide];
P4 = P3 + [0 , L_guide_down];
L45 = L_valve-(L_guide_down+L_guide_up+L_guide+R_neck+H_seat_in+H_seat_out+H_head);
P5 = P4 + [0 , L45];
P6 = P5 + [R_neck , R_neck];
L67 = D_seat_in/2-(D_stem/2+R_neck);
P7 = P6 + [L67 , 0];
P8 = P7 + [(D_seat_out-D_seat_in)/2 , H_seat_in];
P9 = P8 + [(D_head-D_seat_out)/2 , H_seat_out];
P10 = P9 + [0 , H_head];
P11 = P10 + [-D_head/2 , 0];
P12 = P11 + [0 , -L_valve];

X_P = [];
for k=1:Npoints,
   eval(['X_P=[X_P ; P' num2str(k) '];'])
end

% obtenemos las longitudes de cada linea
for k=1:Npoints-1,
   H_lines(k) = norm(X_P(k+1,:)-X_P(k,:));
end
k=Npoints;
H_lines(k)=norm(X_P(k,:)-X_P(1,:));
H_lines=H_lines';

% Hs(8) y Hs(9) en funcion de Hs(12)
nnod_12 = round(H_lines(12)/Hs(12));
nnod_8=round(H_lines(8)/(H_lines(8)+H_lines(9))*nnod_12);
nnod_9=nnod_12-nnod_8+1;
Hs(8)=H_lines(8)/(nnod_8);
Hs(9)=H_lines(9)/(nnod_9);

% CAD - Lines
L_stem = L_guide_down+L_guide_up+L_guide+L45;

ip1=0;
ip2=1;
for k=1:Npoints-1,
   hele = Hs(k);
   ip1  = ip1+1; ip2=ip2+1;
   ref  = [1 1 1];
   [cc] = get_nodes_in_line(X_P([ip1,ip2],:),hele,ref);
   eval(['c' num2str(k) '=cc;']);
end

k=12;
hele = Hs(k);
ip1  = ip1+1; ip2=1;
ref  = [1 1 1];
[cc] = get_nodes_in_line(X_P([ip1,ip2],:),hele,ref);
eval(['c' num2str(k) '=cc;']);

k=5;
hele = Hs(k);
Lon  = R_neck*pi/2;
N    = round(Lon/hele);
angle = pi/2;
psi = (0:N-1)'/(N-1); theta= pi-psi*angle; 
xO = X_P(5,:)+[R_neck 0];
x=xO(1)+R_neck*cos(theta); y=xO(2)+R_neck*sin(theta);
cc = [x,y];
eval(['c' num2str(k) '=cc;']);

tol = 1e-10;
mask = (theta>3*pi/4); 
x0 = R_neck+D_stem/2;
y0 = L_valve-L_stem;
N2 = round(N/(1+y0/x0));
N1 = N-N2;
psi = (0:N1-1)'/(N1-1);
eta = (1:N2)'/N2;
x = [zeros(N1,1);eta*x0];
y = [psi*(L_valve-L_stem)+L_stem;ones(N2,1)*L_valve];
c5p = [x,y];

% Mesh generation of blocks
% block # 1 VASTAGO
C1 = [c1;c2;c3;c4];
C2(:,1) = C1(:,1)-D_stem/2;
C2(:,2) = C1(:,2);
C3 = [c12];
C4(:,1) = C3(:,1);
C4(:,2) = C3(:,2)+L_stem;
[xnod1,icone1] = gen_block_mesh(C2,C1,C3,C4);
figure(1);clf;pltmsh(-xnod1,icone1);axis(-L_valve*[1 0 1 0 ])
clear C1 C2 C3 C4

% block # 2 CUELLO
C1 = c5p;
C2 = c5;
C3 = c12;
C3(:,2) = C3(:,2)+L_stem;
N = length(C3(:,1));
psi = (0:N-1)'/(N-1);
C4(:,2) = L_valve-psi*(H_seat_in+H_seat_out+H_head);
C4(:,1) = ones(N,1)*(D_stem/2+R_neck);
[xnod2,icone2] = gen_block_mesh(C1,C2,C3,C4);
figure(1);pltmsh(-xnod2,icone2);axis(-L_valve*[1 0 1 0])
clear C1 C2 C3 %C4

% block # 3 CABEZA
C2 = [c6;c7];
N = length(C2(:,1));
psi = (0:N-1)'/(N-1);
psi = (C2(:,1)-C2(1,1))/(C2(N,1)-C2(1,1));
C1(:,1) = D_stem/2+R_neck+psi*(D_head/2-(D_stem/2+R_neck));
C1(:,2) = ones(N,1)*L_valve;
if 0,
   C4 = [flipud(c9(2:nnod_9,:));flipud(c8)];
   N = length(C4(:,1));
   psi = (0:N-1)'/(N-1);   
   psi = (C4(:,2)-C4(1,2))/(C4(N,2)-C4(1,2));   
   C3(:,2) = L_valve-psi*(H_seat_in+H_seat_out+H_head);   
   C3(:,1) = ones(N,1)*(D_stem/2+R_neck);
else
   C3=C4;
   C4 = [flipud(c9(2:nnod_9,:));flipud(c8)];
end
[xnod3,icone3] = gen_block_mesh(C1,C2,C3,C4);
figure(1);pltmsh(-xnod3,icone3);axis(-L_valve*[1 0 1 0])
%clear C1 C2 C3 C4

figure(1);hold on;plot(-X_P(:,1),-X_P(:,2),'o');axis(-L_valve*[1 0 1 0])

return

Nx  = 41;
Ny  = 41;
Lx  = 1;  
Ly  = 1; 

psi      = (0:Nx-1)'/(Nx-1);
x0       = 0;
x        = psi*Lx+x0; 
eta      = (0:Ny-1)'/(Ny-1)*1; 
y0       = 0;
y        = eta*Ly+y0;

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
  xnod=[xnod;[x ones(Nx,1)*y(k)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;

if quad2tri,
[numel,nen]=size(icone);

iconew=[];
for k=1:numel,
  vvv  = icone(k,:);
  ico  = Q2T(xnod(vvv,:));
  iconew = [iconew;vvv(ico(1,:));vvv(ico(2,:))];
end
icone=iconew;
end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
ndf = 1;
nodes_bot  = (1:Nx);
nodes_top  = Nx*(Ny-1)+(1:Nx);
nodes_lef  = 1:Nx:Nx*Ny;
nodes_rig  = Nx:Nx:Nx*Ny;

fixa= zeros(numnp,ndf);
fixa(nodes_lef,1)=ones(length(nodes_lef),1);
fixa(nodes_rig,1)=ones(length(nodes_rig),1);
fixa(nodes_top,1)=ones(length(nodes_top),1);
fixa(nodes_bot,1)=ones(length(nodes_bot),1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc = zeros(numnp,ndf);

r_i = 0.5;
r_e = 1.5;
theta = pi/4;
if 0,
% coordenada x
forc(nodes_top,1) = (r_i+(r_e-r_i)*xnod(nodes_top,1))*cos(theta);
forc(nodes_bot,1) = (r_i+(r_e-r_i)*xnod(nodes_top,1))*cos(0);
forc(nodes_lef,1) = r_i*cos(xnod(nodes_lef,2)*theta); 
forc(nodes_rig,1) = r_e*cos(xnod(nodes_rig,2)*theta); 

mask = (xnod(nodes_bot,1)>0.3)&(xnod(nodes_bot,1)<=0.7);
nn   = find(mask==1);
z    = naca4ds(0012,length(nn));
xprf = rot90(rot90(real(z(1:length(nn)))));
forc(nodes_bot(nn),1) = r_i+(0.3+xprf*(0.7-0.3))*(r_e-r_i);
[dmin,imin] = min(abs(xnod(nodes_bot,1)-0.3));
forc(nodes_bot(imin),1) = r_i+0.3*(r_e-r_i);

else
% coordenada y
forc(nodes_top,1) = (r_i+(r_e-r_i)*xnod(nodes_top,1))*sin(theta);
forc(nodes_bot,1) = (r_i+(r_e-r_i)*xnod(nodes_top,1))*sin(0);
forc(nodes_lef,1) = r_i*sin(xnod(nodes_lef,2)*theta); 
forc(nodes_rig,1) = r_e*sin(xnod(nodes_rig,2)*theta); 

mask = (xnod(nodes_bot,1)>0.3)&(xnod(nodes_bot,1)<=0.7);
nn   = find(mask==1);
z    = naca4ds(0012,length(nn));
yprf = rot90(rot90(imag(z(1:length(nn)))));
forc(nodes_bot(nn),1) = yprf;
[dmin,imin] = min(abs(xnod(nodes_bot,1)-0.3));
forc(nodes_bot(imin),1) = 0;

end

%-------- Fuentes de volumen (por elemento)
source =zeros(numel,1);

%--------- Termino reactivo -----------------------------
reac = zeros(numel,1);

%--------- Campo de Velocidad -----------------------------
veloc = zeros(numnp,2);
%-------- Propiedades del material

 rho     = 1    ;
 Cp      = 1    ;
 kappa   = 1    ;
 npg     =  2   ;
 prop(1) =rho   ;
 prop(2) =Cp    ;
 prop(3) =kappa ;
 prop(4) =npg   ;

 upwind  =    0 ;
 shocap  =    0 ;
 axisim  =    0 ;
% [1] SUPG
% [2] (SU+C)PG
 type_upw =    1;   
 lumped   =   0 ;

 prop(6)  = upwind ;
 prop(7)  = shocap ;
 prop(8)  = axisim ;
 prop(9)  = type_upw;
 prop(11) = lumped ;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas
peri = [];

%-------- Solucion inicial (PHI)
bini = zeros(numnp,ndf);

for k=1:ndf,
 nofree        = find(fixa(:,k)~=0);
 bini(nofree,k)   = forc(nofree,k);
end
