function [Kele,Rele] = bc_thermal_contact(xnod,icone,T,thermal_contact_array,Kele,Rele,ijob,axisim)
%
%   [Kele,Rele] = bc_thermal_contact(xnod,icone,T,thermal_contact_array,Kele,Rele,ijob,axisim)
%
%   Thermal contact between 2 layers of nodes
%
%   thermal_contact_array : [ele_master face_master ele_slave face_slave contact_coefficient]
%

teco = thermal_contact_array;

[numel,nen] = size(icone);
[nteco,bas] = size(teco);

npg = 2;

if nteco~=0,
   
  pgauss1d;
   
  ele_teco_master  = teco(:,1);
  ele_teco_slave   = teco(:,3);
  face_teco_master = teco(:,2);
  face_teco_slave  = teco(:,4);
  coef_teco        = teco(:,5);

  nodo_1 = face_teco_master;
  nodo_2 = (nodo_1+1).*(nodo_1<nen)+(nodo_1==nen);
%  nodos_master  = [nodo_1,nodo_2];

  nodog1 = getcol(icone(ele_teco_master,:),nodo_1);
  nodog2 = getcol(icone(ele_teco_master,:),nodo_2);  
%  nodog_master  = [nodog1 , nodog2];

  lteco  = sqrt((xnod(nodog1,1) - xnod(nodog2,1)).^2 + ... 
                (xnod(nodog1,2) - xnod(nodog2,2)).^2 );             
  Jaco   = lteco/2;
    
  re = v2m(xnod(icone(ele_teco_master,:)',1),nen);
  re = [getcol(re,nodo_1), getcol(re,nodo_2) ];

  nodo_1_master = nodo_1;
  nodo_2_master = nodo_2;  
  
  nodo_1_slave = face_teco_slave;
  nodo_2_slave = (nodo_1_slave+1).*(nodo_1_slave<nen)+(nodo_1_slave==nen);
%  nodos_slave  = [nodo_1,nodo_2];
  % intercambiamos los nodos por la forma de numerar la malla
  tmp          = nodo_2_slave;
  nodo_2_slave = nodo_1_slave;
  nodo_1_slave = tmp;
  
  Te_master(:,1) = getcol(T(ele_teco_master,:),nodo_1_master);
  Te_master(:,2) = getcol(T(ele_teco_master,:),nodo_2_master);

  Te_slave(:,1) = getcol(T(ele_teco_slave,:),nodo_1_slave);
  Te_slave(:,2) = getcol(T(ele_teco_slave,:),nodo_2_slave);
  
  for ipg=1:2,
     
   N = ones(nteco,1)*[(1-psipg(ipg))/2 , (1+psipg(ipg))/2 ];
   
% Localizacion , scatter   
   Tpg_master   = zeros(nteco,1);
   Tpg_slave    = zeros(nteco,1);   
   radpg        = zeros(nteco,1);
   
   for inod=1:2,    
    radpg        = radpg +  N(:,inod).*re(:,inod);
    Tpg_master   = Tpg_master +  N(:,inod).*Te_master(:,inod);
    Tpg_slave    = Tpg_slave  +  N(:,inod).*Te_slave (:,inod);    
   end 
        
   xsj = wpg(ipg)*Jaco;
   if axisim,
     xsj = xsj.*radpg;
   end
      
   for iteco=1:nteco,
%       eles  = ele_teco_master(iteco);
       iface = face_teco_master(iteco);
       iface_slave = face_teco_slave(iteco);
       for inod=1:2,
           jnod = iface-1+inod;
           jnod_slave = iface_slave-1+inod;
           if(jnod>nen),jnod=1; end 
           if(jnod_slave>nen),jnod_slave=1; end       
           vaux = N(iteco,inod).*coef_teco(iteco).*(Tpg_master(iteco)-Tpg_slave(iteco)).*xsj(iteco);       
           Rele(ele_teco_master(iteco),jnod,1)       = Rele(ele_teco_master(iteco),jnod,1) - vaux;                 
           Rele(ele_teco_slave (iteco),jnod_slave,1) = Rele(ele_teco_slave (iteco),jnod_slave,1) + vaux;
           for iinod=1:2, 
               jjnod = iface-1+iinod;
               if(jjnod>nen),jjnod=1; end           
               jjnod_slave = iface_slave-1+iinod;
               if(jjnod_slave>nen),jjnod_slave=1; end                          
               vaux = N(iteco,inod).*coef_teco(iteco).*N(iteco,iinod).*xsj(iteco);         
               Kele(ele_teco_master(iteco),jnod,1,jjnod,1) = ...
                   Kele(ele_teco_master(iteco),jnod,1,jjnod,1)+ vaux;                
                
               Kele(ele_teco_master(iteco),jnod,1,jjnod_slave,1) = ...
                   Kele(ele_teco_master(iteco),jnod,1,jjnod_slave,1) - vaux;                
                
               Kele(ele_teco_slave(iteco),jnod_slave,1,jjnod_slave,1) = ...
                   Kele(ele_teco_slave(iteco),jnod_slave,1,jjnod_slave,1) + vaux; 
                
               Kele(ele_teco_slave(iteco),jnod_slave,1,jjnod,1) = ...
                   Kele(ele_teco_slave(iteco),jnod_slave,1,jjnod,1) - vaux; 
                
            end % for iinod
       end  % for inod
   end % for iteco
end % puntos de Gauss del contorno

end % if nteco

