function [tau,delta,dt] = supg_bubbly_2d(Upg,dUpg,Cou,hele,shocap)
%
%  [tau,delta,dt] = supg_bubbly_2d(Upg,UPpg,dUpg,gamma,Cou,hele,shocap)
%

tol = 1e-08;

[N,ndf]     = size(Upg);
alpha_g     = Upg(:,1);
vel_g       = Upg(:,2:3);

velmod  = sqrt(sum(Upg(:,2:3)'.^2))';

if 0,
   velmod  = max([velmod';tol*ones(1,N)])';  
   tau   = 0.5* hele ./ velmod;
   delta = 0*tau;
   delta = 1.0*velmod.*hele/2;
   dt    =  Cou*hele ./ velmod;   
else
   
   tau     = zeros(N,ndf,ndf);    
   delta   = zeros(N,1);
   dt      = zeros(N,1);
   nn      = find(velmod>tol);   
   N       = length(nn);
   alpha_g = alpha_g(nn);
   vel_g   = vel_g(nn,:);
   % usando una descomposicion en autovalores
   beta_0 = alpha_g;
   beta_p = (alpha_g+1/2+1/2*sqrt(4*alpha_g.^2+1));
   beta_m = (alpha_g+1/2-1/2*sqrt(4*alpha_g.^2+1));    
   % eigenvalues of Ax
   mm      = find(abs(vel_g(:,1))<tol);
   if isempty(mm)==0, vel_g(mm,1) = zeros(length(mm),1); end
   mm      = find(abs(vel_g(:,2))<tol);
   if isempty(mm)==0, vel_g(mm,2) = zeros(length(mm),1); end      
   Dx = zeros(N,ndf,ndf);
   Dx(:,1,1) = beta_0.*vel_g(:,1);
   Dx(:,2,2) = beta_p.*vel_g(:,1);
   Dx(:,3,3) = beta_m.*vel_g(:,1);    
   % eigenvalues of Ay
   Dy = zeros(N,ndf,ndf);    
   Dy(:,1,1) = beta_0.*vel_g(:,2);
   Dy(:,2,2) = beta_p.*vel_g(:,2);
   Dy(:,3,3) = beta_m.*vel_g(:,2);    
   mm      = find(abs(vel_g(:,1))<tol);
   if isempty(mm)==0, vel_g(mm,1) = tol*ones(length(mm),1); end
   mm      = find(abs(vel_g(:,2))<tol);
   if isempty(mm)==0, vel_g(mm,2) = tol*ones(length(mm),1); end   
   % eigenvectors of Ax
   Vx = zeros(N,ndf,ndf);    
   Vx(:,1,2) = (beta_p-2*alpha_g)./vel_g(:,1);
   Vx(:,1,3) = (beta_m-2*alpha_g)./vel_g(:,1);
   Vx(:,2,2) = ones(N,1);
   Vx(:,2,3) = ones(N,1);
   Vx(:,3,1) = ones(N,1);    
   Vx(:,3,2) = vel_g(:,2)./vel_g(:,1); 
   Vx(:,3,3) = vel_g(:,2)./vel_g(:,1);    
   % eigenvectors of Ay       
   Vy = zeros(N,ndf,ndf);
   Vy(:,1,2) = (beta_p-2*alpha_g)./vel_g(:,1);
   Vy(:,1,3) = (beta_m-2*alpha_g)./vel_g(:,1);
   Vy(:,2,1) = ones(N,1);
   Vy(:,2,2) = ones(N,1);
   Vy(:,2,3) = ones(N,1);
   Vy(:,3,2) = vel_g(:,2)./vel_g(:,1); 
   Vy(:,3,3) = vel_g(:,2)./vel_g(:,1);    
   % inverse of eigenvectors of Ax
   deno = sqrt(4*alpha_g.^2+1);
   vaux_x = 2*alpha_g-1+deno;
   Vx1 = zeros(N,ndf,ndf);    
   Vx1(:,1,2) = -vel_g(:,2)./vel_g(:,1);
   Vx1(:,1,3) = ones(N,1);
   Vx1(:,2,1) = vel_g(:,1)./deno;
   Vx1(:,2,2) = 0.5*vaux_x./deno;
   vaux_y = 2*alpha_g-1-deno;    
   Vx1(:,3,1) = -vel_g(:,1)./deno;
   Vx1(:,3,2) = -0.5*vaux_y./deno;
   % inverse of eigenvectors of Ay
   Vy1 = zeros(N,ndf,ndf);    
   Vy1(:,1,2) = ones(N,1);
   Vy1(:,1,3) = -vel_g(:,1)./vel_g(:,2);
   Vy1(:,2,1) = vel_g(:,1)./deno;
   Vy1(:,2,3) = 0.5*vaux_x./deno.*vel_g(:,1)./vel_g(:,2);
   Vy1(:,3,1) = -vel_g(:,1)./deno;
   Vy1(:,3,3) = -0.5*vaux_y./deno.*vel_g(:,1)./vel_g(:,2);
    
   vaux = rowprod(abs(Dx),Vx1,ndf,ndf,ndf);
   vaux_1 = rowprod(Vx,vaux,ndf,ndf,ndf);    
   vaux = rowprod(abs(Dy),Vy1,ndf,ndf,ndf);
   vaux_2 = rowprod(Vy,vaux,ndf,ndf,ndf);    
   vaux = vaux_1+vaux_2;
   
   tau(nn,:,:) = row_inv(vaux);
   
%   keyboard;pause
   
end

