% ====================================================================
%
%            gas pipe flow with alpha_g fixed at input
%            axisymmetric case
%
% ====================================================================

% Reference infinity state of variables
alpha_g_ref = 1;
vel_g_ref   = [0,1];

alpha_g_ini = 0.01;
vel_g_ini   = [0,0.00];

rho_g       = 1;
visco_g     = 0.6;
gravity     = 9.8;

H = 4.0;
D = 1.0;
radius_nozzle = 0.1*D;

rho_l       = 1000;
p_ref       = 0;

ndf = 3;
tol = 1e-6;

shocap   =      1 ;
weak_form=      1 ; 

% ====================================================================

xnod     =[];
icone    =[];
Nx       =  20;    Lx =  D/2;
Ny       =  31;    Ly =  H;
psi      = (0:Nx-1)'/(Nx-1);
x        = psi.^2*Lx;
eta      = (0:Ny-1)'/(Ny-1)*1; 
y        = eta.^2*Ly;

%-------- coordenadas (XNOD)
for k=1:Ny,
  xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
nodes_bot  = (1:Nx);
nodes_top  = Nx*(Ny-1)+(1:Nx);
nodes_lef  = 1:Nx:Nx*Ny;
nodes_rig  = Nx:Nx:Nx*Ny;

nn = find(xnod(nodes_bot,1)<radius_nozzle);
nodes_bot_in = nodes_bot(nn);
nn = find(xnod(nodes_bot,1)>=radius_nozzle);
nodes_bot_wall = nodes_bot(nn);

fixa= zeros(numnp,ndf);
% fixations of some dofs
%fixa(:,[2,3])  = ones(numnp,2);
fixa(nodes_bot,[2,3])= ones(length(nodes_bot),2);
fixa(nodes_bot_in,[1,2,3])= ones(length(nodes_bot_in),3);
fixa(nodes_lef,[2])  = ones(length(nodes_lef),1);
fixa(nodes_rig,[2,3])  = ones(length(nodes_rig),2);
fixa(nodes_top,[2])    = ones(length(nodes_top),1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
%forc(:,1)= ones(numnp,1)*alpha_g_ref;
%nn = find(xnod(:,1)>radius_nozzle);
%forc(nn,1) = ones(length(nn),1)*alpha_g_ini;

forc(nodes_bot_in,1:3)= ones(length(nodes_bot_in),1)*[alpha_g_ref,vel_g_ref];
forc(nodes_bot_wall,2:3)= ones(length(nodes_bot_wall),1)*[0,0];

%--------- External fields -----------------------------
% pressure and liquid velocities 
% external_fields = [pressure , vel_liq]
y0 = min(xnod(:,2));
H  = max(xnod(:,2))-y0;
external_fields =  zeros(numnp,3);
external_fields(:,1) = p_ref+rho_l*gravity*(H-(xnod(:,2)-y0));

%-------- Propiedades del material
 Cou     =  1  ;
 npg     =  4  ;

 prop(1) = rho_g  ;
 prop(2) = visco_g;
 prop(3) = gravity;
 prop(4) = npg;
 
 axisim   =      1 ;
 prop(5)  = weak_form;
 prop(6)  =    Cou ;
 prop(7)  = shocap ;
 prop(8)  = axisim ;
 
%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
%teco    = [eles face val1/rho/Cp val2 ];
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
%peri = [];
%slip = [];
lin_con = [];

% Up = [alpha_g, vel_g]
%-------- Solucion inicial (PHI)
uini   = [alpha_g_ini,vel_g_ini];
U      =  ones(numnp,1)*uini;   

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
   bini   = U;
   for k=1:ndf,
      nofree        = find(fixa(:,k)~=0);
      bini(nofree,k)   = forc(nofree,k);
   end   
else
   load bubbly_01_ini
end


