% ====================================================================
%
%            gas pipe flow with alpha_g fixed at input horizontal flow
%
% ====================================================================

% Reference infinity state of variables
alpha_g_ref = 1;
vel_g_ref   = [1,0];

alpha_g_ini = 0.01;
vel_g_ini   = [0,0];

rho_g       = 1;
visco_g     = 0.6;
gravity     = 0;

rho_l       = 1000;
p_ref       = 0;

ndf         = 3;
shocap      = 0;
weak_form   = 1;
% ====================================================================

xnod     =[];
icone    =[];
Nx       =  20;    Lx =  5.0;
Ny       =  21;    Ly =  1.0;
psi      = (0:Nx-1)'/(Nx-1);
x        = psi*Lx;
eta      = (0:Ny-1)'/(Ny-1)*1; 
y        = eta*Ly;

%-------- coordenadas (XNOD)
for k=1:Ny,
  xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
nodes_bot  = (1:Nx);
nodes_top  = Nx*(Ny-1)+(1:Nx);
nodes_lef  = 1:Nx:Nx*Ny;
nodes_rig  = Nx:Nx:Nx*Ny;

fixa= zeros(numnp,ndf);
fixa(nodes_lef,[1,2,3])= ones(length(nodes_lef),3);
%fixa(nodes_top,[2,3])  = ones(length(nodes_top),2);
%fixa(nodes_bot,[2,3])  = ones(length(nodes_bot),2);
fixa(nodes_rig,[3])    = ones(length(nodes_rig),1);
%fixa(:,2:3)            = ones(numnp,2);
%fixa(:,1)            = ones(numnp,1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
forc(nodes_lef,1:3)= ones(length(nodes_lef),1)*[alpha_g_ref,vel_g_ref];
%forc(:,2:3)=ones(numnp,1)*vel_g_ref;
%forc(:,1)=ones(numnp,1)*alpha_g_ref;

%--------- External fields -----------------------------
% pressure and liquid velocities 
% external_fields = [pressure , vel_liq]
y0 = min(xnod(:,2));
H  = max(xnod(:,2))-y0;
external_fields =  zeros(numnp,3);
external_fields(:,1) = p_ref+rho_l*gravity*(H-(xnod(:,2)-y0));

%-------- Propiedades del material
 Cou     =  1  ;
 npg     =  4  ;

 prop(1) = rho_g  ;
 prop(2) = visco_g;
 prop(3) = gravity;
 prop(4) = npg;
 

 axisim   =      0 ; 
 prop(5)  = weak_form;
 prop(6)  =    Cou ;
 prop(7)  = shocap ;
 prop(8)  = axisim ;
 
%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
%teco    = [eles face val1/rho/Cp val2 ];
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
%peri = [];
%slip = [];
lin_con = [];

% Up = [alpha_g, vel_g]
%-------- Solucion inicial (PHI)
uini   = [alpha_g_ini,vel_g_ini];
U      =  ones(numnp,1)*uini;   

% condicion inicial en primitivas
%-----------------------------------------------
bini   = U;

for k=1:ndf,
 nofree        = find(fixa(:,k)~=0);
 bini(nofree,k)   = forc(nofree,k);
end

