alpha_min = -0.1;
alpha_max = 1.1;
u_g_min   = -0.01;
u_g_max   = 0.01;
v_g_min   = -0.1;
v_g_max   = 0.1;
tol       = 1e-15;

rho_g     = 1;
visco_g   = 0;
N         = 50;

psi = (0:N)'/N;

Upg(:,1) = alpha_min + (alpha_max-alpha_min)*psi;
Upg(:,2) = u_g_min + (u_g_max-u_g_min)*psi;
Upg(:,3) = v_g_min + (v_g_max-v_g_min)*psi;

% correccion para evitar valores proximos a cero en las velocidades
mask = (abs(Upg(:,2))<tol);
nn   = find(mask==1);
Upg(nn,2)=tol;

mask = (abs(Upg(:,3))<tol);
nn   = find(mask==1);
Upg(nn,3)=tol;

[N,ndf]     = size(Upg);
alpha_g     = Upg(:,1);
vel_g       = Upg(:,2:3);


    % usando una descomposicion en autovalores
    beta_0 = alpha_g;
    beta_p = (alpha_g+1/2+1/2*sqrt(4*alpha_g.^2+1));
    beta_m = (alpha_g+1/2-1/2*sqrt(4*alpha_g.^2+1));    
    % eigenvalues of Ax
    Dx = zeros(N,ndf,ndf);
    Dx(:,1,1) = beta_0.*vel_g(:,1);
    Dx(:,2,2) = beta_p.*vel_g(:,1);
    Dx(:,3,3) = beta_m.*vel_g(:,1);    
    % eigenvalues of Ay
    Dy = zeros(N,ndf,ndf);    
    Dy(:,1,1) = beta_0.*vel_g(:,2);
    Dy(:,2,2) = beta_p.*vel_g(:,2);
    Dy(:,3,3) = beta_m.*vel_g(:,2);    
    % eigenvectors of Ax
    Vx = zeros(N,ndf,ndf);    
    Vx(:,1,2) = (beta_p-2*alpha_g)./vel_g(:,1);
    Vx(:,1,3) = (beta_m-2*alpha_g)./vel_g(:,1);
    Vx(:,2,2) = ones(N,1);
    Vx(:,2,3) = ones(N,1);
    Vx(:,3,1) = ones(N,1);    
    Vx(:,3,2) = vel_g(:,2)./vel_g(:,1); 
    Vx(:,3,3) = vel_g(:,2)./vel_g(:,1);    
    % eigenvectors of Ay       
    Vy = zeros(N,ndf,ndf);
    Vy(:,1,2) = (beta_p-2*alpha_g)./vel_g(:,1);
    Vy(:,1,3) = (beta_m-2*alpha_g)./vel_g(:,1);
    Vy(:,2,1) = ones(N,1);
    Vy(:,2,2) = ones(N,1);
    Vy(:,2,3) = ones(N,1);
    Vy(:,3,2) = vel_g(:,2)./vel_g(:,1); 
    Vy(:,3,3) = vel_g(:,2)./vel_g(:,1);    
    % inverse of eigenvectors of Ax
    deno = sqrt(4*alpha_g.^2+1);
    vaux_x = 2*alpha_g-1+deno;
    Vx1 = zeros(N,ndf,ndf);    
    Vx1(:,1,2) = -vel_g(:,2)./vel_g(:,1);
    Vx1(:,1,3) = ones(N,1);
    Vx1(:,2,1) = vel_g(:,1)./deno;
    Vx1(:,2,2) = 0.5*vaux_x./deno;
    vaux_y = 2*alpha_g-1-deno;    
    Vx1(:,3,1) = -vel_g(:,1)./deno;
    Vx1(:,3,2) = -0.5*vaux_y./deno;
    % inverse of eigenvectors of Ay
    Vy1 = zeros(N,ndf,ndf);    
    Vy1(:,1,2) = ones(N,1);
    Vy1(:,1,3) = -vel_g(:,1)./vel_g(:,2);
    Vy1(:,2,1) = vel_g(:,1)./deno;
    Vy1(:,2,3) = 0.5*vaux_x./deno.*vel_g(:,1)./vel_g(:,2);
    Vy1(:,3,1) = -vel_g(:,1)./deno;
    Vy1(:,3,3) = -0.5*vaux_y./deno.*vel_g(:,1)./vel_g(:,2);

    prop = [rho_g,visco_g];
    [Fa,Ax,Ay] = Flux_function(Upg,prop);
    
    ddxnorm = [];
    ddynorm = [];
    for k=1:N,
        VVx  = reshape(Vx(k,:,:),3,3);
        VVx1 = reshape(Vx1(k,:,:),3,3);        
        DDx  = reshape(Dx(k,:,:),3,3);        
        AAx  = reshape(Ax(k,:,:),3,3);
        ddx(k,:,:)   = AAx-VVx*DDx*VVx1;
        ddxnorm = [ddxnorm;norm(AAx-VVx*DDx*VVx1)];
        VVy  = reshape(Vy(k,:,:),3,3);
        VVy1 = reshape(Vy1(k,:,:),3,3);        
        DDy  = reshape(Dy(k,:,:),3,3);        
        AAy  = reshape(Ay(k,:,:),3,3);
        ddy(k,:,:)   = AAy-VVy*DDy*VVy1;
        ddynorm = [ddynorm;norm(AAy-VVy*DDy*VVy1)];
        
        tau(k,:,:) = inv(VVx*abs(DDx)*VVx1+VVy*abs(DDy)*VVy1);
        Px(k,:,:) = reshape(tau(k,:,:),3,3)*AAx;
        Py(k,:,:) = reshape(tau(k,:,:),3,3)*AAy;
    end
    
