function [dshapex,N] = shape(X2S,psipg,etapg,nen)
%
%    [dshapex,N] = shape(X2S,psipg,etapg,nen)
%-------------------------------------------------------------------
%          --- FEMCODE routine --- 
%
%   [dNdx,dNdy,N] = shape(S2X,psipg,etapg,nen);
%
%     dNdx = dN_j/dx , j=1,...,nen
%     dNdy = dN_j/dy , j=1,...,nen
%
%-------------------------------------------------------------------

% replaces the old version due to multi index manipulations
% function [dNdx,dNdy,N] = shape(X2S,psipg,etapg,nen)

if nen==4,

psi = [-1 1 1 -1 ];
eta = [-1 -1 1 1 ];

% Calculo de la funcion de forma 

N(1)  = 0.25*(1+psi(1)*psipg)*(1+eta(1)*etapg);
N(2)  = 0.25*(1+psi(2)*psipg)*(1+eta(2)*etapg);
N(3)  = 0.25*(1+psi(3)*psipg)*(1+eta(3)*etapg);
N(4)  = 0.25*(1+psi(4)*psipg)*(1+eta(4)*etapg);

% Calculo de las derivadas de las funciones de forma 
% respecto a cada direccion del elemento master
%                dN_j/dpsi_k

dN1d1 = 0.25*psi(1)*(1+eta(1)*etapg);
dN1d2 = 0.25*eta(1)*(1+psi(1)*psipg);

dN2d1 = 0.25*psi(2)*(1+eta(2)*etapg);
dN2d2 = 0.25*eta(2)*(1+psi(2)*psipg);

dN3d1 = 0.25*psi(3)*(1+eta(3)*etapg);
dN3d2 = 0.25*eta(3)*(1+psi(3)*psipg);

dN4d1 = 0.25*psi(4)*(1+eta(4)*etapg);
dN4d2 = 0.25*eta(4)*(1+psi(4)*psipg);

dNdx(:,1) = X2S(:,1) * dN1d1 + X2S(:,3) * dN1d2;
dNdx(:,2) = X2S(:,1) * dN2d1 + X2S(:,3) * dN2d2;
dNdx(:,3) = X2S(:,1) * dN3d1 + X2S(:,3) * dN3d2;
dNdx(:,4) = X2S(:,1) * dN4d1 + X2S(:,3) * dN4d2;

dNdy(:,1) = X2S(:,2) * dN1d1 + X2S(:,4) * dN1d2;
dNdy(:,2) = X2S(:,2) * dN2d1 + X2S(:,4) * dN2d2;
dNdy(:,3) = X2S(:,2) * dN3d1 + X2S(:,4) * dN3d2;
dNdy(:,4) = X2S(:,2) * dN4d1 + X2S(:,4) * dN4d2;

elseif nen==3,

% Calculo de la funcion de forma 

N(1)  = 1-psipg-etapg;
N(2)  = psipg;
N(3)  = etapg;

% Calculo de las derivadas de las funciones de forma 
% respecto a cada direccion del elemento master
%                dN_j/dpsi_k

dN1d1 = -1;
dN1d2 = -1;

dN2d1 = +1;
dN2d2 =  0;

dN3d1 =  0;
dN3d2 = +1;

dNdx(:,1) = X2S(:,1) * dN1d1 + X2S(:,3) * dN1d2;
dNdx(:,2) = X2S(:,1) * dN2d1 + X2S(:,3) * dN2d2;
dNdx(:,3) = X2S(:,1) * dN3d1 + X2S(:,3) * dN3d2;

dNdy(:,1) = X2S(:,2) * dN1d1 + X2S(:,4) * dN1d2;
dNdy(:,2) = X2S(:,2) * dN2d1 + X2S(:,4) * dN2d2;
dNdy(:,3) = X2S(:,2) * dN3d1 + X2S(:,4) * dN3d2;

else
  error(' ** ERROR ** , Routine SHAPE , Bad type of element')
end

dshapex(:,:,1) = dNdx;
dshapex(:,:,2) = dNdy;
