function [Kg,Rg] = fem001(xnod,icone,fixa,forc,kappa,react,veloc,phi)                   
%------------------------------------------------------------------
%     Inputs:
%
%           xnod(numnp,ndm)
%          icone(numel,nen)
%           fixa(numnp,ndf)
%           forc(numnp,ndf)
%            kappa(numel,1)  difusividad (elemento)
%            react(numel,1)  reaccion (elemento)
%            veloc(numnp,2)  vector velocidad (nodal)
%              phi(numnp,1)
%
%     [Kg,Rg] = fem001(xnod,icone,fixa,forc,kappa,react,veloc,phi)
%------------------------------------------------------------------

%------------------------------------------------------------------
% Datos a entrar por argumento posteriormente junto con xnod e icone
%------------------------------------------------------------------

  numel        = length(icone(:,1));
 [numnp,ndf]   = size(fixa); 

if ndf~=1, 
  error(' NDF > 1 is not implemented yet ')
end

xpg  = sqrt(3)/3;
psipg = [ -xpg xpg xpg -xpg ];
etapg = [ -xpg -xpg xpg xpg ];
wpg   = [    1   1   1    1 ];
npg   = 4;


Kele=zeros(numel,16);
Rele=zeros(numel,4);
Rg  =forc;

%------------------------------------------------------------------
%                Localizacion solucion
%------------------------------------------------------------------
  for inod=1:4,
    phie(:,inod) = phi(icone(:,inod));
    velex(:,inod) = veloc(icone(:,inod),1);
    veley(:,inod) = veloc(icone(:,inod),2);
  end
%------------------------------------------------------------------

for ipg=1:npg

% Calculo de tensor metrico y jacobiano al elemento master
%------------------------------------------------------------------
  [X2S,S2X,Jaco] = xy2st(xnod,icone,psipg(ipg),etapg(ipg));

% Calculo gradiente funcion de peso
%------------------------------------------------------------------
  [dNdx,dNdy,N] = gradN(X2S,psipg(ipg),etapg(ipg)); 

% Calculo gradiente de la solucion en el pto de Gauss
%------------------------------------------------------------------
  dphidx = zeros(numel,1);
  dphidy = zeros(numel,1);
  phipg  = zeros(numel,1);
  velpg  = zeros(numel,2);

  for inod=1:4,
    dphidx(:,1) = dphidx(:,1) + dNdx(:,inod).*phie(:,inod);
    dphidy(:,1) = dphidy(:,1) + dNdy(:,inod).*phie(:,inod);
    phipg(:,1)  = phipg(:,1)  + N(inod).*phie(:,inod);
    velpg(:,1)  = velpg(:,1)  + N(inod).*velex(:,inod);
    velpg(:,2)  = velpg(:,2)  + N(inod).*veley(:,inod);
  end

  P_pg = supg(velpg,dNdx,dNdy,kappa);

% armado matriz tangente elemental y vector residuo
%------------------------------------------------------------------
  for inod=1:4,
   for jnod=1:4,
    ip=(inod-1)*4+jnod;
    Kele(:,ip) = Kele(:,ip) + ... 
       ( dNdx(:,inod).*kappa.*dNdx(:,jnod) + ... 
         dNdy(:,inod).*kappa.*dNdy(:,jnod) + ...
         (N(inod)+P_pg(:,inod)).*react.*N(jnod)           + ...
         (N(inod)+P_pg(:,inod)).*velpg(:,1).*dNdx(:,jnod) + ...
         (N(inod)+P_pg(:,inod)).*velpg(:,2).*dNdy(:,jnod)   ...
       ) * wpg(ipg) .* Jaco;
   end
    Rele(:,inod) = Rele(:,inod) - ...
          ( dNdx(:,inod).*kappa.*dphidx(:,1) + ... 
            dNdy(:,inod).*kappa.*dphidy(:,1) + ...
            (N(inod)+P_pg(:,inod)).*react.*phipg(:,1)       + ...
            (N(inod)+P_pg(:,inod)).*velpg(:,1).*dphidx(:,1) + ...
            (N(inod)+P_pg(:,inod)).*velpg(:,2).*dphidy(:,1)   ...
           ) * wpg(ipg) .* Jaco ;
  end
  
end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------
  [row,col,coe] = assemble(Kele,icone);
  Kg = sparse(row,col,coe);
  for inod=1:4,
    Rg(icone(:,inod)) = Rg(icone(:,inod)) + Rele(:,inod);
  end


