function [Kg,Rg] = fem002(xnod,icone,fixa,forc,teco,prop,veloc,phi,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem002(xnod,icone,fixa,forc,prop,veloc,T)
%
%       Caracteristicas del elemento:
%
%        #.- cuadrangular 
%        #.- incluye cambio de fase
%        #.- estacionario con conveccion
%
%    ijob = 0 ---> matriz tangente y residuo
%    ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

 T = phi(:,1);
 C = phi(:,2);

 [numel,nen]   = size(icone);
 [numnp,ndf]   = size(fixa); 

 Tm       = prop(1) ; 
 slopel   = prop(2) ;
 kp       = prop(3) ;
 kappa_s  = prop(5) ;
 kappa_l  = prop(6) ;
 rho      = prop(7) ;
 Cp       = prop(8) ;
 DeltaH   = prop(9) ;
 U0       = prop(10);
 alfa_min = prop(11);


xpg  = sqrt(3)/3;
psipg = [ -xpg xpg xpg -xpg ];
etapg = [ -xpg -xpg xpg xpg ];
wpg   = [    1   1   1    1 ];
npg   = 4;


Kele=zeros(numel,16);
Rele=zeros(numel,4);
Rg  =forc;

[fsol,dfsdT] = frsol1(T,C,Tm,slopel,kp);

%------------------------------------------------------------------
%                Localizacion solucion
%------------------------------------------------------------------
  for inod=1:4,
    Te(:,inod)    = T(icone(:,inod));
    velex(:,inod) = veloc(icone(:,inod),1);
    veley(:,inod) = veloc(icone(:,inod),2);
    fsole(:,inod) = fsol(icone(:,inod));
    dfsdTe(:,inod) = dfsdT(icone(:,inod));
  end
%------------------------------------------------------------------

for ipg=1:npg

% Calculo de tensor metrico y jacobiano al elemento master
%------------------------------------------------------------------
  [X2S,S2X,Jaco] = xy2st(xnod,icone,psipg(ipg),etapg(ipg));

% Calculo gradiente funcion de peso
%------------------------------------------------------------------
  [dNdx,dNdy,N] = gradN(X2S,psipg(ipg),etapg(ipg)); 

% Calculo gradiente de la solucion en el pto de Gauss
%------------------------------------------------------------------
   Tpg    = zeros(numel,1);
   dTpg   = zeros(numel,2);
   velpg  = zeros(numel,2);
   fspg   = zeros(numel,1);
   dfspg  = zeros(numel,2);
 dfsdTpg  = zeros(numel,1);

  for inod=1:4,
      Tpg(:,1)  =     Tpg(:,1)  +       N(inod).*    Te(:,inod);
     dTpg(:,1)  =    dTpg(:,1)  +  dNdx(:,inod).*    Te(:,inod);
     dTpg(:,2)  =    dTpg(:,2)  +  dNdy(:,inod).*    Te(:,inod);
    velpg(:,1)  =   velpg(:,1)  +       N(inod).* velex(:,inod);
    velpg(:,2)  =   velpg(:,2)  +       N(inod).* veley(:,inod);
     fspg(:,1)  =    fspg(:,1)  +       N(inod).* fsole(:,inod);
    dfspg(:,1)  =   dfspg(:,1)  +  dNdx(:,inod).* fsole(:,inod);
    dfspg(:,2)  =   dfspg(:,2)  +  dNdy(:,inod).* fsole(:,inod);
  dfsdTpg(:,1)  = dfsdTpg(:,1)  +       N(inod).*dfsdTe(:,inod);
  end

  kappa = (kappa_s * fspg + kappa_l * (1-fspg))/(rho*Cp);
  sourc = DeltaH*U0/Cp;
  
  vel_supg = velpg;
  vel_supg(:,2) = vel_supg(:,2) - sourc*dfsdTpg;

  [tau_pg,h_supg] = supg(vel_supg,dNdx,dNdy,kappa);

  for inod=1:nen,
    P_pg(:,inod) = tau_pg .* ... 
       ( vel_supg(:,1) .*dNdx(:,inod) + vel_supg(:,2) .*dNdy(:,inod) ) ;
  end 
 
% armado matriz tangente elemental y vector residuo
%------------------------------------------------------------------
  for inod=1:4,
   if ijob==0,
    for jnod=1:4,
     ip=(inod-1)*4+jnod;
     Kele(:,ip) = Kele(:,ip) + ... 
       ( dNdx(:,inod).*kappa.*dNdx(:,jnod) + ... 
         dNdy(:,inod).*kappa.*dNdy(:,jnod) - ...
         (N(inod)+P_pg(:,inod)).*(sourc*dNdy(:,jnod).*dfsdTe(:,jnod)) + ...
         (N(inod)+P_pg(:,inod)).*velpg(:,1).*dNdx(:,jnod) + ...
         (N(inod)+P_pg(:,inod)).*velpg(:,2).*dNdy(:,jnod)  ...
       ) * wpg(ipg) .* Jaco;
    end
   end
   Rele(:,inod) = Rele(:,inod) - ...
          ( dNdx(:,inod).*kappa.*dTpg(:,1)                + ... 
            dNdy(:,inod).*kappa.*dTpg(:,2)                - ...
            (N(inod)+P_pg(:,inod)).*(sourc*dfspg(:,2))    + ...
            (N(inod)+P_pg(:,inod)).*velpg(:,1).*dTpg(:,1) + ...
            (N(inod)+P_pg(:,inod)).*velpg(:,2).*dTpg(:,2)   ...
           ) * wpg(ipg) .* Jaco ;
  end
  
end

% Contribucion en el contorno

[nteco,bas] = size(teco);

if nteco~=0,

 xpg    = sqrt(3)/3;
 psipg  = [ -xpg xpg ];
 wpg    = [   1   1  ];

 for iteco=1:nteco,

  eles_teco = teco(iteco,1);
  face_teco = teco(iteco,2);
  val1_teco = teco(iteco,3);
  val2_teco = teco(iteco,4);

  nodo_1 = face_teco;
  nodo_2 = rem(nodo_1+1,4);
  nodos  = [nodo_1;nodo_2];
  nodog1 = icone(eles_teco,nodo_1);
  nodog2 = icone(eles_teco,nodo_2);

  lteco  = sqrt((xnod(nodog1,1) - xnod(nodog2,1)).^2 + ... 
                (xnod(nodog1,2) - xnod(nodog2,2)).^2 );

  Te(1,1)  = T(nodog1);
  Te(1,2)  = T(nodog2);

  for ipg=1:2,

   N1     = 0.5*(1-psipg(ipg));
   N2     = 0.5*(1+psipg(ipg));
   N      = [N1 N2];

   Tpg = 0;

   for inod=1:2,
     Tpg  = Tpg + N(inod)*Te(1,inod);
   end 

   for inod=1:2,
    iinod = nodos(inod);
    if ijob==0,
     for jnod=1:2,
      jjnod = nodos(jnod);
      ip=(iinod-1)*4+jjnod;
      Kele(eles_teco,ip) = Kele(eles_teco,ip) + ... 
                N(inod) * val1_teco * N(jnod) .* (lteco/2) * wpg(ipg) ; 
     end % for jnod
    end % if ijob
    Rele(eles_teco,iinod) = Rele(eles_teco,iinod) - ... 
       N(inod)*val1_teco.*(Tpg-val2_teco) .* (lteco/2) * wpg(ipg);
   end  % for inod

  end % puntos de Gauss del contorno

 end % for iteco

end % if teco

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------
  [row,col,coe] = assemble(Kele,icone);
  Kg = sparse(row,col,coe);
  for inod=1:4,
    Rg(icone(:,inod)) = Rg(icone(:,inod)) + Rele(:,inod);
  end