function [Kg,Rg] = ...
       fem003(xnod,icone,fixa,forc,teco,prop,veloc,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem003(xnod,icone,fixa,forc,prop,veloc,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion del soluto
%        #.- elemento cuadrangular 
%        #.- incluye cambio de fase
%        #.- estacionario con conveccion
%
%    ijob = 0 ---> matriz tangente y residuo
%    ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

 [numel,nen]   = size(icone);
 [numnp,ndf]   = size(fixa); 

 Tm       = prop(1) ; 
 slopel   = prop(2) ;
 kp       = prop(3) ;
 kappa_s  = prop(5) ;
 kappa_l  = prop(6) ;
 rho      = prop(7) ;
 Cp       = prop(8) ;
 DeltaH   = prop(9) ;
 U0       = prop(10);
 alfa_min = prop(11);
 Dm_s     = prop(12);
 Dm_l     = prop(13);
 Fou      = prop(14);

T = phi(:,1);
C = phi(:,2);
Told = phiold(:,1);
Cold = phiold(:,2);

xpg  = sqrt(3)/3;
psipg = [ -xpg xpg xpg -xpg ];
etapg = [ -xpg -xpg xpg xpg ];
wpg   = [    1   1   1    1 ];
npg   = 4;

Kele=zeros(numel,16);
Rele=zeros(numel,4);
Rg  =forc;

[fsol,dfsdT,dfsdC,Cs,dcsdT,dcsdC] = frsol2(T,C,Tm,slopel,kp);

%------------------------------------------------------------------
%                Localizacion solucion
%------------------------------------------------------------------

mushy = zeros(numel,1);

  for inod=1:4,
    Ce(:,inod)    = C(icone(:,inod));
    Colde(:,inod) = Cold(icone(:,inod));
    Cse(:,inod)   = Cs(icone(:,inod));
    velex(:,inod) = veloc(icone(:,inod),1);
    veley(:,inod) = veloc(icone(:,inod),2);
    fsole(:,inod) = fsol(icone(:,inod));

mushy = mushy | ((fsole(:,inod)<1) & (fsole(:,inod) >0));

  end

%------------------------------------------------------------------

for ipg=1:npg

% Calculo de tensor metrico y jacobiano al elemento master
%------------------------------------------------------------------
  [X2S,S2X,Jaco] = xy2st(xnod,icone,psipg(ipg),etapg(ipg));

% Calculo gradiente funcion de peso
%------------------------------------------------------------------
  [dNdx,dNdy,N] = gradN(X2S,psipg(ipg),etapg(ipg)); 

% Calculo gradiente de la solucion en el pto de Gauss
%------------------------------------------------------------------
     Cpg    = zeros(numel,1);
  Coldpg    = zeros(numel,1);
    dCpg    = zeros(numel,2);
   dCspg    = zeros(numel,2);
   velpg    = zeros(numel,2);
    fspg    = zeros(numel,1);
 
  for inod=1:4,
      Cpg(:,1)  =     Cpg(:,1)  +       N(inod).*    Ce(:,inod);
   Coldpg(:,1)  =  Coldpg(:,1)  +       N(inod).* Colde(:,inod);
     dCpg(:,1)  =    dCpg(:,1)  +  dNdx(:,inod).*    Ce(:,inod);
     dCpg(:,2)  =    dCpg(:,2)  +  dNdy(:,inod).*    Ce(:,inod);
    dCspg(:,1)  =   dCspg(:,1)  +  dNdx(:,inod).*   Cse(:,inod);
    dCspg(:,2)  =   dCspg(:,2)  +  dNdy(:,inod).*   Cse(:,inod);
    velpg(:,1)  =   velpg(:,1)  +       N(inod).* velex(:,inod);
    velpg(:,2)  =   velpg(:,2)  +       N(inod).* veley(:,inod);
     fspg(:,1)  =    fspg(:,1)  +       N(inod).* fsole(:,inod);
  end

  Dm_pg         = Dm_l/kp + (Dm_s-Dm_l/kp) * fspg ;
  vel_supg      = velpg;

  [tau_pg,h_supg] = supg(vel_supg,dNdx,dNdy,Dm_pg);

  dt = (min(h_supg))^2*Fou/max([Dm_l;Dm_s]);
  for inod=1:nen,
    P_pg(:,inod) = tau_pg .* ... 
       ( vel_supg(:,1) .*dNdx(:,inod) + vel_supg(:,2) .*dNdy(:,inod) ) ;
  end 
 
% armado matriz tangente elemental y vector residuo
%------------------------------------------------------------------
  for inod=1:4,
   if ijob==0,
    for jnod=1:4,
     ip=(inod-1)*4+jnod;
     Kele(:,ip) = Kele(:,ip) + ( ... 
     (N(inod)+P_pg(:,inod))*N(jnod)/dt    + ...
     (N(inod)+P_pg(:,inod)).*velpg(:,1).*dNdx(:,jnod) + ...
     (N(inod)+P_pg(:,inod)).*velpg(:,2).*dNdy(:,jnod) + ...
     (dNdx(:,inod).*dNdx(:,jnod) + dNdy(:,inod).*dNdy(:,jnod)).* Dm_pg ... 
                               ) * wpg(ipg) .* Jaco;
    end
   end
   Rele(:,inod) = Rele(:,inod) - ( ...
      (N(inod)+P_pg(:,inod)).*(Cpg-Coldpg)/dt          + ...
      (N(inod)+P_pg(:,inod)).*velpg(:,1).*dCpg(:,1)    + ...
      (N(inod)+P_pg(:,inod)).*velpg(:,2).*dCpg(:,2)    + ...
     (dNdx(:,inod).*dCspg(:,1)+dNdy(:,inod).*dCspg(:,2)).*Dm_pg  + ...
       mushy .* ( ...
      (N(inod)+P_pg(:,inod)).* velpg(:,1).*(dCspg(:,1)/kp-dCpg(:,1)) + ...
  (N(inod)+P_pg(:,inod)).* (velpg(:,2)-U0).*(dCspg(:,2)/kp-dCpg(:,2)) ) ...  
                                  ) * wpg(ipg) .* Jaco ;
  end
end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------
  [row,col,coe] = assemble(Kele,icone,1);
  Kg = sparse(row,col,coe);
  for inod=1:4,
    Rg(icone(:,inod)) = Rg(icone(:,inod)) + Rele(:,inod);
  end