function [Kg,Rg] = ...
       fem004(xnod,icone,fixa,forc,teco,peri,prop,veloc,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem004(xnod,icone,fixa,forc,prop,veloc,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion acoplada balance de energia y masa del soluto
%        #.- elemento cuadrangular 
%        #.- incluye cambio de fase
%        #.- no estacionario con conveccion
%        #.- NDF == 2
%
%    ijob = 0 ---> matriz tangente y residuo
%    ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

  global Rg_old

  [numel,nen]   = size(icone);
  nen0          = nen;
  ndf           = 2;
  tol           = 1e-6;

  Tm       = prop(1) ; 
  slopel   = prop(2) ;
  kp       = prop(3) ;
  kappa_s  = prop(5) ;
  kappa_l  = prop(6) ;
  rho      = prop(7) ;
  Cp       = prop(8) ;
  DeltaH   = prop(9) ;
  U0       = prop(10);
  alfa_min = prop(11);
  Dm_s     = prop(12);
  Dm_l     = prop(13);
  dt       = prop(14);
  lmushy   = prop(15);
  upwind   = prop(16);
  Dm_l_turb= prop(17);
  shocap   = prop(18);

  T = phi(:,1);
  C = phi(:,2);
  Told = phiold(:,1);
  Cold = phiold(:,2);

  Rg   =forc;
  Kele =zeros(numel,nen0^2*ndf^2);
  Rele =zeros(numel,nen0*ndf);

  [fsolold] = frsol2(Told,Cold,Tm,slopel,kp);
  [fsol,dfsdT,dfsdC,Cs,dCsdT,dCsdC] = frsol2(T,C,Tm,slopel,kp);
  eles     = (1:numel)';
  ceros    = zeros(length(eles),nen);
  fsole    = ceros;
  fsololde = ceros;
  for inod=1:nen,
               nodes    = icone(eles,inod);
    fsole(eles,inod)    = fsol(nodes);
    fsololde(eles,inod) = fsolold(nodes);
  end

  mushy = zeros(numel,1);
  if (lmushy==1),
    iliq = (fsole < tol);
    isol = (fsole > (1-tol));
    imus = (fsole >= tol)&(fsole <= (1-tol));
    ipha = (-1)*isol+0*imus+(1)*iliq;
    fsmax = max(ipha')';
    fsmin = min(ipha')';
    mushy = abs(fsmax-fsmin)>0;
  end

  mushy_old = zeros(numel,1);
  if (lmushy==1),
    iliq = (fsololde < tol);
    isol = (fsololde > (1-tol));
    imus = (fsololde >= tol)&(fsololde <= (1-tol));
    ipha = (-1)*isol+0*imus+(1)*iliq;
    fsmax = max(ipha')';
    fsmin = min(ipha')';
    mushy_old = abs(fsmax-fsmin)>0;
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% [1] Armado contribucion zona no mushy paso actual
% [2] Armado contribucion zona    mushy paso actual
% [3] Armado contribucion zona no mushy paso anterior (solo termino L)
% [4] Armado contribucion zona    mushy paso anterior (solo termino L)

% [1]
% Todos los elementos zona no mushy tiempo actual
  nen   = nen0;
  mask	= (mushy==0);
  eles  = find(mask==1);
  T     = phi(:,1);
  C     = phi(:,2);
  disp([' # eles no mushy en t_n+1 --> ' num2str(length(eles)) ]);

  if length(eles)>0,    
	coe = [1;1;1;1;1];
    mushy_e = mushy(eles);
    fmushy1;
  end

% [2]
% Todos los elementos zona mushy tiempo actual
  nen   = 3;
  mask	= (mushy==1);
  eles  = find(mask==1);
  T     = phi(:,1);
  C     = phi(:,2);
  disp([' # eles mushy en t_n+1 --> ' num2str(length(eles)) ]);
  if length(eles)>0,
	coe = [1;1;1;1;1];

    fmushy2;
  end

% [3]
% Todos los elementos zona no mushy tiempo anterior
  nen   = nen0;
  mask	= (mushy_old==0);
  eles  = find(mask==1);
  T     = phiold(:,1);
  C     = phiold(:,2);
%  disp([' # eles no mushy en t_n --> ' num2str(length(eles)) ]);
  if length(eles)>0,
    ijob= 1;
	coe = [-1;0;0;-1;0];
    mushy_e = mushy_old(eles);
    fmushy1;
  end

% [4]
% Todos los elementos zona mushy tiempo anterior
  nen   = 3;
  mask	= (mushy_old==1);
  eles  = find(mask==1);
  T     = phiold(:,1);
  C     = phiold(:,2);
%  disp([' # eles mushy en t_n --> ' num2str(length(eles)) ]);
  if length(eles)>0,
    ijob= 1;
	coe = [-1;0;0;-1;0];
    fmushy2;
  end

if size(teco)~=[], bc_teco; end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

  [row,col,coe] = assemble(Kele,icone,2);
  Kg = sparse(row,col,coe);

  for iele=1:numel,
  for inod=1:nen0,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

%------------------------------------------------------------------
%       Aplicamos condiciones de periodicidad
%
%    arreglo PERI    i (col 1 ) ----> j (col 2)
%------------------------------------------------------------------

 if isempty(peri)==0,
   n1 = peri(:,1); 
   n2 = peri(:,2);
   Kg((n2-1)*ndf+1,:) = Kg((n2-1)*ndf+1,:)+Kg((n1-1)*ndf+1,:);
   Kg((n2-1)*ndf+2,:) = Kg((n2-1)*ndf+2,:)+Kg((n1-1)*ndf+2,:);
   Kg((n1-1)*ndf+1,:) = 0*Kg((n1-1)*ndf+1,:);
   Kg((n1-1)*ndf+2,:) = 0*Kg((n1-1)*ndf+2,:);

   Rg((n2-1)*ndf+1,1) = Rg((n2-1)*ndf+1,1)+Rg((n1-1)*ndf+1,1);
   Rg((n2-1)*ndf+2,1) = Rg((n2-1)*ndf+2,1)+Rg((n1-1)*ndf+2,1);
   Rg((n1-1)*ndf+1,1) = 0*Rg((n1-1)*ndf+1,1);
   Rg((n1-1)*ndf+2,1) = 0*Rg((n1-1)*ndf+2,1);

   Kg((n1-1)*ndf+1,(n1-1)*ndf+1)= eye(size(n1),size(n1));
   Kg((n1-1)*ndf+2,(n1-1)*ndf+2)= eye(size(n1),size(n1));
   Kg((n1-1)*ndf+1,(n2-1)*ndf+1)=-eye(size(n1),size(n2));
   Kg((n1-1)*ndf+2,(n2-1)*ndf+2)=-eye(size(n1),size(n2));
 end

 Rg_old = v2m(Rg,ndf);
