function [Kg,Rg] = ...
  fem005(xnod,icone,fixa,forc,teco,peri,prop,veloc,source,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem005(xnod,icone,fixa,forc,prop,veloc,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion de Poisson 
%        #.- 2D plano y axisimetrico
%        #.- Triangulos y quads.
%
%    ijob = 0 ---> matriz tangente y residuo
%    ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

  [numel,nen]   = size(icone);
  if sum(icone(:,nen))==0, icone(:,nen)=[]; end
  [numel,nen]   = size(icone);

  nen0          = nen;
  ndf           = 1;
  tol           = 1e-6;

  alpha      = prop(1);
  npg        = prop(2);
  axisim     = prop(3);
  dt         = prop(4);

  if nen==3,
    if npg>=2,npg=3;end
  elseif nen==4,
	if npg>=3, npg=9; end
    if npg==2, npg=4; end
  else
    error(' Bad choice of Gauss point number ')
  end

  T = phi(:,1);
  Told = phiold(:,1);

  Rg   =forc;
  Kele =zeros(numel,nen0^2*ndf^2);
  Rele =zeros(numel,nen0*ndf);

  eles     = (1:numel)';
  ceros    = zeros(length(eles),nen);

  nen   = nen0;
  eles  = (1:numel)';
  T     = phi(:,1);

  if length(eles)>0,    

  ne    = length(eles);
  Kelet = zeros(ne,nen^2*ndf^2);
  Relet = zeros(ne,nen*ndf);

% Puntos de Gauss para elementos cuadrangulares

  if nen==4,
   if npg==4,
    xpg  = sqrt(3)/3;
    psipg = [ -xpg xpg xpg -xpg ];
    etapg = [ -xpg -xpg xpg xpg ];
    wpg   = [    1   1   1    1 ];
   elseif npg==1,
    xpg  = 0;
    psipg = [ xpg ];
    etapg = [ xpg ];
    wpg   = [   4 ];
   end
  else
   if npg==3,
    psipg = [ 1/6  2/3  1/6 ];
    etapg = [ 1/6  1/6  2/3 ];
    wpg   = [ 1/6  1/6  1/6 ];
   elseif npg==1,
    psipg = [ 1/3 ];
    etapg = [ 1/3 ];
    wpg   = [ 1/2 ];
   end
  end

% Localizacion
    ceros = zeros(length(eles),nen);
       Te = ceros;
	Teold = ceros;
	radio = ceros;

  for inod=1:nen,
               nodes = icone(eles,inod);
       Te(:,inod) = T(nodes);
    Teold(:,inod) = Told(nodes);
    radio(:,inod) = xnod(nodes,1);
  end


if nen0==4,
  [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0,0);
  [dNdx0,dNdy0,N0] = shape(X2S,0,0,nen0); 
elseif nen0==3,
  [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),1/3,1/3);
  [dNdx0,dNdy0,N0] = shape(X2S,1/3,1/3,nen0); 
else
  error(' Bad type of element , nen ~=3, nen ~=4 ')
end


 for ipg=1:npg

  [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));

  [dNdx,dNdy,N] = shape(X2S,psipg(ipg),etapg(ipg),nen); 

% Interpolacion en punto de Gauss

     Tpg    = zeros(ne,1);
  Toldpg    = zeros(ne,1);
    dTpg    = zeros(ne,2);
   radpg    = zeros(ne,1);

  for inod=1:nen,
      Tpg(:,1)  =     Tpg(:,1) +      N(inod).*    Te(:,inod);
   Toldpg(:,1)  =  Toldpg(:,1) +      N(inod).* Teold(:,inod);
     dTpg(:,1)  =    dTpg(:,1) + dNdx(:,inod).*    Te(:,inod);
     dTpg(:,2)  =    dTpg(:,2) + dNdy(:,inod).*    Te(:,inod);
	radpg(:,1)  =   radpg(:,1) +      N(inod).* radio(:,inod);    
  end

  xsj =  wpg(ipg) .* Jaco;
  if axisim,
     xsj = xsj.*radpg;
  end

% armado matriz tangente elemental y vector residuo
%------------------------------------------------------------------
  for inod=1:nen,
   if ijob==0,

% Matriz tangente para el balance de energia
%--------------------------------------------------------------------
    for jnod=1:nen,
     ip=((inod-1)*nen+(jnod-1))*ndf^2;
     ter1 = N(inod)*N(jnod)/dt   ;
     ter2 = (dNdx(:,inod).*dNdx(:,jnod) + ... 
             dNdy(:,inod).*dNdy(:,jnod)).* alpha ;
     Kelet(:,ip+1) = Kelet(:,ip+1) + (ter1+ter2) .* xsj;
    end  % jnod

   end % if ijob

% residuo del balance de energia
%-----------------------------------------------------------

    ter1 = N(inod).*(Tpg-Toldpg)/dt ;
    ter2 = (dNdx(:,inod).*dTpg(:,1)+dNdy(:,inod).*dTpg(:,2)).*alpha ;
    ter3 = N(inod).*(source) ;

   Relet(:,(inod-1)*ndf+1) = Relet(:,(inod-1)*ndf+1) - ( ...
	            ter1+ter2+ter3 ) .* xsj ;
							 
  end % inod


 end % ipg

  for itri=1:ne,
    Kele(eles(itri),:)=  Kele(eles(itri),:)+Kelet(itri,:);
    Rele(eles(itri),:)=  Rele(eles(itri),:)+Relet(itri,:);
  end

  end

if size(teco)~=[], bc_teco; end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

  [row,col,coe] = assemble(Kele,icone,ndf);
  Kg = sparse(row,col,coe);

  for iele=1:numel,
  for inod=1:nen0,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

%------------------------------------------------------------------
%       Aplicamos condiciones de periodicidad
%
%    arreglo PERI    i (col 1 ) ----> j (col 2)
%------------------------------------------------------------------

 if isempty(peri)==0,
   n1 = peri(:,1); 
   n2 = peri(:,2);
   for jdf=1:ndf,
     Kg((n2-1)*ndf+jdf,:) = Kg((n2-1)*ndf+jdf,:)+Kg((n1-1)*ndf+jdf,:);
     Kg((n1-1)*ndf+jdf,:) = 0*Kg((n1-1)*ndf+jdf,:);
     Rg((n2-1)*ndf+jdf,1) = Rg((n2-1)*ndf+jdf,1)+Rg((n1-1)*ndf+jdf,1);
     Rg((n1-1)*ndf+jdf,1) = 0*Rg((n1-1)*ndf+jdf,1);
     Kg((n1-1)*ndf+jdf,(n1-1)*ndf+jdf)= eye(size(n1),size(n1));
     Kg((n1-1)*ndf+jdf,(n2-1)*ndf+jdf)=-eye(size(n1),size(n2));
   end
 end
