function [fsol,dfsdt,dfsdc,cs,dcsdt,dcsdc] = ... 
                       frsol2(T,C,Tm,slope_l,kp,mushy_flag)
%
%              calcula la fraccion solida
%
%    [fsol,dfsdt,dfsdc,cs,dcsdt,dcsdc] = frsol2(T,C,Tm,slope_l,kp,mushy_flag)
%
%    fs <=     tol ---> liquid phase
%    fs >= (1-tol) ---> solid phase
%    otherwise     ---> mushy phase
%

tol     = 1e-6;
Cmin    = 1e-6;
mask    = (C<Cmin); 
C       = (1-mask).*C + mask*Cmin;

slope_s =   slope_l/kp ; 
Tsol    =   Tm+slope_s*C;
Tliq    =   Tm+slope_l*C;

mask    = (T<Tm);

deno    = (T-Tm);
deno    = deno + 1e-16*(abs(deno)<1e-16);
fsol    = mask .* (1/(1-kp)*(1-slope_l*C ./ deno)); 

fsol    = max([fsol';  0*ones(1,length(fsol))])';
fsol    = min([fsol';  1*ones(1,length(fsol))])';
mask    = (fsol>tol);
fsol    = fsol.*mask;
mask    = (fsol<(1-tol));
fsol	= fsol.*mask+(1-mask);

%if nargin~=6, mushy = (T<Tliq)&(T>Tsol);end
%fsol    = fsol .* mushy ;
%fsol    = fsol  + (1-mushy).*(T<=Tsol) ;

mushy   = (fsol>tol)&(fsol<(1-tol));
if nargin==6, mushy = mushy_flag;end

dfsdt   = mushy .* (slope_l * C ./ deno.^2 / (1-kp));
dfsdc   = mushy .* (-slope_l ./ deno / (1-kp));

cs    = (((fsol<=tol)*kp+(fsol>=(1-tol))).*C).*(1-mushy) + ... 
          mushy.*(T-Tm)/slope_l*kp;
              
dcsdt =  mushy * kp/slope_l;
dcsdc = ((fsol<tol)*kp+(fsol>(1-tol))).*(1-mushy) ;