function [dNdx,dNdy,N] = gradN(X2S,psipg,etapg)
%-------------------------------------------------------------------
%          --- FEMCODE routine --- 
%
%   [dNdx,dNdy] = fem003(S2X,psipg,etapg);
%
%      Calculo del gradiente de las funciones de forma
%      en un punto de Gauss
%      para elementos cuadrangulares EXCLUSIVAMENTE
%
%     dNdx = [dN_1/dx dN_2/dx dN_3/dx dN_4/dx]
%     dNdy = [dN_1/dy dN_2/dy dN_3/dy dN_4/dy]
%
%-------------------------------------------------------------------

psi = [-1 1 1 -1 ];
eta = [-1 -1 1 1 ];


% Calculo de las derivadas de las funciones de forma 
% respecto a cada direccion del elemento master
%                dN_j/dpsi_k

N(1)  = 0.25*(1+psi(1)*psipg)*(1+eta(1)*etapg);
N(2)  = 0.25*(1+psi(2)*psipg)*(1+eta(2)*etapg);
N(3)  = 0.25*(1+psi(3)*psipg)*(1+eta(3)*etapg);
N(4)  = 0.25*(1+psi(4)*psipg)*(1+eta(4)*etapg);

dN1d1 = 0.25*psi(1)*(1+eta(1)*etapg);
dN1d2 = 0.25*eta(1)*(1+psi(1)*psipg);

dN2d1 = 0.25*psi(2)*(1+eta(2)*etapg);
dN2d2 = 0.25*eta(2)*(1+psi(2)*psipg);

dN3d1 = 0.25*psi(3)*(1+eta(3)*etapg);
dN3d2 = 0.25*eta(3)*(1+psi(3)*psipg);

dN4d1 = 0.25*psi(4)*(1+eta(4)*etapg);
dN4d2 = 0.25*eta(4)*(1+psi(4)*psipg);

dNdx(:,1) = X2S(:,1) * dN1d1 + X2S(:,2) * dN1d2;
dNdx(:,2) = X2S(:,1) * dN2d1 + X2S(:,2) * dN2d2;
dNdx(:,3) = X2S(:,1) * dN3d1 + X2S(:,2) * dN3d2;
dNdx(:,4) = X2S(:,1) * dN4d1 + X2S(:,2) * dN4d2;

dNdy(:,1) = X2S(:,3) * dN1d1 + X2S(:,4) * dN1d2;
dNdy(:,2) = X2S(:,3) * dN2d1 + X2S(:,4) * dN2d2;
dNdy(:,3) = X2S(:,3) * dN3d1 + X2S(:,4) * dN3d2;
dNdy(:,4) = X2S(:,3) * dN4d1 + X2S(:,4) * dN4d2;
