function [Rele] = bc_conv(xnod,icone,T,H,rhoa,teco,Rele,ndf,axisim)
%
% Contribucion de la integral de contorno por debilitar el termino
% convectivo cuando la velocidad normal al borde depende del gradiente
% de una variable nodal
%

[numel,nen] = size(icone);
[nteco,bas] = size(teco);

if nteco~=0,

  xpg    = sqrt(3)/3;
  psipg  = [ -xpg xpg ];
  wpg    = [   1   1  ];

  eles_teco = teco(:,1);
  face_teco = teco(:,2);

  nodo_1 = face_teco;
  nodo_2 = (nodo_1+1).*(nodo_1<nen)+(nodo_1==nen);
  nodos  = [nodo_1,nodo_2];
  nodog1 = getcol(icone(eles_teco,:),nodo_1);
  nodog2 = getcol(icone(eles_teco,:),nodo_2);

  dxbc   = xnod(nodog2,1) - xnod(nodog1,1);
  dybc   = xnod(nodog2,2) - xnod(nodog1,2);
  lteco  = sqrt(dxbc.^2+dybc.^2);
  norbc  = [dybc./lteco, -dxbc./lteco];

  Te    = T(eles_teco,:);
  He    = H(eles_teco,:);
  rhoae = rhoa(eles_teco,:);

  re    = v2m(xnod(icone(eles_teco,:)',1),nen);

  xiglob = zeros(nteco,4); etaglob = xiglob;
  xiglob(:,2)  = +ones(nteco,1);
  xiglob(:,4)  = -ones(nteco,1);
  etaglob(:,1) = -ones(nteco,1);
  etaglob(:,3) = +ones(nteco,1);

  for ipg=1:2,

   xiglob(:,1)  = psipg(ipg)*ones(nteco,1);
   xiglob(:,3)  = (0-psipg(ipg))*ones(nteco,1);
   etaglob(:,2) = psipg(ipg)*ones(nteco,1);;
   etaglob(:,4)  = (0-psipg(ipg))*ones(nteco,1);

   xipg  = getcol(xiglob,face_teco);
   etapg = getcol(etaglob,face_teco);

   [X2S,S2X,Jaco]  = xy2st(xnod,icone(eles_teco,:),xipg,etapg);
   [dNdx,dNdy,N] = shape(X2S,xipg,etapg,nen); 

   Tpg      = zeros(nteco,1);
   dTpg     = zeros(nteco,2);
   Hpg      = zeros(nteco,1);
   dHpg     = zeros(nteco,2);
   rhoapg   = zeros(nteco,1);
   radpg    = zeros(nteco,1);


   for inod=1:nen,
    radpg        =      radpg + N(:,inod).*re(:,inod);
     Tpg         =        Tpg + N(:,inod).*Te(:,inod);
     Hpg         =        Hpg + N(:,inod).*He(:,inod);
     rhoapg      =     rhoapg + N(:,inod).*rhoae(:,inod);
     dTpg(:,1)   =  dTpg(:,1) + dNdx(:,inod).*Te(:,inod);
     dTpg(:,2)   =  dTpg(:,2) + dNdy(:,inod).*Te(:,inod);
     dHpg(:,1)   =  dHpg(:,1) + dNdx(:,inod).*He(:,inod);
     dHpg(:,2)   =  dHpg(:,2) + dNdy(:,inod).*He(:,inod);
   end 

   [rho_s,Cp_s,Cp_s0,Kgz,Kgx,mu_g,Psat,dPsatdT,kappa_z,kappa_x,Cp_g, ...  
                DeltaH,epsilon,Ra,Rv,Patm,MMv,MMa]=coefmat(Tpg,Hpg);
   [HR,dHRdT,dHRdH] = Hrel(Tpg,Hpg);

   Pvpg    = HR.*Psat;
   rhovpg  = Pvpg./(Rv*Tpg);
   dPvpg   = (((dPsatdT.*HR)*[1 1]) + ((Psat.*dHRdT)*[1 1])).*dTpg + ...
             (Psat.*dHRdH*[1 1]).*dHpg ;

   velpg = -(1 ./ mu_g * [Kgx,Kgz]).*dPvpg / epsilon;

% DEBUG
%   velpg = [zeros(nteco,1) ones(nteco,1)];
%   velpg = [zeros(nteco,1) zpg];
% FIN DEBUG

   vn     = velpg(:,1).*norbc(:,1)+velpg(:,2).*norbc(:,2);

   fac1 = epsilon*(rhoapg+rhovpg).*Cp_g./(rho_s*Cp_s0);

   N1     = 0.5*(1-psipg(ipg));
   N2     = 0.5*(1+psipg(ipg));
   Nb     = [N1 N2];

   xsj = wpg(ipg)*lteco/2;
   if axisim,
     xsj = xsj.*radpg;
   end

   for inod=1:2,
    iinod = nodos(:,inod);
    vaux  =  Nb(inod)*vn.*Tpg.*fac1.*xsj;
%  disp(' bc_conv ');keyboard;pause
	Rele(eles_teco,:) = addcol(Rele(eles_teco,:),-vaux,(iinod-1)*ndf+1);
    vaux  =  Nb(inod)*vn.*rhovpg .* xsj;
	Rele(eles_teco,:) = addcol(Rele(eles_teco,:),-vaux,(iinod-1)*ndf+2);
   end  % for inod


  end % puntos de Gauss del contorno

end % if teco

