function [rho_s,Cp_s,Cp_s0,Kgz,Kgx,mu_g,Psat,dPsatdT,kappa_z,kappa_x,Cp_g,...  
             DeltaH,epsilon,Ra,Rv,Patm,MMv,MMa]=coefmat(Tpg,Hpg)
%
%[rho_s,Cp_s,Kgz,Kgx,mu_g,Psat,dPsatdT,kappa_z,kappa_x,Cp_g, ...  
%                DeltaH,epsilon,Ra,Rv,Patm,MMv,MMa]=coefmat(Tpg,Hpg)
%

% Densidad del solido
rho_s = 750 ; % kg/m3
% Calor especifico de la madera
Cp_s0 = 4180/2; 
Cp_s = 4180*(0.268+0.0011*(Tpg-273.15)+Hpg*100)./(1+Hpg*100);
% DEBUG
%Cp_s = 4180*(0.268+0.0011*(350-273.15)+0.11)./(1+0.11)*ones(size(Cp_s));

% Permeabilidad (ec 16)
Kgz   = 1.74e-12*exp(-8.06e-3*rho_s); 

Kgx   = 59*Kgz; 

% DEBUG
%Kgz= Kgz/100;
%Kgx = 0*Kgx;
% END DEBUG

% Viscosidad del gas (sacado del paper de Humphrey & Bolton part II)
mu_g = 1.12e-5*Tpg.^(1.5)./(Tpg+2937.8);
% Presion de saturacion
Psat = 10.^(10.745-2141.0./Tpg);
dPsatdT = log(10)*Psat.*(2141.0./Tpg.^2);
% conductividad de la mezcla
% del paper de CC
%kappa_z = (0.01172+0.0001319*rho_s)*(1.03e-4*Hpg+1).*(1.077e-3*Tpg+0.97);
% del paper de Humphrey II
kappa_z = (0.01172+0.0001319*rho_s)*(1.03e-3*(Hpg*100)+1).*(1.077e-3*(Tpg-273.15)+0.978);
kappa_x = 1.5*kappa_z;
% Calor especifico del gas (considero aire en CNTyP) Joule/Kg/K
Cp_g = 1000;
% Calor de vaporizacion
DeltaH = 2.511e+6-2.48e+3*(Tpg-273.15)+1.176e+6*exp(-0.15*(Hpg*100));
% DEBUG
%DeltaH = 10^6*DeltaH;

% porosidad (ec. 15 paper CC)
rho_f   = 900;
rho_r   = 1100;
yr      = 0.085;
epsilon = 1 - rho_s*(1/rho_f+yr/rho_r)/(1+yr);
% constante de los gases
MMa = 29;
MMv = 18;
Ra = 8314/MMa;
Rv = 8314/MMv;
% Presion atmosferica
Patm = 0*101.3*1000;  % Pa