function [Kg,Rg] = ele_crd(xnod,icone,ifaceb,fixa,forc,teco,peri,prop,...
                          veloc,reac,source,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = ele_crd(xnod,icone,fixa,forc,teco,peri,prop,
%                      veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion adveccion difusion reaccion no estacionaria c/fuente
%        #.- elemento cuadrangular y triangular
%        #.- plano y axisimetrico
%
%        ijob = 1 ---> matriz tangente y residuo
%        ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

  [numel,nen]   = size(icone);
  nen0          = nen;
  [numnp,ndf]   = size(phi);
  if ndf~=1, error(' NDF debe ser 1');end

  rho      = prop(1) ;
  Cp       = prop(2) ;
  kappa    = prop(3) ;
  npg      = prop(4) ;
  upwind   = prop(6) ;
  shocap   = prop(7) ;
  axisim   = prop(8) ;
  type_upw = prop(9) ;
  dtfic    = prop(10);
  lumped   = prop(11);
  tol      = prop(12);
  dt       = prop(13);
  alpha    = prop(14);
  dt       = alpha*dt;
  
  pgauss;
  
  eles  = (1:numel)';
  ceros    = zeros(length(eles),nen);
  
  Told = phiold(:,1);
  T = alpha*phi(:,1)+(1-alpha)*phiold;

  Rg   =forc;
  Kele =zeros(numel,nen^2*ndf^2);
  Rele =zeros(numel,nen*ndf);

  if length(eles)>0,    
    ele_crd_2;
  end

if ~isempty(teco), 
  [Kele,Rele] = bc_teco(xnod,icone,T,teco,Kele,Rele,ijob,1,1);
end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

  [row,col,coe] = assemble(Kele,icone,ndf);
  Kg = sparse(row,col,coe);

  for iele=1:numel,
  for inod=1:nen,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

%------------------------------------------------------------------
%       Aplicamos condiciones de periodicidad
%
%    arreglo PERI    i (col 1 ) ----> j (col 2)
%------------------------------------------------------------------

 if isempty(peri)==0,
   n1 = peri(:,1); 
   n2 = peri(:,2);
   for jdf=1:ndf,
     Kg((n2-1)*ndf+jdf,:) = Kg((n2-1)*ndf+jdf,:)+Kg((n1-1)*ndf+jdf,:);
     Kg((n1-1)*ndf+jdf,:) = 0*Kg((n1-1)*ndf+jdf,:);
     Rg((n2-1)*ndf+jdf,1) = Rg((n2-1)*ndf+jdf,1)+Rg((n1-1)*ndf+jdf,1);
     Rg((n1-1)*ndf+jdf,1) = 0*Rg((n1-1)*ndf+jdf,1);
     Kg((n1-1)*ndf+jdf,(n1-1)*ndf+jdf)= eye(length(n1),length(n1));
     Kg((n1-1)*ndf+jdf,(n2-1)*ndf+jdf)=-eye(length(n1),length(n2));
   end
 end

%disp('fem001');keyboard;pause