function [Mg,Rg] = euler_ale(xnod,icone,fixa,forc,teco,peri,prop,...
                          ifaceb,slip,data_slip,nodes_bou,U,Uold,ijob)
%------------------------------------------------------------------
%
%     [Mg,Rg] = euler_ale(xnod,icone,fixa,forc,teco,peri,prop,
%                      U,Uold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuaciones de Euler 
%        #.- elemento cuadrangular y triangular
%        #.- 2D, plano y axisimetrico
%        #.- no estacionario y explicito (masa diagonal)
%
%        ijob = 1 ---> matriz masa diagonal y residuo
%        ijob = 0 ---> solo residuo
%
% version TET (Tayfun Tezduyar)
% esta version tiene las siguientes caracteristicas
%
% [1] no tiene la opcion de taucte
% [2] tiene el calculo del termino de contorno convectivo incluido
% [3] intento de tratar las condiciones periodicas y slip a nivel
%     del elemento
%------------------------------------------------------------------

global row col

  [numel,nen]   = size(icone);
  [numnp,ndf]   = size(U);
  if ndf~=4, error(' NDF debe ser 4 (Euler 2D) ');end
  ndm = 2;

% tipo de matriz del sistema a armar
  matrix_type = 'full';  % arma el jacobiano lo mejor posible
%  matrix_type = 'diag';  % solo la matriz de masa y lumped 

%%%%%%%%%%%%%% PROPIEDADES DEL MATERIAL 
  gamma    = prop(1) ;
  Cou      = prop(2) ;
  kappa    = prop(3) ;
  npg      = prop(4);
  upwind   = prop(6);
  shocap   = prop(7);
  axisim   = prop(8);
  type_upw = prop(9);
  Cv       = prop(10);
  lumped   = prop(11);

  tol      = prop(12);
  dt       = prop(13);

  visco    = prop(14)  ;
  sigma    = prop(15)  ;
  Pran     = prop(16)  ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  pgauss;

  Rele =zeros(numel,nen*ndf);
  Mele =zeros(numel,nen*ndf);
  Rg   =0*forc;
  if matrix_type=='full',
    Mele = zeros(numel,(nen*ndf)^2);
  end

  eles     = (1:numel)';
  ceros    = zeros(length(eles),nen);


%t0=cputime;
  if length(eles)>0,    
    euler_ale_2;
  end


%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

%t0=cputime;

 if matrix_type=='full',
% version implicita --->matriz jacobiana
  %[row,col,coe] = assemble(Mele,icone,ndf);
   
   coe = m2v(Mele');

   Mg = sparse(row,col,coe);
   
   ijob=1;  %  ijob=0 monta una matriz
   %  ijob=1 monta un vector
   %  usamos row1,col1,coe1 porque row y col van por global para la matriz
   [row1,col1,coe1] = assemble(Rele,icone,ndf,ijob);
   Rg = sparse(row1,col1,coe1);
   Rg = spdiags(Rg);

   apply_peri_bc_full

   apply_slip_bc_full

 else
% version explicita --->matriz masa diagonal (lumped)

   ijob=1;  %  ijob=0 monta una matriz
            %  ijob=1 monta un vector
   [row,col,coe] = assemble(Rele,icone,ndf,ijob);
   Rg = sparse(row,col,coe);
   Rg = spdiags(Rg);

   [row,col,coe] = assemble(Mele,icone,ndf,ijob);
   Mg = sparse(row,col,coe);
   Mg = spdiags(Mg);


   apply_peri_bc_diag

   apply_slip_bc_diag

% duplicamos masa nodos frontera para un problema en particular
  ldupli=0;
  if ldupli,
    for k=1:length(nodes_bou),
       Mg((nodes_bou(k)-1)*ndf+(1:ndf),1) = ...
                  2*Mg((nodes_bou(k)-1)*ndf+(1:ndf),1);
    end
  end

end

%disp('FEM002');keyboard;pause
