function [Kg,Rg] = fem000(xnod,icone,fixa,forc,teco,peri,prop,...
                          veloc,reac,source,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem000(xnod,icone,fixa,forc,teco,peri,prop,
%                      veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion adveccion difusion reaccion no estacionaria c/fuente
%        #.- elemento unidimensional
%
%        ijob = 0 ---> matriz tangente y residuo
%        ijob = 1 ---> solo residuo
%        ijob = -1 --> matriz de masa y residuo (para explicitos)
%
%------------------------------------------------------------------

  [numel,nen]   = size(icone);
  [numnp,ndf]   = size(phi);

  kappa    = prop(1) ;
  npg      = prop(2);
  dt       = prop(3);
  upwind   = prop(4);

  lumped   = prop(11);
  tol      = prop(12);

  T        = phi(:,1);
  Told     = phiold(:,1);

  Rg       = forc;
  Kele     = zeros(numel,nen^2*ndf^2);
  Rele     = zeros(numel,nen*ndf);

  eles     = (1:numel)';
  fem0000;

%disp(' FEM0000');keyboard;pause

%if size(teco)~=[], bc_teco; end
if isempty(teco)==0,bc_teco; end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------
if ijob <0,

Kg=zeros(numnp*ndf,1);
for inod=1:nen,
 for idf=1:ndf,
  ip=(inod-1)*ndf+idf;
  jp=(icone(:,inod)-1)*ndf+idf;
  Kg(jp,1) = Kg(jp,1) + Kele(:,ip);
  Rg(jp,1) = Rg(jp,1) + Rele(:,ip);         
 end
end

%disp(' FEM000 ');keyboard;pause

else
  [row,col,coe] = assemble(Kele,icone,ndf);
  Kg = sparse(row,col,coe);

  for iele=1:numel,
  for inod=1:nen,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

end

%------------------------------------------------------------------
%       Aplicamos condiciones de periodicidad
%
%    arreglo PERI    i (col 1 ) ----> j (col 2)
%------------------------------------------------------------------

 if isempty(peri)==0,
   n1 = peri(:,1); 
   n2 = peri(:,2);
   for jdf=1:ndf,
     Kg((n2-1)*ndf+jdf,:) = Kg((n2-1)*ndf+jdf,:)+Kg((n1-1)*ndf+jdf,:);
     Kg((n1-1)*ndf+jdf,:) = 0*Kg((n1-1)*ndf+jdf,:);
     Rg((n2-1)*ndf+jdf,1) = Rg((n2-1)*ndf+jdf,1)+Rg((n1-1)*ndf+jdf,1);
     Rg((n1-1)*ndf+jdf,1) = 0*Rg((n1-1)*ndf+jdf,1);
     Kg((n1-1)*ndf+jdf,(n1-1)*ndf+jdf)= eye(size(n1),size(n1));
     Kg((n1-1)*ndf+jdf,(n2-1)*ndf+jdf)=-eye(size(n1),size(n2));
   end
 end

%disp('FEM000');keyboard;pause
