%------------------------------------------------------------------
% Continua de fem002 - COMPRESSIBLE NAVIER-STOKES WITH PRECO_CHOI
%------------------------------------------------------------------

%disp(' FEM0023 [0]');keyboard;pause

  ne       = length(eles);
  taucte   = 1;
  ipreco   = 1;
  Mach_min = 1e-6;

%%%%%%%%%%%%%% PROPIEDADES DEL MATERIAL 
  gamma    = prop(1) ;
  Cou      = prop(2) ;
  kappa    = prop(3) ;
  npg      = prop(4);
  dt       = prop(5);
  upwind   = prop(6);
  shocap   = prop(7);
  axisim   = prop(8);
  type_upw = prop(9);
  dtfic    = prop(10);
  lumped   = prop(11);
  tol      = prop(12);
  Cv       = prop(13);
  visco    = prop(14);
  sigma    = prop(15);
  Pran     = prop(16);

  R_gas = Cv*(gamma-1);
  ga1   = gamma-1;
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Puntos de Gauss para elementos cuadrangulares y triangulares
%------------------------------------------------------------------

  if nen==4,
   if npg==4,
    xpg  = sqrt(3)/3;
    psipg = [ -xpg xpg xpg -xpg ];
    etapg = [ -xpg -xpg xpg xpg ];
    wpg   = [    1   1   1    1 ];
   elseif npg==1,
    xpg  = 0;
    psipg = [ xpg ];
    etapg = [ xpg ];
    wpg   = [   4 ];
   end
  else
   if npg==3,
    psipg = [ 1/6  2/3  1/6 ];
    etapg = [ 1/6  1/6  2/3 ];
    wpg   = [ 1/6  1/6  1/6 ];
   elseif npg==1,
    psipg = [ 1/3 ];
    etapg = [ 1/3 ];
    wpg   = [ 1/2 ];
   end
  end

% Localizacion
%------------------------------------------------------------------

    ceros = zeros(length(eles),nen*ndf);
       Ue = ceros;
      U0e = ceros;
	radio = ceros;

  for inod=1:nen,
            nodes = icone(eles,inod);
	         jnod = (inod-1)*ndf+(1:ndf);
       Ue(:,jnod) = U(nodes,1:ndf);
      U0e(:,jnod) = Uold(nodes,1:ndf);
    radio(:,inod) = xnod(nodes,1);
  end

 if taucte,

% Calculo de transformacion isoparametrica y funciones de forma
%------------------------------------------------------------------

if nen==4,
  [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0,0);
  [dNdx0,dNdy0,N0] = shape(X2S,0,0,nen); 
elseif nen==3,
  [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),1/3,1/3);
  [dNdx0,dNdy0,N0] = shape(X2S,1/3,1/3,nen); 
else
  error(' Bad type of element , nen ~=3, nen ~=4 ')
end

%------------------------------------------------------------------
%                      SUPG WITH CHOI PRECO
%------------------------------------------------------------------

  UC_0    = zeros(ne,ndf);
  gU_0    = zeros(ne,2*ndf);

  for inod=1:nen,
    jnod = (inod-1)*ndf+(1:ndf);
    UC_0(:,(1:ndf))  = UC_0(:,(1:ndf)) + N0(inod).* Ue(:,jnod);
	gU_0(:,(1:ndf))  = gU_0(:,(1:ndf)) + ...
	          (dNdx0(:,inod)*ones(1,ndf)).* Ue(:,jnod);
	gU_0(:,(1:ndf)+ndf) = gU_0(:,(1:ndf)+ndf)+ ...
	          (dNdy0(:,inod)*ones(1,ndf)).* Ue(:,jnod);
  end

  UP_0           = cv2pv(UC_0,gamma,+1);
  vele_0         = UP_0(:,2:ndf-1);
  hele_0         = h_supg(vele_0,dNdx0,dNdy0);
  hele           = hele_0;
%  velmod         = sqrt(sum(vele_0'.^2))';
%  velmod         = velmod+(velmod<1e-16)*1e-16;

%  [P1 ]            = InvPreco(UC_0,gamma,visco,sigma,Cou,hele_0*[1,1,1]);
%  [UV,dQvdQ,dQdQv] = cv2vv(UC_0,gamma,R_gas,+1);
%  [Tau]            = Tau_choi(UC_0,P1,dQdQv,dQvdQ, ...
%                gamma,R_gas,visco,sigma,Cou,hele_0*[1,1,1],X2S,Mach_min);

t0 = cputime;

  [Tau]  = ...
    Tau_choi(UC_0,gamma,R_gas,visco,sigma,Cou,hele_0*[1,1,1],X2S,Mach_min);

  [dt]  = autodt(UC_0,gamma,hele_0*[1,1,1],Cou,visco,0,Pran,Pran, ...
                 sigma,ipreco,Mach_min);

t1 = cputime;
disp([' cpu = ' num2str(t1-t0)])

  Ax_0             =  Ajac(UC_0,UP_0,gamma,1);
  Ay_0             =  Ajac(UC_0,UP_0,gamma,2);

 end  % taucte

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------

 for ipg=1:npg,

  [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
  [dNdx,dNdy,N] = shape(X2S,psipg(ipg),etapg(ipg),nen); 

% Interpolacion en punto de Gauss
%------------------------------------------------------------------

     Upg     = zeros(ne,ndf);
     UPpg    = zeros(ne,ndf);
    U0pg     = zeros(ne,ndf);
    dUpg     = zeros(ne,ndm*ndf);
    dUPpg    = zeros(ne,ndm*ndf);
    dTpg     = zeros(ne,ndm);
   radpg     = zeros(ne,1);

    Te       = zeros(ne,nen);
	for inod=1:nen,
      jnod = (inod-1)*ndf;
	  ip = 	jnod+(1:ndf);
	  bas = cv2pv(Ue(:,ip),gamma,+1);
      UPe(:,ip)  = bas;
       Te(:,inod)  = UPe(:,jnod+ndf)./(Cv*ga1*UPe(:,jnod+1));
	end 


%	for inod=1:nen,
%     jnod = (inod-1)*ndf;
%     Te(:,inod)  = UPe(:,jnod+ndf)./(Cv*ga1*UPe(:,jnod+1));
%	end

  for inod=1:nen,
     jnod = (inod-1)*ndf+(1:ndf);
      Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
%     UPpg(:,(1:ndf))  = UPpg(:,(1:ndf)) + N(inod).* UPe(:,jnod);
     U0pg(:,(1:ndf))  = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
	 dUpg(:,(1:ndf))  = dUpg(:,(1:ndf)) + ...
	          (dNdx(:,inod)*ones(1,ndf)).* Ue(:,jnod);
	 dUpg(:,(1:ndf)+ndf) = dUpg(:,(1:ndf)+ndf)+ ...
	          (dNdy(:,inod)*ones(1,ndf)).* Ue(:,jnod);
     dUPpg(:,(1:ndf))  = dUPpg(:,(1:ndf)) + ...
	          (dNdx(:,inod)*ones(1,ndf)).* UPe(:,jnod);
	 dUPpg(:,(1:ndf)+ndf) = dUPpg(:,(1:ndf)+ndf)+ ...
	          (dNdy(:,inod)*ones(1,ndf)).* UPe(:,jnod);
     dTpg(:,1)  = dTpg(:,1) + dNdx(:,inod).* Te(:,inod);	          
	 dTpg(:,2)  = dTpg(:,2) + dNdy(:,inod).* Te(:,inod);
	          
	radpg(:,1)  =   radpg(:,1) +      N(inod).* radio(:,inod);
  end

%------------------------------------------------------------------
%                          KYRELE
%
%  Calcula constribuciones elementales a 
%  la matriz de masa diagonal M y al vector R
%
%------------------------------------------------------------------

  UPpg = cv2pv(Upg,gamma,+1);

  if ~taucte,

%t0  = cputime;

% segun matriz jacobiana al elemento master
   kv = X2S;
   vele_pg         = UPpg(:,2:ndf-1);
   hele_pg         = h_supg(vele_pg,dNdx,dNdy);
   hele            = hele_pg;
%  [tau,dt] = Tau_Hug2(Upg,UPpg,kv,hele_pg,Cou,gamma);

%  [P1 ]            = InvPreco(Upg,gamma,visco,sigma,Cou,hele_pg*[1,1,1]);
%  [UV,dQvdQ,dQdQv] = cv2vv(Upg,gamma,R_gas,+1);
%  [Tau]            = Tau_choi(Upg,P1,dQdQv,dQvdQ, ...
%             gamma,R_gas,visco,sigma,Cou,hele_pg*[1,1,1],X2S,Mach_min);

t0 = cputime;
  [Tau]  = ...
    Tau_choi(Upg,gamma,R_gas,visco,sigma,Cou,hele_pg*[1,1,1],X2S,Mach_min);

t1 = cputime;
disp([' cpu = ' num2str(t1-t0)])


  [dt]  = autodt(Upg,gamma,hele_pg*[1,1,1],Cou,visco,0,Pran,Pran, ...
                 sigma,ipreco,Mach_min);

  end



% jacobianos segun variable de estado en el pto de Gauss actual
  Ax   =  Ajac(Upg,UPpg,gamma,1);
  Ay   =  Ajac(Upg,UPpg,gamma,2);

% Calculo de los flujos advectivos, 
% necesario si debilitamos el flujo advectivo
%  [Fa] = fluxa(Upg,UPpg);

% Calculo del flujo difusivo
   [Fd] = Fluxd(UPpg,dUPpg,dTpg,gamma,Cv,visco,0,Pran,Pran);

% Correccion axisimetrica del jacobiano de la transformacion al elemento master
%------------------------------------------------------------------
  xsj =  wpg(ipg) .* Jaco;
  if axisim,
     xsj = xsj.*radpg;
  end

% Calculo matriz masa diagonal y residuo
%--------------------------------------------------------------------

    Fax = rowprod(Ax,dUpg(:,1:ndf),ndf,ndf,1);
    Fay = rowprod(Ay,dUpg(:,ndf+(1:ndf)),ndf,ndf,1);

    if taucte,
    Ataux = rowprod(Ax_0,Tau,ndf,ndf,ndf);
    Atauy = rowprod(Ay_0,Tau,ndf,ndf,ndf);
	else
    Ataux = rowprod(Ax,Tau,ndf,ndf,ndf);
    Atauy = rowprod(Ay,Tau,ndf,ndf,ndf);
	end

    AtauAdx = rowprod(Ataux,(Fax+Fay),ndf,ndf,1);
	AtauAdy = rowprod(Atauy,(Fax+Fay),ndf,ndf,1);

    [betaMa2] = betama2(UPpg,hele*[1,1,1],gamma,Mach_min,visco,Cou,sigma);
    [Pc1] = Pre_cho2(Upg,betaMa2,gamma,R_gas,-1,+1);

  for inod=1:nen,
    ip=(inod-1)*ndf+(1:ndf);

%------ Masa diagonal
      ter1 = N(inod)./dt;
	  Mele(:,ip) = Mele(:,ip) + (ter1*xsj)*ones(1,ndf);

%------ Residuo

%    ter1 = N(inod)*(Upg(:,(1:ndf))-U0pg(:,(1:ndf)))/dt ;
%    ter2 = (N(inod)+P1_pg(:,ip)).*rowprod(Ax,dUpg(:,1:ndf),ndf,ndf,1)+ ...
%           (N(inod)+P1_pg(:,ip)).*rowprod(Ay,dUpg(:,ndf+(1:ndf)),ndf,ndf,1);

     ter1 = N(inod)*(Fax+Fay);
	 ter2 = rowprod(dNdx(:,inod)*m2v(eye(ndf))',AtauAdx,ndf,ndf,1)+...
            rowprod(dNdy(:,inod)*m2v(eye(ndf))',AtauAdy,ndf,ndf,1);

% Esto se usaria si debilitamos el flujo advectivo
%    ter2 =  rowprod(Fa(:,1:ndf),dNdx(:,inod)*ones(1,ndf),ndf,ndf,1)+ ...
%            rowprod(Fa(:,ndf+(1:ndf)),dNdy(:,inod)*ones(1,ndf),ndf,ndf,1);
%    ter2 = ter2 + (P1_pg(:,ip)).*rowprod(Ax,dUpg(:,1:ndf),ndf,ndf,1)+ ...
%           (P1_pg(:,ip)).*rowprod(Ay,dUpg(:,ndf+(1:ndf)),ndf,ndf,1);

	 ter3 = rowprod(dNdx(:,inod)*m2v(eye(ndf))',Fd(:,1:ndf),ndf,ndf,1)+...
         rowprod(dNdy(:,inod)*m2v(eye(ndf))',Fd(:,ndf+(1:ndf)),ndf,ndf,1);
	Rele1 = ( ter1+ter2+ter3 ) .* (xsj*ones(1,ndf)) ;

%disp(' FEM0023 ');keyboard;pause

	Rele1 = rowprod(Pc1,Rele1,ndf,ndf,1);

    Rele(:,ip) = Rele(:,ip) - Rele1; 
            
 end % inod

 end % ipg



% ------ Terminos de contorno

% si debilitamos aparece un termino de contorno aqui del tipo
%   calculo normal a cada (elemento-cara) del borde
%   [eleb,faceb] = find_bou(????);
%   normal = ???
%   [Fa_n] = fluxan(Fa(eleb,:),normal);
% 	ver rutina bc_teco para armar algo similar


if sum(size(teco))~=0, error(' EULER con TECO '); end
