function [Kg,Rg] = fem004(xnod,icone,fixa,forc,teco,peri,prop,...
                          veloc,reac,source,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem004(xnod,icone,fixa,forc,teco,peri,prop,
%                      veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion vorticidad-funcion de corriente
%        #.- elemento cuadrangular y triangular
%        #.- plano y axisimetrico
%
%        ijob = 0 ---> matriz tangente y residuo
%        ijob = 1 ---> solo residuo
%
%------------------------------------------------------------------



  [numel,nen]   = size(icone);
  nen0          = nen;
  [numnp,ndf]   = size(phi);
  if ndf~=2, error(' NDF debe ser 2');end

  rho      = prop(1) ;
  Cp       = prop(2) ;
  kappa    = prop(3) ;
  npg      = prop(4) ;
  upwind   = prop(6) ;
  shocap   = prop(7) ;
  axisim   = prop(8) ;
  type_upw = prop(9) ;
  dtfic    = prop(10);
  lumped   = prop(11);
  tol      = prop(12);
  dt       = prop(13);

  if nen==3,
	if npg==3,npg=7;end
    if npg==2,npg=3;end
  elseif nen==4,
    if npg==2, npg=4; end
	if npg==3, npg=9; end
  else
    error(' Bad choice of Gauss point number ')
  end

  psi = phi(:,1);
  vor = phi(:,2);
  vorold = phiold(:,2);

  eles     = (1:numel)';
  ceros    = zeros(length(eles),nen);

% Calculo de la vorticidad en aquellos contornos donde se impone la 
% velocidad tangencial (ijob=2)

  if exist('FVTG_BC.M')~=2, 
    disp(' ERROR FEM004 - CALCULO VORTICIDAD EN EL CONTORNO');
    error(' FALTA FICHERO DATOS VELOCIDAD TANGENCIAL EN EL BORDE '); 
  end
  fvtg_bc;
  if sum(size(vtg_bc))~=0,
    ijob = 2;
    vtg = zeros(numnp,1);
	vtg(vtg_bc(:,1),1) = vtg_bc(:,2);
    fem0004a;
    vor = phi(:,2);
  end

% Calculo del update de ambos campos (ijob=0)
  Rg   =forc;
  Kele =zeros(numel,nen0^2*ndf^2);
  Rele =zeros(numel,nen0*ndf);

  ijob = 0;
  fem0004b;

if size(teco)~=[], bc_teco; end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

  [row,col,coe] = assemble(Kele,icone,ndf);
  Kg = sparse(row,col,coe);

  for iele=1:numel,
  for inod=1:nen0,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

%------------------------------------------------------------------
%       Aplicamos condiciones de periodicidad
%
%    arreglo PERI    i (col 1 ) ----> j (col 2)
%------------------------------------------------------------------

 if isempty(peri)==0,
   n1 = peri(:,1); 
   n2 = peri(:,2);
   for jdf=1:ndf,
     Kg((n2-1)*ndf+jdf,:) = Kg((n2-1)*ndf+jdf,:)+Kg((n1-1)*ndf+jdf,:);
     Kg((n1-1)*ndf+jdf,:) = 0*Kg((n1-1)*ndf+jdf,:);
     Rg((n2-1)*ndf+jdf,1) = Rg((n2-1)*ndf+jdf,1)+Rg((n1-1)*ndf+jdf,1);
     Rg((n1-1)*ndf+jdf,1) = 0*Rg((n1-1)*ndf+jdf,1);
     Kg((n1-1)*ndf+jdf,(n1-1)*ndf+jdf)= eye(size(n1),size(n1));
     Kg((n1-1)*ndf+jdf,(n2-1)*ndf+jdf)=-eye(size(n1),size(n2));
   end
 end

