function [Kg,Rg] = ...
       fem005(xnod,icone,iface,fixa,forc,teco,peri,prop,veloc,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem005(xnod,icone,iface,fixa,forc,prop,veloc,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion acoplada balance de energia y masa del soluto
%        #.- incluye cambio de fase
%        #.- no estacionario con conveccion
%        #.- NDF == 2
%
%  CALCULO DE MATRIZ TANGENTE A PARTIR DE PERTURBAR EL RESIDUO
%  A NIVEL DE ELEMENTO
%
%       ijob = 0 matriz y residuo
%       ijob = 1 solo residuo
%------------------------------------------------------------------

  [numel,nen]   = size(icone);
  nen0          = nen;
  if nen~=3, error(' FEM005 - nen ~= 3 ');end
  ndf           = 2;
  tol           = prop(12);

  Tm       = prop(1) ; 
  slopel   = prop(2) ;
  kp       = prop(3) ;
  kappa_s  = prop(5) ;
  kappa_l  = prop(6) ;
  rho      = prop(7) ;
  Cp       = prop(8) ;
  DeltaH   = prop(9) ;
  U0       = prop(10);
  alfa_min = prop(11);
  Dm_s     = prop(14);
  Dm_l     = prop(16);
  dt       = prop(13);
  lmushy   = prop(15);
  upwind   = prop(19);
  Dm_l_turb= prop(17);
  shocap   = prop(18);

  T = phi(:,1);
  C = phi(:,2);
  Told = phiold(:,1);
  Cold = phiold(:,2);

  Rg   =forc;
  Kele =zeros(numel,nen0^2*ndf^2);
  Rele =zeros(numel,nen0*ndf);
     
% Calculo del residuo a partir del vector de estado	actual
% ------------------------------------------------------------------
  eles = (1:numel)';
  T    = phi(:,1);
  C    = phi(:,2);
  coe  = [1;1;1;1;1];

  ceros = zeros(length(eles),nen);
  Te = ceros;
  Ce = ceros;  
  for inod=1:nen,
   nodes = icone(eles,inod);
   Te(:,inod) = T(nodes);
   Ce(:,inod) = C(nodes);
  end

  [Rele] = ...
    Res005(Rele,eles,xnod,icone,iface,Te,Ce,veloc,prop,nen,ndf,coe);

  if ijob==0,

% Calculo del residuo a partir del vector de estado perturbado
% ------------------------------------------------------------------
  eles = (1:numel)';
% norma infinito de cada una de las variables
  Tn   = norm(T,inf);
  Cn   = norm(C,inf);
% perturbacion de cada variable
  eT   = 10^(-7)*Tn;
  eC   = 10^(-7)*Cn;
  ephi = [eT,eC];
  ip0  = [1,3,13,15,25,27]-1;
  iph = [];
  for inod=1:nen,
   for idf=1:ndf,
    Sele1= zeros(numel,nen0*ndf);
    T    = phi(:,1);
    C    = phi(:,2);
    coe  = [1;1;1;1;1];

    ceros = zeros(length(eles),nen);
    Te = ceros;
    Ce = ceros;  
    for jnod=1:nen,
     nodes = icone(eles,jnod);
     Te(:,jnod) = T(nodes);
     Ce(:,jnod) = C(nodes);
    end

    if idf==1,
	  Te(:,inod) = Te(:,inod)+ephi(idf);
	else
	  Ce(:,inod) = Ce(:,inod)+ephi(idf);
	end

    [Sele1] = ...
      Res005(Sele1,eles,xnod,icone,iface,Te,Ce,veloc,prop,nen,ndf,coe);
	ip = ip0+(inod-1)*ndf^2+idf;
	iph = [iph , ip];
    Kele(eles,ip) = -(Sele1(eles,:)-Rele(eles,:))/ephi(idf);

   end 
  end

  end % if ijob

% Calculo del residuo a partir del vector de estado viejo
% ------------------------------------------------------------------
  eles = (1:numel)';
  T    = phiold(:,1);
  C    = phiold(:,2);
  coe  = [-1;0;0;-1;0];

  ceros = zeros(length(eles),nen);
  Te = ceros;
  Ce = ceros;  
  for inod=1:nen,
   nodes = icone(eles,inod);
   Te(:,inod) = T(nodes);
   Ce(:,inod) = C(nodes);
  end

  [Rele] = ...
    Res005(Rele,eles,xnod,icone,iface,Te,Ce,veloc,prop,nen,ndf,coe);

if size(teco)~=[], bc_teco; end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

  [row,col,coe] = assemble(Kele,icone,2);
  Kg = sparse(row,col,coe);

  for iele=1:numel,
  for inod=1:nen0,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

%------------------------------------------------------------------
%       Aplicamos condiciones de periodicidad
%
%    arreglo PERI    i (col 1 ) ----> j (col 2)
%------------------------------------------------------------------

 if isempty(peri)==0,
   n1 = peri(:,1); 
   n2 = peri(:,2);
   Kg((n2-1)*ndf+1,:) = Kg((n2-1)*ndf+1,:)+Kg((n1-1)*ndf+1,:);
   Kg((n2-1)*ndf+2,:) = Kg((n2-1)*ndf+2,:)+Kg((n1-1)*ndf+2,:);
   Kg((n1-1)*ndf+1,:) = 0*Kg((n1-1)*ndf+1,:);
   Kg((n1-1)*ndf+2,:) = 0*Kg((n1-1)*ndf+2,:);

   Rg((n2-1)*ndf+1,1) = Rg((n2-1)*ndf+1,1)+Rg((n1-1)*ndf+1,1);
   Rg((n2-1)*ndf+2,1) = Rg((n2-1)*ndf+2,1)+Rg((n1-1)*ndf+2,1);
   Rg((n1-1)*ndf+1,1) = 0*Rg((n1-1)*ndf+1,1);
   Rg((n1-1)*ndf+2,1) = 0*Rg((n1-1)*ndf+2,1);

   Kg((n1-1)*ndf+1,(n1-1)*ndf+1)= eye(size(n1),size(n1));
   Kg((n1-1)*ndf+2,(n1-1)*ndf+2)= eye(size(n1),size(n1));
   Kg((n1-1)*ndf+1,(n2-1)*ndf+1)=-eye(size(n1),size(n2));
   Kg((n1-1)*ndf+2,(n2-1)*ndf+2)=-eye(size(n1),size(n2));
 end
