function [Kg,Rg] = ...
   fem007(xnod,icone,iface,fixa,forc,teco,bccv,bcpv,peri,prop,veloc,phi,phiold,ijob)
%------------------------------------------------------------------
%
%     [Kg,Rg] = fem007(xnod,icone,fixa,forc,prop,veloc,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion acoplada balance de energia y masa del soluto
%        #.- incluye cambio de fase
%        #.- no estacionario con conveccion
%        #.- NDF == 2
%
%  CALCULO DE MATRIZ TANGENTE A PARTIR DE PERTURBAR EL RESIDUO
%  A NIVEL DE ELEMENTO
%
%       ijob = 0 matriz y residuo
%       ijob = 1 solo residuo
%------------------------------------------------------------------

%global Test Hest rhoaest

%disp('FEM007');  keyboard;pause

  [numel,nen]   = size(icone);
  nen0          = nen;
  ndf           = 3;
  tol           = prop(12);

  T       = phi(:,1);
  H       = phi(:,2);
  rhoa    = phi(:,3);
  Told    = phiold(:,1);
  Hold    = phiold(:,2);
  rhoaold = phiold(:,3);

  Rg   =forc;
  Kele =zeros(numel,nen0^2*ndf^2);
  Rele =zeros(numel,nen0*ndf);
     
% Calculo del residuo a partir del vector de estado	actual
% ------------------------------------------------------------------
  eles = (1:numel)';
%  T       = phi(:,1);
%  H       = phi(:,2);
%  rhoa    = phi(:,3);
  coe  = [1;1;1;1;1];

  ceros = zeros(length(eles),nen);
  Te       = ceros;
  He       = ceros;  
  rhoae    = ceros;
  Teold    = ceros;
  Heold    = ceros;  
  rhoaeold = ceros;

  for inod=1:nen,
   nodes = icone(eles,inod);
   Te(:,inod)      = T(nodes);
   He(:,inod)      = H(nodes);
   rhoae(:,inod)   = rhoa(nodes);
   Teold(:,inod)   = Told(nodes);
   Heold(:,inod)   = Hold(nodes);
   rhoaeold(:,inod) = rhoaold(nodes);
  end

  [Rele] = ...
    Res007(Rele,eles,xnod,icone,iface,Te,He,rhoae,Teold,Heold,rhoaeold,...
	       veloc,prop,nen,ndf,coe,teco,bccv,0);

  if ijob==0,

  ip0=[];
  for k=1:nen,
    ip0 = [ip0;(k-1)*nen*ndf^2 + (1:ndf:ndf^2)'];	
  end
  ip0 = ip0-1;

% Calculo del residuo a partir del vector de estado perturbado
% ------------------------------------------------------------------
  eles = (1:numel)';
% norma infinito de cada una de las variables
  Tn    = norm(T,inf);
  Hn    = norm(H,inf);
  rhoan = norm(rhoa,inf);
% perturbacion de cada variable
  %eepsilon = 1e-7;
  eepsilon = 1e-10;
  eT   = eepsilon*Tn;
  eH   = eepsilon*Hn;
  erhoa   = eepsilon*rhoan;

  ephi = [eT,eH,erhoa];

  for inod=1:nen,
   nodes = icone(eles,inod);
   Teold(:,inod)   = Told(nodes);
   Heold(:,inod)   = Hold(nodes);
   rhoaeold(:,inod) = rhoaold(nodes);
  end

    T    = phi(:,1);
    H    = phi(:,2);
    rhoa = phi(:,3);

  for inod=1:nen,
   for idf=1:ndf,
    Sele1= zeros(numel,nen0*ndf);

    coe  = [1;1;1;1;1];

    ceros = zeros(length(eles),nen);
    Te = ceros;
    He = ceros;  
    rhoae = ceros;

    for jnod=1:nen,
     nodes = icone(eles,jnod);
     Te(:,jnod) = T(nodes);
     He(:,jnod) = H(nodes);
     rhoae(:,jnod) = rhoa(nodes);
    end

    if idf==1,
	  Te(:,inod) = Te(:,inod)+ephi(idf);
	elseif idf==2,
	  He(:,inod) = He(:,inod)+ephi(idf);
	else
	  rhoae(:,inod) = rhoae(:,inod)+ephi(idf);
	end

    [Sele1] = ...
    Res007(Sele1,eles,xnod,icone,iface,Te,He,rhoae,Teold,Heold,rhoaeold,... 
	         veloc,prop,nen,ndf,coe,teco,bccv,0);

    ip = ip0 + (inod-1)*ndf^2+idf;
    Kele(eles,ip) = -(Sele1(eles,:)-Rele(eles,:))/ephi(idf);

   end 
  end

  end % if ijob

%if ~isempty(teco), 
%  [Kele,Rele] = bc_teco(xnod,icone,T,teco,Kele,Rele,ijob);
%end

%------------------------------------------------------------------
%        Montaje
%------------------------------------------------------------------

  [row,col,coe] = assemble(Kele,icone,ndf);
  Kg = sparse(row,col,coe);


  for iele=1:numel,
  for inod=1:nen0,
   for idf=1:ndf,
    Rg((icone(iele,inod)-1)*ndf+idf,1) = ... 
         Rg((icone(iele,inod)-1)*ndf+idf,1) + Rele(iele,(inod-1)*ndf+idf);
   end
  end
  end

% condicion de contorno presion vapor impuesta
  pvatm = prop(20);
  [Rg,Kg] = bc_pvap(phi,Rg,Kg,bcpv,pvatm);  

%disp(' FEM007');keyboard;pause

% condiciones periodicas
  [Rg,Kg] = bc_peri(Rg,Kg,peri,ndf);  

