function [dNdx,dNdy,dNdz,N] = shape3d(X2S,psipg,etapg,zetapg,nen)
%-------------------------------------------------------------------
%          --- FEMCODE routine --- 
%
%   [dNdx,dNdy,dNdz,N] = shape(S2X,psipg,etapg,zetapg,nen);
%
%     dNdx = dN_j/dx , j=1,...,nen
%     dNdy = dN_j/dy , j=1,...,nen
%     dNdz = dN_j/dz , j=1,...,nen
%
%-------------------------------------------------------------------

if nen==8,
   
psi  = [-0.5,0.5,0.5,-0.5,-0.5,0.5,0.5,-0.5];
eta  = [-0.5,-0.5,0.5,0.5,-0.5,-0.5,0.5,0.5];
zeta = [-0.5,-0.5,-0.5,-0.5,0.5,0.5,0.5,0.5];

% hexahedro
   for i=1:8
         N(i) =  (0.5+psi(i)*psipg)*(0.5+eta(i)*etapg)*(0.5+zeta(i)*zetapg);
      dNd1(i) =    psi(i)*(0.5+zeta(i)*zetapg)* (0.5+eta(i)*etapg);
      dNd2(i) =    eta(i)*(0.5+zeta(i)*zetapg)* (0.5+psi(i)*psipg);
      dNd3(i) =   zeta(i)*(0.5+ eta(i)* etapg)* (0.5+psi(i)*psipg);      
   end
   
   for i=1:8,
      dNdx(:,i) = X2S(:,1) * dNd1(i) + X2S(:,4) * dNd2(i) + X2S(:,7) * dNd3(i);
      dNdy(:,i) = X2S(:,2) * dNd1(i) + X2S(:,5) * dNd2(i) + X2S(:,8) * dNd3(i);
      dNdz(:,i) = X2S(:,3) * dNd1(i) + X2S(:,6) * dNd2(i) + X2S(:,9) * dNd3(i);      
   end   
   
elseif nen==4,
   
% tetraedro
      N(1) = 1-psipg-etapg-zetapg;
      N(2) = psipg;  
      N(3) = etapg; 
      N(4) = zetapg; 
      dNd1(1) = -1.0;
      dNd1(2) =  1.0;
      dNd1(3) =  0.0;
      dNd1(4) =  0.0;
      
      dNd2(1) = -1.0;
      dNd2(2) =  0.0;
      dNd2(3) =  1.0;
      dNd2(4) =  0.0;
      
      dNd3(1) = -1.0;
      dNd3(2) =  0.0;
      dNd3(3) =  0.0;
      dNd3(4) =  1.0;
      
   for i=1:4,
      dNdx(:,i) = X2S(:,1) * dNd1(i) + X2S(:,4) * dNd2(i) + X2S(:,7) * dNd3(i);
      dNdy(:,i) = X2S(:,2) * dNd1(i) + X2S(:,5) * dNd2(i) + X2S(:,8) * dNd3(i);
      dNdz(:,i) = X2S(:,3) * dNd1(i) + X2S(:,6) * dNd2(i) + X2S(:,9) * dNd3(i);      
   end   

else
  error(' ** ERROR ** , Routine SHAPE , Bad type of element')
end