function [beta,gamma] = sucpg(vele,hele,difu,reac,source)
%
%  calculo del upwind usando (SU+C)PG
%
%   [beta,gamma] = sucpg(vele,hele,difu,reac,source)
%

% definicion de algunas constantes
limit = 1e-16;

[ne,ibas]=size(hele);

P20 = 0;
aa  = -2/9;
mm  = aa/2;

velmod = sqrt(vele(:,1).^2+vele(:,2).^2);

% Calculo del Peclet y el Nro de Reaccion
Pe = velmod.*hele/2 ./ difu;
Re =  reac.*hele.^2 ./ difu;
vmin = limit*ones(size(Pe));
vmax = 100*ones(size(Pe));

Pe = max([Pe';vmin'])';
Re = max([Re';vmin'])';
Pe = min([Pe';vmax'])';
Re = min([Re';vmax'])';

% Calculo de ambos lambdas
lamb1h = (Pe+sqrt(Pe.^2+Re));
lamb2h = (Pe-sqrt(Pe.^2+Re));

lamb1h = min([lamb1h';40*ones(1,ne)])';   
lamb1h = max([lamb1h';-40*ones(1,ne)])';
lamb2h = min([lamb2h';40*ones(1,ne)])';
lamb2h = max([lamb2h';-40*ones(1,ne)])';

% Calculo del sistema de 2 x 2 para hallar alfa y gamma

mask = (abs(Re)<1e-03*abs(Pe));

g11   = 4*Pe.*(1-cosh(lamb1h))-Re.*sinh(lamb1h);
f1    = -2*(cosh(lamb1h).*(Re/6-1)+Pe.*sinh(lamb1h)+(1+Re/3));
alfa  = f1 ./ g11;
gamma = 0*alfa;
alfa1 = alfa;
gamma1= gamma;

g11 = 4*Pe.*(1-cosh(lamb1h))-Re.*sinh(lamb1h);
f1 = -2*(cosh(lamb1h).*(Re/6-1)+Pe.*sinh(lamb1h)+(1+Re/3));
g21 = 4*Pe.*(1-cosh(lamb2h))-Re.*sinh(lamb2h);
g12 = 2 * ... 
   (cosh(lamb1h).*(Re*mm-P20)+2*aa*Pe.*sinh(lamb1h)+(P20-Re*(mm-aa)));
g22 = 2 * ... 
   (cosh(lamb2h).*(Re*mm-P20)+2*aa*Pe.*sinh(lamb2h)+(P20-Re*(mm-aa)));
f2 = -2*(cosh(lamb2h).*(Re/6-1)+Pe.*sinh(lamb2h)+(1+Re/3));

AA    = [g11, g12 , g21 , g22]; FF = [f1 , f2]; 
detA  = g11.*g22-g12.*g21;
AA1   = [g22, -g12, -g21, g11];
AA1   = AA1 ./ (detA*ones(1,4));
alfa  = AA1(:,1).*FF(:,1)+AA1(:,2).*FF(:,2);
gamma = AA1(:,3).*FF(:,1)+AA1(:,4).*FF(:,2);

alfa2 = alfa;
gamma2= gamma;

alfa  = mask .* alfa1 +(1-mask) .* alfa2;
gamma = mask .* gamma1+(1-mask) .* gamma2;

beta  = alfa ./ Pe ; 

%disp('sucpg'); keyboard;pause
