function [Fd] = Dflux(Up,dUp,prop);
%--------------------------------------------------------------------
%
%  [Fd] = Dflux(Up,dUp,prop);
%
%  Diffusive fluxes for Navier-Stokes equations
%
%--------------------------------------------------------------------

% ijob = 0  Up = [  p,u,v,T]
% ijob = 1  Up = [rho,u,v,p]
ijob = 1;

gamma    = prop(1);
visco    = prop(20);
kappa    = prop(3);
Rgas     = prop(11);

if ijob==0,
   %       Up = [p,u,v,T]
   grad_T = dUp(:,4,1:2);
elseif ijob==1,
   grad_T(:,1)  = (dUp(:,4,1)-Up(:,4)./Up(:,1).*dUp(:,1,1))./Up(:,1)/Rgas;
   grad_T(:,2)  = (dUp(:,4,2)-Up(:,4)./Up(:,1).*dUp(:,1,2))./Up(:,1)/Rgas;   
else
   error(' BAD IJOB @ DFLUX')
end
  
[numnp,ndf] = size(Up);    

Fd  = zeros(numnp,ndf,ndf-2);

grad_u = reshape(dUp(:,2,1:2),numnp,2);
grad_v = reshape(dUp(:,3,1:2),numnp,2);

defo(:,1,1) = grad_u(:,1)+grad_u(:,1);
defo(:,1,2) = grad_u(:,2)+grad_v(:,1);
defo(:,2,1) = grad_v(:,1)+grad_u(:,2);
defo(:,2,2) = grad_v(:,2)+grad_v(:,2);
defo = defo/2;
divu = grad_u(:,1)+grad_v(:,2);
tau = 2*visco*defo;
tau(:,1,1) = tau(:,1,1)-2/3*visco*divu;
tau(:,2,2) = tau(:,2,2)-2/3*visco*divu;

Fd(:,2:3,1:2) = tau;
Fd(:,4,1) = tau(:,1,1).*Up(:,2)+tau(:,1,2).*Up(:,3)+kappa*grad_T(:,1);
Fd(:,4,2) = tau(:,2,1).*Up(:,2)+tau(:,2,2).*Up(:,3)+kappa*grad_T(:,2);
