function [H,Cp] = enthalpy_function(Up,prop)
%
%      [H,Cp] = enthalpy_function(Up,prop)
%
%       Computes enthalpy function to handle primitive variables
%       in 2D 
%

% ijob = 0  Up = [  p,u,v,T]
% ijob = 1  Up = [rho,u,v,p]

ijob = 1;

if ijob==0,
%       Up = [p,u,v,T]
gamma       = prop(1);
Rgas        = prop(11);
[numnp,ndf] = size(Up);
p           = Up(:,1);
T           = Up(:,4);
u           = Up(:,2);
v           = Up(:,3);
rho         = p/Rgas ./ T;
Cv          = Rgas/(gamma-1);
ene         = Cv*T+0.5*(u.^2+v.^2);
H           = [rho , rho.*Up(:,2), rho.*Up(:,3) , rho.*ene];
cero        = zeros(numnp,1);

Cp(:,1:4:16) = [rho ./p,  cero, cero , -rho ./ T ];
Cp(:,2:4:16) = [rho ./p .*u, rho, cero, -rho ./T .*u ];
Cp(:,3:4:16)= [rho ./p .*v, cero, rho, -rho ./T .*v ];
Cp(:,4:4:16) = [ ...
  rho ./p.*(Cv*T+0.5*(u.^2+v.^2)), rho.*u, rho.*v,-rho./T.*(Cv*T+0.5*(u.^2+v.^2))+rho*Cv];

Cp = reshape(Cp,numnp,ndf,ndf);
elseif ijob==1
   
gamma       = prop(1);
[numnp,ndf] = size(Up);
H           = cv2pv(Up,gamma,-1);  
[Q,Q1]      = cv2pv_Jaco(Up,gamma);
Cp          = reshape(Q,numnp,ndf,ndf);

else
   error(' BAD IJOB @ enthalpy function ')
end

