function [Kg,Rg] = elem_Euler_2D(xnod,icone,fixa,forc,teco,lin_con,prop,...
                       ifaceb,Up,Upold,ijob)                       
%------------------------------------------------------------------
%
%
%       Caracteristicas del elemento:
%
%        #.- ecuaciones de Euler 
%        #.- elemento cuadrangular y triangular
%        #.- 2D, plano y axisimetrico
%        #.- no estacionario y explicito (masa diagonal)
%
%        ijob = 1 ---> matriz masa diagonal y residuo
%        ijob = 0 ---> solo residuo
%
% version TET (Tayfun Tezduyar)
% esta version tiene las siguientes caracteristicas
%
% [1] no tiene la opcion de taucte
% [2] tiene el calculo del termino de contorno convectivo incluido
% [3] intento de tratar las condiciones periodicas y slip a nivel
%     del elemento
%------------------------------------------------------------------

global row col


%%%%%%%%%%%%%% PROPIEDADES DEL MATERIAL 
  gamma    = prop(1) ;
  Cou      = prop(2) ;
  kappa    = prop(3) ;
  npg      = prop(4) ;
  shocap   = prop(7) ;
  axisim   = prop(8) ;
  tol      = prop(12);
  dt       = prop(13);
  weak_form= prop(5) ;
  tol_upw  = prop(10);
  Rgas     = prop(11);
  if prop(6)==0, type_upw='TET'; elseif prop(6)==1 type_upw='HUG'; else type_upw='NON';end
  if prop(9)==0, matrix_type='diag'; else matrix_type='full'; end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% conversion de primitivas y conservativas para hacer el calculo de Rg,Mg 
%  U = pv2cv(Up,gamma);
%  Uold = pv2cv(Upold,gamma);
  
  U    = Up;
  Uold = Upold;

  [numel,nen]   = size(icone);
  [numnp,ndf]   = size(U);
  if ndf~=4, error(' NDF debe ser 4 (Euler 2D) ');end
  ndm = 2;

 if matrix_type=='diag',
     if ijob>=0, 
        error(' Diagonal matrix needs ijob < 0 - check type solver and matrix type');
     end
  end
% tipo de matriz del sistema a armar
%  matrix_type = 'full';  % arma el jacobiano lo mejor posible
%  matrix_type = 'diag';  % solo la matriz de masa y lumped 

  if ndm==1,
     pgauss1d;
  elseif ndm==2, 
     pgauss2d; 
  elseif ndm==3,
     pgauss3d; 
  end

Rele =zeros(numel,nen,ndf);
if matrix_type=='diag',
   Mele = zeros(numel,nen,ndf);
else
   Mele = zeros(numel,nen,ndf,nen,ndf);
end
  Rg   =0*forc;
  eles     = (1:numel)';
  ceros    = zeros(length(eles),nen);
      
  if length(eles)>0,    
    elem_euler_2d_ele;
  end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
  [Rg,Kg] = gather(Rele,Mele,icone,nen,ndf);
  
%  disp(' VERSION 2 ')
%  disp([' || Rg ||=' num2str(norm(Rg))])
%  disp([' || Kg ||=' num2str(normest(Kg))])
  
  
  if 0,
if matrix_type=='full',
   
%  t0=cputime;   
% aplicamos transformacion a la matriz para hacer el update en primitivas
[Q,Q1] = cv2pv_Jaco(Up,gamma);   
indx = (1:numnp)';
row2 = []; col2=[]; coe2=[];
for idf=1:ndf,
   for jdf=1:ndf,
      row2 = [row2;(indx-1)*ndf+idf];
      col2 = [col2;(indx-1)*ndf+jdf];
      coe2 = [coe2;Q(:,(jdf-1)*ndf+idf)];         
   end
end
Qg = sparse(row2,col2,coe2);

Kg = Kg*Qg;
end

  end

% Linear restrictions
%------------------------------------------------------------------
if matrix_type=='full',
   if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end
end


