%   
%   dominio cuadrado con una perturbacion en un flujo uniforme
%  
%  using new set of primitive variables 
%       U = [ p , u , v T ]
%

% ====================================================================
% Reference infinity state of variables
rho_infty  = 1.15;
p_infty    = 1e5;
Mach_infty = 0.1;
gamma      = 1.4;
Rgas       = 287;

cc_infty   = sqrt(gamma*p_infty/rho_infty);
u_infty    = Mach_infty*cc_infty;
T_infty    = p_infty/rho_infty/Rgas;


% ====================================================================

xnod     =[];
icone    =[];
Nx       =  15;   Lx =  1.0;
Ny       =  15;   Ly =  1.0;
psi      = (0:Nx-1)'/(Nx-1);
x        = psi*Lx;
eta      = (0:Ny-1)'/(Ny-1)*1; 
y        = eta*Ly;

%-------- coordenadas (XNOD)
for k=1:Ny,
  xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
ndf = 4;
nodes_bot  = (1:Nx);
nodes_top  = Nx*(Ny-1)+(1:Nx);
nodes_lef  = 1:Nx:Nx*Ny;
nodes_rig  = Nx:Nx:Nx*Ny;

fixa= zeros(numnp,ndf);
fixa(nodes_lef,[1,2,3,4])=ones(length(nodes_lef),4);
fixa(nodes_rig,[1])=ones(length(nodes_rig),1);
fixa(nodes_top,[3])=ones(length(nodes_top),1);
fixa(nodes_bot,[3])=ones(length(nodes_bot),1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
forc(nodes_lef,1:4)= ones(length(nodes_lef),1)*[p_infty,u_infty,0,T_infty];
forc(nodes_rig,1)= ones(length(nodes_rig),1)*[p_infty];

%-------- Fuentes de volumen (por elemento)
 xc=0;yc=0;
 factor = 1/nen;
 for kk=1:nen,
  xc = xc+factor*xnod(icone(:,kk),1);
  yc = yc+factor*xnod(icone(:,kk),2);
 end
 mask=(yc>0.1)&(yc<0.3);
 source = 0*mask;

%--------- Termino reactivo -----------------------------
reac = zeros(numel,1);
reac = [];

%--------- Campo de Velocidad -----------------------------
veloc =  zeros(numnp,2);
veloc = [];

%-------- Propiedades del material
% gamma   = 1.4    ; ver arriba
 Cou     = 1    ;
 % Rgas     =     287; ver arriba
 visco    =      1e-5; % laminar
 visco    =      1e+2; % algo bastante turbulento
 Pr      = 0.72;
 Cv      = Rgas/(gamma-1);
 Cp      = gamma*Cv;
 kappa   = Cp*visco/Pr;
 
 npg     =  4     ;

 prop(1) =gamma   ;
 prop(2) =Cou    ;
 prop(3) =kappa ;
 prop(4) =npg   ;

 shocap   =      0 ;
 axisim   =      0 ;
 type_upw =      1 ;   
 weak_form=      1 ;
 
 prop(5)  = weak_form;
 prop(6)  = type_upw; 
 prop(7)  = shocap ;
 prop(8)  = axisim ;
 prop(11) = Rgas ;
 prop(20) = visco;
 
 matrix_type = 'full';  % arma el jacobiano lo mejor posible
% matrix_type = 'diag';  % solo la matriz de masa y lumped 
if exist('matrix_type'),
   if matrix_type=='diag', prop(9) = 0; else prop(9) = 1; end 
end
 

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
%teco    = [eles face val1/rho/Cp val2 ];
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
%peri = [];
%slip = [];
lin_con = [];

%-------- Solucion inicial (PHI)
rhoini = rho_infty;
pini   = p_infty;
Tini   = pini/rhoini/Rgas;
uini   = u_infty;
U      =  ones(numnp,1)*[pini,uini,0,Tini];
% pressure perturbation
DP     = 0.01*pini;
sigma  = 0.05;
xc     = (max(xnod)+min(xnod))/2;
r      = sqrt(sum((xnod-ones(numnp,1)*xc)'.^2))';
U(:,1) = U(:,1)+DP*exp(-r.^2/sigma);

% condicion inicial en primitivas
%-----------------------------------------------
bini   = U;

for k=1:ndf,
 nofree        = find(fixa(:,k)~=0);
 bini(nofree,k)   = forc(nofree,k);
end

