function [Kg,Rg] = advdif_1d(xnod,icone,fixa,forc,teco,lin_con,prop,...
                          veloc,reac,source,phi,phiold,ijob)
%------------------------------------------------------------------
%
%   [Kg,Rg] = advdif_1d(xnod,icone,fixa,forc,teco,lin_con,prop,...
%                               veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion adveccion difusion reaccion no estacionaria c/fuente
%        #.- elemento unidimensional
%
%        ijob = 0 ---> matriz tangente y residuo
%        ijob = 1 ---> solo residuo
%        ijob = -1 --> matriz de masa y residuo (para explicitos)
%
%------------------------------------------------------------------

  [numel,nen]   = size(icone);
  [numnp,ndf]   = size(phi);
  [numnp,ndm]   = size(xnod);

  kappa    = prop(1) ;
  npg      = prop(2);
  dt       = prop(3);
  upwind   = prop(4);

  lumped   = prop(11);
  tol      = prop(12);
  
  if ndm==1,
     pgauss1d;
  elseif ndm==2, 
     pgauss2d; 
  elseif ndm==3,
     pgauss3d; 
  end
  
  T        = phi(:,1);
  Told     = phiold(:,1);
    
  Rg       = forc;
  Kele     = zeros(numel,nen,ndf,nen,ndf);
  Rele     = zeros(numel,nen,ndf);

  advdif_1d_ele;

% Mixed boundary condition
%------------------------------------------------------------------
if isempty(teco)==0,bc_teco; end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
[Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);

% Linear restrictions
%------------------------------------------------------------------
if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end
