%addpath 'c:\proyecto\edival\matlab\my_tools' 
%addpath 'c:\proyecto\edival\matlab\femcode'
%
%   Ejemplo Generacion de malla para una valvula
%  

%  Dividimos la valvula en partes:
%  [1] Cabeza - guia
%  [2] Cuello
%  [3] Cabeza - asiento

% Datos geometricos en mm
D_head       =  45;   % diametro de la cabeza
D_stem       =   9;   % diametro del vastago
L_valve      =  126;  % longitud total de la valvula
R_neck       =  14.6;   % Radio del cuello
D_seat_in    =  38.8; % diametro interior del asiento
D_seat_out   =  40.5; % diametro exterior del asiento
H_seat_in    =   0.5; % altura asiento parte interior
H_seat_out   =   1.2; % altura asiento parte exterior
H_head       =   2.8; % altura cabeza de valvula
L_guide      =  49;   % longitud de la guia
L_guide_up   =  10;   % longitud por sobre la guia
L_guide_down =  10;   % longitud por debajo de la guia

width_seat   =  4.0;  % ancho del asiento
height_seat  =  7.2;  % altura del asiento
gap_seat     =  0.2;  % gap entre cabeza de valvula y asiento

Npoints      = 12;    % numero de puntos que definen la geometria
h_default    = L_valve/90; % longitud de los elementos por default

Hs = h_default*ones(Npoints,1);
Hs(12) = 0.4;
Hs(6:9)= 0.4;
Hs(5)  = 0.4;
Hs(7)  = 0.25;
Hs(8)  = 0.25;
% datos del movimiento de la valvula en el tiempo
theta_VO    = 0*pi/180;
theta_VC    = 220*pi/180;    %% en estos tengo 20 grados de diferencia con el modelo samcef (abre a 700 grados y cierra a 200)
Lvmax       = 10;
x_guide_min = 15;
x_guide_max = 25;
rpm         = 1000;

% datos de transferencia de calor
h_guide = 100;
T_guide = 130;
h_fluid = 100;     % convection film coefficient between valve and gas at stem and guide region
T_fluid = 65;     
h_seat  = 240;     % convection film coefficient between valve and gas at seat region
h_seat_out = 240;  % convection film coefficient between seat and gas at seat region
C_seat  = 1000;    % thermal contact between seat face and valve face at seat
h_cylinder = 440;
T_cylinder = 1000;

T_ini   = 200;

transient = 0;

bini_file = 'valve_0_steady_Tini';

% generamos el CAD
% puntos
P1 = [ D_stem/2 , 0];
P2 = P1 + [0 , L_guide_up];
P3 = P2 + [0 , L_guide];
P4 = P3 + [0 , L_guide_down];
L45 = L_valve-(L_guide_down+L_guide_up+L_guide+R_neck+H_seat_in+H_seat_out+H_head);
P5 = P4 + [0 , L45];
P6 = P5 + [R_neck , R_neck];
L67 = D_seat_in/2-(D_stem/2+R_neck);
P7 = P6 + [L67 , 0];
P8 = P7 + [(D_seat_out-D_seat_in)/2 , H_seat_in];
P9 = P8 + [(D_head-D_seat_out)/2 , H_seat_out];
P10 = P9 + [0 , H_head];
P11 = P10 + [-D_head/2 , 0];
P12 = P11 + [0 , -L_valve];

X_P = [];
for k=1:Npoints,
   eval(['X_P=[X_P ; P' num2str(k) '];'])
end

% obtenemos las longitudes de cada linea
for k=1:Npoints-1,
   H_lines(k) = norm(X_P(k+1,:)-X_P(k,:));
end
k=Npoints;
H_lines(k)=norm(X_P(k,:)-X_P(1,:));
H_lines=H_lines';

% Hs(8) y Hs(9) en funcion de Hs(12)
nnod_12 = round(H_lines(12)/Hs(12));
%nnod_8=round(H_lines(8)/(H_lines(8)+H_lines(9))*nnod_12);
%nnod_9=nnod_12-nnod_8+1;
nnod_9=nnod_12;
%Hs(8)=H_lines(8)/(nnod_8);
Hs(9)=H_lines(9)/(nnod_9);

% CAD - Lines
L_stem = L_guide_down+L_guide_up+L_guide+L45;

ip1=0;
ip2=1;
for k=1:Npoints-1,
   hele = Hs(k);
   ip1  = ip1+1; ip2=ip2+1;
   ref  = [1 1 1];
   [cc] = set_nodes_in_line(X_P([ip1,ip2],:),hele,ref);
   eval(['c' num2str(k) '=cc;']);
end

k=12;
hele = Hs(k);
ip1  = ip1+1; ip2=1;
ref  = [1 1 1];
[cc] = set_nodes_in_line(X_P([ip1,ip2],:),hele,ref);
eval(['c' num2str(k) '=cc;']);

k=5;
hele = Hs(k);
Lon  = R_neck*pi/2;
N    = round(Lon/hele);
angle = pi/2;
psi = (0:N-1)'/(N-1); theta= pi-psi*angle; 
xO = X_P(5,:)+[R_neck 0];
x=xO(1)+R_neck*cos(theta); y=xO(2)+R_neck*sin(theta);
cc = [x,y];
eval(['c' num2str(k) '=cc;']);

tol = 1e-10;
mask = (theta>3*pi/4); 
x0 = R_neck+D_stem/2;
y0 = L_valve-L_stem;
N2 = round(N/(1+y0/x0));
N1 = N-N2;
psi = (0:N1-1)'/(N1-1);
eta = (1:N2)'/N2;
x = [zeros(N1,1);eta*x0];
y = [psi*(L_valve-L_stem)+L_stem;ones(N2,1)*L_valve];
c5p = [x,y];

% Mesh generation of blocks
% block # 1 VASTAGO
C1 = [c1;c2;c3;c4];
C2(:,1) = C1(:,1)-D_stem/2;
C2(:,2) = C1(:,2);
C3 = [c12];
C4(:,1) = C3(:,1);
C4(:,2) = C3(:,2)+L_stem;
[xnod1,icone1] = gen_block_mesh(C2,C1,C3,C4);
%figure(1);clf;pltmsh(-xnod1,icone1);axis(-L_valve*[1 0 1 0 ])
clear C1 C2 C3 C4

% block # 2 CUELLO
C1 = c5p;
C2 = c5;
C3 = c12;
C3(:,2) = C3(:,2)+L_stem;
N = length(C3(:,1));
psi = (0:N-1)'/(N-1);
C4(:,2) = L_valve-psi*(H_seat_in+H_seat_out+H_head);
C4(:,1) = ones(N,1)*(D_stem/2+R_neck);
[xnod2,icone2] = gen_block_mesh(C1,C2,C3,C4);
%figure(1);pltmsh(-xnod2,icone2);axis(-L_valve*[1 0 1 0])
clear C1 C2 C3 %C4

% block # 3 CABEZA
C2 = [c6;c7];
C2 = [c6;c7;c8];
N = length(C2(:,1));
psi = (0:N-1)'/(N-1);
psi = (C2(:,1)-C2(1,1))/(C2(N,1)-C2(1,1));
C1(:,1) = D_stem/2+R_neck+psi*(D_head/2-(D_stem/2+R_neck));
C1(:,2) = ones(N,1)*L_valve;
C3=C4;
%C4 = [flipud(c9(2:nnod_9,:));flipud(c8)];
C4 = [flipud(c9)];
[xnod3,icone3] = gen_block_mesh(C1,C2,C3,C4);
%figure(1);pltmsh(-xnod3,icone3);axis(-L_valve*[1 0 1 0])
clear C1 C2 C3 C4
%figure(1);hold on;plot(-X_P(:,1),-X_P(:,2),'o');axis(-L_valve*[1 0 1 0])

x = [xnod1;xnod2;xnod3];
numnp_1 = length(xnod1(:,1));
numnp_2 = length(xnod2(:,1));
ico = [icone1;icone2+numnp_1;icone3+numnp_1+numnp_2];

[xnod,icone]=sew_mesh(x,ico);
[icone] = check_mesh_orientation(xnod,icone);

% block # 4 ASIENTO
c7p = [];
C1  = [c7;c8]; 
N   = length(C1(:,1));
Lon = norm(C1(N,:)-C1(1,:));
vec = (C1-ones(N,1)*C1(1,:));
psi = sqrt(sum(vec'.^2))'/Lon;
%psi = (0:N-1)'/(N-1);
C1  = C1 + gap_seat*[psi , 0-(1-psi)] ;
%C1  = C1 - gap_seat * [zeros(N,1) ones(N,1)] ;
c21 = C1;
N1  = length(c7(:,1));
psi = (0:N1-1)'/(N1-1);
x   = c7(1,1)+psi*width_seat;
y   = c7(1,2)-(height_seat-(H_seat_in+H_seat_out))*ones(size(x));
c7p = [x,y];
N2  = length(c8(:,1));
psi = (0:N2-1)'/(N2-1);
y   = y(N1)+psi*height_seat;
x   = x(N1)*ones(size(y));
c7p = [c7p;[x,y]];
C2  = c7p;
N   = length(C2(:,1));
psi = (0:N-1)'/(N-1);
C3  = ones(N,1)*C1(1,:)+psi*(C2(1,:)-C1(1,:));
C4  = ones(N,1)*C1(N,:)+psi*(C2(N,:)-C1(N,:));
c22 = C3;
c23 = C4;
[xnod4,icone4] = gen_block_mesh(C1,C2,C3,C4);
clear C1 C2 C3 C4

[xnod4,icone4]=sew_mesh(xnod4,icone4,(Hs(7)+Hs(8))/100);
[icone4] = check_mesh_orientation(xnod4,icone4);

[numnp,ndm]=size(xnod);
xnod = [xnod;xnod4];
icone = [icone;icone4+numnp];
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);

figure(1);clf;pltmsh(-xnod,icone);axis(-1.2*L_valve*[1 0 1 0]);

%>==============================================================================>%

% generamos la geometria y la malla de la valvula
%gen_valve;

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri
   quad_to_tri;
end

% generamos la malla del contorno
[ifaceb,nodesb] = pfbou2d(xnod,icone,1);

figure(2);clf;hold on

% identificamos diferentes partes del contorno para imponer BC
tol=1e-6;
[nodes_guide] = get_nodes_in_line(xnod,[c1;c2;c3],tol);
nn = get_eles_by_nodes(nodes_guide,nodesb,1);
ifaceb_guide = ifaceb(nn,:);
icone_guide = nodesb(nn,:);
nodes_guide = rmdblist(m2v(icone_guide));
nn = nodes_guide;
plot(-xnod(nn,1),-xnod(nn,2),'y.')

[nodes_fluid] = get_nodes_in_line(xnod,[c4;c5;c6],tol);
nn = get_eles_by_nodes(nodes_fluid,nodesb,1);
ifaceb_fluid = ifaceb(nn,:);
icone_fluid = nodesb(nn,:);
nodes_fluid = rmdblist(m2v(icone_fluid));
nn = nodes_fluid;
plot(-xnod(nn,1),-xnod(nn,2),'g.')

[nodes_seat] = get_nodes_in_line(xnod,[c7;c8],tol);
nn = get_eles_by_nodes(nodes_seat,nodesb,1);
ifaceb_seat = ifaceb(nn,:);
icone_seat = nodesb(nn,:);
nodes_seat = rmdblist(m2v(icone_seat));
nn = nodes_seat;
plot(-xnod(nn,1),-xnod(nn,2),'c.')

[nodes_top_stem] = get_nodes_in_line(xnod,[c12],tol);
nn = get_eles_by_nodes(nodes_top_stem,nodesb,1);
ifaceb_top_stem = ifaceb(nn,:);
icone_top_stem = nodesb(nn,:);
nodes_top_stem = rmdblist(m2v(icone_top_stem));
nn = nodes_top_stem;
plot(-xnod(nn,1),-xnod(nn,2),'b.')

if 0,
[nodes_cyl] = get_nodes_in_line(xnod,[c10],tol);
nn = get_eles_by_nodes(nodes_cyl,nodesb,1);
ifaceb_cyl = ifaceb(nn,:);
icone_cyl = nodesb(nn,:);
nodes_cyl = rmdblist(m2v(icone_cyl));
nn = nodes_cyl;
plot(-xnod(nn,1),-xnod(nn,2),'m.')

[nodes_axis] = get_nodes_in_line(xnod,[c11],tol);
nn = get_eles_by_nodes(nodes_axis,nodesb,1);
ifaceb_axis = ifaceb(nn,:);
icone_axis = nodesb(nn,:);
nodes_axis = rmdblist(m2v(icone_axis));
nn = nodes_axis;
plot(-xnod(nn,1),-xnod(nn,2),'w.')
else
nodes_cyl = get_nodes_in_line(xnod,[c9],tol);
nodes_cyl = [nodes_cyl;find(abs(xnod(:,2)-L_valve)<tol)];
nn = get_eles_by_nodes(nodes_cyl,nodesb,1);
ifaceb_cyl = ifaceb(nn,:);
icone_cyl = nodesb(nn,:);
nn = nodes_cyl;
plot(-xnod(nn,1),-xnod(nn,2),'m.')

nodes_axis = find(abs(xnod(:,1)-0)<tol);
nn = get_eles_by_nodes(nodes_axis,nodesb,1);
ifaceb_axis = ifaceb(nn,:);
icone_axis = nodesb(nn,:);
nn = nodes_axis;
plot(-xnod(nn,1),-xnod(nn,2),'w.')

end

[nodes_seat_out] = get_nodes_in_line(xnod,[c21],tol);
nn = get_eles_by_nodes(nodes_seat_out,nodesb,1);
ifaceb_seat_out = ifaceb(nn,:);
icone_seat_out = nodesb(nn,:);
nodes_seat_out = rmdblist(m2v(icone_seat_out));
nn = nodes_seat_out;
plot(-xnod(nn,1),-xnod(nn,2),'ko')

[nodes_seat_fluid] = get_nodes_in_line(xnod,c22,tol);
nn = get_eles_by_nodes(nodes_seat_fluid,nodesb,1);
ifaceb_seat_fluid = ifaceb(nn,:);
icone_seat_fluid = nodesb(nn,:);
nodes_seat_fluid = rmdblist(m2v(icone_seat_fluid));
nn = nodes_seat_fluid;
plot(-xnod(nn,1),-xnod(nn,2),'y*')

[nodes_seat_cylinder] = get_nodes_in_line(xnod,c23,tol);
nn = get_eles_by_nodes(nodes_seat_cylinder,nodesb,1);
ifaceb_seat_cylinder = ifaceb(nn,:);
icone_seat_cylinder = nodesb(nn,:);
nodes_seat_cylinder = rmdblist(m2v(icone_seat_cylinder));
nn = nodes_seat_cylinder;
plot(-xnod(nn,1),-xnod(nn,2),'yo')

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
ndf = 1;
fixa= zeros(numnp,ndf);
%fixa(nodes_cyl,1)=ones(length(nodes_cyl),1);
fixa(nodes_top_stem,1)=ones(length(nodes_top_stem),1);
[nodes_seat_wall] = get_nodes_in_line(xnod,[c7p],tol);
fixa(nodes_seat_wall,1)=ones(length(nodes_seat_wall),1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc = zeros(numnp,ndf);
forc(nodes_top_stem,1) = 100;
%forc(nodes_cyl,1)      = T_cylinder;
forc(nodes_seat_wall,1)= 150;

%-------- Fuentes de volumen (por elemento)
source =zeros(numel,1);

%--------- Termino reactivo -----------------------------
reac = zeros(numel,1);

%--------- Campo de Velocidad -----------------------------
veloc = zeros(numnp,2);

%-------- Propiedades del material
% global definition (default properties)
 id_material = 1;
 rho     = 7800 ;  % Kg/m3
 Cp      = 460  ;  % Joule/Kg/C
 kappa   = 32   ;  % Watt/m/C
 npg     =  2   ;
 prop(1) =rho   ;
 prop(2) =Cp    ;
 prop(3) =kappa ;
 prop(4) =npg   ;
 upwind  =    0 ;
 shocap  =    0 ;
 axisim  =    1 ;
 type_upw =   1 ;   
 lumped   =   0 ;
 prop(6)  = upwind ;
 prop(7)  = shocap ;
 prop(8)  = axisim ;
 prop(9)  = type_upw;
 prop(11) = lumped ;
 
 prop_elem(1:3,1) = prop(1:3)'; 
 nummat = 1;
 % for more materials
 if 0,
    id_material_seat = 2;
    id_material = id_material_seat;
    prop_elem(1,id_material) =2   ;
    prop_elem(2,id_material) =2    ;
    prop_elem(3,id_material) =1e-1 ;
    [numel4,nen4]=size(icone4);
    xx = reshape(xnod4(icone4,1),numel4,nen4);
    yy = reshape(xnod4(icone4,2),numel4,nen4);
    [eles_in_seat] = get_eles_by_xnod(xnod,icone,[xx,yy],tol);
    icone = [icone , -ones(numel,1)];
    icone(eles_in_seat,nen+1) = -id_material_seat;    
 end
 
%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Contacto termico
%ther_contact =  [icone_seat(:,[2,1]) icone_seat_out(:,[2,1]) ones(length(icone_seat(:,1)),1)*1000];
ther_contact = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];
if 0,  
   % ponemos las condiciones TECO en un archivo temporal
eles    = ifaceb_seat(:,1);
face    = ifaceb_seat(:,2);
val1    = 1*ones(size(eles));
val2    = 150*ones(size(eles));
teco    = [eles face val1/rho/Cp val2 ];
end

if ~transient,
   valve_0_teco_fixed;
end

%-------- Restricciones lineales
lin_con = [];


%-------- Solucion inicial (PHI)
if transient
   eval(['load ' bini_file ';'])
   bc_time_flag = 1;
else
bini = T_ini*ones(numnp,ndf);
for k=1:ndf,
 nofree        = find(fixa(:,k)~=0);
 bini(nofree,k)   = forc(nofree,k);
end
bc_time_flag = 0;
end

% archivos con coeficientes de conveccion pelicular en funcion del tiempo
bc_time_file_amplitude = 'valve_0_teco_time_seat';
% archivo con la ley de la alzada de la valvula en el tiempo
bc_time_file_lift_valve = 'valve_0_lift_valve';
% Cylinder temperature vs crank angle
bc_time_file_T_cyl = 'valve_0_T_cyl';

analysis = 1;  % [0] : only for some preliminar data test, [1] : full simulation

Lvmax       = Lvmax/1000;
x_guide_min = x_guide_min/1000;
x_guide_max = x_guide_max/1000;
xnod = xnod/1000;