function [Kg,Rg] = advdif_system(xnod,icone,fixa,forc,teco,lin_con,veloc,reac,source,prop,...
                       ifaceb,Up,Upold,ijob)                       
%------------------------------------------------------------------
%
%         Advective-diffusive-reactive system of equations
%
%  Each equation with its own parameters (veloc,reac,source)
%
%  size(veloc)  = [numnp,ndf,ndim]
%  size(reac)   = [numel,ndf]
%  size(source) = [numel,ndf]
%
%------------------------------------------------------------------

global row col

ijob = 'get';  % to be used by handle_prop_elem to get element properties
handle_prop_elem;
 
  U    = Up;
  Uold = Upold;

  [numel,nen]   = size(icone);
  [numnp,ndf]   = size(U);
  ndm = 2;

  if ndm==1,
     pgauss1d;
  elseif ndm==2, 
     pgauss2d; 
  elseif ndm==3,
     pgauss3d; 
  end

  Rele = zeros(numel,nen,ndf);
  Mele = zeros(numel,nen,ndf,nen,ndf);
  
  Rg    = 0*forc;
  eles  = (1:numel)';
  ceros = zeros(length(eles),nen);
      
  if length(eles)>0,    
    if 1,
        % Hauke upwind version + GLS
        advdif_system_ele;
    else
        % (SU+C)PG upwind version  
        advdif_system_ele_with_sucpg;
    end
  end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
  [Rg,Kg] = gather(Rele,Mele,icone,nen,ndf);
  
% Linear restrictions
%------------------------------------------------------------------
  if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end
