function [Fc,C] = Cflux(Up,grad_p,vel_l,prop)
%
%   [Fc,C] = Cflux(Up,grad_p,vel_l,prop);
%

%comp_interphase_terms = 0;
%drag_factor = 1;

ijob = 'get';
handle_prop_elem;

%rho_g       = prop(1);
%visco_g     = prop(2);
%gravity     = prop(3);
[N,ndf]     = size(Up);
alpha_g     = Up(:,1);
vel_g       = Up(:,2:3);

Fc          = zeros(N,ndf);
C           = zeros(N,ndf,ndf);

% reactive fluxes due to bouyancy forces
Fc(:,3)     = -rho_g*gravity*alpha_g;
C(:,3,1)    = -rho_g*gravity*ones(N,1);

% reactive fluxes due to pressure gradient
Fc(:,2)     = Fc(:,2)+alpha_g.*grad_p(:,1);
Fc(:,3)     = Fc(:,3)+alpha_g.*grad_p(:,2);
C(:,2,1)    = grad_p(:,1);
C(:,3,1)    = grad_p(:,2);

if comp_interphase_terms,   
   % reactive and source terms due to drag interaction forces
   tol      = 1e-06;
visco_l     = visco_g;
d_bubble    = 6/1000;
rho_l       = 1000;
v_rel       = vel_g - vel_l;
v_slip      = sqrt(sum(v_rel'.^2))';
Re          = rho_l*d_bubble*v_slip/visco_l;
mask        = Re>tol;
nn          = find(mask==1);

%Re          = max([Re';tol*ones(1,N)])';
f_Re        = 24.0 ./ Re(nn);
C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
C_drag      = C1_drag*Re(nn).*f_Re(nn);  
df_RedRe    = -24.0 ./Re(nn) ./Re(nn);

Fc(nn,2)     = Fc(nn,2)+C_drag.*alpha_g(nn).*(v_rel(nn,1));
Fc(nn,3)     = Fc(nn,3)+C_drag.*alpha_g(nn).*(v_rel(nn,2));

if 1,
C(nn,2,1)    = C(nn,2,1) + C_drag.*(v_rel(nn,1));
C(nn,3,1)    = C(nn,3,1) + C_drag.*(v_rel(nn,2));
end

if 1,
dRedU       = rho_l*d_bubble/visco_l;

tmp4_drag   = f_Re+Re(nn).*df_RedRe;
%tmp4_drag   = -dRedU * tmp4_drag ./ v_slip(nn);
%for idm=1:2,
%   for jdm=1:2,
%      Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
%      Phi(:,idm,jdm) = C1_drag*alpha_g(nn).*(Phi(:,idm,jdm) - Re(nn).*f_Re);
%   end
%end

tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);
for idm=1:2,
   for jdm=1:2,
      Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
  end
  Phi(:,idm,idm) = Phi(:,idm,idm)+C_drag.*alpha_g(nn);
end

C(nn,2,2)    = C(nn,2,2) + Phi(:,1,1);
C(nn,2,3)    = C(nn,2,3) + Phi(:,1,2);
C(nn,3,2)    = C(nn,3,2) + Phi(:,2,1);
C(nn,3,3)    = C(nn,3,3) + Phi(:,2,2);
end

end

%keyboard;pause

if 0,
   % DEBUG , add a div(vel) term 
   C(:,1,1)    = C(:,1,1)+rho_g*div_vel;   
end
