function [Fd,Kxx,Kxy,Kyx,Kyy] = Dflux(Up,dUp,prop,flag_predictor);
%--------------------------------------------------------------------
%
%         [Fd,Kxx,Kxy,Kyx,Kyy] = Dflux(Up,dUp,prop,flag_predictor);
%
%  Diffusive fluxes and jacobians for the gas phase in the two phase flow eqs 
%
%--------------------------------------------------------------------

if nargin < 4, flag_predictor=0; end

alpha_g_ctf = 0;
rho_g       = prop(1);
visco_g     = prop(2);
[N,ndf]     = size(Up);
alpha_g     = Up(:,1);
vel_g       = Up(:,2:3);

Fd     = zeros(N,ndf,2);
Kxx    = zeros(N,ndf,ndf);
Kxy    = zeros(N,ndf,ndf);
Kyx    = zeros(N,ndf,ndf);
Kyy    = zeros(N,ndf,ndf);
   
if 0,
% DEBUG and fixme
file = 'Dflux_nc';
line = '27';
lp_debug(file,line)
return
end

grad_u      = reshape(dUp(:,2,1:2),N,2);
grad_v      = reshape(dUp(:,3,1:2),N,2);

defo(:,1,1) = grad_u(:,1)+grad_u(:,1);
defo(:,1,2) = grad_u(:,2)+grad_v(:,1);
defo(:,2,1) = grad_v(:,1)+grad_u(:,2);
defo(:,2,2) = grad_v(:,2)+grad_v(:,2);
defo        = defo/2;

tau         = 2*defo;

alpha_g     = max([alpha_g_ctf*ones(1,N);alpha_g'])';
alpha_g     = min([(1-alpha_g_ctf)*ones(1,N);alpha_g'])';

if flag_predictor,   
   visco_mix   = visco_g*ones(N,1);
else
   visco_mix   = alpha_g*visco_g;   
   %visco_mix   = visco_g;      
end

for k=1:2,
    for m=1:2,
       Fd(:,k+1,m) = visco_mix.*tau(:,k,m);       
    end
end

vaux        = visco_mix;

delta       = eye(2);
uno_1       = [1;0];
uno_2       = [0;1];

tmp_xx = (delta(1,1)*eye(2)+uno_1*uno_1');
tmp_xy = (delta(1,2)*eye(2)+uno_2*uno_1');
tmp_yx = (delta(2,1)*eye(2)+uno_1*uno_2');
tmp_yy = (delta(2,2)*eye(2)+uno_2*uno_2');

for j=1:2
   for k=1:2,
      Kxx(:,j+1,k+1)  = vaux*tmp_xx(j,k);
      Kxy(:,j+1,k+1)  = vaux*tmp_xy(j,k);
      Kyx(:,j+1,k+1)  = vaux*tmp_yx(j,k);
      Kyy(:,j+1,k+1)  = vaux*tmp_yy(j,k);      
   end
end
