function [Fa,Ax,Ay] = Flux_function_nc(Up,prop,flag_predictor)
%
%            [Fa,Ax,Ay] = Flux_function_nc(Up,prop,flag_predictor)
%
%       Computes advective fluxes F_a
%                   2D    
%	 Dimension of Fa array = numnp x ndm*ndf 
%             

if nargin < 3, flag_predictor=0; end

ndm         = 2;
rho_g       = prop(1);
visco_g     = prop(2);
[N,ndf]     = size(Up);
alpha_g     = Up(:,1);
vel_g       = Up(:,2:3);

if 0,
% DEBUG and fixme
file = 'Flux_function_nc';
line = '26';
lp_debug(file,line)
end

Fa            = zeros(N,ndf,2);
A             = zeros(N,ndm,ndf,ndf);

if flag_predictor,
   
% el flujo no se usa en la fase predictora porque no se usa formulacion debil   

% advective fluxes

% Continuity equation (mass)
A(:,1:2,1,1)  = rho_g* vel_g;

% Momentum equations
A(:,1,2,2)    = rho_g* vel_g(:,1);
A(:,1,3,3)    = rho_g* vel_g(:,1);

A(:,2,2,2)    = rho_g* vel_g(:,2);
A(:,2,3,3)    = rho_g* vel_g(:,2);

else
   
% advective fluxes
Fa(:,1,1:2)   = rho_g*(alpha_g*[1,1]).*vel_g;
Fa(:,2,1)     = rho_g*alpha_g.*vel_g(:,1).*vel_g(:,1);
Fa(:,3,1)     = rho_g*alpha_g.*vel_g(:,1).*vel_g(:,2);
Fa(:,2,2)     = rho_g*alpha_g.*vel_g(:,2).*vel_g(:,1);
Fa(:,3,2)     = rho_g*alpha_g.*vel_g(:,2).*vel_g(:,2);

% Continuity equation (mass)
A(:,1:2,1,1)  = rho_g*vel_g;
%A(:,1,1,2)    = rho_g*alpha_g;
%A(:,2,1,3)    = rho_g*alpha_g;

% Momentum equations
A(:,1,2,1)    = rho_g*vel_g(:,1).*vel_g(:,1);
A(:,1,3,1)    = rho_g*vel_g(:,2).*vel_g(:,1);
A(:,2,2,1)    = rho_g*vel_g(:,1).*vel_g(:,2);
A(:,2,3,1)    = rho_g*vel_g(:,2).*vel_g(:,2);

A(:,1,2,2)    = rho_g*alpha_g.*(vel_g(:,1)+vel_g(:,1));
A(:,1,3,2)    = rho_g*alpha_g.* vel_g(:,2);
A(:,1,3,3)    = rho_g*alpha_g.* vel_g(:,1);

A(:,2,2,2)    = rho_g*alpha_g.* vel_g(:,2);
A(:,2,2,3)    = rho_g*alpha_g.* vel_g(:,1);
A(:,2,3,3)    = rho_g*alpha_g.*(vel_g(:,2)+vel_g(:,2));

end

Ax         = reshape(A(:,1,:,:),N,ndf,ndf);
Ay         = reshape(A(:,2,:,:),N,ndf,ndf);      
