% ====================================================================
%
%            gas pipe flow with alpha_g fixed at input
%
% ====================================================================

% Reference infinity state of variables
alpha_g_ref = 1;
vel_g_ref   = [0,1];

alpha_g_ini = 0.1;
vel_g_ini   = [0,0.01];

rho_g       = 1;
visco_g     = 1.0e-5;
gravity     = -9.8;
visco_l     = 0.6;

H = 0.1;
D = 0.01;
radius_nozzle = 0.1*D;
radius_nozzle = 10*D;

rho_l       = 1000;
p_ref       = 0;

ndf = 3;
tol = 1e-6;

shocap   =      1 ;
weak_form=      0 ; 
axisim   =      0 ;
comp_interphase_terms = 1;
drag_factor           = 0.3;
d_bubble              = 0.006;

Cou     =  1  ;
npg     =  4  ;

% ====================================================================

xnod     =[];
icone    =[];
Nx       =  10;    
Ny       =  51;    
psi      = (0:Nx-1)'/(Nx-1);
x        = psi*D;
eta      = (0:Ny-1)'/(Ny-1)*1; 
y        = eta.^3*H;

%-------- coordenadas (XNOD)
for k=1:Ny,
  xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
nodes_bot  = (1:Nx);
nodes_top  = Nx*(Ny-1)+(1:Nx);
nodes_lef  = 1:Nx:Nx*Ny;
nodes_rig  = Nx:Nx:Nx*Ny;
nodes_bot_in = find(xnod(nodes_bot,1)<radius_nozzle);
nodes_bot_wall = find(xnod(nodes_bot,1)>=radius_nozzle);

fixa= zeros(numnp,ndf);
fixa(nodes_bot_in,[1,2,3])= ones(length(nodes_bot_in),3);
fixa(nodes_bot_wall,[2,3])= ones(length(nodes_bot_wall),2);
fixa(nodes_lef,[2])  = ones(length(nodes_lef),1);
fixa(nodes_rig,[2])  = ones(length(nodes_rig),1);
fixa(nodes_top,[2])    = ones(length(nodes_top),1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
forc(nodes_bot_in,1:3)= ones(length(nodes_bot_in),1)*[alpha_g_ref,vel_g_ref];

%--------- External fields -----------------------------
% pressure and liquid velocities 
% external_fields = [pressure , vel_liq]
external_fields =  zeros(numnp,3);
external_fields(:,1) = p_ref+rho_l*abs(gravity)*(H-(xnod(:,2)));

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;
 
%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
%teco    = [eles face val1/rho/Cp val2 ];
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
%peri = [];
%slip = [];
lin_con = [];

% Up = [alpha_g, vel_g]
%-------- Solucion inicial (PHI)
uini   = [alpha_g_ini,vel_g_ini];
U      =  ones(numnp,1)*uini;   

% condicion inicial en primitivas
%-----------------------------------------------
bini   = U;

for k=1:ndf,
 nofree        = find(fixa(:,k)~=0);
 bini(nofree,k)   = forc(nofree,k);
end

