%
%  Incompressible Navier-Stokes equations using SUPG-PSPG + transport equations for void fraction
%  ASM bubbly version
%

type_upwind         = 1;
tau_centroide       = 1;
debug               = 0;
alpha_min           = 1e-3;

%------------------------------------------------------------------
ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

nphases = ndf-(ndm+1);

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);
%Uenorm = zeros(ne,nen);

if ndf_fields>0,
    Uf_e = zeros(ne,nen*ndf_fields);
    Uf_e_old = zeros(ne,nen*ndf_fields);
end

for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    
    Ue(:,jnod)  = U(nodes,1:ndf);		 
    U0e(:,jnod) = Uold(nodes,1:ndf);   
    re(:,inod)  = xnod(nodes,1);		        
    %   Uenorm(:,inod) = sqrt(sum(Uold(nodes,1:ndf)'.^2)');
    
    if ndf_fields>0,
        knod  = (inod-1)*ndf_fields+(1:ndf_fields);                   
        Uf_e(:,knod)  = extra_nodal_fields(nodes,1:ndf_fields);
        Uf_e_old(:,knod)  = extra_nodal_fields_old(nodes,1:ndf_fields); 
    end
end

if tau_centroide==1,
    
    % UPWIND EN EL CENTROIDE 
    %------------------------------------------------------------------
    if nen==4,
        [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0,0);
        [dshapex0,N0] = shape2d(X2S,0,0,nen); 
    elseif nen==3,
        [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),1/3,1/3);
        [dshapex0,N0] = shape2d(X2S,1/3,1/3,nen); 
    else
        error(' Bad type of element , nen ~=3, nen ~=4 ')
    end
    Upg     = zeros(ne,ndf);
    dUpg    = zeros(ne,ndf,ndm);
    vel_k   = zeros(ne,ndf_fields);
    grad_vel_k =  zeros(ne,ndf_fields,ndm);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N0(inod).*  U0e(:,jnod);
        dUpg(:,(1:ndf),1) = dUpg(:,(1:ndf),1) + (dshapex0(:,inod,1)*ones(1,ndf)).* U0e(:,jnod);                   
        dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2) + (dshapex0(:,inod,2)*ones(1,ndf)).* U0e(:,jnod);
        
        if ndf_fields>0,
            for iif=1:ndf_fields/2,
                ipt = (iif-1)*2+(1:2);
                value = Uf_e(:,(inod-1)*ndf_fields+ipt);
                vel_k(:,ipt)  =  vel_k(:,ipt) + N0(inod).* value;
                grad_vel_k(:,ipt,1)  =  grad_vel_k(:,ipt,1) + (dshapex0(:,inod,1)*ones(1,2)).*value;             
                grad_vel_k(:,ipt,2)  =  grad_vel_k(:,ipt,2) + (dshapex0(:,inod,2)*ones(1,2)).*value;             
            end      
        end   
    end          
    
    for iif=1:nphases
        alpha(:,iif)=Upg(:,ndm+1+iif);
    end
    
    velpg   = Upg(:,2:3);  % velocidad de la mezcla
    hele    = comp_hele(velpg,dshapex0,Jaco0,nen);
    
    % calculamos la viscosidad turbulenta de la fase continua
    vel_liq = vel_k(:,1:2);
%    grad_vel_liq = grad_vel_k(:,1:2,:);
    grad_vel_liq = dUpg(:,2:3,:);
    visco_t = get_turbulent_visco(grad_vel_liq,hele); 
    % calculamos la densidad y viscosidad de la mezcla 
    [rho_m,visco_m] = get_mixture_props(alpha,prop,visco_t,0);
    
    % estabilizacion del bloque NSI para la mezcla
    [tau_m,delta_m] = supg_nsi_tet(Upg,hele,rho_m,visco_m);   
    tau_m   = tau_factor*tau_m;
    delta_m = tau_factor*delta_m;
    % estabilizacion de las ecs de transporte para las fases dispersas 
    tau_k = tau_m;
    delta_k = delta_m;
    for iif=1:nphases
        ipt = (iif-1)*2+(1:2);       
        vel = vel_k(:,ipt+2);
        [rho_k,visco_k] = get_mixture_props(alpha,prop,visco_t,iif);       
%    keyboard;pause        
        [tau,delta] = supg_advdif(vel,hele,rho_k,visco_k);
        tau   = tau_factor*tau;
        delta = tau_factor*delta; 
        tau_k = [tau_k,tau];
        delta_k = [delta_k,delta];
    end
end

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
    
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    
    Upg    = zeros(ne,ndf);
    U0pg    = zeros(ne,ndf);
    dUpg    = zeros(ne,ndf,ndm);
    radpg    = zeros(ne,1);
    vel_k   = zeros(ne,ndf_fields);
    grad_vel_k =  zeros(ne,ndf_fields,ndm);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).* Ue(:,jnod);
        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
        dUpg(:,(1:ndf),1) = dUpg(:,(1:ndf),1) + (dshapex(:,inod,1)*ones(1,ndf)).* Ue(:,jnod);               
        dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2) + (dshapex(:,inod,2)*ones(1,ndf)).* Ue(:,jnod);          
        
        if ndf_fields>0,
            for iif=1:ndf_fields/2,
                ipt = (iif-1)*2+(1:2);
                value = Uf_e(:,(inod-1)*ndf_fields+ipt);
                vel_k(:,ipt)  =  vel_k(:,ipt) + N(inod).* value;
                grad_vel_k(:,ipt,1)  =  grad_vel_k(:,ipt,1) + (dshapex(:,inod,1)*ones(1,2)).*value;             
                grad_vel_k(:,ipt,2)  =  grad_vel_k(:,ipt,2) + (dshapex(:,inod,2)*ones(1,2)).*value;             
            end      
        end   
        
    end
    
    for iif=1:nphases
        alpha(:,iif)=Upg(:,ndm+1+iif);
        alpha_old(:,iif)=U0pg(:,ndm+1+iif);      
    end
    
    % calculamos la viscosidad turbulenta de la fase continua
    vel_liq = vel_k(:,1:2);
%    grad_vel_liq = grad_vel_k(:,1:2,:);
    grad_vel_liq = dUpg(:,2:3,:);    
    visco_t = get_turbulent_visco(grad_vel_liq,hele); 
    % calculamos la densidad y viscosidad de la mezcla 
    [rho_m,visco_m] = get_mixture_props(alpha,prop,visco_t,0);
    [rho_m_old,bas] = get_mixture_props(alpha_old,prop,visco_t,0);    
    
    [H,Cp]     =  enthalpy_function(Upg,rho_m);
    [H0,bas]   =  enthalpy_function(U0pg,rho_m_old);      
    
    [Fa,Ax,Ay] =  Flux_function(Upg,rho_m);        
    
    Fc = zeros(ne,ndf);
    Fc(:,3) = Fc(:,3) - rho_m*gravity;
    
    [Fd,Kxx,Kxy,Kyx,Kyy]  = Dflux(dUpg,alpha,prop,visco_t);  
    
    % agregado ecuaciones adicionales
    for iif=1:nphases
        ipt = ndm+1+iif;
        [rho_k,visco_k] = get_mixture_props(alpha,prop,visco_t,iif);
        vel_g = vel_k(:,2+(iif-1)*2+(1:2));      
        vel_m = Upg(:,2:3);
        H(:,ipt) = alpha(:,iif);
        H0(:,ipt) = alpha_old(:,iif);      
        Cp(:,ipt,ipt) = 1;
%        Cp(:,1,ipt) = rho_g-rho_l;
%        Cp(:,2:3,ipt) = (rho_g-rho_l).*vel_m;

        difu = visco_k ./rho_k;
        for idm=1:ndm
            Fa(:,ipt,idm) = alpha(:,iif).*vel_g(:,idm);
            Fd(:,ipt,idm) = difu.*dUpg(:,ipt,idm);
        end
        Ax(:,ipt,ipt) = vel_g(:,1);
        Ay(:,ipt,ipt) = vel_g(:,2); 
        
        Ax(:,1,ipt) = (rho_g-rho_l)*vel_m(:,1);
        Ay(:,1,ipt) = (rho_g-rho_l)*vel_m(:,2);        
        Ax(:,2,ipt) = (rho_g-rho_l)*vel_m(:,1).*vel_m(:,1);
        Ax(:,3,ipt) = (rho_g-rho_l)*vel_m(:,1).*vel_m(:,2);        
        Ay(:,2,ipt) = (rho_g-rho_l)*vel_m(:,2).*vel_m(:,1);
        Ay(:,3,ipt) = (rho_g-rho_l)*vel_m(:,2).*vel_m(:,2);        
        
        Kxx(:,ipt,ipt) = difu;
        Kyy(:,ipt,ipt) = difu;      
    end
    
    % Correccion axisimetrica del jacobiano de la transformacion al elemento master
    %------------------------------------------------------------------
    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
    
    AgU = rowprod(Ax,reshape(dUpg(:,1:ndf,1),ne,ndf),ndf,ndf,1);
    AgU = AgU + rowprod(Ay,reshape(dUpg(:,1:ndf,2),ne,ndf),ndf,ndf,1);
    
    
    % P_i Petrov-Galerkin perturbation weight function
    for inod=1:nen,   
        
        compute_P_supg;
        W_supg = P_supg;      
        for idf=1:ndf,
            W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
        end
        
        % W_i N_j term
        
        for jnod=1:nen,        
            % A_gN     
            tmpx = dshapex(:,jnod,1);
            tmpy = dshapex(:,jnod,2);
            for idf=1:ndf,
                for jdf=1:ndf,
                    A_gN(:,idf,jdf,jnod) = tmpx.*Ax(:,idf,jdf) + tmpy.*Ay(:,idf,jdf) ;
                end 
            end    
        end
        
        for jnod=1:nen,
            
            % temporal terms
            if 1,
                W_Cp_N = rowprod(W_supg,Cp,ndf,ndf,ndf)*N(jnod) ;        
            else
                % DEBUG
                W_Cp_N = N(inod)*Cp*N(jnod) ;         
            end
            
            % N_A_gN or gN_A_N according to weak form
            if weak_form,
                N_A_gN = -A_gN(:,:,:,inod)*N(jnod);                 
            else          
                N_A_gN =  A_gN(:,:,:,jnod)*N(inod);              
            end
            
            
            P_A_gN = rowprod(P_supg,A_gN(:,:,:,jnod),ndf,ndf,ndf);
            
            % shock capturing term
            gN_D_gN = zeros(numel,ndf,ndf);        
            if (shocap>0)
                for iif=1:nphases
                    ipt = ndm+1+iif;
                    [rho_k,bas] = get_mixture_props(Upg(:,ndm+1+iif),prop,visco_t,iif);                                                
                    for idm=1:ndm 
                        
                        gN_D_gN(:,ipt,ipt) = gN_D_gN(:,ipt,ipt) + ...
                            dshapex(:,inod,idm).*(shocap*rho_k.*delta_k(:,iif+1)).*dshapex(:,jnod,idm);
                    end
                end
            end
            
            gN_K_gN = zeros(numel,ndf,ndf);                    
            %            keyboard;pause
            for idf=1:ndf,
                for jdf=1:ndf,
                    gN_K_gN(:,idf,jdf) = ...
                        -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                        (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                        (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                        (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
                end             
            end
            
            % assembling
            vaux =  N_A_gN + P_A_gN - gN_K_gN  + gN_D_gN ;
            
            for idf=1:ndf,
                for jdf=1:ndf,              
                    Mele(:,inod,idf,jnod,jdf) = Mele(:,inod,idf,jnod,jdf) + ... 
                        xsj.*( W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );           
                end
            end
            
        end % jnod
        
        % temporal term
        if 1,
            W_dUdt   = rowprod(W_supg,(H-H0),ndf,ndf,1);
        else
            % DEBUG
            W_dUdt   = N(inod)*(H-H0);
        end
        
        % advective fluxes
        if weak_form,            
            gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf)).*Fa(:,1:ndf,1) ... 
                -(dshapex(:,inod,2)*ones(1,ndf)).*Fa(:,1:ndf,2);
        else
            gN_Fa = (N(inod)*ones(numel,ndf)).* AgU(:,1:ndf);
        end
        
        % diffusive fluxes
        gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1) ... 
            -(dshapex(:,inod,2)*ones(1,ndf)).*Fd(:,1:ndf,2);
        
        % stabilization SUPG terms        
        P_A_gU   = rowprod(P_supg,AgU,ndf,ndf,1);
        
        gN_D_gU = zeros(ne,ndf);      
        if(shocap>0)                
            for iif=1:nphases,
                ipt = ndm+1+iif;             
                [rho_k,bas] = get_mixture_props(Upg(:,ndm+1+iif),prop,visco_t,iif);                            
                for idm=1:ndm              
                    gN_D_gU(:,ipt) = gN_D_gU(:,ipt) + ...
                        dshapex(:,inod,idm).*(shocap*rho_k.*delta_k(:,iif+1)).*dUpg(:,ipt,idm);
                end
            end         
        end        
        
        % source terms        
        if 1,
            W_Fc   = rowprod(W_supg,Fc,ndf,ndf,1) ;
        else  % DEBUG
            W_Fc   = N(inod)*Fc; 
        end
        
        vaux =  gN_Fa + P_A_gU - gN_Fd + W_Fc + gN_D_gU ;  
        
        for idf=1:ndf,
            Rele(:,inod,idf) = Rele(:,inod,idf) - (vaux(:,idf) + W_dUdt(:,idf) ./ dt ).* xsj ;
        end

%    disp('bubbly_2d_ele (liquid) phase ');keyboard;pause   
    
    end % inod
    

    
end % ipg


% ------ Terminos de contorno
if weak_form,
    [Mele,Rele] = bc_conv_nsi_tet(xnod,icone,Ue,ifaceb,Rele,Mele,prop,Uf_e);      
    %    [bas,ncc]=size(Uf_e);
    [nnR,nrR,ncR]=size(Rele);
    for iif=1:nphases

        ipt0 = 2+(iif-1)*2+(1:2);
        ipt = ipt0;
        for knod=2:nen, ipt=[ipt,ipt0+(knod-1)*2*(nphases+1)]; end
        ipt1 = [ipt(1:2:end),ipt(2:2:end)];

        ipt0 = ndm+1+iif;
        ipt = ipt0;
        for knod=2:nen, ipt=[ipt,ipt0+(knod-1)*(nphases+1+ndm)]; end        
        ipt2 = ipt;

        Rele_k = Rele(:,:,ndm+1+iif);
        Mele_k = reshape(Mele(:,:,ndm+1+iif,:,ndm+1+iif),nnR,nrR,1,nrR,1);
%        keyboard;pause        
        [Rele_k,Mele_k] = bcconv_advdif_2d(xnod,icone,Uf_e(:,ipt1),Ue(:,ipt2),ifaceb,Rele_k,Mele_k,axisim);    
        Rele(:,:,ndm+1+iif) = Rele_k;
        Mele(:,:,ndm+1+iif,:,ndm+1+iif) = Mele_k;
    end
end












