%
%  Incompressible Navier-Stokes equations
%  using SUPG-PSPG like advdif elemset
%

type_upwind         = 1;
tau_centroide       = 1;
debug               = 0;
alpha_min           = 1e-3;

%------------------------------------------------------------------
ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

alpha = extra_nodal_fields;
for iif=1:ndf_fields,
    mask = alpha(:,iif)<alpha_min;
    nn=find(mask==1);
    alpha(nn,iif)=alpha_min;
end      
extra_nodal_fields = alpha;

alpha = extra_nodal_fields_old;
for iif=1:ndf_fields,
    mask = alpha(:,iif)<alpha_min;
    nn=find(mask==1);
    alpha(nn,iif)=alpha_min;
end      
extra_nodal_fields_old = alpha;

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);
%Uenorm = zeros(ne,nen);

if ndf_fields>0,
   Uf_e = zeros(ne,nen*ndf_fields);
   Uf_e_old = zeros(ne,nen*ndf_fields);
end

for inod=1:nen,
   nodes  = icone(eles,inod);
   jnod  = (inod-1)*ndf+(1:ndf);
   knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
   Ue(:,jnod)  = U(nodes,1:ndf);		 
   U0e(:,jnod) = Uold(nodes,1:ndf);   
   re(:,inod)  = xnod(nodes,1);		        
   %   Uenorm(:,inod) = sqrt(sum(Uold(nodes,1:ndf)'.^2)');
   
   if ndf_fields>0,
      Uf_e(:,knod)  = extra_nodal_fields(nodes,1:ndf_fields);
      Uf_e_old(:,knod)  = extra_nodal_fields_old(nodes,1:ndf_fields); 
   end
end

if tau_centroide==1,
   
   % UPWIND EN EL CENTROIDE 
   %------------------------------------------------------------------
   if nen==4,
      [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0,0);
      [dshapex0,N0] = shape2d(X2S,0,0,nen); 
   elseif nen==3,
      [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),1/3,1/3);
      [dshapex0,N0] = shape2d(X2S,1/3,1/3,nen); 
   else
      error(' Bad type of element , nen ~=3, nen ~=4 ')
   end
   Upg          = zeros(ne,ndf);
   dUpg         = zeros(ne,ndf,ndm);
   
   if ndf_fields>0,
      alpha      = zeros(ne,ndf_fields);
   end
   
   for inod=1:nen,
      jnod = (inod-1)*ndf+(1:ndf);
      Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N0(inod).*  U0e(:,jnod);
      dUpg(:,(1:ndf),1) = dUpg(:,(1:ndf),1) + ...
         (dshapex0(:,inod,1)*ones(1,ndf)).* U0e(:,jnod);          
      dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2)+ ...
         (dshapex0(:,inod,2)*ones(1,ndf)).* U0e(:,jnod);             
      
      %   grad_Unorm(:,1) = grad_Unorm(:,1) + dshapex0(:,inod,1).* Uenorm(:,inod);
      %   grad_Unorm(:,2) = grad_Unorm(:,2) + dshapex0(:,inod,2).* Uenorm(:,inod);   
      
      if ndf_fields>0,
         for iif=1:ndf_fields,
            alpha(:,iif)  =  alpha(:,iif) + N0(inod).* Uf_e(:,(inod-1)*ndf_fields+iif);
         end      
      end   
   end    
      
   [rho_m,visco_m] = get_mixture_props(alpha,prop);
   
   velpg   = Upg(:,2:3);
   hele    = comp_hele(velpg,dshapex0,Jaco0,nen);
   [tau,delta] = supg_nsi_tet(Upg,hele,rho_m,visco_m);
   
   tau = tau_factor*tau;
   delta = tau_factor*delta;
   
end


% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
   
   [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
   [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
   
   % Interpolacion en punto de Gauss
   %------------------------------------------------------------------
   
   Upg    = zeros(ne,ndf);
   U0pg    = zeros(ne,ndf);
   dUpg    = zeros(ne,ndf,ndm);
   radpg    = zeros(ne,1);
   
   if ndf_fields>0,      
      alpha      = zeros(ne,ndf_fields);
      alpha_old  = alpha;      
      grad_alpha = zeros(ne,ndf_fields,ndm);      
   end
   
   for inod=1:nen,
      jnod = (inod-1)*ndf+(1:ndf);
      Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).* Ue(:,jnod);
      U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
      radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
      
      dUpg(:,(1:ndf),1) = dUpg(:,(1:ndf),1) + (dshapex(:,inod,1)*ones(1,ndf)).* Ue(:,jnod);               
      dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2) + (dshapex(:,inod,2)*ones(1,ndf)).* Ue(:,jnod);          
      
      if ndf_fields>0,         
         for iif=1:ndf_fields,
            alpha(:,iif)     =  alpha(:,iif) + N(inod).* Uf_e(:,(inod-1)*ndf_fields+iif);
            alpha_old(:,iif) =  alpha_old(:,iif) + N(inod).* Uf_e_old(:,(inod-1)*ndf_fields+iif);
            grad_alpha(:,iif,1) =  grad_alpha(:,iif,1) + dshapex(:,inod,1).* Uf_e(:,(inod-1)*ndf_fields+iif);
            grad_alpha(:,iif,2) =  grad_alpha(:,iif,2) + dshapex(:,inod,2).* Uf_e(:,(inod-1)*ndf_fields+iif);            
         end      
      end      
   end
   
   [rho_m,visco_m] = get_mixture_props(alpha,prop);
   [rho_m_old,bas] = get_mixture_props(alpha_old,prop);
   
   [H,Cp]     =  enthalpy_function(Upg,rho_m);
   [H0,bas]   =  enthalpy_function(U0pg,rho_m_old);      
   
   [Fa,Ax,Ay] =  Flux_function(Upg,rho_m);        
   
   Fc = zeros(ne,ndf);
   Fc(:,3) = Fc(:,3) - rho_m*gravity;
   
   [Fd,Kxx,Kxy,Kyx,Kyy]  = Dflux(Upg,dUpg,visco_m);  
   
   % Correccion axisimetrica del jacobiano de la transformacion al elemento master
   %------------------------------------------------------------------
   xsj =  wpg(ipg) .* Jaco;
   if axisim,
      xsj = xsj.*radpg;
   end
   
   AgU = rowprod(Ax,reshape(dUpg(:,1:ndf,1),ne,ndf),ndf,ndf,1);
   AgU = AgU + rowprod(Ay,reshape(dUpg(:,1:ndf,2),ne,ndf),ndf,ndf,1);
   
   
   % P_i Petrov-Galerkin perturbation weight function
   for inod=1:nen,   
      
      compute_P_supg;
      W_supg = P_supg;      
      for idf=1:ndf,
         W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
      end
      
      % W_i N_j term
      
      for jnod=1:nen,        
         % A_gN     
         tmpx = dshapex(:,jnod,1);
         tmpy = dshapex(:,jnod,2);
         for idf=1:ndf,
            for jdf=1:ndf,
               A_gN(:,idf,jdf,jnod) = tmpx.*Ax(:,idf,jdf) + tmpy.*Ay(:,idf,jdf) ;
            end 
         end    
      end
      
      for jnod=1:nen,
         
         % temporal terms
         if 1,
            W_Cp_N = rowprod(W_supg,Cp,ndf,ndf,ndf)*N(jnod) ;        
         else
            % DEBUG
            W_Cp_N = N(inod)*Cp*N(jnod) ;         
         end
         
         % N_A_gN or gN_A_N according to weak form
         if weak_form,
            N_A_gN = -A_gN(:,:,:,inod)*N(jnod);                 
         else          
            N_A_gN =  A_gN(:,:,:,jnod)*N(inod);              
         end
         
         
         P_A_gN = rowprod(P_supg,A_gN(:,:,:,jnod),ndf,ndf,ndf);
         
         gN_K_gN = zeros(numel,ndf,ndf);        
         for idf=1:ndf,
            for jdf=1:ndf,
               gN_K_gN(:,idf,jdf) = ...
                  -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                  (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                  (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                  (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
            end             
         end
         
         % assembling
         vaux =  N_A_gN + P_A_gN - gN_K_gN ;
         
         for idf=1:ndf,
            for jdf=1:ndf,              
               Mele(:,inod,idf,jnod,jdf) = Mele(:,inod,idf,jnod,jdf) + ... 
                  xsj.*( W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );           
            end
         end
         
      end % jnod
      
      % temporal term
      if 1,
         W_dUdt   = rowprod(W_supg,(H-H0),ndf,ndf,1);
      else
         % DEBUG
         W_dUdt   = N(inod)*(H-H0);
      end
      
      % advective fluxes
      if weak_form,            
         gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf)).*Fa(:,1:ndf,1) ... 
            -(dshapex(:,inod,2)*ones(1,ndf)).*Fa(:,1:ndf,2);
      else
         gN_Fa = (N(inod)*ones(numel,ndf)).* AgU(:,1:ndf);
      end
      
      % diffusive fluxes
      gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1) ... 
         -(dshapex(:,inod,2)*ones(1,ndf)).*Fd(:,1:ndf,2);
      
      % stabilization SUPG terms        
      P_A_gU   = rowprod(P_supg,AgU,ndf,ndf,1);
      
      % source terms        
      if 1,
         W_Fc   = rowprod(W_supg,Fc,ndf,ndf,1) ;
      else  % DEBUG
         W_Fc   = N(inod)*Fc; 
      end
      
      vaux =  gN_Fa + P_A_gU - gN_Fd + W_Fc;  
      
      for idf=1:ndf,
         Rele(:,inod,idf) = Rele(:,inod,idf) - (vaux(:,idf) + W_dUdt(:,idf) ./ dt ).* xsj ;
      end
      
   end % inod
   
   %   disp('bubbly_2d_ele (liquid) phase ');keyboard;pause   
   
end % ipg


% ------ Terminos de contorno
if weak_form,
   [Mele,Rele] = bc_conv_nsi_tet(xnod,icone,Ue,ifaceb,Rele,Mele,prop,Uf_e);      
end
