% ====================================================================
%
%  2 fases , gas fijo concentrado sobre una pared de un contenedor vertical
%
% ====================================================================


read_from_file = 0;
filestate = 'last_state';

nnwt = 3;
nnwt_outer = 1;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  4;
ndf   =  4;
tol   = 1e-6;

L = 2;                     % Depth of the fluid
R = 1;                   % Radius of the container
Nr = 21;                     % Number of elements in r dir
Nz = 21;
Nx = Nr;
Ny = Nz;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8*1;
factor   =  1e-3;
turbulence_factor = 100;
rho_l    =  1000   * factor;
rho_g    =  1      * factor;
visco_l  =  0.8e-3 * factor;          % liquid phase viscosity
visco_l  =  visco_l * turbulence_factor ;
visco_g  =  visco_l;


axisim                = 0;
comp_interphase_terms = 0;
drag_factor           = 1;
virtual_mass          = 0;
C_vm                  = 0.5;
d_bubble              = 0.004;
model_visco_std       = 0;   % std --> alpha_k * visco_k  sino avg(alpha_k) * visco_k
qpot_drag_model       = 1;   % F_drag propto alpha_l ^ qpot
fRe_drag_model        = 0;   % [0] Laminar model [1] Cd=0.44
lumping_drag_forces   = 0;
lumping_time_terms    = 0;
tau_factor            = 1;


% BOUNDARY CONDITIONS
p_ref      = 0;
p_bot      = p_ref+rho_l*abs(gravity)*L;
vel_l_wall = [0,0];
vel_l_in   = [0,0];
alpha_g_ini = 0;
alpha_g_in = 0.8;
vslip    = 0.1;

% INITIAL VALUES
p_ini       = p_ref;
vel_l_ini   = vel_l_in;

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([1,1,1],Nx)*R;
y        = refh([1,1,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
if read_from_file==1,
   eval(['load ' filestate]);
   U = phi;
else
   uini    = [p_ini,vel_l_ini,alpha_g_ini];
   U       =  ones(numnp,1)*uini;   
   U(:,1)  = (p_bot-rho_l*abs(gravity)*xnod(:,2));
end

%-------- fijaciones (FIXA)
nodes_bot   = (1:Nx)';
nodes_top   = Nx*(Ny-1)+(1:Nx)';
nodes_lef   = (1:Nx:Nx*Ny)';
nodes_rig   = (Nx:Nx:Nx*Ny)';

fixa= zeros(numnp,ndf);

fixa(nodes_bot,2:3) = ones(length(nodes_bot),2);
fixa(nodes_bot,4)   = ones(length(nodes_bot),1);
fixa(nodes_top,3)   = ones(length(nodes_top),1);
fixa(nodes_top,1)   = 1;
fixa(nodes_lef,2)   = ones(length(nodes_lef),1);
fixa(nodes_rig,2:3) = ones(length(nodes_rig),2);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
forc(nodes_top,1) = p_ref;
mask      = xnod(nodes_bot,1)<R/4;
nn        = find(mask==1);
forc(nodes_bot(nn),4) = alpha_g_in;  
alpha_l_in = 1-alpha_g_in;
rho_m_in = rho_l*alpha_l_in+rho_g*alpha_g_in;
vel_m_in = (0*rho_l*alpha_l_in+vslip*rho_g*alpha_g_in)./rho_m_in;
forc(nodes_bot(nn),3) = vel_m_in;  

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;