function [Fc,C] = Cflux_gas(Up,grad_p,vel_l,Dmat_vl,prop,job)
%
%   [Fc,C] = Cflux(Up,grad_p,vel_l,Dmat_vl,prop,job);
%
%  job = 0 ; los 3 terminos (pressure, bouyancy y drag)
%  job = 1 ; los terminos de pressure y bouyancy
%  job = 2 ; solo el termino de drag
%

ijob = 'get';
handle_prop_elem;

qpot = qpot_drag_model;

[N,ndf]     = size(Up);
alpha_g     = Up(:,1);
vel_g       = Up(:,2:3);

Fc          = zeros(N,ndf);
C           = zeros(N,ndf,ndf);

field = [1,1,1];

if job==1, field(3)=0; end
if job==2, field(1:2)=zeros(2,1); end

if field(1),
    % reactive fluxes due to bouyancy forces
    Fc(:,3)     = -rho_g*gravity*alpha_g;
    %    C(:,3,1)    = -rho_g*gravity*ones(N,1);
end

if field(2),
    % reactive fluxes due to pressure gradient
    Fc(:,2)     = Fc(:,2)+alpha_g.*grad_p(:,1);
    Fc(:,3)     = Fc(:,3)+alpha_g.*grad_p(:,2);
    %    C(:,2,1)    = C(:,2,1)+grad_p(:,1);
    %    C(:,3,1)    = C(:,3,1)+grad_p(:,2);
end

if field(3),
    if comp_interphase_terms,   
        % reactive and source terms due to drag interaction forces
        tol         = 1e-06;
        v_rel       = vel_g - vel_l;
        v_slip      = sqrt(sum(v_rel'.^2))';
        Re          = rho_l*d_bubble*v_slip/visco_l;
        mask        = Re>tol;
        nn          = find(mask==1);
        
        if isempty(nn)==0,
            
            if fRe_drag_model==0,
                f_Re        = 24.0 ./ Re(nn);
                df_RedRe    = -24.0 ./Re(nn) ./Re(nn);
            elseif fRe_drag_model==1,
                f_Re        = 0.44;
                df_RedRe    = 0;     
            elseif fRe_drag_model==2,                
                tmp0_drag   = 1.0+0.15*Re(nn).^(0.687);
                f_Re        = tmp0_drag.*(24.0 ./ Re(nn));
                df_RedRe    = -24.0 ./Re(nn) ./Re(nn) .* tmp0_drag + (24.0 ./ Re(nn)) .* (0.15*0.687*(Re(nn).^(0.687-1.0))); 
            end    

            C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
            C_drag      = C1_drag*Re(nn).*f_Re;  
            f_alpha     = alpha_g(nn).*(1-alpha_g(nn)).^qpot;
            
            Fc(nn,2)     = Fc(nn,2)+C_drag.*f_alpha.*(v_rel(nn,1));
            Fc(nn,3)     = Fc(nn,3)+C_drag.*f_alpha.*(v_rel(nn,2));
            
            if 0,
                C(nn,2,1)    = C(nn,2,1) + C_drag.*(v_rel(nn,1));
                C(nn,3,1)    = C(nn,3,1) + C_drag.*(v_rel(nn,2));
            end
            
            dRedU       = rho_l*d_bubble/visco_l;
            
            tmp4_drag   = f_Re+Re(nn).*df_RedRe;
            
            tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);
            for idm=1:2,
                for jdm=1:2,
                    Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
                end
                Phi(:,idm,idm) = Phi(:,idm,idm)+f_Re.*Re(nn);                
            end
            
            C(nn,2,2)    = C(nn,2,2) + C1_drag.*f_alpha.*Phi(:,1,1);
            C(nn,2,3)    = C(nn,2,3) + C1_drag.*f_alpha.*Phi(:,1,2);
            C(nn,3,2)    = C(nn,3,2) + C1_drag.*f_alpha.*Phi(:,2,1);
            C(nn,3,3)    = C(nn,3,3) + C1_drag.*f_alpha.*Phi(:,2,2);
            
        end
        %disp('Cflux');keyboard;pause      
    end
    
end

if virtual_mass,
    Fc(:,2:3) = Fc(:,2:3)-C_vm*rho_l*(alpha_g*[1,1]).*Dmat_vl;
end
