function [Fc,C] = Cflux_liq(Up,alpha_g,grad_alpha_g,vel_g,prop,job)
%
%   [Fc,C] = Cflux(Up,alpha_g,grad_alpha_g,vel_g,prop,job)
%
%  job = 0 ; los 3 terminos (pressure, bouyancy y drag)
%  job = 1 ; los terminos de pressure y bouyancy
%  job = 2 ; solo el termino de drag
%

ijob = 'get';
handle_prop_elem;

qpot = 1;

[N,ndf]      = size(Up);
pre          = Up(:,1);
vel_l        = Up(:,2:3);

alpha_l      = 1-alpha_g;
grad_alpha_l = -grad_alpha_g;

Fc           = zeros(N,ndf);
C            = zeros(N,ndf,ndf);

field = [1,1,1];

if job==1, field(3)=0; end
if job==2, field(1:2)=zeros(2,1); end

if field(1),
    % reactive fluxes due to bouyancy forces
    Fc(:,3)     = -rho_l*gravity*alpha_l;
end

if field(2),
    % reactive fluxes due to pressure gradient
    Fc(:,2)     = Fc(:,2)-grad_alpha_l(:,1).*pre;
    Fc(:,3)     = Fc(:,3)-grad_alpha_l(:,2).*pre;
    C(:,2,1)    = C(:,2,1)-grad_alpha_l(:,1);
    C(:,3,1)    = C(:,3,1)-grad_alpha_l(:,2);    
end

if field(3),
    if comp_interphase_terms,   
        % reactive and source terms due to drag interaction forces
        tol      = 1e-06;
        v_rel       = vel_l - vel_g;
        v_slip      = sqrt(sum(v_rel'.^2))';
        Re          = rho_l*d_bubble*v_slip/visco_l;
        mask        = Re>tol;
        nn          = find(mask==1);
        
        if isempty(nn)==0,
            %Re          = max([Re';tol*ones(1,N)])';
            f_Re        = 24.0 ./ Re(nn);
            C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
            C_drag      = C1_drag*Re(nn).*f_Re;  
            df_RedRe    = -24.0 ./Re(nn) ./Re(nn);
            f_alpha     = alpha_g(nn).*(1-alpha_g(nn)).^qpot;
            
            Fc(nn,2)     = Fc(nn,2)+C_drag.*f_alpha.*(v_rel(nn,1));
            Fc(nn,3)     = Fc(nn,3)+C_drag.*f_alpha.*(v_rel(nn,2));
                        
            dRedU       = rho_l*d_bubble/visco_l;
            
            tmp4_drag   = f_Re+Re(nn).*df_RedRe;
            
            tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);
            for idm=1:2,
                for jdm=1:2,
                    Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
                end
%                Phi(:,idm,idm) = Phi(:,idm,idm)+C_drag.*f_alpha;
                Phi(:,idm,idm) = Phi(:,idm,idm)+f_Re.*Re(nn);
            end
            
            C(nn,2,2)    = C(nn,2,2) + C1_drag.*f_alpha.*Phi(:,1,1);
            C(nn,2,3)    = C(nn,2,3) + C1_drag.*f_alpha.*Phi(:,1,2);
            C(nn,3,2)    = C(nn,3,2) + C1_drag.*f_alpha.*Phi(:,2,1);
            C(nn,3,3)    = C(nn,3,3) + C1_drag.*f_alpha.*Phi(:,2,2);
            
        end
        %disp('Cflux liq ');keyboard;pause
    end
    
end

