% ====================================================================
%
%                           laddle_1d
%
% ====================================================================
read_from_file = 1;
filestate = 'tempo_w5';

nnwt_liq = 4;
nnwt_gas = 4;

shocap                = 1;
weak_form             = 1; 

Cou   =  1;
npg   =  4;
ndf   =  6;
ndf_liq = 3;
ndf_gas = 3;
tol   = 1e-6;

indx_gas = [2,5,6];
indx_liq = [1,3,4];

H = 0.4;                     % Depth of the fluid
R = 0.5/2;                   % Radius of the container
Nr = 2;                     % Number of elements in r dir
Nz = 100;
Nx = Nr;
Ny = Nz;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size
nozzle_radius = 6e-2/2;      
factor_area_nozzle = 1.;
nozzle_radius = nozzle_radius*factor_area_nozzle;
Q_in = 2.0e-4;               % flow rate at inlet nozzle

% PHYSICAL DATA
gravity  = -9.8;
visco_l  =  0.8e-3;          % liquid phase viscosity
%visco_l  =  0.01;
visco_g  =  1.000e-5;        % gas phase viscosity
%visco_g  =  0.01;        % gas phase viscosity
rho_l    =  1000.0;
rho_g    =  1.0;

d_bubble              = 0.001;
comp_interphase_terms = 1;
axisim                = 0;
drag_factor           = 1;

% BOUNDARY CONDITIONS
p_ref      = 0;
p_bot      = p_ref+rho_l*abs(gravity)*H;
alpha_g_in = 1.0;
alpha_g_in = alpha_g_in/ (factor_area_nozzle*factor_area_nozzle);
Area_in    = pi*nozzle_radius*nozzle_radius;
vel_g_in   = 1*[0,Q_in/Area_in/rho_g/alpha_g_in];
vel_l_wall = [0,0];
vel_g_wall = [0,0];
vel_l_in   = [0,0];

% INITIAL VALUES
p_ini       = p_ref;
alpha_g_ini = 0.001*alpha_g_in;
vel_l_ini   = 1e-6*vel_g_in;
%vel_l_ini   = 0*vel_g_in;
vel_g_ini   = vel_g_in;

% ESCALEO
if 1,
   visco_l=visco_l/sqrt(rho_l);
   visco_g=visco_g/sqrt(rho_l);
   rho_g=rho_g/sqrt(rho_l);
   p_bot=p_bot/sqrt(rho_l);    
   p_ini=p_ini/sqrt(rho_l);    
   p_ref=p_ref/sqrt(rho_l);    
   rho_l=rho_l/sqrt(rho_l);
end

% ====================================================================
x        = (0:Nr-1)'/(Nr-1)*R;
y        = refh([1,6*zratio,8*zratio],Nz)*H;

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
   xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
   icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
if read_from_file==1,
   eval(['load ' filestate]);
   U = phi;
else
   uini    = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
   U       =  ones(numnp,1)*uini;   
   
   U(:,indx_liq(1)) = (p_bot-rho_l*abs(gravity)*xnod(:,2));
   
   Ampl   = alpha_g_in-alpha_g_ini;
   sigma  = 0.2;
   sigma  = 0.05;
   xc     = 0;
   z      = xnod(:,2)-xc;
   %U(:,indx_gas(1)) = alpha_g_ini+Ampl*exp(-z.^2/sigma);   
   U(:,indx_gas(1)) = alpha_g_in;
   nn = find(xnod(:,2)>H/2);
   U(nn,indx_gas(1)) = alpha_g_ini;   
end

%-------- fijaciones (FIXA)
xmin = min(xnod);
xmax = max(xnod);
tol  = 1e-6;
nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_wall= find(abs(xnod(:,1)-xmax(1))<tol);
nodes_axis= find(abs(xnod(:,1)-xmin(1))<tol);

fixa = zeros(numnp,ndf);
forc = zeros(numnp,ndf);

% entrada
nodes = nodes_bot;
idfs=[2,3,4,5,6]; 
values = [0,alpha_g_in,vel_l_in,vel_g_in];
for icompo=idfs
   value = values(icompo)*ones(length(nodes),1);
   fixa(nodes,icompo) = ones(length(nodes),1);
   forc(nodes,icompo) = value;  
end

% en la tapa de arriba la presion de referencia, velocidad liquido normal nula
nodes = nodes_top;
idfs=[1]; 
values = [p_ref,alpha_g_in,0,0,0,0];
for icompo=idfs
   value = values(icompo)*ones(length(nodes),1);
   fixa(nodes,icompo) = ones(length(nodes),1);
   forc(nodes,icompo) = value;  
end

if 0,
   % fijamos todo el liquido
   fixa(:,indx_liq(1:3))= ones(numnp,3);
   forc(:,indx_liq(2:3))= 0*ones(numnp,2);
   forc(:,indx_liq(1))= p_bot-rho_l*abs(gravity)*(xnod(:,2));
end    

if 0,
   % fijamos toda la velocidad del liquido
   fixa(:,indx_liq(2:3))= ones(numnp,2);
   forc(:,indx_liq(2:3))= ones(numnp,1)*[0,0.00000];
end

if 0,
   % fijamos solo la presion con un gradiente algo mayor que el hidrostatico
   
   if 1,
      fixa(:,indx_liq(1))= ones(numnp,1);      
      %      forc(:,indx_liq(1))= 1*(p_bot-rho_l*abs(gravity)*(xnod(:,2)));
      forc(:,indx_liq(1))= phi(:,indx_liq(1));
   elseif 0,   
      z = xnod(:,2);
      f_dp = 100;
      Cte = p_ref/(rho_l*abs(gravity)/H)-H^2*(f_dp+0.5*(1-f_dp));
      p = -rho_l*abs(gravity)/H*(f_dp*H*z+(1-f_dp)/2*z.^2+Cte);
      fixa(:,indx_liq(1))= ones(numnp,1);
      forc(:,indx_liq(1))= p;
   else
      z = xnod(:,2);      
      z0 = min(z); 
      zf = max(z); 
      z1 = z0+0.05*(zf-z0); 
      z2 = z0+(0.4:0.1:0.9)*(zf-z0);       
      x  = [z0,z1,z2,zf];  
      y0 = p_bot;
      yf = p_bot-rho_l*abs(gravity)*zf;
      y2 = p_bot-rho_l*abs(gravity)*z2;
      y  = [y0,1.1*y0,y2,yf]; 
      p = spline(x,y,z);
      figure(2);clf;plot(x,y,'o',z,p)      
      pause
      fixa(:,indx_liq(1))= ones(numnp,1);
      forc(:,indx_liq(1))= p;      
   end
   
end

if 0,
   % fijamos la fraccion del gas
   fixa(:,indx_gas(1))= ones(numnp,1);
   Ampl   = alpha_g_in-alpha_g_ini;
   sigma  = 0.2;
   sigma  = 0.005;
   xc     = 0.1;
   z      = xnod(:,2)-xc;
   %forc(:,indx_gas(1)) = alpha_g_ini+Ampl*exp(-z.^2/sigma);   
   forc(:,indx_gas(1))= alpha_g_in*ones(numnp,1);
   nn = find(xnod(:,2)>H/2);
   forc(nn,indx_gas(1))= alpha_g_ini*ones(length(nn),1);
   %forc(nn,indx_gas(1))= phi(:,indx_gas(1));   
end

if 0,
   % fijamos toda la velocidad del gas
   fixa(:,indx_gas(2:3))= ones(numnp,2);
   forc(:,indx_gas(2:3))= ones(numnp,1)*[0,0.01];
end


%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];

for icompo=[1,2,3,4,5,6]
   lin_con = [lin_con ; ... 
         [ones(length(nodes_axis),1) nodes_axis icompo*ones(length(nodes_axis),1) ...
            -ones(length(nodes_wall),1) nodes_wall icompo*ones(length(nodes_wall),1)]];      
end

for icompo=[2,3,4,5,6]   
   lin_con = [lin_con ; ... 
         [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
            -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];         
end



% condicion inicial en primitivas
%-----------------------------------------------

bini   = U;
for k=1:ndf,
   nofree        = find(fixa(:,k)~=0);
   bini(nofree,k)   = forc(nofree,k);
end   


bc_time_flag = 0;
bc_time_t_end = 1;