% ====================================================================
%
%                           laddle_2d
%
% ====================================================================

read_from_file = 1;
filestate = 'tempo_2d_53';

nnwt_liq = 4;
nnwt_gas = 4;

shocap                = 2;
weak_form             = 1; 

Cou   =  1;
npg   =  4;
ndf   =  6;
ndf_liq = 3;
ndf_gas = 3;
tol   = 1e-6;

indx_gas = [2,5,6];
indx_liq = [1,3,4];


H = 0.4;                     % Depth of the fluid
R = 0.5/2;                   % Radius of the container
Nr = 30;                     % Number of elements in r dir
Nz = 45;
Nz = 30;
Nx = Nr;
Ny = Nz;
rratio = 0.25;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size
nozzle_radius = 6e-2/2;      
factor_area_nozzle = 1.4;
nozzle_radius = nozzle_radius*factor_area_nozzle;
Q_in = 2.0e-4;               % flow rate at inlet nozzle

% PHYSICAL DATA
visco_l  =  0.8e-3;          % liquid phase viscosity
visco_l  =  0.8e-1;          % liquid phase viscosity
visco_g  =  1.000e-5;        % gas phase viscosity
gravity  = -9.8;
rho_l    =  1000.0;
rho_g    =  1.0;
d_bubble = 0.004;
comp_interphase_terms = 1;
axisim                = 0;
drag_factor           = 1.0;

% BOUNDARY CONDITIONS
p_ref      = 0;
p_bot      = p_ref+rho_l*abs(gravity)*H;
alpha_g_in = 1.0;
alpha_g_in = alpha_g_in/ (factor_area_nozzle*factor_area_nozzle);
Area_in = pi*nozzle_radius*nozzle_radius;
vel_g_in  = [0,Q_in/Area_in/rho_g/alpha_g_in];
vel_l_wall  = [0,0];
vel_g_wall  = [0,0];
vel_l_in    = [0,0];

% INITIAL VALUES
p_ini       = p_ref;
alpha_g_ini = 0.01*alpha_g_in;
vel_l_ini   = 1e-6*[0,1];
vel_g_ini   = 1e-6*[0,1];

% ESCALEO
if 1,
   visco_l=visco_l/sqrt(rho_l);
   visco_g=visco_g/sqrt(rho_l);
   rho_g=rho_g/sqrt(rho_l);
   p_bot=p_bot/sqrt(rho_l);    
   p_ini=p_ini/sqrt(rho_l);    
   p_ref=p_ref/sqrt(rho_l);    
   rho_l=rho_l/sqrt(rho_l);
end

% ====================================================================
x        = refh([rratio,5*rratio,2*rratio],Nr)*R;
y        = refh([1,4*zratio,8*zratio],Nz)*H;
%w=zhomo([0 R 0 H],Nr+1,Nz+1,[rratio 5*rratio 2*rratio 1 4*zratio 8*zratio]);

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
   xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
   icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
xmin = min(xnod);
xmax = max(xnod);

if read_from_file==1,
   eval(['load ' filestate]);
   U = phi;
else
   uini    = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
   U       =  ones(numnp,1)*uini;   
   U(:,indx_liq(1)) = rho_l*abs(gravity)*(H-xnod(:,2));
   
   Ampl   = alpha_g_in-alpha_g_ini;
   sigma  = 0.2;
   sigma  = 0.05;
   xc     = xmin;
   xx     = xnod-ones(numnp,1)*xc;
   z      = sqrt(sum(xx'.^2))';
   U(:,indx_gas(1)) = alpha_g_ini+Ampl*exp(-z.^2/sigma);   
end

%-------- fijaciones (FIXA)
tol  = 1e-6;
nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_wall= find(abs(xnod(:,1)-xmax(1))<tol);
nodes_axis= find(abs(xnod(:,1)-xmin(1))<tol);
nn = find(abs(xnod(nodes_bot,1))<nozzle_radius);
nodes_nozzle = nodes_bot(nn);
nodes_bot(nn)=[];

fixa = zeros(numnp,ndf);
forc = zeros(numnp,ndf);

% pared fondo
nodes = nodes_bot;
idfs = [3,4,5,6]; 
values = [0,alpha_g_in,vel_l_wall,vel_g_wall];
for icompo=idfs
%  compo = icompo*ones(length(nodes),1);
  value = values(icompo)*ones(length(nodes),1);
%  fixa  = [ fixa ; [nodes compo value ]];
  fixa(nodes,icompo) = ones(length(nodes),1);
  forc(nodes,icompo) = value;
end

% entrada tapon
nodes = nodes_nozzle;
idfs=[2,4,5,6]; 
values = [0,alpha_g_in,vel_l_in,vel_g_in];
for icompo=idfs
%  compo = icompo*ones(length(nodes),1);
  value = values(icompo)*ones(length(nodes),1);
%  fixa  = [ fixa ; [nodes compo value ]];
  fixa(nodes,icompo) = ones(length(nodes),1);
  forc(nodes,icompo) = value;  
end

% en la pared las velocidades son nulas
nodes = nodes_wall;
values = [0,alpha_g_in,vel_l_wall,vel_g_wall];
idfs =[3,4,5,6]; 
for icompo=idfs
%  compo = icompo*ones(length(nodes),1);
  value = values(icompo)*ones(length(nodes),1);
%  fixa  = [ fixa ; [nodes compo value ]];
  fixa(nodes,icompo) = ones(length(nodes),1);
  forc(nodes,icompo) = value;  
end

% en la tapa de arriba la presion de referencia, velocidad liquido normal nula
nodes = nodes_top;
idfs=[1,4]; 
values = [p_ref,alpha_g_in,0,0,0,0];
for icompo=idfs
%  compo = icompo*ones(length(nodes),1);
  value = values(icompo)*ones(length(nodes),1);
%  fixa  = [ fixa ; [nodes compo value ]];
  fixa(nodes,icompo) = ones(length(nodes),1);
  forc(nodes,icompo) = value;  
end

% en el eje la velocidad normal es nula
nodes = nodes_axis;
values = [0,alpha_g_in,0,0,0,0]; 
idfs=[3,5];
for icompo=idfs
%  compo = icompo*ones(length(nodes),1);
  value = values(icompo)*ones(length(nodes),1);
%  fixa  = [ fixa ; [nodes compo value ]];
  fixa(nodes,icompo) = ones(length(nodes),1);
  forc(nodes,icompo) = value;  
end

if 0,
% fijamos toda la fraccion de gas
fixa(:,indx_gas(1))= ones(numnp,1);
forc(:,indx_gas(1))= alpha_g_in*ones(numnp,1);
end

if 0,
fixa(nodes_bot,indx_liq(3)) = ones(length(nodes_bot),1);
fixa(nodes_top,indx_liq(3)) = ones(length(nodes_top),1);
fixa(nodes_lef,indx_liq(2)) = ones(length(nodes_lef),1);
fixa(nodes_rig,indx_liq(2:3)) = ones(length(nodes_rig),2);
%fixa(:,indx_liq(2:3)) = ones(numnp,2);

fixa(nodes_bot,indx_liq(1))  = ones(length(nodes_bot),1);
fixa(nodes_top,indx_liq(1))  = ones(length(nodes_top),1);
%fixa(:,indx_liq(1))  = ones(numnp,1);

fixa(nodes_bot,indx_gas(2:3)) = ones(length(nodes_bot),2);
fixa(nodes_top,indx_gas(2)) = ones(length(nodes_top),1);
fixa(nodes_lef,indx_gas(2)) = ones(length(nodes_lef),1);
fixa(nodes_rig,indx_gas(2:3)) = ones(length(nodes_rig),2);

fixa(nodes_bot,indx_gas(1))  = ones(length(nodes_bot),1);

%-------- Cargas/Desplazamientos nodales(FORC)
fixa   = zeros(numnp,ndf);
forc= zeros(numnp,ndf);

forc(nodes_bot,indx_liq(1))  = rho_l*abs(gravity)*(Ly-xnod(nodes_bot,2));
forc(nodes_top,indx_liq(1))  = 0;
%forc(        :,indx_liq(1))  = rho_l*abs(gravity)*(Ly-xnod(:,2));

forc(nodes_bot,indx_gas(1))  = alpha_g_ref;
forc(nodes_bot,indx_gas(3))  = vel_g_ref(2);
end

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
   for icompo=[1,2,3]
      lin_con = [lin_con ; ... 
            [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
               -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
      
      lin_con = [lin_con ; ... 
            [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
               -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
   end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
   bini   = U;
   for k=1:ndf,
      nofree        = find(fixa(:,k)~=0);
      bini(nofree,k)   = forc(nofree,k);
   end   
end

bc_time_flag = 0;
bc_time_t_end = 1;