% ====================================================================
%
%                           laddle_1d_homo
%
% ====================================================================
read_from_file = 0;
filestate = 'bubbly_1d_homo';

nnwt_outer = 2;
nnwt_liq = 3;
nnwt_gas = 3;

shocap                = 1;
weak_form             = 1; 

Cou   =  1;
npg   =  4;
ndf   =  6;
ndf_liq = 3;
ndf_gas = 3;
tol   = 1e-6;

indx_gas = [2,5,6];
indx_liq = [1,3,4];

H = 0.4;                     % Depth of the fluid
H = 2.0;                     % Depth of the fluid
R = 0.5/2;                   % Radius of the container
Nr = 2;                     % Number of elements in r dir
Nz = 100;
Nx = Nr;
Ny = Nz;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8*0;
visco_l  =  0.8e-3;          % liquid phase viscosity
visco_g  =  1.000e-5;        % gas phase viscosity

rho_l    =  1.0;
rho_g    =  1.0;

d_bubble              = 0.004;
comp_interphase_terms = 1;
axisim                = 0;
drag_factor           = 1;
virtual_mass          = 0;
C_vm                  = 0.5;
model_visco_std       = 0;   % std --> alpha_k * visco_k  sino avg(alpha_k) * visco_k
qpot_drag_model       = 1;   % F_drag propto alpha_l ^ qpot
fRe_drag_model        = 0;   % [0] Laminar model [1] Cd=0.44

lumping_drag_forces   = 1;
lumping_time_terms    = 1;
tau_factor            = 1;

% BOUNDARY CONDITIONS
% es flujo homogeneo por lo tanto los contornos son todos periodicos

% INITIAL VALUES
p_ini       = 0;
alpha_g_ini = 0.1;
vel_l_ini   = [0,1];
vel_g_ini   = [0,1];
uini        = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
Ampl        = uini.*[0,0.1,0,0,0,0];

% ESCALEO
if 0,
    visco_l=visco_l/sqrt(rho_l);
    visco_g=visco_g/sqrt(rho_l);
    rho_g=rho_g/sqrt(rho_l);
    rho_l=rho_l/sqrt(rho_l);
    p_ini=p_ini/(rho_l);    
else
    visco_l=visco_l/(rho_l);
    visco_g=visco_g/(rho_l);
    rho_g=rho_g/(rho_l);
    p_ini=p_ini/(rho_l);    
    rho_l=rho_l/(rho_l);   
end

% ====================================================================
x        = (0:Nr-1)'/(Nr-1)*R;
%y        = refh([1,6*zratio,8*zratio],Nz)*H;
y        = refh([1,1,1],Nz)*H;

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate]);
    U = phi;
else
    Ly   = max(xnod(:,2))-min(xnod(:,2));
    U    = ones(numnp,1)*uini;          
    U    = U+sin(2*pi*xnod(:,2)/Ly)*Ampl;    
end

%-------- fijaciones (FIXA)
fixa = zeros(numnp,ndf);
forc = zeros(numnp,ndf);

xmin = min(xnod);
xmax = max(xnod);
tol  = 1e-6;

nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_wall= find(abs(xnod(:,1)-xmax(1))<tol);
nodes_axis= find(abs(xnod(:,1)-xmin(1))<tol);

% fijamos la presion en un nodo
fixa(1,1)=1;

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];

for icompo=[1,2,3,4,5,6]
    lin_con = [lin_con ; ... 
            [ones(length(nodes_axis),1) nodes_axis icompo*ones(length(nodes_axis),1) ...
                -ones(length(nodes_wall),1) nodes_wall icompo*ones(length(nodes_wall),1)]];      
end

for icompo=[2,3,4,5,6]   
    lin_con = [lin_con ; ... 
            [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                -ones(length(nodes_bot),1) nodes_bot icompo*ones(length(nodes_bot),1)]];         
end

% condicion inicial en primitivas
%-----------------------------------------------

bini   = U;
for k=1:ndf,
    nofree        = find(fixa(:,k)~=0);
    bini(nofree,k)   = forc(nofree,k);
end   


bc_time_flag = 0;
bc_time_t_end = 1;