%
%  Incompressible Navier-Stokes equations
%  using SUPG-PSPG like advdif elemset
%
%             LIQUID PHASE
%

type_upwind         = 1;
tau_centroide       = 0;
debug               = 0;
factor_alpha_liq    = 1;  % multiplica o relaja al grad_alpha_g
flag_tempo          = 1;  % [0] como antes [1] modif para hacer que coincida con el Petscfem
%------------------------------------------------------------------

ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);
Uf_e = zeros(ne,nen*ndf_fields);
Uf_e_old = zeros(ne,nen*ndf_fields);
Uenorm = zeros(ne,nen);
for inod=1:nen,
   nodes  = icone(eles,inod);
   jnod  = (inod-1)*ndf+(1:ndf);
   knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
   Ue(:,jnod)  = U(nodes,1:ndf);		 
   U0e(:,jnod) = Uold(nodes,1:ndf);   
   re(:,inod)  = xnod(nodes,1);		        
   Uf_e(:,knod)  = extra_nodal_fields(nodes,1:ndf_fields);
   Uf_e_old(:,knod)  = extra_nodal_fields_old(nodes,1:ndf_fields); 
   Uenorm(:,inod) = sqrt(sum(Uold(nodes,1:ndf)'.^2)');
end

% UPWIND EN EL CENTROIDE 
%------------------------------------------------------------------
if nen==4,
   [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0,0);
   [dshapex0,N0] = shape2d(X2S,0,0,nen); 
elseif nen==3,
   [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),1/3,1/3);
   [dshapex0,N0] = shape2d(X2S,1/3,1/3,nen); 
else
   error(' Bad type of element , nen ~=3, nen ~=4 ')
end
Upg        = zeros(ne,ndf);
dUpg       = zeros(ne,ndf,ndm);
dUpg_0   = zeros(ne,ndf,ndm);
grad_Unorm = zeros(ne,ndm);
alpha_g    = zeros(ne,1);
vel_g      = zeros(ne,ndm);
grad_alpha_g   = zeros(ne,ndm);   
vel_g_old  = zeros(ne,ndm);
grad_vel_g = zeros(ne,ndm,ndm);

for inod=1:nen,
   jnod = (inod-1)*ndf+(1:ndf);
   Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N0(inod).*  U0e(:,jnod);
   dUpg(:,(1:ndf),1) = dUpg(:,(1:ndf),1) + ...
      (dshapex0(:,inod,1)*ones(1,ndf)).* U0e(:,jnod);          
   dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2)+ ...
      (dshapex0(:,inod,2)*ones(1,ndf)).* U0e(:,jnod);             
   dUpg_0(:,(1:ndf),1) = dUpg_0(:,(1:ndf),1) + ...
      (dshapex0(:,inod,1)*ones(1,ndf)).* Ue(:,jnod);          
   dUpg_0(:,(1:ndf),2) = dUpg_0(:,(1:ndf),2)+ ...
      (dshapex0(:,inod,2)*ones(1,ndf)).* Ue(:,jnod);             
   
   grad_Unorm(:,1) = grad_Unorm(:,1) + dshapex0(:,inod,1).* Uenorm(:,inod);
   grad_Unorm(:,2) = grad_Unorm(:,2) + dshapex0(:,inod,2).* Uenorm(:,inod);   
   
   alpha_g    =     alpha_g + N0(inod).* Uf_e(:,(inod-1)*ndf_fields+1);
   vel_g(:,1) = vel_g (:,1) + N0(inod).* Uf_e(:,(inod-1)*ndf_fields+2);
   vel_g(:,2) = vel_g (:,2) + N0(inod).* Uf_e(:,(inod-1)*ndf_fields+3);      
   grad_alpha_g (:,1)= grad_alpha_g(:,1) + dshapex0(:,inod,1).* Uf_e(:,(inod-1)*ndf_fields+1); 
   grad_alpha_g (:,2)= grad_alpha_g(:,2) + dshapex0(:,inod,2).* Uf_e(:,(inod-1)*ndf_fields+1);       
   
   vel_g_old(:,1)   = vel_g_old (:,1) + N0(inod).* Uf_e_old(:,(inod-1)*ndf_fields+2);
   vel_g_old(:,2)   = vel_g_old (:,2) + N0(inod).* Uf_e_old(:,(inod-1)*ndf_fields+3);      
   
   grad_vel_g(:,1,1)  = grad_vel_g(:,1,1) + ...
      dshapex0(:,inod,1).* Uf_e(:,(inod-1)*ndf_fields+2);
   grad_vel_g(:,1,2)  = grad_vel_g(:,1,2) + ...
      dshapex0(:,inod,2).* Uf_e(:,(inod-1)*ndf_fields+2); 
   
   grad_vel_g(:,2,1)  = grad_vel_g(:,2,1) + ...
      dshapex0(:,inod,1).* Uf_e(:,(inod-1)*ndf_fields+3);
   grad_vel_g(:,2,2)  = grad_vel_g(:,2,2) + ...
      dshapex0(:,inod,2).* Uf_e(:,(inod-1)*ndf_fields+3); 
   
end    

grad_alpha_g_0 = grad_alpha_g;

if tau_centroide==1,
   velpg_old   = Upg(:,2:3);
   hele    = comp_hele(velpg_old,dshapex,Jaco,nen);
   [tau,delta,bas] = supg_bubbly_2d_liq(Upg,dUpg,hele,prop,type_upwind);
end

Vol = zeros(ne,nen);

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
   
   [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
   [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
   
   % Interpolacion en punto de Gauss
   %------------------------------------------------------------------
   
   Upg    = zeros(ne,ndf);
   U0pg    = zeros(ne,ndf);
   dUpg    = zeros(ne,ndf,ndm);
   radpg    = zeros(ne,1);
   alpha_g   = zeros(ne,1);
   grad_alpha_g   = zeros(ne,ndm);   
   vel_g    = zeros(ne,ndm);
   alpha_g_old    = zeros(ne,1);
   vel_g_old    = zeros(ne,ndm);
   
   for inod=1:nen,
      jnod = (inod-1)*ndf+(1:ndf);
      Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
      U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
      radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
      
      dUpg(:,(1:ndf),1)   = dUpg(:,(1:ndf),1) + ...
         (dshapex(:,inod,1)*ones(1,ndf)).* Ue(:,jnod);
      
      dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2)+ ...
         (dshapex(:,inod,2)*ones(1,ndf)).* Ue(:,jnod);          
      alpha_g      =      alpha_g + N(inod).* Uf_e(:,(inod-1)*ndf_fields+1);
      grad_alpha_g (:,1)= grad_alpha_g(:,1) + dshapex(:,inod,1).* Uf_e(:,(inod-1)*ndf_fields+1); 
      grad_alpha_g (:,2)= grad_alpha_g(:,2) + dshapex(:,inod,2).* Uf_e(:,(inod-1)*ndf_fields+1);       
      vel_g(:,1) = vel_g (:,1) + N(inod).* Uf_e(:,(inod-1)*ndf_fields+2);
      vel_g(:,2) = vel_g (:,2) + N(inod).* Uf_e(:,(inod-1)*ndf_fields+3);      
      
      alpha_g_old      =   alpha_g_old   + N(inod).* Uf_e_old(:,(inod-1)*ndf_fields+1);         
      vel_g_old(:,1)   = vel_g_old (:,1) + N(inod).* Uf_e_old(:,(inod-1)*ndf_fields+2);
      vel_g_old(:,2)   = vel_g_old (:,2) + N(inod).* Uf_e_old(:,(inod-1)*ndf_fields+3);      
      
   end
   
   [H,Cp]     =  enthalpy_function_liq(Upg,alpha_g,prop);
   if flag_tempo>0,
      [H0,bas]   =  enthalpy_function_liq(U0pg,alpha_g_old,prop);
   else
      [H0,bas]   =  enthalpy_function_liq(U0pg,alpha_g,prop);      
   end
   
   % advective flux function  
   [Fa,Ax,Ay]     =  Flux_function_liq(Upg,alpha_g,prop);        
   % reactive and source terms due to:
   % .- pressure gradient 
   % .- bouyancy forces
   % .- phase interactions drag  
   
   if lumping_drag_forces,
      job  = 1; % [0] en Cflux todas las 3 fuerzas (pressure, buoyancy y pressure)
      % [1] en Cflux solo las fuerzas de pressure y buoyancy
      % [2] en Cflux solo la fuerza de drag
      %job  = 2;
   else
      job = 0;
   end
   
   for idm=1:2,
      Dmat_vg(:,idm)    = (vel_g(:,idm)-vel_g_old(:,idm))/dt + ...
         vel_g(:,1).*grad_vel_g(:,idm,1)+vel_g(:,2).*grad_vel_g(:,idm,2);
      Dmat_vl(:,idm)    = (Upg(:,idm+1)-U0pg(:,idm+1))/dt + ...
         Upg(:,2).*dUpg(:,idm+1,1)+Upg(:,3).*dUpg(:,idm+1,2);      
%         Upg(:,idm+1).*dUpg(:,idm+1,1)+Upg(:,idm+1).*dUpg(:,idm+1,2);          
   end
   
   [Fc_1,C_1] = Cflux_liq(Upg,dUpg,alpha_g,factor_alpha_liq*grad_alpha_g, ...
      vel_g,Dmat_vg,Dmat_vl,prop,job);      
   if lumping_drag_forces, Fc_1 = 0*Fc_1; C_1=0*C_1; end
   
   [Fd,Kxx,Kxy,Kyx,Kyy]  = Dflux_liq(Upg,dUpg,alpha_g,prop);  
   
   if debug,
      Fc_1=0*Fc_1;
      C_1=0*C_1;
      Fd=0*Fd;
      Kxx=0*Kxx;
      Kxy=0*Kxy;
      Kyx=0*Kyx;
      Kyy=0*Kyy;
   end
   
   % Correccion axisimetrica del jacobiano de la transformacion al elemento master
   %------------------------------------------------------------------
   xsj =  wpg(ipg) .* Jaco;
   if axisim,
      xsj = xsj.*radpg;
   end
   
   AgU = rowprod(Ax,reshape(dUpg(:,1:ndf,1),ne,ndf),ndf,ndf,1);
   AgU = AgU + rowprod(Ay,reshape(dUpg(:,1:ndf,2),ne,ndf),ndf,ndf,1);
   
   
   if tau_centroide==0,
      velpg_old   = U0pg(:,2:3);
      hele    = comp_hele(velpg_old,dshapex,Jaco,nen);
      %      hele    = 0.4/100*ones(size(hele));
      [tau,delta,bas] = supg_bubbly_2d_liq(U0pg,dUpg,hele,prop,type_upwind);
   end
   
   
   % P_i Petrov-Galerkin perturbation weight function
   for inod=1:nen,   
      
      P_supg = zeros(ne,ndf,ndf);       
      
      % Element Volume
      Vol(:,inod)=Vol(:,inod)+N(inod)*xsj;
      
      % W_i = N_i + P_i
      tmp = tau.*(velpg_old(:,1).*dshapex(:,inod,1)+velpg_old(:,2).*dshapex(:,inod,2));
      for idm=1:ndm,
         P_supg(:,idm+1,idm+1) = tmp;
      end
      P_supg(:,1,2) = tau/rho_l.*dshapex(:,inod,1);
      P_supg(:,1,3) = tau/rho_l.*dshapex(:,inod,2);
      %P_supg(:,1,2) = tau.*dshapex(:,inod,1);
      %P_supg(:,1,3) = tau.*dshapex(:,inod,2);
      
      if (shocap>0 & 1),
         P_supg(:,2,1) = rho_l*delta.*dshapex(:,inod,1);
         P_supg(:,3,1) = rho_l*delta.*dshapex(:,inod,2);
      end      
      
      %   disp(['IPG = ' num2str(ipg) ' Nodo = ' num2str(inod)])
      %   keyboard;pause
      
      W_supg = P_supg;      
      for idf=1:ndf,
         W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
      end
      
      % W_i N_j term
      
      for jnod=1:nen,        
         % A_gN     
         tmpx = dshapex(:,jnod,1);
         tmpy = dshapex(:,jnod,2);
         for idf=1:ndf,
            for jdf=1:ndf,
               A_gN(:,idf,jdf,jnod) = tmpx.*Ax(:,idf,jdf) + tmpy.*Ay(:,idf,jdf) ;
            end 
         end    
      end
      
      for jnod=1:nen,
         
         % temporal terms
         if 1,
            W_Cp_N = rowprod(W_supg,Cp,ndf,ndf,ndf)*N(jnod) ;        
         else
            % DEBUG
            W_Cp_N = N(inod)*Cp*N(jnod) ;         
         end
         
         % N_A_gN or gN_A_N according to weak form
         if weak_form,
            N_A_gN = -A_gN(:,:,:,inod)*N(jnod);                 
         else          
            N_A_gN =  A_gN(:,:,:,jnod)*N(inod);              
         end
         
         
         P_A_gN = rowprod(P_supg,A_gN(:,:,:,jnod),ndf,ndf,ndf);
         
         % shock capturing term
         gN_D_gN = zeros(numel,ndf,ndf);        
         if (shocap > 0 & 0)        
            for idf=1:ndf,
               for idm=1:ndm
                  gN_D_gN(:,idf,idf) = gN_D_gN(:,idf,idf) + ...
                     dshapex(:,inod,idm).*delta.*dshapex(:,jnod,idm);
               end
            end
         end
         
         gN_K_gN = zeros(numel,ndf,ndf);        
         for idf=1:ndf,
            for jdf=1:ndf,
               gN_K_gN(:,idf,jdf) = ...
                  -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                  (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                  (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                  (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
            end             
         end
         
         % reactive terms
         
         if 1,
            W_C_N = rowprod(W_supg,C_1,ndf,ndf,ndf)*N(jnod) ;        
         else
            % DEBUG
            W_C_N = C_1*N(jnod)*N(inod) ;
         end    
         
         
         % assembling
         vaux =  N_A_gN + P_A_gN + gN_D_gN - gN_K_gN + W_C_N;
         
         
         for idf=1:ndf,
            for jdf=1:ndf,              
               Mele(:,inod,idf,jnod,jdf) = Mele(:,inod,idf,jnod,jdf) + ... 
                  xsj.*( (1-lumping_time_terms)*W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );           
            end
         end
         
      end % jnod
      
      % temporal term
      if 1,
         W_dUdt   = rowprod(W_supg,(H-H0),ndf,ndf,1);
      else
         % DEBUG
         W_dUdt   = N(inod)*(H-H0);
      end
      
      % advective fluxes
      if weak_form,            
         gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf)).*Fa(:,1:ndf,1) ... 
            -(dshapex(:,inod,2)*ones(1,ndf)).*Fa(:,1:ndf,2);
      else
         gN_Fa = (N(inod)*ones(numel,ndf)).* AgU(:,1:ndf);
      end
      
      % diffusive fluxes
      gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1) ... 
         -(dshapex(:,inod,2)*ones(1,ndf)).*Fd(:,1:ndf,2);
      
      % stabilization SUPG terms        
      P_A_gU   = rowprod(P_supg,AgU,ndf,ndf,1);
      
      gN_D_gU = zeros(ne,ndf);
      
      if(shocap>0 & 0)                
         for idf=1:ndf,
            for idm=1:ndm              
               gN_D_gU(:,idf) = gN_D_gU(:,idf) + ...
                  dshapex(:,inod,idm).*(rho_l*delta).*dUpg(:,idf,idm);
            end
         end
      end        
      
      % reactive and source terms        
      if 1,
         W_Fc_1   = rowprod(W_supg,Fc_1,ndf,ndf,1) ;
      else  % DEBUG
         W_Fc_1   = N(inod)*Fc_1; 
      end
      
      vaux =  gN_Fa + P_A_gU + gN_D_gU - gN_Fd + W_Fc_1;  
      
      for idf=1:ndf,
         Rele(:,inod,idf) = Rele(:,inod,idf) - ... 
            (vaux(:,idf) +(1-lumping_time_terms)*W_dUdt(:,idf) ./ dt ).* xsj ;
      end
      
   end % inod
   
   %   disp('bubbly_2d_ele (liquid) phase ');keyboard;pause   
   
end % ipg


if lumping_drag_forces | lumping_time_terms,
   
   for inod=1:nen,
      % reactive and source terms        
      job         = 2; % [0] en Cflux todas las 3 fuerzas (pressure, buoyancy y pressure)
      % [1] en Cflux solo las 3 fuerzas de pressure y buoyancy
      % [2] en Cflux solo la fuerza de drag
      job = 0;
      
      jnod           = (inod-1)*ndf+(1:ndf);
      Upg(:,(1:ndf)) = Ue(:,jnod);
      U0pg(:,(1:ndf)) = U0e(:,jnod);   
      alpha_g        = Uf_e(:,(inod-1)*ndf_fields+1);
      vel_g(:,1)     = Uf_e(:,(inod-1)*ndf_fields+2);
      vel_g(:,2)     = Uf_e(:,(inod-1)*ndf_fields+3);      
      
      alpha_g_old     = Uf_e_old(:,(inod-1)*ndf_fields+1);      
      vel_g_old(:,1)  = Uf_e_old(:,(inod-1)*ndf_fields+2);
      vel_g_old(:,2)  = Uf_e_old(:,(inod-1)*ndf_fields+3);      
      
      [H,Cp]     =  enthalpy_function_liq(Upg,alpha_g,prop);
      if flag_tempo>0,
         [H0,bas]   =  enthalpy_function_liq(U0pg,alpha_g_old,prop);
      else
         [H0,bas]   =  enthalpy_function_liq(U0pg,alpha_g,prop);      
      end
      
      for idm=1:2,
         Dmat_vg(:,idm)    = (vel_g(:,idm)-vel_g_old(:,idm))/dt + ...
            vel_g(:,1).*grad_vel_g(:,idm,1)+vel_g(:,2).*grad_vel_g(:,idm,2);
         Dmat_vl(:,idm)    = (Upg(:,idm+1)-U0pg(:,idm+1))/dt + ...
            Upg(:,2).*dUpg_0(:,idm+1,1)+Upg(:,3).*dUpg_0(:,idm+1,2);               
%            Upg(:,idm+1).*dUpg_0(:,idm+1,1)+Upg(:,idm+1).*dUpg_0(:,idm+1,2);                      
      end
      
      [Fc_2,C_2] = Cflux_liq(Upg,dUpg_0,alpha_g,factor_alpha_liq*grad_alpha_g_0,vel_g, ...
         Dmat_vg,Dmat_vl,prop,job);
      
      %disp('bubbly_2d_ele (liquid) phase ');keyboard;pause      
      
      for idf=1:ndf,
         for jdf=1:ndf,              
            Mele(:,inod,idf,inod,jdf)=Mele(:,inod,idf,inod,jdf)+...
               lumping_drag_forces*Vol(:,inod).*C_2(:,idf,jdf);               
            Mele(:,inod,idf,inod,jdf)=Mele(:,inod,idf,inod,jdf)+...
               lumping_time_terms*Vol(:,inod).*Cp(:,idf,jdf)./dt;           
         end
      end
      
      for idf=1:ndf,
         Rele(:,inod,idf) = Rele(:,inod,idf) - ...
            lumping_drag_forces*Vol(:,inod).*Fc_2(:,idf);
         Rele(:,inod,idf) = Rele(:,inod,idf) - ...
            lumping_time_terms*Vol(:,inod).*(H(:,idf)-H0(:,idf))./dt;      
      end
      
   end
end

% ------ Terminos de contorno
if weak_form,
   %[Mele,Rele] = bc_conv_bubbly_2d_liq(xnod,icone,Ue,ifaceb,Rele,Mele,prop,Uf_e(:,1:ndf_fields:nen*ndf_fields));   
   [Mele,Rele] = bc_conv_bubbly_2d_liq(xnod,icone,Ue,ifaceb,Rele,Mele,prop,extra_nodal_fields(:,1));      
end
