% ====================================================================
%
%            gas pipe flow with alpha_g fixed at input
%            1D case
%
% ====================================================================

% Reference infinity state of variables
p_ref      = 0;
vel_ref    = [1,0];
p_ini      = 0.01*p_ref;
vel_l_ini  = 1e-3*[1,1];

alpha_g_ref = 0.1;
vel_g_ref   = [0,0];

rho_g       = 1;
rho_l       = 1000;
gravity     = -9.8;

visco_g     = 1.0e-5;
visco_l     = 1.0e-3;

if 1,
    visco_l     = visco_l/rho_l;
    rho_l       = 1;
end

H = 0.4;
D = 0.1;
radius_nozzle = 0.1*D;
radius_nozzle = 10*D;

shocap                = 0;
weak_form             = 0; 
axisim                = 0;
comp_interphase_terms = 1;
drag_factor           = 0.5;
d_bubble              = 0.006;

Cou   =  1  ;
npg   =  4  ;
ndf   = 3;
tol   = 1e-6;

% ====================================================================

xnod     =[];
icone    =[];
Nx       =  21;    Lx =  1;
Ny       =  21;    Ly =  1;

%psi      = (0:Nx-1)'/(Nx-1);
%x        = psi*Lx;
x        = refh([1,4,1],Nx)*Lx;

%eta      = (0:Ny-1)'/(Ny-1)*1; 
%y        = eta*Ly;
y        = refh([1,4,1],Ny)*Ly;

%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
nodes_bot  = (1:Nx)';
nodes_top  = Nx*(Ny-1)+(1:Nx)';
nodes_lef  = (1:Nx:Nx*Ny)';
nodes_rig  = (Nx:Nx:Nx*Ny)';

nodes_bot_in = nodes_bot;
nodes_bot_wall = [];

fixa= zeros(numnp,ndf);
fixa(nodes_bot,[2,3]) = ones(length(nodes_bot),2);
fixa(nodes_top,[2,3]) = ones(length(nodes_top),2);
fixa(nodes_lef,[2,3]) = ones(length(nodes_lef),2);
fixa(nodes_rig,[2,3]) = ones(length(nodes_rig),2);
fixa(1,1)             = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
forc(nodes_top,2:3)= ones(length(nodes_top),1)*[vel_ref];

%--------- External fields -----------------------------
% void fraction and gas velocities 
% external_fields = [alpha_g , vel_gas]
y0 = min(xnod(:,2));
H  = max(xnod(:,2))-y0;
external_fields =  ones(numnp,1)*[alpha_g_ref,vel_g_ref];

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

%-------- Solucion inicial (PHI)
uini    = [p_ini,vel_l_ini];

if 1,
    U       =  ones(numnp,1)*uini;   
else
    Ampl   = (vel_g_ref(:,2)-vel_g_ini(:,2));
    sigma  = 0.2;
    sigma  = 0.05;
    xc     = 0;
    z      = xnod(:,2)-xc;
    U(:,3) = vel_g_ini(:,2)+Ampl*exp(-z.^2/sigma);
    
    Ampl   = alpha_g_ref-alpha_g_ini;
    sigma  = 0.2;
    sigma  = 0.05;
    xc     = 0;
    z      = xnod(:,2)-xc;
    U(:,1) = alpha_g_ini+Ampl*exp(-z.^2/sigma);
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;