function [Fc,C] = Cflux(Up,dUp,Dmat_vg,Dmat_vl,prop,job,Up_old,dUp_old)
%
%   [Fc,C] = Cflux(Up,alpha_g,grad_alpha_g,vel_g,prop,job,Up_old,dUp_old)
%
%  job = 0 ; los 3 terminos (pressure, bouyancy y drag)
%  job = 1 ; los terminos de pressure y bouyancy
%  job = 2 ; solo el termino de drag
%

if nargin <7,
   Up_old = Up;
   dUp_old = dUp;
end

ijob = 'get';
handle_prop_elem;

[N,ndf]  = size(Up);
ndm = (ndf-2)/2;

if ndm==1,
   indx_l = [1,3];
   indx_g = [2,4];   
elseif ndm==2,
   indx_l = [1,3,4];
   indx_g = [2,5,6];
end

alpha_g     = Up(:,indx_g(1));
vel_g       = Up(:,indx_g(2:ndm+1));
pre         = Up(:,indx_l(1));
vel_l       = Up(:,indx_l(2:ndm+1));
alpha_l     = 1-alpha_g;


pre_old      = Up_old(:,indx_l(1));

grad_alpha_g = dUp(:,indx_g(1),1:ndm);
grad_alpha_l = -grad_alpha_g;
grad_v_l     = dUp(:,indx_l(2:ndm+1),1:ndm);
grad_v_g     = dUp(:,indx_g(2:ndm+1),1:ndm);
grad_p       = dUp(:,indx_l(1),1:ndm);

grad_alpha_g_old = dUp_old(:,indx_g(1),1:ndm);
grad_alpha_l_old = -grad_alpha_g_old;

qpot = qpot_drag_model;

Fc           = zeros(N,ndf);
C            = zeros(N,ndf,ndf);

field = [1,1,1];

if job==1, field(3)=0; end
if job==2, field(1:2)=zeros(2,1); end
if field(1),
   % reactive fluxes due to bouyancy forces
   Fc(:,indx_l(ndm+1))  = -rho_l*gravity*alpha_l;
   Fc(:,indx_g(ndm+1))  = -rho_g*gravity*alpha_g;    
end

grad_p = reshape(grad_p,N,ndm);

if field(2)&0,
   Fc(:,2)     = Fc(:,2)-alpha_l.*grad_p(:,1);
   Fc(:,3)     = Fc(:,3)-alpha_l.*grad_p(:,2);
   Fc(:,5)     = Fc(:,5)-alpha_g.*grad_p(:,1);
   Fc(:,6)     = Fc(:,6)-alpha_g.*grad_p(:,2);    
   C(:,5,2)    = C(:,5,2)-grad_p(:,1);
   C(:,6,2)    = C(:,6,2)-grad_p(:,2);        
end

if field(3),
   if comp_interphase_terms,   
      % reactive and source terms due to drag interaction forces
      tol      = 1e-06;
      v_rel       = vel_l - vel_g;
      v_slip      = sqrt(sum(v_rel'.^2))';
      Re          = rho_l*d_bubble*v_slip/visco_l;
      mask        = Re>tol;
      nn          = find(mask==1);
      
      if isempty(nn)==0,
         
         if fRe_drag_model==0,
            f_Re        = 24.0 ./ Re(nn);
            df_RedRe    = -24.0 ./Re(nn) ./Re(nn);
         elseif fRe_drag_model==1,
            f_Re        = 0.44;
            df_RedRe    = 0;     
         elseif fRe_drag_model==2,                
            tmp0_drag   = 1.0+0.15*Re(nn).^(0.687);
            f_Re        = tmp0_drag.*(24.0 ./ Re(nn));
            df_RedRe    = -24.0 ./Re(nn) ./Re(nn) .* tmp0_drag + (24.0 ./ Re(nn)) .* (0.15*0.687*(Re(nn).^(0.687-1.0))); 
         end    
         
         C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
         C_drag      = C1_drag*Re(nn).*f_Re;  
         f_alpha     = alpha_g(nn).*(1-alpha_g(nn)).^qpot;
         dRedU       = rho_l*d_bubble/visco_l;            
         tmp4_drag   = f_Re+Re(nn).*df_RedRe;            
         tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);
         
         for idm=1:ndm
            Fc(nn,indx_l(idm+1)) = Fc(nn,indx_l(idm+1))+C_drag.*f_alpha.*(v_rel(nn,idm));               
            Fc(nn,indx_g(idm+1)) = Fc(nn,indx_g(idm+1))-C_drag.*f_alpha.*(v_rel(nn,idm));               
         end
         
         %            Fc(nn,indx_l(2)) = Fc(nn,indx_l(2))+C_drag.*f_alpha.*(v_rel(nn,1));
         %            Fc(nn,indx_l(3)) = Fc(nn,indx_l(3))+C_drag.*f_alpha.*(v_rel(nn,2));            
         %            Fc(nn,indx_g(2)) = Fc(nn,indx_g(2))-C_drag.*f_alpha.*(v_rel(nn,1));
         %            Fc(nn,indx_g(3)) = Fc(nn,indx_g(3))-C_drag.*f_alpha.*(v_rel(nn,2));
         
         for idm=1:ndm,
            for jdm=1:ndm,
               Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
            end
            %                Phi(:,idm,idm) = Phi(:,idm,idm)+C_drag.*f_alpha;
            Phi(:,idm,idm) = Phi(:,idm,idm)+f_Re.*Re(nn);
         end
         
         for idm=1:ndm
            for jdm=1:ndm
               tmp(:,idm,jdm) =  C1_drag.*f_alpha.*Phi(:,idm,jdm);
            end
         end
         %            tmp(:,1,1) =  C1_drag.*f_alpha.*Phi(:,1,1);
         %            tmp(:,1,2) =  C1_drag.*f_alpha.*Phi(:,1,2);
         %            tmp(:,2,1) =  C1_drag.*f_alpha.*Phi(:,2,1);
         %            tmp(:,2,2) =  C1_drag.*f_alpha.*Phi(:,2,2);
         
         for idm=1:ndm
            for jdm=1:ndm
               C(nn,indx_l(idm+1),indx_l(jdm+1)) = C(nn,indx_l(idm+1),indx_l(jdm+1)) + tmp(:,idm,jdm);
               C(nn,indx_l(idm+1),indx_g(jdm+1)) = C(nn,indx_l(idm+1),indx_g(jdm+1)) - tmp(:,idm,jdm);
               C(nn,indx_g(idm+1),indx_g(jdm+1)) = C(nn,indx_g(idm+1),indx_g(jdm+1)) + tmp(:,idm,jdm);
               C(nn,indx_g(idm+1),indx_l(jdm+1)) = C(nn,indx_g(idm+1),indx_l(jdm+1)) - tmp(:,idm,jdm);               
            end
         end
         
         if 0,
            C(nn,indx_l(2),indx_l(2)) = C(nn,indx_l(2),indx_l(2)) + tmp(:,1,1);
            C(nn,indx_l(2),indx_l(3)) = C(nn,indx_l(2),indx_l(3)) + tmp(:,1,2);
            C(nn,indx_l(3),indx_l(2)) = C(nn,indx_l(3),indx_l(2)) + tmp(:,2,1);
            C(nn,indx_l(3),indx_l(3)) = C(nn,indx_l(3),indx_l(3)) + tmp(:,2,2);
            
            C(nn,indx_l(2),indx_g(2)) = C(nn,indx_l(2),indx_g(2)) - tmp(:,1,1);
            C(nn,indx_l(2),indx_g(3)) = C(nn,indx_l(2),indx_g(3)) - tmp(:,1,2);
            C(nn,indx_l(3),indx_g(2)) = C(nn,indx_l(3),indx_g(2)) - tmp(:,2,1);
            C(nn,indx_l(3),indx_g(3)) = C(nn,indx_l(3),indx_g(3)) - tmp(:,2,2);
            
            C(nn,indx_g(2),indx_g(2)) = C(nn,indx_g(2),indx_g(2)) + tmp(:,1,1);
            C(nn,indx_g(2),indx_g(3)) = C(nn,indx_g(2),indx_g(3)) + tmp(:,1,2);
            C(nn,indx_g(3),indx_g(2)) = C(nn,indx_g(3),indx_g(2)) + tmp(:,2,1);
            C(nn,indx_g(3),indx_g(3)) = C(nn,indx_g(3),indx_g(3)) + tmp(:,2,2);
            
            C(nn,indx_g(2),indx_l(3)) = C(nn,indx_g(2),indx_l(3)) - tmp(:,1,2);
            C(nn,indx_g(3),indx_l(2)) = C(nn,indx_g(3),indx_l(2)) - tmp(:,2,1);
            C(nn,indx_g(3),indx_l(3)) = C(nn,indx_g(3),indx_l(3)) - tmp(:,2,2);
            
         end
         
         
      end
      %disp('Cflux liq ');keyboard;pause
   end
end

if field(3),
   if virtual_mass,
      if stadtke_method==1,
         
         if 0,  % esto se calcula en Cp (enthalpy) y en A (flux_function)
            rho = rho_l*alpha_l+rho_g*alpha_g;
            factor = C_vm*rho.*alpha_g.*alpha_l;
            
            % LIQUID            
            Fc(:,indx_l(2:3))        =  Fc(:,indx_l(2:3))        - factor*[1,1].*(Dmat_vg-Dmat_vl);
            C(:,indx_l(2),indx_l(2)) =  C(:,indx_l(2),indx_l(2)) + factor.*(1/dt*ones(N,1)-grad_v_g(:,1,1));
            C(:,indx_l(2),indx_l(3)) =  C(:,indx_l(2),indx_l(3)) + factor.*(-grad_v_g(:,1,2));
            C(:,indx_l(3),indx_l(2)) =  C(:,indx_l(3),indx_l(2)) + factor.*(-grad_v_g(:,2,1));    
            C(:,indx_l(3),indx_l(3)) =  C(:,indx_l(3),indx_l(3)) + factor.*(1/dt*ones(N,1)-grad_v_g(:,2,2));    
            
            C(:,indx_l(2),indx_g(2)) =  C(:,indx_l(2),indx_g(2)) + factor.*(-1/dt*ones(N,1)+grad_v_l(:,1,1));
            C(:,indx_l(2),indx_g(3)) =  C(:,indx_l(2),indx_g(3)) + factor.*(grad_v_g(:,1,2));
            C(:,indx_l(3),indx_g(2)) =  C(:,indx_l(3),indx_g(2)) + factor.*(grad_v_g(:,2,1));    
            C(:,indx_l(3),indx_g(3)) =  C(:,indx_l(3),indx_g(3)) + factor.*(-1/dt*ones(N,1)+grad_v_l(:,2,2));    
            
            % GAS
            Fc(:,indx_g(2:3))        =  Fc(:,indx_g(2:3))        + factor*[1,1].*(Dmat_vg-Dmat_vl);
            C(:,indx_g(2),indx_g(2)) =  C(:,indx_g(2),indx_g(2)) + factor.*(1/dt*ones(N,1)-grad_v_l(:,1,1));
            C(:,indx_g(2),indx_g(3)) =  C(:,indx_g(2),indx_g(3)) + factor.*(-grad_v_l(:,1,2));
            C(:,indx_g(3),indx_g(2)) =  C(:,indx_g(3),indx_g(2)) + factor.*(-grad_v_l(:,2,1));    
            C(:,indx_g(3),indx_g(3)) =  C(:,indx_g(3),indx_g(3)) + factor.*(1/dt*ones(N,1)-grad_v_l(:,2,2));    
            
            C(:,indx_g(2),indx_l(2)) =  C(:,indx_g(2),indx_l(2)) + factor.*(-1/dt*ones(N,1)+grad_v_g(:,1,1));
            C(:,indx_g(2),indx_l(3)) =  C(:,indx_g(2),indx_l(3)) + factor.*(grad_v_g(:,1,2));
            C(:,indx_g(3),indx_l(2)) =  C(:,indx_g(3),indx_l(2)) + factor.*(grad_v_g(:,2,1));    
            C(:,indx_g(3),indx_l(3)) =  C(:,indx_g(3),indx_l(3)) + factor.*(-1/dt*ones(N,1)+grad_v_g(:,2,2));    
         end
         
         
      else
         if 1,
         % LIQUID
         for idm=1:ndm
            Fc(:,indx_l(idm+1)) = Fc(:,indx_l(idm+1)) - C_vm*rho_l*(alpha_g.*Dmat_vg(:,idm)-Dmat_vl(:,idm)); 
            Fc(:,indx_g(idm+1)) = Fc(:,indx_g(idm+1)) - C_vm*rho_l*alpha_g.*Dmat_vl(:,idm); 
            C(:,indx_l(idm+1),indx_l(idm+1)) =  C(:,indx_l(idm+1),indx_l(idm+1)) + C_vm*rho_l*(1/dt*ones(N,1)); 
            C(:,indx_l(idm+1),indx_g(idm+1)) =  C(:,indx_l(idm+1),indx_g(idm+1)) - C_vm*rho_l*alpha_g.*(1/dt*ones(N,1));
            C(:,indx_g(idm+1),indx_l(idm+1)) =  C(:,indx_g(idm+1),indx_l(idm+1)) - C_vm*rho_l*alpha_g.*(1/dt*ones(N,1));            
            for jdm=1:ndm
               C(:,indx_l(idm+1),indx_l(jdm+1)) =  C(:,indx_l(idm+1),indx_l(jdm+1)) + C_vm*rho_l*(grad_v_l(:,idm,jdm));
               C(:,indx_l(idm+1),indx_g(jdm+1)) =  C(:,indx_l(idm+1),indx_g(jdm+1)) - C_vm*rho_l*alpha_g.*(grad_v_l(:,idm,jdm));
               C(:,indx_g(idm+1),indx_l(jdm+1)) =  C(:,indx_g(idm+1),indx_l(jdm+1)) - C_vm*rho_l*alpha_g.*(grad_v_l(:,idm,jdm));               
            end            
         end
         
         else
            Fc(:,indx_l(2:3))        =  Fc(:,indx_l(2:3))        - C_vm*rho_l*((alpha_g*[1,1]).*Dmat_vg-Dmat_vl);
            
            C(:,indx_l(2),indx_l(2)) =  C(:,indx_l(2),indx_l(2)) + C_vm*rho_l*(1/dt*ones(N,1)+grad_v_l(:,1,1));
            C(:,indx_l(2),indx_l(3)) =  C(:,indx_l(2),indx_l(3)) + C_vm*rho_l*(grad_v_l(:,1,2));
            C(:,indx_l(3),indx_l(2)) =  C(:,indx_l(3),indx_l(2)) + C_vm*rho_l*(grad_v_l(:,2,1));    
            C(:,indx_l(3),indx_l(3)) =  C(:,indx_l(3),indx_l(3)) + C_vm*rho_l*(1/dt*ones(N,1)+grad_v_l(:,2,2));    
            
            C(:,indx_l(2),indx_g(2)) =  C(:,indx_l(2),indx_g(2)) - C_vm*rho_l*alpha_g.*(1/dt*ones(N,1)+grad_v_l(:,1,1));
            C(:,indx_l(2),indx_g(3)) =  C(:,indx_l(2),indx_g(3)) - C_vm*rho_l*alpha_g.*(grad_v_l(:,1,2));
            C(:,indx_l(3),indx_g(2)) =  C(:,indx_l(3),indx_g(2)) - C_vm*rho_l*alpha_g.*(grad_v_l(:,2,1));    
            C(:,indx_l(3),indx_g(3)) =  C(:,indx_l(3),indx_g(3)) - C_vm*rho_l*alpha_g.*(1/dt*ones(N,1)+grad_v_l(:,2,2));    
            
            % GAS
            Fc(:,indx_g(2:3))        = Fc(:,indx_g(2:3))         - C_vm*rho_l*(alpha_g*[1,1]).*Dmat_vl;                
            
            C(:,indx_g(2),indx_l(2)) =  C(:,indx_g(2),indx_l(2)) - C_vm*rho_l*alpha_g.*(1/dt*ones(N,1)+grad_v_l(:,1,1));
            C(:,indx_g(2),indx_l(3)) =  C(:,indx_g(2),indx_l(3)) - C_vm*rho_l*alpha_g.*(grad_v_l(:,1,2));
            C(:,indx_g(3),indx_l(2)) =  C(:,indx_g(3),indx_l(2)) - C_vm*rho_l*alpha_g.*(grad_v_l(:,2,1));    
            C(:,indx_g(3),indx_l(3)) =  C(:,indx_g(3),indx_l(3)) - C_vm*rho_l*alpha_g.*(1/dt*ones(N,1)+grad_v_l(:,2,2));    
            
         end         
      end      
   end   
end


%disp('Cflux');keyboard;pause
