function [MCp] = PMM_function(Up,prop,Cp)

%-------------------------------------------------------------------%
% Aplication: TWO PHASE FLOW                                        %
%-------------------------------------------------------------------%
% Funcion :                                                         %
%          function [MCp] = PMM_function(Up,prop,Cp)                %
%-------------------------------------------------------------------%
% Parametros:                                                       %
%            Up: vector de estado.                                  %
%            prop: vector de propiedades.                           %
%            Cp: matriz temporal                                    %
%-------------------------------------------------------------------%
% Descripcion:                                                      %
%             Calcula el producto de la matriz preco. M por la      %
%             matriz temporal Cp                                    %
%-------------------------------------------------------------------%
% Subrutinas:                                                       %
%             handle_prop_elem.                                     %
%-------------------------------------------------------------------%


%---------------------< Lista de propiedades  >---------------------%
ijob = 'get';
handle_prop_elem;
limit=1e-10; % Esta por verse no definitivo.
beta1=1;     % Esta por verse no definitivo. 
[N,ndf]  = size(Up);
ndm = (ndf-2)/2;

%---------------------<       Indices        >---------------------%
if ndm==1,
    indx_l = [1,3];
    indx_g = [2,4];   
elseif ndm==2,
    indx_l = [1,3,4];
    indx_g = [2,5,6];
end

%---------------------< Variables dependientes >--------------------%
alpha_g_ctf=0.01;
alpha_g     = Up(:,indx_g(1));
alpha_g     = max([alpha_g_ctf*ones(1,N);alpha_g'])'; % alpha_g_ctf --> es global?
alpha_g     = min([(1-alpha_g_ctf)*ones(1,N);alpha_g'])';
u_g       = Up(:,indx_g(2:ndm+1));
pre         = Up(:,indx_l(1));
u_l       = Up(:,indx_l(2:ndm+1));
alpha_l     = 1-alpha_g;

%---------------< Relacion entre var. dependientes >----------------%
% Relacion de alpha
A=alpha_l./alpha_g;
% Relacion de densidades
B=rho_l./rho_g;
% Relacion de velocidades
% Pensar una mejor forma calcular el limite par beta
C=u_l./u_g;

% 
%-----------------------< Calculo de Beta >-------------------------%
beta=ones(N,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculo de beta_2 para valores de C menores a 0.2 y mayores a 2.4 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
index=[];
index1=[];
% Beta para todas las fracciones de vacio
index=find(C<0.2);
index1=find(C>2.4);
index=[index;index1];
beta(index,1)=[ -1/2*(rho_l*(u_l(index,1).^2).*alpha_g(index,1)-2*rho_l*u_l(index,1).*(alpha_g(index,1).*u_g(index,1))+rho_g*u_g(index,1).^2-rho_g*(u_g(index,1).^2).*alpha_g(index,1)+ ...
        ((-rho_g*(u_g(index,1).^2).*alpha_g(index,1)+rho_g*u_g(index,1).^2+rho_l*(u_l(index,1).^2).*alpha_g(index,1)-4*rho_l*u_l(index,1).*(alpha_g(index,1).*u_g(index,1))+4*rho_l*(u_g(index,1).^2).*alpha_g(index,1)).* ...
        (rho_g*u_g(index,1).^2-rho_g*(u_g(index,1).^2).*alpha_g(index,1)+rho_l*(u_l(index,1).^2).*alpha_g(index,1))).^(1/2))/rho_g./(u_g(index,1).^2)./(-1+alpha_g(index,1))];

% Beta para alpha tendiendo a 0;
index_aux=find(alpha_g(index,1)==0);
index_1=index(index_aux);
beta(index_1,1)=ones(length(index_1),1);

% Beta para alpha tendiendo a 1;
index_aux=find(alpha_g(index,1)==1)
index_2=index(index_aux);
beta(index_2,1)=[ 1/2*(99*rho_l*u_l(index_2,1).^2+rho_g*u_g(index_2,1).^2-198*u_l(index_2,1)*rho_l.*u_g(index_2,1)+(9801*rho_l^2*u_l(index_2,1).^4-(396*u_l(index_2,1)*rho_l).*u_g(index_2,1).^3*rho_g+ ...
        (198*rho_l*u_l(index_2,1).^2*rho_g).*u_g(index_2,1).^2-(39204*rho_l^2*u_l(index_2,1).^3).*u_g(index_2,1)+rho_g^2*u_g(index_2,1).^4+396*rho_g*u_g(index_2,1).^4*rho_l+(39204*rho_l^2*u_l(index_2,1).^2).*u_g(index_2,1).^2).^(1/2))/rho_g./u_g(index_2,1).^2];



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculo de beta_1 para valores de C menores a 1.8 y mayores a 0.4 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Beta para todas las fracciones de vacio
index=find(C<1.8);
index1=find(C(index)>0.4);
index=index(index1);
beta(index,1)=[ -1/2*(rho_l*(u_l(index,1).^2).*alpha_g(index,1)-2*rho_l*u_l(index,1).*(alpha_g(index,1).*u_g(index,1))+rho_g*u_g(index,1).^2-rho_g*(u_g(index,1).^2).*alpha_g(index,1)- ...
        ((-rho_g*(u_g(index,1).^2).*alpha_g(index,1)+rho_g*u_g(index,1).^2+rho_l*(u_l(index,1).^2).*alpha_g(index,1)-4*rho_l*u_l(index,1).*(alpha_g(index,1).*u_g(index,1))+4*rho_l*(u_g(index,1).^2).*alpha_g(index,1)).* ...
        (rho_g*u_g(index,1).^2-rho_g*(u_g(index,1).^2).*alpha_g(index,1)+rho_l*(u_l(index,1).^2).*alpha_g(index,1))).^(1/2))/rho_g./(u_g(index,1).^2)./(-1+alpha_g(index,1))]

% Beta para alpha tendiendo a 0;
index_aux=find(alpha_g(index,1)==0)
index_1=index(index_aux);
beta(index_1,1)=zeros(length(index_1),1);

% Beta para alpha tendiendo a 1;
index_aux=find(alpha_g(index,1)==1)
index_2=index(index_aux);
beta(index_2,1)=[ 1/2*(99*rho_l*u_l(index_2,1).^2+rho_g*u_g(index_2,1).^2-198*u_l(index_2,1)*rho_l.*u_g(index_2,1)-(9801*rho_l^2*u_l(index_2,1).^4-(396*u_l(index_2,1)*rho_l).*u_g(index_2,1).^3*rho_g+ ...
        (198*rho_l*u_l(index_2,1).^2*rho_g).*u_g(index_2,1).^2-(39204*rho_l^2*u_l(index_2,1).^3).*u_g(index_2,1)+rho_g^2*u_g(index_2,1).^4+396*rho_g*u_g(index_2,1).^4*rho_l+(39204*rho_l^2*u_l(index_2,1).^2).*u_g(index_2,1).^2).^(1/2))/rho_g./u_g(index_2,1).^2]


% Punteros a casos singulares

% Puntero a velocidad del gas igual a cero.
p_vel_g0=find(u_g==0);
if length(p_vel_g0)>1,
    flag_vel_g0=1;
else
    flag_vel_g0=0;
end

if flag_vel_g0==1,
    index=p_vel_g0;
    beta(index,1)=[ -1/2*(100000000*rho_l*(u_l(index,1).^2).*alpha_g(index,1)-20000*(u_l(index,1)*rho_l).*alpha_g(index,1)+rho_g-rho_g*alpha_g(index,1)+ ...
            ((-rho_g*alpha_g(index,1)+rho_g+4*rho_l*alpha_g(index,1)-40000*(u_l(index,1)*rho_l).*alpha_g(index,1)+100000000*rho_l*(u_l(index,1).^2).*alpha_g(index,1)).*(-rho_g*alpha_g(index,1)+ ...
            rho_g+100000000*rho_l*(u_l(index,1).^2).*alpha_g(index,1))).^(1/2))/rho_g./(-1+alpha_g(index,1))];
    % Punteros Alpha igual a 0 
    p_vel_g0_a0=find(alpha_g(p_vel_g0)==0);
    if length(p_vel_g0_a0)>1,
        beta(p_vel_g0(p_vel_g0_a0),1)=ones(length(p_vel_g0_a0),1);
    end
    % Punteros Alpha igual a 1
    p_vel_g0_a1=find(alpha_g(p_vel_g0)==1)
    if length(p_vel_g0_a1)>1,
        index=p_vel_g0(p_vel_g0_a1);
        beta(index,1)=[ 1/2*(9900000000*rho_l*u_l(index,1).^2+rho_g-1980000*u_l(index,1).*rho_l+((rho_g+396*rho_l- ...
                3960000*u_l(index,1)*rho_l+9900000000*rho_l*u_l(index,1).^2).*(rho_g+9900000000*rho_l*u_l(index,1).^2)).^(1/2))./rho_g];
    end
end

%---------------------<      Matriz Cp        >---------------------%
MCp = zeros(N,ndf,ndf);
gamma=ones(N,1);
gamma=1;

% Producto de 1 fila por beta
MCp(:,indx_g(1),indx_g(1)) = gamma*beta.*Cp(:,indx_g(1),indx_g(1));
% Producto por gamma
MCp(:,indx_l(1),indx_g(1))= gamma.*Cp(:,indx_l(1),indx_g(1));
for idm=1:ndm
    MCp(:,indx_l(idm+1),indx_g(1)) = gamma.*Cp(:,indx_l(idm+1),indx_g(1));
    MCp(:,indx_g(idm+1),indx_g(1)) = gamma.*Cp(:,indx_g(idm+1),indx_g(1));         
    MCp(:,indx_l(idm+1),indx_l(idm+1)) = gamma.*Cp(:,indx_l(idm+1),indx_l(idm+1));
    MCp(:,indx_g(idm+1),indx_g(idm+1)) = gamma.*Cp(:,indx_g(idm+1),indx_g(idm+1));
end