%-------------------------------------------------------------------%
% Aplication: TWO PHASE FLOW                                        %
%-------------------------------------------------------------------%
% Rutina:                                                           %
%             bubbly_2d_ele                                         %
%-------------------------------------------------------------------%
% Descripcion:                                                      %
%             Calculo del matriz jacobiana y residuo del elem.      %
%-------------------------------------------------------------------%
% Subrutinas:                                                       %
%                                                                   %
%-------------------------------------------------------------------%

%-----------------< Determina inex segun fase >---------------------%
if ndm==1,
    indx_l = [1,3];
    indx_g = [2,4];   
elseif ndm==2,
    indx_l = [1,3,4];
    indx_g = [2,5,6];
end

% Numero de puntos nodales
N=numnp;

%-----------------<   cut-off void fraction   >---------------------% 
alpha_g_ctf  = 1e-3;
alpha_g      = U(:,indx_g(1));
alpha_g      = max([alpha_g_ctf*ones(1,N);alpha_g'])';
alpha_g      = min([(1-alpha_g_ctf)*ones(1,N);alpha_g'])';
alpha_l      = 1-alpha_g;

U(:,indx_g(1)) = alpha_g;

% Lumping sobre los terminos de Gas
use_lumping_for_gas  = lumping_time_terms;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     UPWIND EN EL CENTROIDE                        % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

type_upwind         = 0;      
tau_centroide       = 1;
debug               = 0;

%-------------------------------------------------------------------%

% Numero de elementos
ne  = numel;

[bas,ndf_fields] = size(extra_nodal_fields);

%---------------< Localizacion de las variables >-------------------% 

ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);
%Uf_e = zeros(ne,nen*ndf_fields);
%Uf_e_old = zeros(ne,nen*ndf_fields);
%Uenorm = zeros(ne,nen);

% Mapeo a formato [nodes;ndf*nen]=zise(Ue) idem U0e - Ref.(1)

for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
    Ue(:,jnod)  = U(nodes,1:ndf);		 
    U0e(:,jnod) = Uold(nodes,1:ndf);   
    %re(:,inod)  = xnod(nodes,1);		        
    %    Uf_e(:,knod)  = extra_nodal_fields(nodes,1:ndf_fields);
    %    Uf_e_old(:,knod)  = extra_nodal_fields_old(nodes,1:ndf_fields); 
    %    Uenorm(:,inod) = sqrt(sum(Uold(nodes,1:ndf)'.^2)');
end

%----------< Calculo del Tensor Metrico y Det(Jacobiano) >----------% 

if ndm==2,
    if nen==4,
        [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0,0);
        [dshapex0,N0]   = shape2d(X2S,0,0,nen); 
    elseif nen==3,
        [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),1/3,1/3);
        [dshapex0,N0]   = shape2d(X2S,1/3,1/3,nen); 
    else
        error(' Bad type of element , nen ~=3, nen ~=4 ')
    end
else
    [X2S,S2X,Jaco0] = xy2st(xnod,icone(eles,:),0);
    [dshapex0,N0]   = shape1d(X2S,0,nen);    
end



%--------< Interpolacion en los PG de v.dep. y derivadas  >----------%

Upg            = zeros(ne,ndf);
Upg_old        = zeros(ne,ndf);
dUpg           = zeros(ne,ndf,ndm);
dUpg_0         = zeros(ne,ndf,ndm);

for inod=1:nen,
    jnod = (inod-1)*ndf+(1:ndf);

%    Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N0(inod).*  Ue(:,jnod);
    Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N0(inod).*  U0e(:,jnod);   % MODIFICADO POR BETO
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                    Inicio_modif. Angel                  %
    
    Upg_old(:,(1:ndf))    =  Upg_old(:,(1:ndf)) + N0(inod).*  U0e(:,jnod);
    
    %                                                         %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    for idm=1:ndm,
        dUpg(:,(1:ndf),idm) = dUpg(:,(1:ndf),idm) + ...
            (dshapex0(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);          
        %   dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2)+ ...
        %      (dshapex0(:,inod,2)*ones(1,ndf)).* U0e(:,jnod);             
        dUpg_0(:,(1:ndf),idm) = dUpg_0(:,(1:ndf),idm) + ...
            (dshapex0(:,inod,idm)*ones(1,ndf)).* Ue(:,jnod);          
        %   dUpg_0(:,(1:ndf),2) = dUpg_0(:,(1:ndf),2)+ ...
        %      (dshapex0(:,inod,2)*ones(1,ndf)).* Ue(:,jnod);                
    end
end    

dUpg_0_old = dUpg;

if tau_centroide==1,
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Inicio_Modificacion: 01, Fecha:03/08/04, Modifico: Angel Zanotti  %

    if 1,
        [H,Cp]      =  enthalpy_function(Upg,prop,Upg_old);
        [Fa,Ax,Ay]  =  Flux_function(Upg,prop);        
    else
        [H,Cp]      =  enthalpy_function_prueba(Upg,Upg_old);
        [Fa,Ax,Ay]  =  Flux_function_prueba(Upg,prop);        
    end
    
    % Fin Modificacion                                                  %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % original: 01, Fecha:03/08/04, Modifico: Angel Zanotti             %
    
    %if 1,
    %    [H,Cp]      =  enthalpy_function(Upg,prop);
    %    [Fa,Ax,Ay]  =  Flux_function(Upg,prop);        
    %else
    %    [H,Cp]      =  enthalpy_function_prueba(Upg,prop);
    %    [Fa,Ax,Ay]  =  Flux_function_prueba(Upg,prop);        
    %end
    
    %                                                                   %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    velpg_l         = Upg(:,indx_l(2:ndm+1));
    velpg_g         = Upg(:,indx_g(2:ndm+1));
    
    if ndm==2, 
        hele_l      = comp_hele(velpg_l,dshapex0,Jaco0,nen);
        hele_g      = comp_hele(velpg_g,dshapex0,Jaco0,nen);          
    else
        hele_l      = diff(xnod);
        hele_g      = hele_l;
    end
    
    if strcmp(supg_method,'standard')==1,
        [tau_l,tau_g,delta_l,delta_g,bas] = supg_bubbly(Upg,0*dUpg,hele_l,hele_g,prop); 
    else
        %      [tau_l,tau_g,P_supg_l,P_supg_g] = comp_eig_stadtke_1d(Upg,prop,Cp,Ax,hele_l,hele_g);
        [tau_l,tau_g,P_supg_full,delta] = comp_eig_stadtke_1d(Upg,prop,Cp,Ax,hele_l,hele_g);
        delta_g = delta;      
        
    end
end

if 0,
    nnn=randperm(5);nnn=nnn(1);
    vaux_Cp2=reshape(Cp(nnn,:,:),6,6);
    vaux_Ax2=reshape(Ax(nnn,:,:),6,6);
    vaux_Ay2=reshape(Ay(nnn,:,:),6,6);
    vvv = eig(inv(vaux_Ay2)*vaux_Cp2);
    nn=find(abs(vvv)>1e-08);
    rho=rho_l*(1-Upg(:,2))+rho_g*Upg(:,2);
    V1=velpg_l(:,2).*(1-Upg(:,2))+velpg_g(:,2).*Upg(:,2);
    V2=(velpg_l(:,2).*(1-Upg(:,2))*rho_l+velpg_g(:,2).*Upg(:,2)*rho_g)./rho;
    tmp1 = C_vm*rho.^2/rho_l/rho_g;
    Ve = (V1+tmp1.*V2) ./(1+tmp1);
    disp(' Eigenvalues : ' )
    1 ./vvv(nn)
    [velpg_g(nnn,2);velpg_l(nnn,2);velpg_g(nnn,2)-velpg_l(nnn,2)]
    keyboard;pause
end

%keyboard;pause

Vol = zeros(ne,nen);

mask_Cp = fix(mask_matrix/1000);
mask_A = fix((mask_matrix-1000*mask_Cp)/100);
mask_C = fix((mask_matrix-1000*mask_Cp-100*mask_A)/10);
mask_D = fix((mask_matrix-1000*mask_Cp-100*mask_A-10*mask_C));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Loop sobre los puntos de Gauss                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ipg=1:npg
    %----------< Calculo del Tensor Metrico y Det(Jacobiano) >----------% 
    %---------------< Funcion de forma y su derivada >-----------------%
    if ndm==2
        [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
        [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
    else
        [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg));
        [dshapex,N] = shape1d(X2S,psipg(ipg),nen);    
    end
    
    %--------< Interpolacion en los PG de v.dep. y derivadas  >----------%
    
    Upg    = zeros(ne,ndf);
    U0pg    = zeros(ne,ndf);
    dUpg    = zeros(ne,ndf,ndm);
    dU0pg    = zeros(ne,ndf,ndm);   
    radpg    = zeros(ne,1);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);      
        for idm=1:ndm,
            dUpg(:,(1:ndf),idm)   = dUpg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* Ue(:,jnod);      
            dU0pg(:,(1:ndf),idm)   = dU0pg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);            
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Inicio_Modificacion: 02, Fecha:03/08/04, Modifico: Angel Zanotti  %

    if 1,
        [H,Cp]   =  enthalpy_function(Upg,prop,U0pg);
        [H0,bas]  =  enthalpy_function(U0pg,prop);      
        [Fa,Ax,Ay]     =  Flux_function(Upg,prop);        
    else
        [H,Cp]     =  enthalpy_function_prueba(Upg,prop,U0pg);
        [H0,bas]   =  enthalpy_function_prueba(U0pg,prop);      
        [Fa,Ax,Ay] =  Flux_function_prueba(Upg,prop);           
    end
    
    %                                                                   %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Inicio_Modificacion: 02, Fecha:03/08/04, Modifico: Angel Zanotti  %

    %if 1,
    %    [H,Cp]   =  enthalpy_function(Upg,prop);
    %    [H0,bas]  =  enthalpy_function(U0pg,prop);      
    %    [Fa,Ax,Ay]     =  Flux_function(Upg,prop);        
    %else
    %    [H,Cp]     =  enthalpy_function_prueba(Upg,prop);
    %    [H0,bas]   =  enthalpy_function_prueba(U0pg,prop);      
    %    [Fa,Ax,Ay] =  Flux_function_prueba(Upg,prop);           
    %end
    
    %                                                                   %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
    
    vel_g      = Upg(:,indx_g(2:ndm+1));
    vel_g_old  = U0pg(:,indx_g(2:ndm+1));       
    vel_l      = Upg(:,indx_l(2:ndm+1));
    vel_l_old  = U0pg(:,indx_l(2:ndm+1));       
    grad_vel_g = dUpg(:,indx_g(2:ndm+1),:);
    grad_vel_l = dUpg(:,indx_l(2:ndm+1),:);
    
    if stadtke_method==1,
        
        if 0,
            for idm=1:ndm,
                Dmat_vg(:,idm)    = (vel_g(:,idm)-vel_g_old(:,idm))/dt + ...
                    vel_l(:,1).*grad_vel_g(:,idm,1)+vel_l(:,2).*grad_vel_g(:,idm,2);
                Dmat_vl(:,idm)    = (vel_l(:,idm)-vel_l_old(:,idm))/dt + ...
                    vel_g(:,1).*grad_vel_l(:,idm,1)+vel_g(:,2).*grad_vel_l(:,idm,2);      
            end       
            
        else
            for idm=1:ndm,
                Dmat_vg(:,idm) = (vel_g(:,idm)-vel_g_old(:,idm))/dt;
                Dmat_vl(:,idm) = (vel_l(:,idm)-vel_l_old(:,idm))/dt;
                for jdm=1:ndm
                    Dmat_vg(:,idm) = Dmat_vg(:,idm) + ...
                        vel_l(:,jdm).*grad_vel_g(:,idm,jdm);
                    Dmat_vl(:,idm) = Dmat_vl(:,idm) + ...
                        vel_g(:,jdm).*grad_vel_l(:,idm,jdm);         
                end
            end       
            
        end
        
        
    else       
        
        if 0,
            for idm=1:2,
                Dmat_vg(:,idm)    = (vel_g(:,idm)-vel_g_old(:,idm))/dt + ...
                    vel_g(:,1).*grad_vel_g(:,idm,1)+vel_g(:,2).*grad_vel_g(:,idm,2);
                Dmat_vl(:,idm)    = (vel_l(:,idm)-vel_l_old(:,idm))/dt + ...
                    vel_l(:,1).*grad_vel_l(:,idm,1)+vel_l(:,2).*grad_vel_l(:,idm,2);      
            end
        else
            
            for idm=1:ndm,
                Dmat_vg(:,idm) = (vel_g(:,idm)-vel_g_old(:,idm))/dt;
                Dmat_vl(:,idm) = (vel_l(:,idm)-vel_l_old(:,idm))/dt;
                for jdm=1:ndm
                    Dmat_vg(:,idm) = Dmat_vg(:,idm) + ...
                        vel_g(:,jdm).*grad_vel_g(:,idm,jdm);
                    Dmat_vl(:,idm) = Dmat_vl(:,idm) + ...
                        vel_l(:,jdm).*grad_vel_l(:,idm,jdm);         
                end
            end       
            
        end
        
    end
    
    if lumping_drag_forces,
        job  = 1; % [0] en Cflux todas las 3 fuerzas (pressure, buoyancy y pressure)
        % [1] en Cflux solo las fuerzas de pressure y buoyancy
        % [2] en Cflux solo la fuerza de drag
        %job  = 2;
    else
        job = 0;
    end
    
    [Fc_1,C_1] = Cflux(Upg,dUpg,Dmat_vg,Dmat_vl,prop,job,U0pg,dU0pg);
    
    [Fd,Kxx,Kxy,Kyx,Kyy]  = Dflux(Upg,dUpg,prop);  
    
    % Correccion axisimetrica del jacobiano de la transformacion al elemento master
    %------------------------------------------------------------------
    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
    
    AgU = rowprod(Ax,reshape(dUpg(:,1:ndf,1),ne,ndf),ndf,ndf,1);
    if ndm==2, AgU = AgU + rowprod(Ay,reshape(dUpg(:,1:ndf,2),ne,ndf),ndf,ndf,1); end
    
    if tau_centroide==0,
        
        velpg_l        = Upg(:,indx_l(2:ndm+1));
        velpg_g        = Upg(:,indx_g(2:ndm+1));
        
        if ndm==2, 
            hele_l      = comp_hele(velpg_l,dshapex0,Jaco0,nen);
            hele_g      = comp_hele(velpg_g,dshapex0,Jaco0,nen);          
        else
            hele_l      = diff(xnod);
            hele_g      = hele_l;
        end
        
        if strcmp(supg_method,'standard')==1,
            [tau_l,tau_g,delta_l,delta_g,bas] = supg_bubbly(Upg,0*dU0pg,hele_l,hele_g,prop); 
        else
            %         [tau_l,tau_g,P_supg_l,P_supg_g] = comp_eig_stadtke_1d(U0pg,prop,Cp,Ax,hele_l,hele_g);         
            [tau_l,tau_g,P_supg_full,delta] = comp_eig_stadtke_1d(Upg,prop,Cp,Ax,hele_l,hele_g);                  
        end
        
    end
    
    % masking matrices for debugging
    
    if 0,
        Cp = mask_Cp*Cp;   
        Ax = mask_A*Ax;
        Ay = mask_A*Ay;   
        C_1 = mask_C*C_1;
        
        Kxx = mask_D*Kxx;
        Kxy = mask_D*Kxy;
        Kyx = mask_D*Kyx;
        Kyy = mask_D*Kyy;
    end
    
    % P_i Petrov-Galerkin perturbation weight function
    for inod=1:nen,   
        
        % Element Volume
        Vol(:,inod)=Vol(:,inod)+N(inod)*xsj;
        
        compute_P_supg;         
        
        
        W_supg = P_supg;      
        for idf=1:ndf,
            W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
        end
        
        % W_i N_j term      
        for jnod=1:nen,        
            % A_gN     
            tmpx = dshapex(:,jnod,1);
            if ndm==2, tmpy = dshapex(:,jnod,2); end
            for idf=1:ndf,
                for jdf=1:ndf,
                    if ndm==2, 
                        A_gN(:,idf,jdf,jnod) = tmpx.*Ax(:,idf,jdf) + tmpy.*Ay(:,idf,jdf) ;
                    else
                        A_gN(:,idf,jdf,jnod) = tmpx.*Ax(:,idf,jdf) ;                  
                    end               
                end 
            end    
        end
        
        for jnod=1:nen,
            
            % temporal terms
            if 1,
                W_Cp_N = rowprod(W_supg,Cp,ndf,ndf,ndf)*N(jnod) ;        
            else
                % DEBUG
                W_Cp_N = N(inod)*Cp*N(jnod) ;         
            end
            
            % N_A_gN or gN_A_N according to weak form
            if weak_form,
                N_A_gN = -A_gN(:,:,:,inod)*N(jnod);                 
            else          
                N_A_gN =  A_gN(:,:,:,jnod)*N(inod);              
            end
            
            
            P_A_gN = rowprod(P_supg,A_gN(:,:,:,jnod),ndf,ndf,ndf);
            
            % shock capturing term
            gN_D_gN = zeros(numel,ndf,ndf);        
            if ((shocap>0))%&(mask_D==1)) 
                % GAS
                for idf=1:ndm+1,
                    for idm=1:ndm
                        gN_D_gN(:,indx_g(idf),indx_g(idf)) = gN_D_gN(:,indx_g(idf),indx_g(idf)) + ...
                            dshapex(:,inod,idm).*(shocap*rho_g*delta_g).*dshapex(:,jnod,idm);
                    end
                end
            end
            
            gN_K_gN = zeros(numel,ndf,ndf);        
            for idf=1:ndf,
                for jdf=1:ndf,
                    if ndm==2,
                        gN_K_gN(:,idf,jdf) = ...
                            -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                            (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                            (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                            (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
                    else
                        gN_K_gN(:,idf,jdf) = ...
                            -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1)));               
                    end            
                end             
            end
            
            % reactive terms
            if 1,
                W_C_N = rowprod(W_supg,C_1,ndf,ndf,ndf)*N(jnod) ;        
            else
                % DEBUG
                W_C_N = C_1*N(jnod)*N(inod) ;
            end    
            
            % assembling
            vaux =  N_A_gN + P_A_gN + gN_D_gN - gN_K_gN + W_C_N;
            
            W_Cp_N_no_lumped = W_Cp_N;
            if lumping_time_terms==1,
                if use_lumping_for_gas==1,                 
                    W_Cp_N_no_lumped(:,indx_g,indx_g)=0;
                    W_Cp_N_no_lumped(:,indx_l,indx_g)=0;
                    %             keyboard;pause
                else
                    W_Cp_N_no_lumped = 0*W_Cp_N;                 
                end
            end
            
            for idf=1:ndf,
                for jdf=1:ndf,              
                    Mele(:,inod,idf,jnod,jdf) = Mele(:,inod,idf,jnod,jdf) + ... 
                        xsj.*( W_Cp_N_no_lumped(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );                              
                    %                  xsj.*( (1-lumping_time_terms)*W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );           
                end
            end
            
        end % jnod
        
        if 1,
            dH = rowprod(Cp,Upg-U0pg,ndf,ndf,1);
        else
            
            dH = H-H0;
        end
        
        %%% DEBUG
        % if inod==1 & ipg==1, keyboard;pause; end
        
        % temporal term
        if 1,
            W_dUdt   = rowprod(W_supg,dH,ndf,ndf,1);
            %         W_dUdt   = rowprod(W_supg,(H-H0),ndf,ndf,1);         
        else
            % DEBUG
            W_dUdt   = N(inod)*dH;         
            %         W_dUdt   = N(inod)*(H-H0);
        end
        
        % advective fluxes
        if weak_form,    
            if ndm==2,
                gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf)).*Fa(:,1:ndf,1) ... 
                    -(dshapex(:,inod,2)*ones(1,ndf)).*Fa(:,1:ndf,2);
            else
                gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf)).*Fa(:,1:ndf,1) ;         
            end
            
        else
            gN_Fa = (N(inod)*ones(numel,ndf)).* AgU(:,1:ndf);
        end
        
        % diffusive fluxes
        if ndm==2
            gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1) ... 
                -(dshapex(:,inod,2)*ones(1,ndf)).*Fd(:,1:ndf,2);
        else
            gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1);      
        end
        
        % stabilization SUPG terms        
        P_A_gU   = rowprod(P_supg,AgU,ndf,ndf,1);
        
        gN_D_gU = zeros(ne,ndf);
        
        if(shocap>0)                
            for idf=1:ndm+1,
                for idm=1:ndm              
                    gN_D_gU(:,indx_g(idf)) = gN_D_gU(:,indx_g(idf)) + ...
                        dshapex(:,inod,idm).*(shocap*rho_g*delta_g).*dUpg(:,indx_g(idf),idm);
                end
            end
        end        
        
        % reactive and source terms        
        if 1,
            W_Fc_1   = rowprod(W_supg,Fc_1,ndf,ndf,1) ;
        else  % DEBUG
            W_Fc_1   = N(inod)*Fc_1; 
        end
        
        vaux =  mask_A*(gN_Fa + P_A_gU + gN_D_gU) - mask_D*gN_Fd + mask_C*W_Fc_1;  
        
        W_dUdt_no_lumped = W_dUdt;
        if lumping_time_terms==1,
            if use_lumping_for_gas==1,
                W_dUdt_no_lumped(:,indx_g)=0;
            else
                W_dUdt_no_lumped = 0*W_dUdt;              
            end
            %             keyboard;pause
        end
        
        for idf=1:ndf,
            Rele(:,inod,idf) = Rele(:,inod,idf) - ... 
                (vaux(:,idf) + mask_Cp*W_dUdt_no_lumped(:,idf) ./ dt ).* xsj ;              
            %            (vaux(:,idf) +(1-lumping_time_terms)*W_dUdt(:,idf) ./ dt ).* xsj ;
        end
        
    end % inod
    
end % ipg


if lumping_drag_forces | lumping_time_terms,
    
    if ndm==1, error(' PLEASE --- CHECK validity of lumping in 1D '); end 
    
    for inod=1:nen,
        
        jnod            = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  = Ue(:,jnod);
        U0pg(:,(1:ndf)) = U0e(:,jnod);   
        
        %      [H,Cp]   =  enthalpy_function(Upg,prop,U0pg);
        [H,Cp]   =  enthalpy_function(Upg,prop);
        %      [H0,bas]  =  enthalpy_function(U0pg,prop);      
        [H0,Cp0]  =  enthalpy_function(U0pg,prop);            
        
        vel_g      = Upg(:,indx_g(2:3));
        vel_g_old  = U0pg(:,indx_g(2:3));       
        vel_l      = Upg(:,indx_l(2:3));
        vel_l_old  = U0pg(:,indx_l(2:3));       
        grad_vel_g = dUpg(:,indx_g(2:3),:);
        grad_vel_l = dUpg(:,indx_l(2:3),:);
        
        %        disp('bubbly_2d_ele ');keyboard;pause      
        if stadtke_method==1,
            for idm=1:2,
                Dmat_vg(:,idm)    = (vel_g(:,idm)-vel_g_old(:,idm))/dt + ...
                    vel_l(:,1).*grad_vel_g(:,idm,1)+vel_l(:,2).*grad_vel_g(:,idm,2);
                Dmat_vl(:,idm)    = (vel_l(:,idm)-vel_l_old(:,idm))/dt + ...
                    vel_g(:,1).*grad_vel_l(:,idm,1)+vel_g(:,2).*grad_vel_l(:,idm,2);      
            end       
        else                 
            for idm=1:2,
                Dmat_vg(:,idm)    = (vel_g(:,idm)-vel_g_old(:,idm))/dt + ...
                    vel_g(:,1).*grad_vel_g(:,idm,1)+vel_g(:,2).*grad_vel_g(:,idm,2);
                Dmat_vl(:,idm)    = (vel_l(:,idm)-vel_l_old(:,idm))/dt + ...
                    vel_l(:,1).*grad_vel_l(:,idm,1)+vel_l(:,2).*grad_vel_l(:,idm,2);      
            end
        end
        
        % reactive and source terms        
        % [0] en Cflux todas las 3 fuerzas (pressure, buoyancy y pressure)
        % [1] en Cflux solo las fuerzas de pressure y buoyancy
        % [2] en Cflux solo la fuerza de drag
        job = 2; 
        
        [Fc_2,C_2] = Cflux(Upg,dUpg_0,Dmat_vg,Dmat_vl,prop,job);
        
        if 0, %%%%% DEBUG
            Fc_2 = 0*Fc_2; C_2=0*C_2; 
        end
        
        
        Cp_lumped = Cp;
        if (lumping_time_terms==1),
            if (use_lumping_for_gas==1),
                Cp_lumped(:,indx_l,indx_l)=0;
                Cp_lumped(:,indx_g,indx_l)=0;
                %             keyboard;pause
            end
        else
            Cp_lumped = 0*Cp;
        end
        
        for idf=1:ndf,
            for jdf=1:ndf,              
                Mele(:,inod,idf,inod,jdf)=Mele(:,inod,idf,inod,jdf)+...
                    lumping_drag_forces*Vol(:,inod).*(mask_C*C_2(:,idf,jdf));               
                Mele(:,inod,idf,inod,jdf)=Mele(:,inod,idf,inod,jdf)+...
                    Vol(:,inod).*(mask_Cp*Cp_lumped(:,idf,jdf))./dt;                            
                %                  lumping_time_terms*Vol(:,inod).*(mask_Cp*Cp(:,idf,jdf))./dt;           
            end
        end
        
        %dH = rowprod(Cp,Upg-U0pg,ndf,ndf,1);
        dH = H-H0;
        % dH = rowprod(Cp,Upg,ndf,ndf,1)-rowprod(Cp0,U0pg,ndf,ndf,1);      
        
        dH_lumped = dH;
        if (lumping_time_terms==1),
            if (use_lumping_for_gas==1),
                dH_lumped(:,indx_l)=0;
            end
        else
            dH_lumped = 0*dH;          
        end
        
        for idf=1:ndf,
            Rele(:,inod,idf) = Rele(:,inod,idf) - ...
                lumping_drag_forces*Vol(:,inod).*(mask_C*Fc_2(:,idf));
            Rele(:,inod,idf) = Rele(:,inod,idf) - ...
                Vol(:,inod).*(mask_Cp*dH_lumped(:,idf))./dt;                  
            %            lumping_time_terms*Vol(:,inod).*(mask_Cp*dH(:,idf))./dt;      
        end
        
    end
end

%disp('bubbly_2d_ele phase ');keyboard;pause   

% ------ Terminos de contorno
if weak_form,
    disp(' weak form ')
    [Mele,Rele] = bc_conv_bubbly_2d(xnod,icone,Ue,ifaceb,Rele,Mele,prop);      
end
