function [H,Cp] = enthalpy_function(Up,prop,Up_old)

%-------------------------------------------------------------------%
% Aplication: TWO PHASE FLOW                                        %
%-------------------------------------------------------------------%
% Funcion :                                                         %
%          [H,Cp] = enthalpy_function(Up,prop,Up_old)               %
%-------------------------------------------------------------------%
% Parametros:                                                       %
%            Up: vector de estado new.                              %
%            prop: vector de propiedades                            %
%            Up_old: vector de estado old                           %
%-------------------------------------------------------------------%
% Descripcion:                                                      %
%             Calcula "H" y "Cp". permite manipular las             %
%             variables primitiva                                   %
%-------------------------------------------------------------------%
% Subrutinas:                                                       %
%             handle_prop_elem.                                     %
%-------------------------------------------------------------------%


%if nargin <3, Up_old=Up; end
%if nargin >2, flag=1; else flag=0; end
flag=0;

%---------------------< Lista de propiedades  >---------------------%
ijob = 'get';
handle_prop_elem;
limit=1e-10;
beta1=1;
[N,ndf]  = size(Up);
ndm = (ndf-2)/2;

% Inicio de mod.
if PMM_method==1,
          %flag=0;
end
% Fin de mod.

%---------------------<       Punteros        >---------------------%
if ndm==1,
    indx_l = [1,3];
    indx_g = [2,4];   
elseif ndm==2,
    indx_l = [1,3,4];
    indx_g = [2,5,6];
end


%---------------------< Variables dependientes >--------------------%
alpha_g     = Up(:,indx_g(1));
vel_g       = Up(:,indx_g(2:ndm+1));
pre         = Up(:,indx_l(1));
vel_l       = Up(:,indx_l(2:ndm+1));
alpha_l     = 1-alpha_g;


%---------------------<      Masa Virtual     >---------------------%
if virtual_mass,
    if stadtke_method==1,
        rho_g_eff = rho_g;
        rho_l_eff = rho_l;           
    else        
        rho_g_eff = rho_g + C_vm*rho_l;
        rho_l_eff = rho_l*(1-C_vm);   
    end
else
    rho_g_eff = rho_g;
    rho_l_eff = rho_l;   
end

%---------------------<      Matriz Cp        >---------------------%
Cp                           = zeros(N,ndf,ndf);
Cp(:,indx_l(1),indx_g(1))    = -rho_l*ones(N,1);
Cp(:,indx_g(1),indx_g(1))    =  rho_g*ones(N,1);

for idm=1:ndm
    Cp(:,indx_l(idm+1),indx_l(idm+1))    =  rho_l_eff*alpha_l;
    Cp(:,indx_g(idm+1),indx_g(idm+1))    =  rho_g_eff*alpha_g;   
end
%Cp(:,indx_l(2),indx_l(2))    =  rho_l_eff*alpha_l;
%Cp(:,indx_l(3),indx_l(3))    =  rho_l_eff*alpha_l;
%Cp(:,indx_g(2),indx_g(2))    =  rho_g_eff*alpha_g;
%Cp(:,indx_g(3),indx_g(3))    =  rho_g_eff*alpha_g;

%---------------------< Como Sistema Adv-Diff >---------------------%
if Advdif_scheme==1,   
    if 1,
        for idm=1:ndm
            Cp(:,indx_l(idm+1),indx_g(1))  = Cp(:,indx_l(idm+1),indx_g(1))-rho_l_eff*vel_l;
            Cp(:,indx_g(idm+1),indx_g(1))  = Cp(:,indx_g(idm+1),indx_g(1))+rho_g_eff*vel_g;         
        end      
    else   
        Cp(:,indx_l(2:3),indx_g(1))  = Cp(:,indx_l(2:3),indx_g(1))-rho_l_eff*vel_l;
        Cp(:,indx_g(2:3),indx_g(1))  = Cp(:,indx_g(2:3),indx_g(1))+rho_g_eff*vel_g;
    end
end

if stadtke_method & virtual_mass,
    rho    = rho_l*alpha_l+rho_g*alpha_g;
    factor = C_vm*rho.*alpha_g.*alpha_l;
    
    if 1,
        for idm=1:ndm
            Cp(:,indx_l(idm+1),indx_l(idm+1)) =  Cp(:,indx_l(idm+1),indx_l(idm+1)) + factor;
            Cp(:,indx_l(idm+1),indx_g(idm+1)) =  Cp(:,indx_l(idm+1),indx_g(idm+1)) - factor;      
            Cp(:,indx_g(idm+1),indx_g(idm+1)) =  Cp(:,indx_g(idm+1),indx_g(idm+1)) + factor;
            Cp(:,indx_g(idm+1),indx_l(idm+1)) =  Cp(:,indx_g(idm+1),indx_l(idm+1)) - factor;      
        end
        
    else
        
        Cp(:,indx_l(2),indx_l(2)) =  Cp(:,indx_l(2),indx_l(2)) + factor;
        Cp(:,indx_l(3),indx_l(3)) =  Cp(:,indx_l(3),indx_l(3)) + factor;    
        Cp(:,indx_l(2),indx_g(2)) =  Cp(:,indx_l(2),indx_g(2)) - factor;
        Cp(:,indx_l(3),indx_g(3)) =  Cp(:,indx_l(3),indx_g(3)) - factor;    
        
        Cp(:,indx_g(2),indx_g(2)) =  Cp(:,indx_g(2),indx_g(2)) + factor;
        Cp(:,indx_g(3),indx_g(3)) =  Cp(:,indx_g(3),indx_g(3)) + factor;    
        Cp(:,indx_g(2),indx_l(2)) =  Cp(:,indx_g(2),indx_l(2)) - factor;
        Cp(:,indx_g(3),indx_l(3)) =  Cp(:,indx_g(3),indx_l(3)) - factor;    
        
    end
end

%-------------------< Compresibilidad Artificial >-------------------%
if artif_compres
    Cp(:,indx_l(1),indx_l(1)) = Cp(:,indx_l(1),indx_l(1)) + alpha_l./s_sound_l.^2;
    Cp(:,indx_g(1),indx_l(1)) = Cp(:,indx_g(1),indx_l(1)) + alpha_g./s_sound_g.^2;      
end

if (liq_mass_eq_mod~=1),
    Cp(:,indx_l(1),indx_g(1))=liq_mass_eq_mod*Cp(:,indx_l(1),indx_g(1)); 
end

if flag==1,
    alpha_g     = Up_old(:,indx_g(1));
    vel_g       = Up_old(:,indx_g(2:ndm+1));
    pre         = Up_old(:,indx_l(1));
    vel_l       = Up_old(:,indx_l(2:ndm+1));
    
    alpha_g     = max([alpha_g_ctf*ones(1,N);alpha_g'])';
    alpha_g     = min([(1-alpha_g_ctf)*ones(1,N);alpha_g'])';
    alpha_l     = 1-alpha_g;   
end

H(:,indx_l)   = [rho_l*alpha_l,rho_l_eff*(alpha_l*ones(1,ndm)).*vel_l];
H(:,indx_g)   = [rho_g*alpha_g,rho_g_eff*(alpha_g*ones(1,ndm)).*vel_g];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inicio_Modificacion: 01, Fecha:03/08/04, Modifico: Angel Zanotti  %

%---------------------<  Calculo Matriz Preco >---------------------%

if nargin==3,
    if(ndm==1 & PMM_method==1),
        if (Advdif_scheme==1),
            [Cp] = PMM_function(Up_old,prop,Cp);  
        else
            disp('El PMM_method esta solamente implementado para sist. adv-diff ');
        end        
    elseif (PMM_method==1),
        disp('El PMM_method esta solamente implementado para sitemas 1D');
    end
end
if (nargin==2 & PMM_method==1),
    disp('No existe Vector de Estado "Old" para PMM_method');
    disp('SISTEMA SIN PRECONDICIONAR');
end

% Fin_Modificacion: 01, Fecha:03/08/04, Modifico: Angel Zanotti     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


