%-------------------------------------------------------------------%
% Aplication: TWO PHASE FLOW                                        %
%-------------------------------------------------------------------%
% Rutina:                                                           %
%             homo_1d_beta                                          %
%-------------------------------------------------------------------%
% Descripcion:                                                      %
%           1. Datos del problema.                                  %
%			2. Geometra.                                           %
%			3. Datos Fisicos.                                       %
%			4. Condiciones Iniciales.                               %
%			5. Escaleo --> [1] sqrt(rho_l), [2] rho_l.              %
%			6. Malla.                                               %
%			7. Determinacion estado inicial.                        %
%                          Tipos: a) Cte. p,vl,vg.                  %
%						          b) Onda sinusoidal de 2 periodos  %
%			8. Condiciones de contorno.                             %
%			9. Fijacion de Campos.                                  %
%           10. Manipulacion de las Prop. del Material.             %
%           11. Cargas Repartidas.                                  %
%           12. Condiciones Periodicas.                             %        
%			10.C.Iniciales en Primitivas./Aplica CC. a CI.          %
%                                                                   %
%-------------------------------------------------------------------%
% Subrutinas:                                                       %
%            handle_prop_elem                                       %
%-------------------------------------------------------------------%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       DATOS DEL PROBLEMA                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%---------------------<  Archivo de Estado >------------------------%
% read_from_file = 1 ==> Toma solucion inicial del archivo filestate%
% read_from_file = 0 ==> Toma solucion inicial de uini              %

read_from_file = 0;     
filestate = 'homo_1d_beta';

% mask_matrix = [mask_Cp , mask_A , mask_C , mask_D]
mask_matrix = 1111;

tol_adt_high = 3.5;
tol_adt_low  = 2.5;
tol_adt_min  = 1.5;

automatic_time_control = 0;

%---------------------<     Metodo PMM     >------------------------%
PMM_method = 0;

%-------------------<  Formulacion Adv-Diff  >----------------------%
Advdif_scheme = 1;

liq_mass_eq_mod = 0;        %%%%%%%???????????       
smoothed_alpha_freq = 0;    %%%%%%%???????????
stadtke_method = 0;         

%------------------< Tipo de Estab. Empleada >----------------------%
supg_method = 'finlandes';
supg_method = 'non_standard';
supg_method = 'standard';

%---------------------<  Comp. Artifiical  >------------------------%
artif_compres = 0;
s_sound_g = 1000;
s_sound_l = 1000;

%gp_reduced = 2;           %%%%%%%???????????

%-----------<  Define si el Probl. esta acoplado o no  >------------%
coupled    = 0;

%----------------<  Control de lazo de Newton  >--------------------%
nnwt_outer = 1;
nnwt       = 3;
nnwt_liq   = 3;
nnwt_gas   = 3;

%----------------<  Numero de Pasos de tiempo  >--------------------%
nstep_liq  = 1;
nstep_gas  = 1;

%----------------< Banderas para fijar Campos  >--------------------%
flag_p_all_fixed = 0;
flag_v_l_all_fixed = 0;
flag_v_g_all_fixed = 0;
flag_alpha_g_all_fixed = 0;

%-------------------------<  Shocap  >------------------------------%
shocap        = 1;

%---------------------<  Formulacion debil  >-----------------------%
weak_form     = 0; 

%------------------<  Factor que afecta a Tau  >--------------------%
tau_factor    = 1;

%---------------------<  Numero de Courant  >-----------------------%
Cou   =  0.4;

npg   =  2;
ndf   =  4;
tol   = 1e-6;
% Vector de Estado U=[p,alpha,u_l,u_g]; 
indx_gas = [2,4];       
indx_liq = [1,3];       

%------------------------<  Axisimetria  >--------------------------%
axisim = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           GEOMETRIA                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

H  = 0.4;                    % Depth of the fluid.
Nx = 300;                    % Numero de nodos en x. 
xratio = 1;                  % Ratio between axis / side element size.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                         DATOS FISICOS                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gravity  =  9.8*0;
visco_l  =  1.000e-8;          
visco_g  =  1.000e-8;        

rho_l    =  2;
rho_g    =  1;

%------------------<  Modelo de Sato Turb.  >-----------------------%
Sato_model_coef = 0.6;
Sato_model_coef = 0;

d_bubble              = 0.002;

%-------------<  Incluye terminos entre Fases  >--------------------%
comp_interphase_terms = 0;

%----------------------<  Fuerza de Drag  >-------------------------%
drag_factor           = 1;

%-----------------<  Fuerza de Masa Virtual  >----------------------%
virtual_mass          = 0;
C_vm                  = 0.5;

%-------------------<  Presion Interfacial  >-----------------------%
flag_interf_pressure  = 0;

%------------------<  Modelo de viscosidad ? >----------------------%
model_visco_std       = 1;      % [0]  visco_k 
                                % [1]  alpha_k * visco_k  
                                % [2]  avg(alpha_k) * visco_k

qpot_drag_model       = 1;      % F_drag propto alpha_l ^ qpot

%-------------------<  Coeficiente de Drag  >-----------------------%
fRe_drag_model        = 0;      % [0] Laminar model. 
                                % [1] Cd=0.44. 
                                % [2] turbulent model.

%-------------------------<  Lumping  >-----------------------------%
lumping_drag_forces   = 0;
lumping_time_terms    = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    CONDICIONES INICIALES                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p_ini       = 0;
alpha_g_ini = 0.1;
vel_l_ini   = [2];
vel_g_ini   = [2.5];
%alpha_g_in  = 0.2;
%vel_l_in    = [10];
%vel_g_in    = [0];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             ESCALEO                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 0,
    if 0,
        visco_l=visco_l/sqrt(rho_l);
        visco_g=visco_g/sqrt(rho_l);
        rho_g=rho_g/sqrt(rho_l);
        rho_l=rho_l/sqrt(rho_l);
        p_ini=p_ini/(rho_l);    
    else
        visco_l=visco_l/(rho_l);
        visco_g=visco_g/(rho_l);
        rho_g=rho_g/(rho_l);
        p_ini=p_ini/(rho_l);       
        rho_l=rho_l/(rho_l);   
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             MALLA                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x        = (0:Nx-1)'/(Nx-1)*H;
%-------- coordenadas (XNOD)
xnod     = x ;

%-------- conectividades (ICONE)
icone = [ (1:Nx-1)' (2:Nx)' ];

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

xmin = min(xnod);
xmax = max(xnod);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   DETERMINA EL ESTADO INICIAL                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%---------------------<  Solucion inicial (PHI) >-------------------%

if read_from_file==1,
    eval(['load ' filestate]);
    U = phi;
elseif 0,
    % Modifica Alfa ( Campana de Gauss) y Presion (Lineal)
    uini    = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
    U       =  ones(numnp,1)*uini;          
    %sigma  = 0.2;
    %sigma  = 0.1;      
    %xc     = H/2;
    %z      = xnod(:,2)-xc;
    U(:,indx_gas(1)) = alpha_g_in+alpha_g_ini*exp(-z.^2/sigma);  
    U(:,indx_liq(1)) = p_ini+rho_l*abs(gravity)*(H-xnod(:,2));       
else
    uini    = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
    U       =  ones(numnp,1)*uini;              
    Ampl   = 0.05;
    U(:,indx_gas(1)) = alpha_g_ini*ones(length(U),1)+Ampl*sin((4*pi*xnod)./H)
end

xmin = min(xnod);
xmax = max(xnod);
tol  = 1e-6;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     BOUNDARY CONDITIONS                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%nodes_bot = find(abs(xnod(:,1)-xmin(1))<tol);
%nodes_top = find(abs(xnod(:,1)-xmax(1))<tol);
fixa = zeros(numnp,ndf);
forc = zeros(numnp,ndf);

fixa(2,1)=1;

%-----------------------<  Fijaciones Salida >----------------------%
%fixa(nodes_top,indx_liq(1))= 1;
%fixa([nodes_top],indx_gas(1))= 1;
%forc(nodes_top,indx_liq(1))= p_ini;
%forc(nodes_top,indx_gas(1))= alpha_g_in;

%----------------------<  Fijaciones Entrada >----------------------%
%fixa(nodes_bot,indx_liq(2))=1;
%fixa(nodes_bot,indx_gas(1))=1;
%fixa(nodes_bot,indx_gas(2))=1;
%forc(nodes_bot,indx_liq(2))=vel_l_in;
%forc(nodes_bot,indx_gas(1))=alpha_g_in;
%forc(nodes_bot,indx_gas(2))=vel_g_in;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     FIJACIONES DE CAMPOS                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if flag_p_all_fixed==1,
    fixa(:,indx_liq(1))=1;
    forc(:,indx_liq(1))=U(:,indx_liq(1));    
end

if flag_v_l_all_fixed==1,
    fixa(:,indx_liq(2))=1;
    forc(:,indx_liq(2))=U(:,indx_liq(2));    
end

if flag_v_g_all_fixed==1,
    fixa(:,indx_gas(2))=1;
    forc(:,indx_gas(2))=U(:,indx_gas(2));
    %    forc(nodes_bot,indx_gas(2))=vel_g_in;        
end

if flag_alpha_g_all_fixed==1,
    fixa(:,indx_gas(1))=1;
    forc(:,indx_gas(1))=U(:,indx_gas(1));    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              MANIPULA LAS PROPIEDADES DEL MATERIAL                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ijob = 'set';
handle_prop_elem;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                        CARGAS REPRTIDAS                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
strg    = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 CONDICIONES DE CONTORNO MIXTAS                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
teco     = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    CONDICIONES DE PERIODICAS                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lin_con = [];

if 0,
    for icompo=[1,2,3,4]
        lin_con = [lin_con ; ... 
                [ 1 1 icompo -1 Nx icompo]];
    end
else
    for icompo=[1,2,3,4] % Ojo hay que elegir el menor bloque LI, 
                         % porque usamos LU.las CP son entonces. 
        lin_con = [lin_con ; ... 
                [ 1 1 icompo -1 Nx-1 icompo];
            [ 1 Nx icompo -1 2 icompo]    ];
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               CONDICIONES INICIALES EN PRIMITIVAS                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bini   = U;
for k=1:ndf,
    nofree        = find(fixa(:,k)~=0);
    bini(nofree,k)   = forc(nofree,k);
end   

bc_time_flag = 0;
bc_time_t_end = 1e-3;