% ====================================================================
%
%                           laddle_1d_homo
%
% ====================================================================
read_from_file = 0;
filestate = 'bubbly_1d_homo';

PMM_method = 1;
Advdif_scheme = 1;

flag_p_all_fixed = 0;
flag_v_l_all_fixed = 0;
flag_v_g_all_fixed = 0;
flag_alpha_g_all_fixed = 0;

nnwt = 5;

shocap                = 1;
weak_form             = 1; 

cc    = 1000000;  % velocidad del sonido para poder regularizar la matriz de masa que es nearly singular

Cou   =  1;
npg   =  4;
ndf   =  6;
tol   = 1e-6;

indx_gas = [2,5,6];
indx_liq = [1,3,4];

H = 5;                     % Depth of the fluid
R = 0.1;                   % Radius of the container
Nr = 2;                     % Number of elements in r dir
Nz = 50;
Nx = Nr;
Ny = Nz;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8*0;
visco_l  =  0.8e-3;          % liquid phase viscosity
visco_g  =  1.000e-5;        % gas phase viscosity
visco_l  =  1e-1;          % liquid phase viscosity
visco_g  =  1e-1;        % gas phase viscosity

rho_l    =  1000;
rho_g    =  1;

d_bubble              = 0.004;
comp_interphase_terms = 0;
axisim                = 0;
drag_factor           = 1;
virtual_mass          = 0;
C_vm                  = 0.5;
model_visco_std       = 0;   % std --> alpha_k * visco_k  sino avg(alpha_k) * visco_k
qpot_drag_model       = 1;   % F_drag propto alpha_l ^ qpot
fRe_drag_model        = 0;   % [0] Laminar model [1] Cd=0.44

lumping_drag_forces   = 1;
lumping_time_terms    = 1;
tau_factor            = 1;

% BOUNDARY CONDITIONS
% es flujo homogeneo por lo tanto los contornos son todos periodicos

% INITIAL VALUES
p_ini       = 0;
alpha_g_ini = 0.1;
vel_l_ini   = [0,2.00];
vel_g_ini   = [0,1.50];

% ESCALEO
if 1,
if 0,
   visco_l=visco_l/sqrt(rho_l);
   visco_g=visco_g/sqrt(rho_l);
   rho_g=rho_g/sqrt(rho_l);
   rho_l=rho_l/sqrt(rho_l);
   p_ini=p_ini/(rho_l);    
else
   visco_l=visco_l/(rho_l);
   visco_g=visco_g/(rho_l);
   rho_g=rho_g/(rho_l);
   p_ini=p_ini/(rho_l);    
   rho_l=rho_l/(rho_l);   
end
end

% ====================================================================
x        = (0:Nr-1)'/(Nr-1)*R;
%y        = refh([1,6*zratio,8*zratio],Nz)*H;
y        = refh([1,1,1],Nz)*H;

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
   xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
   icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

xmin = min(xnod);
xmax = max(xnod);

%-------- Solucion inicial (PHI)
if read_from_file==1,
   eval(['load ' filestate]);
   U = phi;
else
   uini    = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
   U       =  ones(numnp,1)*uini;      
%   U       =  rand(numnp,1)*[p_ini,alpha_g_ini,-[1,1],[1,1]];      
%   U(:,indx_liq(1)) = (p_bot-rho_l*abs(gravity)*xnod(:,2));   
   if 0,
      Ampl   = 0.01*alpha_g_ini;
      sigma  = 0.2;
      sigma  = 0.0001;      
      xc     = H/2;
      z      = xnod(:,2)-xc;
      U(:,indx_gas(1)) = alpha_g_ini+Ampl*exp(-z.^2/sigma);   
   elseif 0,
      U(:,indx_gas(1)) = alpha_g_in;
      nn = find(xnod(:,2)>H/2);
      U(nn,indx_gas(1)) = alpha_g_ini;   
   else
      Ampl   = 0.01*alpha_g_ini;
      kappa  = 2;
      z = (xnod(:,2)-xmin(2))/(max(xnod(:,2))-xmin(2));
      U(:,indx_gas(1)) = alpha_g_ini+Ampl*sin(kappa*2*pi*z);
   end
end

%-------- fijaciones (FIXA)
fixa = zeros(numnp,ndf);
forc = zeros(numnp,ndf);

xmin = min(xnod);
xmax = max(xnod);
tol  = 1e-6;

nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_wall= find(abs(xnod(:,1)-xmax(1))<tol);
nodes_axis= find(abs(xnod(:,1)-xmin(1))<tol);

% fijamos la presion en algunos nodos
if flag_p_all_fixed==1,
    fixa(:,indx_liq(1))=1;
    forc(:,indx_liq(1))=p_ini;    
    forc(:,indx_liq(1))=p_ini+0.001*xnod(:,2);   
    %   fixa(:,indx_gas(1))=1;
    %   forc(:,indx_gas(1))=U(:,indx_gas(1));           
else
    fixa(nodes_top,indx_liq(1))=1;
    forc(nodes_top,indx_liq(1))=p_ini;
    fixa(nodes_bot,indx_liq(1))=1;
    forc(nodes_bot,indx_liq(1))=p_ini;    
end

fixa(nodes_wall,[3,5])=1;
fixa(nodes_axis,[3,5])=1;

if flag_v_l_all_fixed==1,
   fixa(:,indx_liq(2:3))= ones(numnp,2);
   forc(:,indx_liq(2:3))= ones(numnp,1)*vel_l_ini;   
end

if flag_v_g_all_fixed==1,
   fixa(:,indx_gas(2:3))= ones(numnp,2);
   forc(:,indx_gas(2:3))= ones(numnp,1)*vel_g_ini;   
end

if flag_alpha_g_all_fixed==1,
   fixa(:,indx_gas(1))= ones(numnp,1);
   forc(:,indx_gas(1))= U(:,indx_gas(1));   
end

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];

for icompo=[1,2,3,4,5,6]
   lin_con = [lin_con ; ... 
         [ones(length(nodes_axis),1) nodes_axis icompo*ones(length(nodes_axis),1) ...
            -ones(length(nodes_wall),1) nodes_wall icompo*ones(length(nodes_wall),1)]];      
end

if flag_p_all_fixed==1, idfs=[2,3,4,5,6]; else idfs=[1,2,3,4,5,6]; end
idfs=[2,3,4,5,6];

for icompo = idfs,    
    if 0,
        lin_con = [lin_con ; ...                         
                [ones(length(nodes_bot),1) nodes_bot icompo*ones(length(nodes_bot),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]
            [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_bot),1) nodes_bot+2 icompo*ones(length(nodes_bot),1)] ];         
    else
        lin_con = [lin_con ; ...                     
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_bot),1) nodes_bot icompo*ones(length(nodes_bot),1)]];         
    end
end

% condicion inicial en primitivas
%-----------------------------------------------

bini   = U;
for k=1:ndf,
   nofree        = find(fixa(:,k)~=0);
   bini(nofree,k)   = forc(nofree,k);
end   


bc_time_flag = 0;
bc_time_t_end = 1;