function [H,Cp,Cpi,beta] = enthalpy_function(Up,prop,Up_old)
%
%      [H,Cp,Cpi,beta] = enthalpy_function(Up,prop,Up_old)
%
%       Computes enthalpy function to handle primitive variables in 2D 
%

%if nargin <3, Up_old=Up; end
if nargin >2, flag=1; else flag=0; end

ijob = 'get';
handle_prop_elem;

alpha_g_ctf  = 1e-3; 

[N,ndf]     = size(Up);

indx_l = [1,3,4];
indx_g = [2,5,6];

alpha_g     = Up(:,indx_g(1));
vel_g       = Up(:,indx_g(2:3));
pre         = Up(:,indx_l(1));
vel_l       = Up(:,indx_l(2:3));

alpha_g     = max([alpha_g_ctf*ones(1,N);alpha_g'])';
alpha_g     = min([(1-alpha_g_ctf)*ones(1,N);alpha_g'])';
alpha_l     = 1-alpha_g;   

if virtual_mass,
    rho_g_eff = rho_g + C_vm*rho_l;
    rho_l_eff = rho_l*(1-C_vm);   
else
    rho_g_eff = rho_g;
    rho_l_eff = rho_l;   
end


Cp                           = zeros(N,ndf,ndf);
Cp(:,indx_l(1),indx_g(1))    = -rho_l*ones(N,1);
Cp(:,indx_g(1),indx_g(1))    =  rho_g*ones(N,1);
Cp(:,indx_l(2),indx_l(2))    =  rho_l_eff*alpha_l;
Cp(:,indx_l(3),indx_l(3))    =  rho_l_eff*alpha_l;
Cp(:,indx_g(2),indx_g(2))    = rho_g_eff*alpha_g;
Cp(:,indx_g(3),indx_g(3))    = rho_g_eff*alpha_g;
% esta parte es para incluir la ecuacion de continuidad o no en la ec de cantidad de movimiento
if Advdif_scheme==1,
    Cp(:,indx_l(2:3),indx_g(1))  = -rho_l_eff*  vel_l;
    Cp(:,indx_g(2:3),indx_g(1))  = rho_g_eff*  vel_g;
end

if (liq_mass_eq_mod~=1),Cp(:,indx_l(1),indx_g(1))=liq_mass_eq_mod*Cp(:,indx_l(1),indx_g(1)); end

if flag==1,
alpha_g     = Up_old(:,indx_g(1));
vel_g       = Up_old(:,indx_g(2:3));
pre         = Up_old(:,indx_l(1));
vel_l       = Up_old(:,indx_l(2:3));

alpha_g     = max([alpha_g_ctf*ones(1,N);alpha_g'])';
alpha_g     = min([(1-alpha_g_ctf)*ones(1,N);alpha_g'])';
alpha_l     = 1-alpha_g;   
end

if PMM_method==1,    
    compute_beta_limits;
else
    beta=1;
end

for idf=1:ndf,
    Cp(:,indx_g(1),idf)=Cp(:,indx_g(1),idf).*beta;
end

Cpi           = zeros(N,ndf,ndf);
if 1,
    % calculamos la inversa de Cp con compresibilidad artificial y luego afectamos por 1/beta
    % inverse of Cp matrix corregida por compresibilidad ariticial
    rho_m = alpha_l*rho_g_eff+alpha_g*rho_l_eff;
    Cpi(:,1,1)    = rho_g_eff*cc^2./rho_m;
    Cpi(:,1,2)    = rho_l_eff*cc^2./rho_m;    
    Cpi(:,2,2)    =  alpha_l./rho_m;
    Cpi(:,2,1)    = -alpha_g./rho_m;    
    Cpi(:,3,2)    = vel_l(:,1)./rho_m;    
    Cpi(:,4,2)    = vel_l(:,2)./rho_m;    
    Cpi(:,3,1)    = -alpha_g./alpha_l.*vel_l(:,1)./rho_m;    
    Cpi(:,4,1)    = -alpha_g./alpha_l.*vel_l(:,2)./rho_m;    
    Cpi(:,3,3)    = 1/rho_l_eff ./alpha_l;    
    Cpi(:,4,4)    = 1/rho_l_eff ./alpha_l;        
    Cpi(:,5,1)    = vel_g(:,1)./rho_m;    
    Cpi(:,6,1)    = vel_g(:,2)./rho_m;    
    Cpi(:,5,2)    = -alpha_l./alpha_g.*vel_g(:,1)./rho_m;    
    Cpi(:,6,2)    = -alpha_l./alpha_g.*vel_g(:,2)./rho_m;    
    Cpi(:,5,5)    = 1/rho_g_eff ./alpha_g;    
    Cpi(:,6,6)    = 1/rho_g_eff ./alpha_g;    
    
    for idf=1:ndf,
        Cpi(:,idf,indx_g(1))=Cpi(:,idf,indx_g(1))./beta;        
%        Cpi(:,indx_g(1),idf)=Cpi(:,indx_g(1),idf)./beta;                
    end    
end

H(:,indx_l)   = [rho_l*alpha_l,rho_l_eff*(alpha_l*[1,1]).*vel_l];
H(:,indx_g)   = [rho_g*alpha_g.*beta,rho_g_eff*(alpha_g*[1,1]).*vel_g];

%disp('enthalpy function');keyboard;pause
