function [Fc,C] = Cflux_liq(Up,dUp,alpha_g,grad_alpha_g,vel_g,Dmat_vg,Dmat_vl,prop,job)
%
%   [Fc,C] = Cflux(Up,alpha_g,grad_alpha_g,vel_g,prop,job)
%
%  job = 0 ; los 3 terminos (pressure, bouyancy y drag)
%  job = 1 ; los terminos de pressure y bouyancy
%  job = 2 ; solo el termino de drag
%

ijob = 'get';
handle_prop_elem;

qpot = qpot_drag_model;

[N,ndf]      = size(Up);
pre          = Up(:,1);
vel_l        = Up(:,2:3);

alpha_l      = 1-alpha_g;
grad_alpha_l = -grad_alpha_g;

grad_v_l = dUp(:,2:3,1:2);

Fc           = zeros(N,ndf);
C            = zeros(N,ndf,ndf);

field = [1,1,1];

if job==1, field(3)=0; end
if job==2, field(1:2)=zeros(2,1); end

if field(1),
    % reactive fluxes due to bouyancy forces
    Fc(:,3)     = -rho_l*gravity*alpha_l;
end

if field(2)&0,
    % reactive fluxes due to pressure gradient
    Fc(:,2)     = Fc(:,2)-grad_alpha_l(:,1).*pre;
    Fc(:,3)     = Fc(:,3)-grad_alpha_l(:,2).*pre;
    C(:,2,1)    = C(:,2,1)-grad_alpha_l(:,1);
    C(:,3,1)    = C(:,3,1)-grad_alpha_l(:,2);    
end

if field(3),
    if comp_interphase_terms,   
        % reactive and source terms due to drag interaction forces
        tol      = 1e-06;
        v_rel       = vel_l - vel_g;
        v_slip      = sqrt(sum(v_rel'.^2))';
        Re          = rho_l*d_bubble*v_slip/visco_l;
        mask        = Re>tol;
        nn          = find(mask==1);
        
        if isempty(nn)==0,
            
            if fRe_drag_model==0,
                f_Re        = 24.0 ./ Re(nn);
                df_RedRe    = -24.0 ./Re(nn) ./Re(nn);
            elseif fRe_drag_model==1,
                f_Re        = 0.44;
                df_RedRe    = 0;     
            elseif fRe_drag_model==2,                
                tmp0_drag   = 1.0+0.15*Re(nn).^(0.687);
                f_Re        = tmp0_drag.*(24.0 ./ Re(nn));
                df_RedRe    = -24.0 ./Re(nn) ./Re(nn) .* tmp0_drag + (24.0 ./ Re(nn)) .* (0.15*0.687*(Re(nn).^(0.687-1.0))); 
            end    
            
            C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
            C_drag      = C1_drag*Re(nn).*f_Re;  
            f_alpha     = alpha_g(nn).*(1-alpha_g(nn)).^qpot;
            
            Fc(nn,2)     = Fc(nn,2)+C_drag.*f_alpha.*(v_rel(nn,1));
            Fc(nn,3)     = Fc(nn,3)+C_drag.*f_alpha.*(v_rel(nn,2));
            
            dRedU       = rho_l*d_bubble/visco_l;
            
            tmp4_drag   = f_Re+Re(nn).*df_RedRe;
            
            tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);
            for idm=1:2,
                for jdm=1:2,
                    Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
                end
                %                Phi(:,idm,idm) = Phi(:,idm,idm)+C_drag.*f_alpha;
                Phi(:,idm,idm) = Phi(:,idm,idm)+f_Re.*Re(nn);
            end
            
            C(nn,2,2)    = C(nn,2,2) + C1_drag.*f_alpha.*Phi(:,1,1);
            C(nn,2,3)    = C(nn,2,3) + C1_drag.*f_alpha.*Phi(:,1,2);
            C(nn,3,2)    = C(nn,3,2) + C1_drag.*f_alpha.*Phi(:,2,1);
            C(nn,3,3)    = C(nn,3,3) + C1_drag.*f_alpha.*Phi(:,2,2);
            
        end
        %disp('Cflux liq ');keyboard;pause
        
    end
    
end

if 1,
if virtual_mass,
    Fc(:,2:3) = Fc(:,2:3)- C_vm*rho_l*((alpha_g*[1,1]).*Dmat_vg-Dmat_vl);
    C(:,2,2) =  C(:,2,2) + C_vm*rho_l*(1/dt*ones(N,1)+grad_v_l(:,1,1));
    C(:,2,3) =  C(:,2,3) + C_vm*rho_l*(grad_v_l(:,1,2));
    C(:,3,2) =  C(:,3,2) + C_vm*rho_l*(grad_v_l(:,2,1));    
    C(:,3,3) =  C(:,3,3) + C_vm*rho_l*(1/dt*ones(N,1)+grad_v_l(:,2,2));    
end
end