function [Fa,Ax,Ay] = Flux_function(Up,prop)
%
%        [Fa,Ax,Ay] = Flux_function(Up,prop)
%
%       Computes advective fluxes F_a
%                   2D    
%	 Dimension of Fa array = numnp x ndm*ndf 
%             

ndm         = 2;

ijob = 'get';
handle_prop_elem;
[N,ndf]     = size(Up);

indx_l = [1,3,4];
indx_g = [2,5,6];

if virtual_mass,
    if stadtke_method==1,
        rho_g_eff = rho_g;
        rho_l_eff = rho_l;           
    else        
        rho_g_eff = rho_g + C_vm*rho_l;
        rho_l_eff = rho_l*(1-C_vm);   
    end
else
    rho_g_eff = rho_g;
    rho_l_eff = rho_l;   
end

%clear rho_l rho_g

alpha_g     = Up(:,indx_g(1));
vel_g       = Up(:,indx_g(2:3));
pre         = Up(:,indx_l(1));
vel_l       = Up(:,indx_l(2:3));
alpha_l     = 1-alpha_g;

v_rel       = vel_g - vel_l;
v_slip      = sqrt(sum(v_rel'.^2))';

% advective fluxes
Fa            = zeros(N,ndf,2);
A             = zeros(N,ndm,ndf,ndf);

% Fase liq

% continuity eq
Fa(:,indx_l(1),1:2)   = rho_l*(alpha_l*[1,1]).*vel_l;
% momentum eq
Fa(:,indx_l(2),1)     = alpha_l.*(rho_l_eff*vel_l(:,1).*vel_l(:,1));
Fa(:,indx_l(3),1)     = alpha_l.*(rho_l_eff*vel_l(:,1).*vel_l(:,2));
Fa(:,indx_l(2),2)     = alpha_l.*(rho_l_eff*vel_l(:,2).*vel_l(:,1));
Fa(:,indx_l(3),2)     = alpha_l.*(rho_l_eff*vel_l(:,2).*vel_l(:,2));

% continuity eq
A(:,1,indx_l(1),indx_l(2))    =  rho_l*alpha_l;
A(:,2,indx_l(1),indx_l(3))    =  rho_l*alpha_l;
A(:,1:2,indx_l(1),indx_g(1))  = -rho_l*vel_l;

% momentum eq
A(:,1,indx_l(2),indx_l(1))    = alpha_l;
A(:,2,indx_l(3),indx_l(1))    = alpha_l;


% esta parte es para incluir la ecuacion de continuidad o no en la ec de cantidad de movimiento
if Advdif_scheme==1,
    if 1,
        A(:,1,indx_l(2),indx_g(1))    = A(:,1,indx_l(2),indx_g(1))-rho_l_eff*vel_l(:,1).*vel_l(:,1);
        A(:,1,indx_l(3),indx_g(1))    = A(:,1,indx_l(3),indx_g(1))-rho_l_eff*vel_l(:,1).*vel_l(:,2);
        A(:,2,indx_l(2),indx_g(1))    = A(:,2,indx_l(2),indx_g(1))-rho_l_eff*vel_l(:,2).*vel_l(:,1);
        A(:,2,indx_l(3),indx_g(1))    = A(:,2,indx_l(3),indx_g(1))-rho_l_eff*vel_l(:,2).*vel_l(:,2);
    end

    A(:,1,indx_l(2),indx_l(2))    = A(:,1,indx_l(2),indx_l(2))+2*rho_l_eff*alpha_l.*vel_l(:,1);
    A(:,1,indx_l(3),indx_l(2))    = A(:,1,indx_l(3),indx_l(2))+rho_l_eff*alpha_l.* vel_l(:,2);
    A(:,1,indx_l(3),indx_l(3))    = A(:,1,indx_l(3),indx_l(3))+rho_l_eff*alpha_l.* vel_l(:,1);
    
    A(:,2,indx_l(2),indx_l(2))    = A(:,2,indx_l(2),indx_l(2))+rho_l_eff*alpha_l.*vel_l(:,2);
    A(:,2,indx_l(2),indx_l(3))    = A(:,2,indx_l(2),indx_l(3))+rho_l_eff*alpha_l.*vel_l(:,1);
    A(:,2,indx_l(3),indx_l(3))    = A(:,2,indx_l(3),indx_l(3))+2*rho_l_eff*alpha_l.*vel_l(:,2);
else
    A(:,1,indx_l(2),indx_l(2))    = rho_l_eff*alpha_l.*vel_l(:,1);
    A(:,2,indx_l(2),indx_l(2))    = rho_l_eff*alpha_l.*vel_l(:,2);
    A(:,1,indx_l(3),indx_l(3))    = rho_l_eff*alpha_l.*vel_l(:,1);
    A(:,2,indx_l(3),indx_l(3))    = rho_l_eff*alpha_l.*vel_l(:,2);        
end


% Fase gas
% continuity eq
Fa(:,indx_g(1),1:2)   = rho_g*(alpha_g*[1,1]).*vel_g;
% momentum eq
Fa(:,indx_g(2),1)     = alpha_g.*(rho_g_eff*vel_g(:,1).*vel_g(:,1));
Fa(:,indx_g(3),1)     = alpha_g.*(rho_g_eff*vel_g(:,1).*vel_g(:,2));
Fa(:,indx_g(2),2)     = alpha_g.*(rho_g_eff*vel_g(:,2).*vel_g(:,1));
Fa(:,indx_g(3),2)     = alpha_g.*(rho_g_eff*vel_g(:,2).*vel_g(:,2));

% continuity eq.
A(:,1,indx_g(1),indx_g(2))    = rho_g*alpha_g;
A(:,2,indx_g(1),indx_g(3))    = rho_g*alpha_g;
A(:,1:2,indx_g(1),indx_g(1))  = rho_g*vel_g;

% momentum eq.
A(:,1,indx_g(2),indx_l(1))    = alpha_g;
A(:,2,indx_g(3),indx_l(1))    = alpha_g;

if(flag_interf_pressure>0),

    if stadtke_method==1,
        factor = flag_interf_pressure*(rho_g+rho_l)*alpha_g.*alpha_l;
        vaux = factor.*v_rel(:,1).^2;
        A(:,1,indx_l(2),indx_g(1))    = A(:,1,indx_l(2),indx_g(1))-vaux;        
        A(:,1,indx_g(2),indx_g(1))    = A(:,1,indx_g(2),indx_g(1))+vaux;                
        vaux = factor.*v_rel(:,2).^2;        
        A(:,2,indx_l(3),indx_g(1))    = A(:,2,indx_l(3),indx_g(1))-vaux;                   
        A(:,2,indx_g(3),indx_g(1))    = A(:,2,indx_g(3),indx_g(1))+vaux;                           
    else        
        vaux = rho_g*flag_interf_pressure*v_slip.^2;
        A(:,1,indx_l(2),indx_g(1))    = A(:,1,indx_l(2),indx_g(1))-vaux;
        A(:,2,indx_l(3),indx_g(1))    = A(:,2,indx_l(3),indx_g(1))-vaux;           
        A(:,1,indx_g(2),indx_g(1))    = A(:,1,indx_g(2),indx_g(1))+vaux;
        A(:,2,indx_g(3),indx_g(1))    = A(:,2,indx_g(3),indx_g(1))+vaux;                   
    end
    
end

% esta parte es para incluir la ecuacion de continuidad o no en la ec de cantidad de movimiento
if Advdif_scheme==1,
    A(:,1,indx_g(2),indx_g(1))    = A(:,1,indx_g(2),indx_g(1))+rho_g_eff*vel_g(:,1).*vel_g(:,1);
    A(:,1,indx_g(3),indx_g(1))    = A(:,1,indx_g(3),indx_g(1))+rho_g_eff*vel_g(:,2).*vel_g(:,1);
    A(:,2,indx_g(2),indx_g(1))    = A(:,2,indx_g(2),indx_g(1))+rho_g_eff*vel_g(:,1).*vel_g(:,2);
    A(:,2,indx_g(3),indx_g(1))    = A(:,2,indx_g(3),indx_g(1))+rho_g_eff*vel_g(:,2).*vel_g(:,2);
    
    A(:,1,indx_g(2),indx_g(2))    = 2*rho_g_eff*alpha_g.*vel_g(:,1);
    A(:,1,indx_g(3),indx_g(2))    = rho_g_eff*alpha_g.* vel_g(:,2);
    A(:,1,indx_g(3),indx_g(3))    = rho_g_eff*alpha_g.* vel_g(:,1);
    
    A(:,2,indx_g(2),indx_g(2))    = rho_g_eff*alpha_g.* vel_g(:,2);
    A(:,2,indx_g(2),indx_g(3))    = rho_g_eff*alpha_g.* vel_g(:,1);
    A(:,2,indx_g(3),indx_g(3))    = 2*rho_g_eff*alpha_g.*vel_g(:,2);
else    
    A(:,1,indx_g(2),indx_g(2))    = rho_g_eff*alpha_g.*vel_g(:,1);
    A(:,2,indx_g(2),indx_g(2))    = rho_g_eff*alpha_g.*vel_g(:,2);
    A(:,1,indx_g(3),indx_g(3))    = rho_g_eff*alpha_g.*vel_g(:,1);
    A(:,2,indx_g(3),indx_g(3))    = rho_g_eff*alpha_g.*vel_g(:,2);
end

% agregado termino segun paper Stadtke
if (stadtke_method & 0),
    factor = alpha_g.*alpha_l.*(rho_l*alpha_g-rho_g*alpha_l);

    % LIQ
    A(:,1,indx_l(2),indx_l(2))    = A(:,1,indx_l(2),indx_l(2))+factor.*v_rel(:,1) ;
    A(:,1,indx_l(3),indx_l(3))    = A(:,1,indx_l(3),indx_l(3))+factor.*v_rel(:,1) ;
    
    A(:,2,indx_l(2),indx_l(2))    = A(:,2,indx_l(2),indx_l(2))+factor.*v_rel(:,2) ;
    A(:,2,indx_l(3),indx_l(3))    = A(:,2,indx_l(3),indx_l(3))+factor.*v_rel(:,2) ;

    A(:,1,indx_l(2),indx_g(2))    = A(:,1,indx_l(2),indx_g(2))-factor.*v_rel(:,1) ;
    A(:,1,indx_l(3),indx_g(3))    = A(:,1,indx_l(3),indx_g(3))-factor.*v_rel(:,1) ;
    
    A(:,2,indx_l(2),indx_g(2))    = A(:,2,indx_l(2),indx_g(2))-factor.*v_rel(:,2) ;
    A(:,2,indx_l(3),indx_g(3))    = A(:,2,indx_l(3),indx_g(3))-factor.*v_rel(:,2) ;
    
    % GAS
    A(:,1,indx_g(2),indx_g(2))    = A(:,1,indx_g(2),indx_g(2))-factor.*v_rel(:,1) ;
    A(:,1,indx_g(3),indx_g(3))    = A(:,1,indx_g(3),indx_g(3))-factor.*v_rel(:,1) ;
    
    A(:,2,indx_g(2),indx_g(2))    = A(:,2,indx_g(2),indx_g(2))-factor.*v_rel(:,2) ;
    A(:,2,indx_g(3),indx_g(3))    = A(:,2,indx_g(3),indx_g(3))-factor.*v_rel(:,2) ;

    A(:,1,indx_g(2),indx_l(2))    = A(:,1,indx_g(2),indx_l(2))+factor.*v_rel(:,1) ;
    A(:,1,indx_g(3),indx_l(3))    = A(:,1,indx_g(3),indx_l(3))+factor.*v_rel(:,1) ;
    
    A(:,2,indx_g(2),indx_l(2))    = A(:,2,indx_g(2),indx_l(2))+factor.*v_rel(:,2) ;
    A(:,2,indx_g(3),indx_l(3))    = A(:,2,indx_g(3),indx_l(3))+factor.*v_rel(:,2) ;
    
end


Ax = reshape(A(:,1,:,:),N,ndf,ndf);
Ay = reshape(A(:,2,:,:),N,ndf,ndf);      

%disp(' Flux Function ');keyboard;pause
