function [Fc,C] = Cflux_nc(Up,grad_p,vel_l,div_vg,prop,job,flag_predictor)
%
%   [Fc,C] = Cflux_nc(Up,grad_p,vel_l,div_vg,prop,job,flag_predictor);
%
%  job = 0 ; los 3 terminos (pressure, bouyancy y drag)
%  job = 1 ; los terminos de pressure y bouyancy
%  job = 2 ; solo el termino de drag

if nargin < 7, flag_predictor=0; end

ijob = 'get';
handle_prop_elem;

[N,ndf]     = size(Up);
alpha_g     = Up(:,1);
vel_g       = Up(:,2:3);

Fc          = zeros(N,ndf);
C           = zeros(N,ndf,ndf);

field = [1,1,1];

if job==1, field(3)=0; end
if job==2, field(1:2)=zeros(2,1); end

if flag_predictor,
   
if field(1),
% reactive fluxes due to bouyancy forces
Fc(:,3)     = Fc(:,3)-rho_g*gravity;
end

if field(2),
% reactive fluxes due to pressure gradient
Fc(:,2)     = Fc(:,2)+grad_p(:,1);
Fc(:,3)     = Fc(:,3)+grad_p(:,2);
end

if field(3),
% non conservative treatment of gas continuity equation
C(:,1,1) =  C(:,1,1) + rho_g*div_vg;
Fc(:,1)  =  Fc(:,1) + rho_g*div_vg.*alpha_g;   

if comp_interphase_terms,   
   % reactive and source terms due to drag interaction forces
   tol      = 1e-06;
%visco_l     = visco_g;
d_bubble    = 6/1000;
rho_l       = 1000;
v_rel       = vel_g - vel_l;
v_slip      = sqrt(sum(v_rel'.^2))';
Re          = rho_l*d_bubble*v_slip/visco_l;
mask        = Re>tol;
nn          = find(mask==1);

if isempty(nn)==0,

f_Re        = 24.0 ./ Re(nn);
C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
C_drag      = C1_drag*Re(nn).*f_Re;  
df_RedRe    = -24.0 ./Re(nn) ./Re(nn);

Fc(nn,2)     = Fc(nn,2)+C_drag.*(v_rel(nn,1));
Fc(nn,3)     = Fc(nn,3)+C_drag.*(v_rel(nn,2));

dRedU       = rho_l*d_bubble/visco_l;
tmp4_drag   = f_Re+Re(nn).*df_RedRe;
tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);

for idm=1:2,
   for jdm=1:2,
      Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
  end
  Phi(:,idm,idm) = Phi(:,idm,idm)+C_drag;
end
C(nn,2,2)    = C(nn,2,2) + Phi(:,1,1);
C(nn,2,3)    = C(nn,2,3) + Phi(:,1,2);
C(nn,3,2)    = C(nn,3,2) + Phi(:,2,1);
C(nn,3,3)    = C(nn,3,3) + Phi(:,2,2);
end

end
end

else
   
if 0,
% DEBUG and fixme
file = 'Cflux_nc';
line = '89';
lp_debug(file,line)
return
end

if field(1),
% reactive fluxes due to bouyancy forces
Fc(:,3)     = Fc(:,3)-rho_g*gravity*alpha_g;

C(:,3,1)    = C(:,3,1)-rho_g*gravity*ones(N,1);
end

if field(2),
% reactive fluxes due to pressure gradient
Fc(:,2)     = Fc(:,2)+alpha_g.*grad_p(:,1);
Fc(:,3)     = Fc(:,3)+alpha_g.*grad_p(:,2);

C(:,2,1)    = C(:,2,1)+grad_p(:,1);
C(:,3,1)    = C(:,3,1)+grad_p(:,2);
end

if field(3),
% non conservative treatment of gas continuity equation
C(:,1,1) =  C(:,1,1) + rho_g*div_vg;
Fc(:,1)  =  Fc(:,1) + rho_g*div_vg.*alpha_g;   

if comp_interphase_terms,   
   % reactive and source terms due to drag interaction forces
   tol      = 1e-06;
visco_l     = visco_g;
d_bubble    = 6/1000;
rho_l       = 1000;
v_rel       = vel_g - vel_l;
v_slip      = sqrt(sum(v_rel'.^2))';
Re          = rho_l*d_bubble*v_slip/visco_l;
mask        = Re>tol;
nn          = find(mask==1);

if isempty(nn)==0,
f_Re        = 24.0 ./ Re(nn);
C1_drag     = drag_factor * 3/4*visco_l/d_bubble^2;
C_drag      = C1_drag*Re(nn).*f_Re;  
df_RedRe    = -24.0 ./Re(nn) ./Re(nn);

Fc(nn,2)     = Fc(nn,2)+C_drag.*alpha_g(nn).*(v_rel(nn,1));
Fc(nn,3)     = Fc(nn,3)+C_drag.*alpha_g(nn).*(v_rel(nn,2));

C(nn,2,1)    = C(nn,2,1) + C_drag.*(v_rel(nn,1));
C(nn,3,1)    = C(nn,3,1) + C_drag.*(v_rel(nn,2));

dRedU       = rho_l*d_bubble/visco_l;

tmp4_drag   = f_Re+Re(nn).*df_RedRe;

tmp4_drag   = dRedU * tmp4_drag ./ v_slip(nn);
for idm=1:2,
   for jdm=1:2,
      Phi(:,idm,jdm) = tmp4_drag.*(v_rel(nn,idm).*v_rel(nn,jdm));
  end
  Phi(:,idm,idm) = Phi(:,idm,idm)+C_drag.*alpha_g(nn);
end

C(nn,2,2)    = C(nn,2,2) + Phi(:,1,1);
C(nn,2,3)    = C(nn,2,3) + Phi(:,1,2);
C(nn,3,2)    = C(nn,3,2) + Phi(:,2,1);
C(nn,3,3)    = C(nn,3,3) + Phi(:,2,2);

end
%disp('Cflux');keyboard;pause
end

end

end