% ====================================================================
%
%            gas pipe flow with alpha_g fixed at input
%            1D case
%
% ====================================================================

% Reference infinity state of variables
alpha_g_ref = 1;
vel_g_ref   = [0,1];

alpha_g_ini = 0.1;
vel_g_ini   = [0,0.01];

rho_g       = 1;
visco_g     = 1e-1;
visco_g     = 1e-5;
visco_l     = 0.6;
gravity     = 9.8;

H = 4.0;
D = 1.0;
radius_nozzle = 0.1*D;

rho_l       = 1000;
p_ref       = 0;

ndf = 3;
tol = 1e-6;

shocap   =      0 ;
weak_form=      0 ; 
axisim   =      0 ;
comp_interphase_terms = 1;
drag_factor           = 1;

 Cou     =  1  ;
 npg     =  4  ;

% ====================================================================

xnod     =[];
icone    =[];
Nx       =  2;    Lx =  0.1*H;
Ny       =  31;    Ly =  H;
psi      = (0:Nx-1)'/(Nx-1);
x        = psi*Lx;
eta      = (0:Ny-1)'/(Ny-1)*1; 
y        = eta.^2*Ly;

%-------- coordenadas (XNOD)
for k=1:Ny,
  xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
nodes_bot  = (1:Nx)';
nodes_top  = Nx*(Ny-1)+(1:Nx)';
nodes_lef  = (1:Nx:Nx*Ny)';
nodes_rig  = (Nx:Nx:Nx*Ny)';

nodes_bot_in = nodes_bot;
nodes_bot_wall = [];

fixa= zeros(numnp,ndf);
% fixations of some dofs
fixa(:,[1])  = ones(numnp,1);
fixa(nodes_bot_in,[1,2,3])= ones(length(nodes_bot_in),3);
fixa(nodes_top,[2])    = ones(length(nodes_top),1);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);

if 1,
Ampl   = alpha_g_ref-alpha_g_ini;
sigma  = 0.005;
xc     = 0;
z      = xnod(:,2)-xc;
forc(:,1) = alpha_g_ini+Ampl*exp(-z.^2/sigma);
elseif 0,
    mask = xnod(:,2)>0.5*H;
    vaux = mask*alpha_g_ini+(1-mask)*alpha_g_ref;
    forc(:,[1]) = vaux;
else
    forc(:,[1]) = ones(numnp,1)*alpha_g_ini;       
end
forc(nodes_bot_in,1:3)= ones(length(nodes_bot_in),1)*[alpha_g_ref,vel_g_ref];

%--------- External fields -----------------------------
% pressure and liquid velocities 
% external_fields = [pressure , vel_liq]
y0 = min(xnod(:,2));
H  = max(xnod(:,2))-y0;
external_fields =  zeros(numnp,3);
external_fields(:,1) = p_ref+rho_l*gravity*(H-(xnod(:,2)-y0));

%-------- Propiedades del material

ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
%teco    = [eles face val1/rho/Cp val2 ];
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
%peri = [];
%slip = [];
lin_con = [];
for icompo=[1,2,3]
   lin_con = [lin_con ; ... 
      [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
   -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];

   lin_con = [lin_con ; ... 
      [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
   -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];

end


% Up = [alpha_g, vel_g]
%-------- Solucion inicial (PHI)
uini    = [alpha_g_ini,vel_g_ini];
U       =  ones(numnp,1)*uini;   

if 1,
Ampl   = (vel_g_ref(:,2)-vel_g_ini(:,2));
sigma  = 0.05;
sigma  = 0.2;
xc     = 0;
z      = xnod(:,2)-xc;
U(:,3) = vel_g_ini(:,2)+Ampl*exp(-z.^2/sigma);

Ampl   = alpha_g_ref-alpha_g_ini;
sigma  = 0.2
sigma  = 0.05;
xc     = 0;
z      = xnod(:,2)-xc;
U(:,1) = alpha_g_ini+Ampl*exp(-z.^2/sigma);
elseif 0,
    mask = xnod(:,2)>0.5*H;
    vaux = mask*alpha_g_ini+(1-mask)*alpha_g_ref;
    U(:,1) = vaux;    
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
   bini   = U;
   for k=1:ndf,
      nofree        = find(fixa(:,k)~=0);
      bini(nofree,k)   = forc(nofree,k);
   end   
else
   load bubbly_01_ini
end

bc_time_flag = 0;
bc_time_t_end = 1;