%
%            VISCOUS STEP
%

ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

% Localizacion de las variables
%------------------------------------------------------------------

ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);
Uf_e = zeros(ne,nen*ndf_fields);
Uenorm = zeros(ne,nen);
for inod=1:nen,
   nodes  = icone(eles,inod);
   jnod  = (inod-1)*ndf+(1:ndf);
   knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
   Ue(:,jnod)  = U(nodes,1:ndf);		 
   U0e(:,jnod) = Uold(nodes,1:ndf);   
   re(:,inod)  = xnod(nodes,1);		        
   Uf_e(:,knod)  = extra_nodal_fields(nodes,1:ndf_fields);       
   Uenorm(:,inod) = sqrt(sum(Uold(nodes,1:ndf)'.^2)');
end

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
   
   [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
   [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
   
   % Interpolacion en punto de Gauss
   %------------------------------------------------------------------
   
   Upg    = zeros(ne,ndf);
   U0pg    = zeros(ne,ndf);
   dUpg    = zeros(ne,ndf,ndm);
   dU0pg    = zeros(ne,ndf,ndm);    
   radpg    = zeros(ne,1);
   grad_p   = zeros(ne,ndm);
   vel_l    = zeros(ne,ndm);
   
   for inod=1:nen,
      jnod = (inod-1)*ndf+(1:ndf);
      Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
      U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
      radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
      
      dUpg(:,(1:ndf),1)   = dUpg(:,(1:ndf),1) + ...
         (dshapex(:,inod,1)*ones(1,ndf)).* Ue(:,jnod);
      
      dUpg(:,(1:ndf),2) = dUpg(:,(1:ndf),2)+ ...
         (dshapex(:,inod,2)*ones(1,ndf)).* Ue(:,jnod);          
      
      dU0pg(:,(1:ndf),1)   = dU0pg(:,(1:ndf),1) + ...
         (dshapex(:,inod,1)*ones(1,ndf)).* U0e(:,jnod);
      
      dU0pg(:,(1:ndf),2) = dU0pg(:,(1:ndf),2)+ ...
         (dshapex(:,inod,2)*ones(1,ndf)).* U0e(:,jnod);          
      
      grad_p(:,1)  = grad_p(:,1) + dshapex(:,inod,1).* Uf_e(:,(inod-1)*ndf_fields+1);
      grad_p(:,2)  = grad_p(:,2) + dshapex(:,inod,2).* Uf_e(:,(inod-1)*ndf_fields+1);          
      vel_l(:,1)   = vel_l (:,1) + N(inod).* Uf_e(:,(inod-1)*ndf_fields+2);
      vel_l(:,2)   = vel_l (:,2) + N(inod).* Uf_e(:,(inod-1)*ndf_fields+3);      
   end
   
   [H,Cp]     =  enthalpy_function(Upg,prop,1);
   [H0,bas]   =  enthalpy_function(U0pg,prop,1);
   
   [Fd,Kxx,Kxy,Kyx,Kyy]  = Dflux(Upg,dUpg,prop,0);  
   
   % Correccion axisimetrica del jacobiano de la transformacion al elemento master
   %------------------------------------------------------------------
   xsj =  wpg(ipg) .* Jaco;
   if axisim,
      xsj = xsj.*radpg;
   end
   
   for inod=1:nen,   
      
      W_supg = zeros(ne,ndf,ndf);
      
      for idf=1:ndf,
         W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
      end
            
      for jnod=1:nen,
         
         % temporal terms
         W_Cp_N = rowprod(W_supg,Cp,ndf,ndf,ndf)*N(jnod) ;        
         
         gN_K_gN = zeros(numel,ndf,ndf);        
         for idf=1:ndf,
            for jdf=1:ndf,
               gN_K_gN(:,idf,jdf) = ...
                  -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                  (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                  (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                  (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
            end             
         end
         
         % assembling
         vaux =  - gN_K_gN ;
         
         for idf=1:ndf,
            for jdf=1:ndf,              
               Mele(:,inod,idf,jnod,jdf) = Mele(:,inod,idf,jnod,jdf) + ... 
                  xsj.*( W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );           
            end
         end
         
      end % jnod
      
      % temporal term
      W_dUdt   = rowprod(W_supg,(H-H0),ndf,ndf,1);
      
      % diffusive fluxes
      gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1) ... 
         -(dshapex(:,inod,2)*ones(1,ndf)).*Fd(:,1:ndf,2);
      
      vaux =  - gN_Fd;  
      
      for idf=1:ndf,
         Rele(:,inod,idf) = Rele(:,inod,idf) - ... 
            (vaux(:,idf) +W_dUdt(:,idf) ./ dt ).* xsj ;
      end
      
   end % inod
   
end % ipg

