%
%   Generacion de una malla a partir de un Laplaciano
%        
%   Problema : Back_step 2D
%
%  6O================================================O5 
%   |
%   |
%   O======O| 
%   1      2|
%           |
%           |3
%           |O=======================================O4


% Coordenadas a resolver 
% coor = 'x'; grado de libertad # 1 
% coor = 'y'; grado de libertad # 2 

N12 = 11;
N23 = 21;
N34 = 31;
N16 = 21;
N45 = N16;
N65 = N12+N23+N34;

P1 = [0,0];
P2 = [5,0];
P3 = [5,-4.9];
P4 = [150,-4.9];
P5 = [150,+5.2];
P6 = [0,+5.2];

Nx = N65;
Ny = N16;

psi = (0:N12-1)'/(N12);
psi1= psi.^2;
psi2= 1-(1-psi).^2;
psi3= sin(psi*pi/2).^2;
x12 = ones(length(psi),1)*P1+psi2*(P2-P1);

psi = (0:N23-1)'/(N23);
psi1= psi.^2;
psi2= 1-(1-psi).^2;
psi3= sin(psi*pi/2).^2;
x23 = ones(length(psi),1)*P2+psi3*(P3-P2);

psi = (0:N34-1)'/(N34-1);
psi1= psi.^2;
psi2= 1-(1-psi).^2;
psi3= sin(psi*pi/2).^2;
x34 = ones(length(psi),1)*P3+psi1*(P4-P3);

psi = (0:N45-1)'/(N45-1);
psi1= psi.^2;
psi2= 1-(1-psi).^2;
psi3= sin(psi*pi/2).^2;
x45 = ones(length(psi),1)*P4+psi*(P5-P4);

psi = (0:N65-1)'/(N65-1);
psi1= psi.^2;
psi2= 1-(1-psi).^2;
psi3= sin(psi*pi/2).^2;
x65 = ones(length(psi),1)*P6+psi*(P5-P6);

psi = (0:N16-1)'/(N16-1);
psi1= psi.^2;
psi2= 1-(1-psi).^2;
psi3= sin(psi*pi/2).^2;
x16 = ones(length(psi),1)*P1+psi*(P6-P1);

Lx  = 1;  
Ly  = 1; 

psi      = (0:Nx-1)'/(Nx-1);
x0       = 0;
x        = psi*Lx+x0; 
eta      = (0:Ny-1)'/(Ny-1)*1; 
y0       = 0;
y        = eta*Ly+y0;

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
  xnod=[xnod;[x ones(Nx,1)*y(k)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end

%----------------------------------------
% convertimos cuadrangulos en triangulos
quad2tri=0;

if quad2tri,
   icone=quad2tri_mesh(icone);
end

%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
ndf = 2;
nodes_bot  = (1:Nx);
nodes_top  = Nx*(Ny-1)+(1:Nx);
nodes_lef  = 1:Nx:Nx*Ny;
nodes_rig  = Nx:Nx:Nx*Ny;

fixa= zeros(numnp,ndf);
fixa(nodes_lef,:)=ones(length(nodes_lef),ndf);
fixa(nodes_rig,:)=ones(length(nodes_rig),ndf);
fixa(nodes_top,:)=ones(length(nodes_top),ndf);
fixa(nodes_bot,:)=ones(length(nodes_bot),ndf);

%-------- Cargas/Desplazamientos nodales(FORC)
forc = zeros(numnp,ndf);
% coordenada x
forc(nodes_top,1) = x65(:,1);
forc(nodes_bot,1) = [x12(:,1);x23(:,1);x34(:,1)];
forc(nodes_lef,1) = x16(:,1); 
forc(nodes_rig,1) = x45(:,1); 

% coordenada y
forc(nodes_top,2) = x65(:,2);
forc(nodes_bot,2) = [x12(:,2);x23(:,2);x34(:,2)];
forc(nodes_lef,2) = x16(:,2); 
forc(nodes_rig,2) = x45(:,2); 


%-------- Fuentes de volumen (por elemento)
source =zeros(numel,ndf);
xc = zeros(numel,ndf);
for k=1:nen
   xc = xc + xnod(icone(:,k),:);
end
xc = xc ./ (nen*ones(numel,2));

PP2 = xnod(N12,:);
ff2 = gauss_bell(xc,PP2,0.02);
PP3 = xnod(N12+N23,:);
ff3 = gauss_bell(xc,PP3,0.02);

source(:,1) = source(:,1) + (50)*ff2 + 500*ff3;
source(:,2) = source(:,2) + (-100)*ff2 + 100*ff3;    

%keyboard;pause

%--------- Termino reactivo -----------------------------
reac = zeros(numel,ndf);

%--------- Campo de Velocidad -----------------------------
veloc = zeros(numnp,ndm,ndf);
%-------- Propiedades del material

 rho     = 1    ;
 Cp      = 1    ;
 kappa   = 1    ;
 npg     =  2   ;
 prop(1) =rho   ;
 prop(2) =Cp    ;
 prop(3) =kappa ;
 prop(4) =npg   ;

 upwind  =    0 ;
 shocap  =    0 ;
 axisim  =    0 ;
% [1] SUPG
% [2] (SU+C)PG
 type_upw =   0 ;   
 lumped   =   0 ;

 prop(6)  = upwind ;
 prop(7)  = shocap ;
 prop(8)  = axisim ;
 prop(9)  = type_upw;
 prop(11) = lumped ;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas
lin_con = [];

%-------- Solucion inicial (PHI)
bini = zeros(numnp,ndf);

for k=1:ndf,
 nofree        = find(fixa(:,k)~=0);
 bini(nofree,k)   = forc(nofree,k);
end
