function [Kg,Rg] = advdif_2d(xnod,icone,ifaceb,fixa,forc,teco,lin_con,prop,...
                          veloc,reac,source,phi,phiold,ijob)                       
%------------------------------------------------------------------
%
%     [Kg,Rg] = advdif_2d(xnod,icone,fixa,forc,teco,peri,prop,
%                      veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion adveccion difusion reaccion no estacionaria c/fuente
%        #.- elemento cuadrangular y triangular
%        #.- plano y axisimetrico
%
%        ijob = 1 ---> matriz tangente y residuo
%        ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

  [numel,nen]   = size(icone);
  [numnp,ndf]   = size(phi);
  [numnp,ndm]   = size(xnod);
  
  if ndf~=1, error(' NDF debe ser 1');end

  rho      = prop(1) ;
  Cp       = prop(2) ;
  kappa    = prop(3) ;
  npg      = prop(4) ;
  tau_fac  = prop(6) ;
  shocap   = prop(7) ;
  axisim   = prop(8) ;
  type_upw = prop(9) ;
  dtfic    = prop(10);
  lumped   = prop(11);
  tol      = prop(12);
  dt       = prop(13);
  alpha    = prop(14);
  dt       = alpha*dt;
  
  if ndm==1,
     pgauss1d;
  elseif ndm==2, 
     pgauss2d; 
  elseif ndm==3,
     pgauss3d; 
  end
  
  eles  = (1:numel)';
%  ceros    = zeros(length(eles),nen);
  
  Told = phiold(:,1);
  T = alpha*phi(:,1)+(1-alpha)*phiold;

  Rg   =forc;
  Kele =zeros(numel,nen,ndf,nen,ndf);
  Rele =zeros(numel,nen,ndf);
  mass_lumped = zeros(numnp,1);

  if length(eles)>0,    
     advdif_2d_ele;
  end


% Gather contributions from eles to nodes
%------------------------------------------------------------------
[Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);

 if lumped,
    Rg = Rg - diag(mass_lumped)*phi;
    Kg = Kg + diag(mass_lumped);
 end

% Linear restrictions
%------------------------------------------------------------------
if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end

%disp('fem001');keyboard;pause