function [r] = r_abso_bc_tubes(U)
%
% CALCULAMOS EL RESIDUO DE LAS UNIONES
%

global datos_abso_bc_tubes

ga       = datos_abso_bc_tubes.ga;
type_end = datos_abso_bc_tubes.type_end;

tol    = 1e-16;
dx     = datos_abso_bc_tubes.dx;    % tamao celda extrema de cada tubo en la union 
ntubes = length(type_end);

U_ref = U(2*ntubes+(1:ntubes),:);  % estado en el nodo de referencia de cada tubo
U_end = U(1*ntubes+(1:ntubes),:);  % estado en el nodo del extremo de cada tubo
U_in  = U(1:ntubes,:);  % estado en el nodo inmediato interior a cada tubo

% conversion a conservativas
U_ref = cv2pv(U_ref,ga,-1);
U_end = cv2pv(U_end,ga,-1);
U_in  = cv2pv(U_in ,ga,-1);

% obtengo matrices de diagonalizacion
[P,P1,D] = diagsys(U_end,ga);

V_in = zeros(ntubes,3);
V_ref = V_in;
for k=1:3,
    for m=1:3,
        V_in(:,k)  = V_in(:,k)  + P1(:,k,m).*  U_in(:,m);	
        V_ref(:,k) = V_ref(:,k) + P1(:,k,m).* U_ref(:,m);	
    end
end

% aplico la normal [-1,+1] for [left,right] ends
for k=1:ntubes
    D(k,:) = D(k,:)*type_end(k);    
end

for k=1:3
    lambda=D(:,k);
    V(:,k)=V_in(:,k).*(lambda>0)+V_ref(:,k).*(lambda<=0);
%    V(:,k)=V_in(:,k).*(lambda<=0)+V_ref(:,k).*(lambda>0);    
end

U_bc = zeros(ntubes,3);
for k=1:3,
    for m=1:3,
        U_bc(:,k)  = U_bc(:,k) + P(:,k,m).* V(:,m);	
    end
end

U_end = cv2pv(U_end,ga,1);
U_bc  = cv2pv(U_bc,ga,1);
%disp('r_abso_bc_tubes.m');keyboard;pause
r = reshape((U_end-U_bc)',3*ntubes,1);

return

