function [r] = r_junction(U)
%
% CALCULAMOS EL RESIDUO DE LAS UNIONES
%

global datos

ga=datos.ga;
type_end=datos.type_end;
Area=datos.Area;
tol=1e-16;
dx=datos.dx;    % tamao celda extrema de cada tubo en la union 
ntubes = length(type_end);

U_in = U(ntubes+(1:ntubes),:);  % estado en el nodo inmediato interior a cada tubo
U(ntubes+(1:ntubes),:)=[];

ip = 0;
iq = 0;
%Up = cv2pv(U,ga,+1);  % conservative --> primitive
%Up_in = cv2pv(U_in,ga,+1);  % conservative --> primitive
Up = U;
Up_in = U_in;

r = zeros(3*ntubes,1);

%disp('r_junction.m');keyboard;pause
for j=1:ntubes
    rho = Up(j,1);
    vel = Up(j,2);
    pre = Up(j,3);
    
    rho_in = Up_in(j,1);
    vel_in = Up_in(j,2);
    pre_in = Up_in(j,3);
    
    aa   = sqrt(ga*pre/rho);
    dpdx    = type_end(j)*(pre-pre_in)/dx(j);
    dvdx    = type_end(j)*(vel-vel_in)/dx(j);
    drhodx  = type_end(j)*(rho-rho_in)/dx(j);
    
    mdot = rho*(type_end(j)*vel)*Area(j);
    edot = mdot*(aa^2/(ga-1.0)+0.5*vel^2);
    inflow = ( (mdot>tol) | (abs(mdot)<tol & type_end(j)>0));
    % mass balance
    r(1) = r(1) + mdot;
    % energy balance
    r(2) = r(2) + edot;
    
    % Mach line characteristic
    %    r(j+2) = -(vel+type_end(j)*aa)*(dpdx+type_end(j)*rho*aa*dvdx); 
    r(j+2) = (dpdx+type_end(j)*rho*aa*dvdx);     
    
    % path line characteristic
    if(inflow)
        ip = ip+1;
        r(ntubes+2+ntubes-1+ip) = -vel*(dpdx-aa^2*drhodx);
    end
    
    if(ip>ntubes)
        error(' ====== ERROR ====== @ r_junction [001] ');
    end
end

% equalize all the pressures
pre_1 = Up(1,3);
for j=2:ntubes
    pre = Up(j,3);
    r(2+ntubes+j-1) = pre-pre_1;
end 

% equalize all the outgoing entropy 

if(inflow==0)
    if(iq>0)
        r(2+ntubes+ip+ntubes-1+iq) = aa-aa_1;
        iq = iq+1;
    else
        iq=1;
        rho_1 = Up(j,1);
        pre_1 = Up(j,3);
        aa_1  = sqrt(ga*pre_1/rho_1);
    end
end

return
