function [RHS1,RHS2m,RHS2p] = rhs1y2(U1,U2m,U2p,dt,valve)
%
%   [RHS1,RHS2m,RHS2p] = rhs1y2(U1,U2m,U2p,dt,valve)
%
%   U1 state for RHS1 computation
%   U2m state for RHS2m computation (lambda minus)
%   U2p state for RHS2p computation (lambda plus)
%

get_phys_prop;
g1 = ga-1;

% for path line
rho = U1(:,1);
vel = U1(:,2);
pre = U1(:,3);
Temp = pre./(R_gas*rho);
cc  = sqrt(ga*pre./rho);

Area   = valve.Area_pipe;
dAreax = valve.dAreax;
T_wall = valve.Twall;

[alfa,G,q] = source4(U1,dAreax,T_wall,ga,R_gas,Area);

RHS1 = g1./(ga*R_gas*Temp).*(q+vel.*G)*dt;

% for lambda minus (Mach line)
rho = U2m(:,1);
vel = U2m(:,2);
pre = U2m(:,3);
Temp = pre./(R_gas*rho);
cc  = sqrt(ga*pre./rho);

[alfa,G,q] = source4(U2m,dAreax,T_wall,ga,R_gas,Area);

RHS2m = (rho.*(ga-1).*(q+vel.*G)+rho.*cc.*G-rho.*vel.*cc.^2.*alfa)*dt;
RHS2m = -RHS2m./(rho.*cc);

% for lambda plus (Mach line)
rho = U2p(:,1);
vel = U2p(:,2);
pre = U2p(:,3);
Temp = pre./(R_gas*rho);
cc  = sqrt(ga*pre./rho);

[alfa,G,q] = source4(U2p,dAreax,T_wall,ga,R_gas,Area);

RHS2p = (rho.*(ga-1).*(q+vel.*G)-rho.*cc.*G-rho.*vel.*cc.^2.*alfa)*dt;
RHS2p = -RHS2p./(rho.*cc);