%
%  routine <test_valve_model_2.m> 
%
%  Performs test valve model with upstream and downstream
%  states variables in time according to a user defined law
%  Only one valve is used
%

relax0 = 0.1;
bfh = [];
p_P_ref = 0.8e5;
dt = 0.01/200;
u_P_ref = 50;
rho_P_ref = 1;
p_C_ref = 1e5;
rho_C_ref = 1;
%Tem_C_ref = p_C_ref/rho_C_ref/287;
Tem_C_ref = 300;

datos_gral.ttime = 0;datos_gral.dt    = dt;
phi_pipe_int = ones(1,1)*[rho_P_ref,u_P_ref,p_P_ref];
phi_pipe_end = ones(1,1)*[1,u_P_ref,1e5];
phi_valve_th = ones(1,1)*[1,u_P_ref,1e5]; 
phi_cylinder = ones(1,1)*[rho_C_ref,p_C_ref,Tem_C_ref];

datos_valve.dx       = [0.1];
datos_valve.Area_pipe   = [1];
datos_valve.Area_throat = [0.5];
datos_valve.dAreax      = [0];
datos_valve.Twall       = [300];
phi   = [phi_pipe_int;phi_pipe_end;phi_valve_th;phi_cylinder];
datos_valve.nodes_end = [1,2,3,4];
type_end = [1];

datos_valve.type_end = type_end;

Kg = [];
Rg = [];

tol = 1e-6;
itermax = 100;
ntime = 200;

figure(1);clf;
jterh = [];

for itime=1:ntime
    ttime = itime*dt;
    Ampl = -0.1;
    T = ntime*dt/4;
    omega = 2*pi/T;
    phi_pipe_int(3) = p_P_ref*(1+Ampl*sin(omega*ttime));
    % phi_pipe_int(2) = u_P_ref*(1+Ampl*sin(omega*ttime));    
    % phi_pipe_int(1) = rho_P_ref*(1-Ampl*sin(omega*ttime));        
    Ampl = -0.0;
    T = ntime*dt/7;
    omega = 2*pi/T;
    phi_cylinder(2) = p_C_ref*(1-Ampl*sin(omega*ttime));
    Ampl = -0.0;
    T = ntime*dt/5;
    omega = 2*pi/T;
    phi_cylinder(1) = rho_C_ref*(1-Ampl*sin(omega*ttime));    
%    phi_cylinder(3) = phi_cylinder(2)/phi_cylinder(1)/287;
    
    if 0
        phi   = [phi_pipe_int;phi_pipe_end;phi_valve_th;phi_cylinder];        
        phi = [...
               9.9195e-001  1.0000e+002  1.0982e+005
               1.0000e+000  1.0000e+002  1.0000e+005
               1.0000e+000  1.0000e+002  1.0000e+005
               1.0000e+000  2.5313e+005  3.0000e+002];
    else
        phi(4,:) = phi_cylinder;
        phi(1,:) = phi_pipe_int;        
    end
    
    iter = 0;
    jter = 0;
    sigma_jter = 5;
    lconv = 1;
    jtermax = 0;
    
    while lconv
        iter=iter+1;
        jter=jter+1;
        if iter==1, phi0 = phi; end
        
        [Kg,Rg] = do_valve(Kg,Rg,phi,datos_valve,datos_gral);
        
        % free = [3+(1:2*3),15+(1:2*3)];
        free = [3+(1:2*3)];
        
        Kg = Kg(free,free);
        Rg = Rg(free);
        relax = min(relax0 + tanh(jter/sigma_jter),1);
        dphi = relax*inv(Kg)*Rg;    
        
        % phi([2,3,6,7],:) = phi([2,3,6,7],:) + reshape(dphi,3,4)';
        
        phi(2:3,:)=phi(2:3,:)+reshape(dphi,3,2)';
        
        if iter==1, Rg0=Rg; Rgh=[]; end 
        Rgh = [Rgh;[relax,norm(Rg)]];
        lconv = (norm(dphi)>tol)&(iter<itermax);
        %        lconv = (norm(dphi)>tol);        
        %        disp([' || dphi || = ' num2str(norm(dphi))])
        disp([' || Rg || = ' num2str(norm(Rg))])
        
        %phi,pause
        
        if iter>5 & norm(Rg)>norm(Rg0)
            disp('test_valve_model');keyboard;pause                    
        end
        
    end
    
    jterh = [jterh;iter];
    jtermax = max(jtermax,iter);
    
    Kg = 0*Kg;
    Rg = 0*Rg;
    bfh = [bfh,[ttime;phi(1,3);reshape(phi(2:3,:)',6,1);phi(4,2)]];
    figure(1);clf;plot(bfh(1,:),bfh(2,:)/p_P_ref,bfh(1,:),bfh([4,7],:),bfh(1,:),bfh(9,:)/p_C_ref),pause(0.1);
    
    %    keyboard;pause
    
    if 0
        if 1
            disp([' [ p_P_int  -  u_P  -  u_G  -  p_C  ]'])
            disp([phi(1,3), phi(2:3,2)',phi(4,2)'])
            pause
        else
            bfh([4,7],:)'
            pause
        end
    end
end

return

