%
%  routine <test_valve_model_2.m> 
%
%  Performs test valve model using two valves for imntake & exhaust
%  with the cylinder and the pipe at intake and exhaust side as 
%  references
%

relax0 = 0.1;
bfh = []; bfh2=[];
dt = 0.01/200;
datos_gral.ttime = 0;datos_gral.dt    = dt;

% intake
p_P_ref = 0.8e5;
u_P_ref = -50;
rho_P_ref = 1;
phi_pipe_adm_int = ones(1,1)*[rho_P_ref,u_P_ref,p_P_ref];
phi_pipe_adm_end = ones(1,1)*[1,u_P_ref,1e5];
phi_valve_adm_th = ones(1,1)*[1,u_P_ref,1e5]; 

% cylinder
p_C_ref = 0.9e5;
rho_C_ref = 1;
Tem_C_ref = 300;
phi_cylinder = ones(1,1)*[rho_C_ref,p_C_ref,Tem_C_ref];

% exhaust
p_P_ref = 0.8e5;
u_P_ref = 50;
rho_P_ref = 1;
phi_pipe_exh_int = ones(1,1)*[rho_P_ref,u_P_ref,p_P_ref];
phi_pipe_exh_end = ones(1,1)*[1,u_P_ref,1e5];
phi_valve_exh_th = ones(1,1)*[1,u_P_ref,1e5]; 

datos_valve(1).dx       = 0.1;
datos_valve(2).dx       = 0.1;
datos_valve(1).Area_pipe = 1;
datos_valve(2).Area_pipe = 1;
datos_valve(1).Area_throat = 0.5;
datos_valve(2).Area_throat = 0.5;
datos_valve(1).dAreax      = 0;
datos_valve(2).dAreax      = 0;
datos_valve(1).Twall       = 300;
datos_valve(2).Twall       = 300;

phi = [phi_pipe_adm_int;phi_pipe_adm_end;phi_valve_adm_th;phi_cylinder;
    phi_valve_exh_th;phi_pipe_exh_end;phi_pipe_exh_int];

datos_valve(1).nodes_end = [1,2,3,4];
datos_valve(2).nodes_end = [7,6,5,4];
type_end = [-1;1];

datos_valve(1).type_end = type_end(1);
datos_valve(2).type_end = type_end(2);

Kg = [];
Rg = [];

tol = 1e-6;
itermax = 100;
ntime = 300;

figure(1);clf;
jterh = [];

for itime=1:ntime
    ttime = itime*dt;
    
    if 1
        Ampl = 0.1;
        T = ntime*dt/5;
        omega = 2*pi/T;
        phi_pipe_adm_int(3) = p_P_ref*(1+Ampl*sin(omega*ttime));
        Ampl = 0.25;
        T = ntime*dt/7;
        omega = 2*pi/T;
        phi_cylinder(2) = p_C_ref*(1-Ampl*sin(omega*ttime));
        Ampl = -0.0;
        T = ntime*dt/5;
        omega = 2*pi/T;
        phi_cylinder(1) = rho_C_ref*(1-Ampl*sin(omega*ttime));    
        
        phi(4,:) = phi_cylinder;
        phi(1,:) = phi_pipe_adm_int;        
        phi(7,:) = phi_pipe_exh_int;            
        
    else
        phi = [...         
         1.0000e+000 -5.0000e+001  8.7984e+004
  9.9410e-001 -1.5662e+001  7.6287e+004
  9.9545e-001 -3.1281e+001  7.6287e+004
  1.0000e+000  7.0283e+004  3.0000e+002
  9.7982e-001  6.6059e+001  7.4615e+004
  9.7387e-001  3.3231e+001  7.4615e+004
  1.0000e+000  5.0000e+001  8.0000e+004];

        phi = [...         
  1.0000e+000 -5.0000e+001  8.7956e+004
  6.2966e-001  9.7192e-001  7.3925e+004
  6.2966e-001  1.9439e+000  7.3924e+004
  1.0000e+000  7.3924e+004  3.0000e+002
  9.8653e-001  5.2914e+001  7.2534e+004
  9.8253e-001  2.6565e+001  7.2534e+004
  1.0000e+000  5.0000e+001  8.0000e+004];

    end
    iter = 0;
    jter = 0;
    sigma_jter = 5;
    lconv = 1;
    jtermax = 0;
    
    while lconv
        iter=iter+1;
        jter=jter+1;
        if iter==1, phi0 = phi; end
        
        [Kg,Rg] = do_valve(Kg,Rg,phi,datos_valve,datos_gral);
        
        free = [3+(1:2*3),12+(1:2*3)];
        
        Kg = Kg(free,free);
        Rg = Rg(free);
        relax = min(relax0 + tanh(jter/sigma_jter),1);
        dphi = relax*inv(Kg)*Rg;    
        
        phi([2,3,5,6],:)=phi([2,3,5,6],:)+reshape(dphi,3,4)';
        
        if iter==1, Rg0=Rg; Rgh=[]; end 
        Rgh = [Rgh;[relax,norm(Rg)]];
        lconv = (norm(dphi)>tol)&(iter<itermax);
        %        lconv = (norm(dphi)>tol);        
        %        disp([' || dphi || = ' num2str(norm(dphi))])
        disp([' || Rg || = ' num2str(norm(Rg))])
        
        %phi,pause
        
        if (iter>5 & norm(Rg)>norm(Rg0))            
%            if iter>10,
                disp('test_valve_model');keyboard;pause                    
                %            else
%                phi = phi0;
%                iter=0;
%                jter = 0;
%                sigma_jter = 20;
%                relax0 = 0.1*relax0;
%                lconv = 1;
%            end
        end
        
    end
    
    jterh = [jterh;iter];
    jtermax = max(jtermax,iter);
    
    Kg = 0*Kg;
    Rg = 0*Rg;
    bfh = [bfh,[ttime;phi(1,3);reshape(phi([2,3,5,6],:)',4*3,1);phi(4,2)]];
    figure(1);clf;plot(bfh(1,:),bfh(2,:)/p_P_ref,bfh(1,:),bfh([4,7,10,13],:),bfh(1,:),bfh(15,:)/p_C_ref),pause(0.1);

    bfh2 = [bfh2,phi];
    if isreal(phi)==0; error(' *** ERROR ***'); end
    
    %    keyboard;pause
    
    if 0
        if 1
            disp([' [ p_P_int  -  u_P  -  u_G  -  p_C  ]'])
            disp([phi(1,3), phi(2:3,2)',phi(4,2)'])
            pause
        else
            bfh([4,7],:)'
            pause
        end
    end
end

return

