function [U_S,U_R,indx_tubes_out,x_S,x_R]=valve_model_get_state(U_int,U_end,U_ext,dx,dt,x_P,type_end)
%
%  [U_S,U_R,indx_tubes_out,x_S,x_R] = valve_model_get_state(U_int,U_end,U_ext,dx,dt,x_P,type_end)
%
%  get interpolated points following the characteristics
%  get some utilities for applying the valve model
%
%  from laurea thesis by Marco Alessandri
%
%  U_int : internal tubes nodes
%  U_end : end of tubes connected to the exterior
%  U_ext : external state, atmosphere, tank, cylinder
%

get_phys_prop;

% unpack the states
U_P = U_end;
U_O = U_int;
U_C = U_ext;
% get the characteristics velocities 
v_P = U_end(:,2);
a_P = sqrt(ga*U_end(:,3)./U_end(:,1));
lambda_p = v_P.*type_end+a_P;
lambda_m = v_P.*type_end-a_P;
lambda_0 = v_P.*type_end;

% compute the interpolated points to build the final point
x_O   = x_P + type_end.*dx;

% determines which ends are in and which are out according to
% type of end and flow sense
type_end_flow = sign(type_end.*v_P);
indx_tubes_out = find(type_end_flow<0);

dx_SP = -type_end.*lambda_m*dt;
dx_SP = sign(dx_SP).*min([abs(dx_SP)';dx'])';
x_S   = x_P + dx_SP;

x_R = [];
if isempty(indx_tubes_out)==0
    dx_RP = -type_end(indx_tubes_out).*lambda_0(indx_tubes_out)*dt;
    dx_RP = sign(dx_RP).*min([abs(dx_RP)';dx(indx_tubes_out)'])';
    x_R   = x_P(indx_tubes_out) + dx_RP;        
end

[ntubes,ndf] = size(U_P);

% state at x_S
psi = abs(dx_SP)./dx;
U_S = U_P.*((1-psi)*ones(1,ndf))+(psi*ones(1,ndf)).*U_O;

U_R = [];
% state at x_R if any
if isempty(indx_tubes_out)==0
    U_R = 0*U_S;
    psi = abs(dx_RP)./dx(indx_tubes_out);
    U_R(indx_tubes_out,:) = U_P(indx_tubes_out,:).*((1-psi)*ones(1,ndf)) + ...
        (psi*ones(1,ndf)).*U_O(indx_tubes_out,:);
end