function [r] = r_cylinder(U)
%
% CALCULAMOS EL RESIDUO DEL CILINDRO
%

global datos_cylinder datos_valve datos_gral

tol    = 1e-16;

ga        = datos_cylinder.ga;
ncylinder = datos_cylinder.ncylinder;
theta_0   = datos_cylinder.theta_0;
rpm       = datos_cylinder.rpm;
dt        = datos_cylinder.dt;
Rgas      = datos_cylinder.Rgas;
Twall_cyl = datos_cylinder.Twall_cyl;

ttime     = datos_gral.ttime;

omega     = rpm*2*pi/60;
theta     = omega*ttime+theta_0;

Rgas_i    = Rgas;
Rgas_e    = Rgas;
Rgas_cyl  = Rgas;

U_exh = U(2*ncylinder+(1:ncylinder),:);  % estado en el nodo del escape
U_cyl = U(1*ncylinder+(1:ncylinder),:);  % estado en el nodo del cilindro
U_int = U(1:ncylinder,:);             % estado en el nodo de la admision

rho_i = U_int(:,1);
rho_e = U_exh(:,1);
vel_i = U_int(:,2);
vel_e = U_exh(:,2);
pre_i = U_int(:,3);
pre_e = U_exh(:,3);
Tem_i = pre_i./rho_i/Rgas_i;
Tem_e = pre_e./rho_e/Rgas_e;
Cp_i = ga*Rgas_i/(ga-1);
Cp_e = ga*Rgas_e/(ga-1);

rho_cyl = U_cyl(:,1);
pre_cyl = U_cyl(:,2);
Tem_cyl = U_cyl(:,3);
Cv_cyl = Rgas_cyl/(ga-1);
Cp_cyl = ga*Cv_cyl;


% entalpias
h_i = Cp_i.*Tem_i;
h_e = Cp_e.*Tem_e;
h_cyl = Cp_cyl.*Tem_cyl;

% calculo del area de pasaje de cada valvula
type_valve = strvcat(datos_valve(:).type_valve);

if strmatch(type_valve(2,:),'exhaust','exact')
    indx_valve_e = 2;
    indx_valve_i = 1;
else
    indx_valve_e = 1;
    indx_valve_i = 2;
end    

[A_valve] = Area_valve(theta,datos_valve);
Area_i = A_valve(indx_valve_i);
Area_e = A_valve(indx_valve_e);

% calculo del flujo masico por las valvulas
mdot_i = rho_i.*vel_i.*Area_i;
mdot_e = rho_e.*vel_e.*Area_e;
mdot = mdot_i - mdot_e;

% calculo del flujo entalpico por las valvulas
hdot_i = mdot_i.*((vel_i>0).*h_i+(vel_i<=0.*h_cyl));
hdot_e = mdot_e.*((vel_e<0).*h_e+(vel_i>=0.*h_cyl));
hdot = hdot_i - hdot_e;

% Calculo de la cinematica del mecanismo biela manivela
[V_cyl,A_cyl,Vdot_cyl] = Cyl_geo(theta,datos_cylinder);
V_cyl_old              = V_cyl - dt * Vdot_cyl;
datos_cylinder.V_cyl   = V_cyl;

% calculo de las perdidas de calor por refrigeracion
hfilm_cyl = 0;
Q_ht = hfilm_cyl.*A_cyl.*(Tem_cyl-Twall_cyl);
    
% calculo del trabajo del piston
Wp = pre_cyl.*Vdot_cyl;

% residuo de masa
rho_cyl_old = datos_cylinder.rho_old;
mass_cyl = rho_cyl.*V_cyl;
mass_cyl_old = rho_cyl_old.*V_cyl_old;
r(1,1) = (mass_cyl-mass_cyl_old)/dt - mdot;

% residuo de energia
Tem_cyl_old = datos_cylinder.Tem_old;
ene_cyl_old = Cv_cyl*Tem_cyl_old;
ene_cyl     = Cv_cyl*Tem_cyl;

%r(2,1) = (rho_cyl.*ene_cyl-(rho_cyl_old.*ene_cyl_old))/dt - (hdot + Q_ht - Wp);
r(2,1) = (mass_cyl.*ene_cyl-(mass_cyl_old.*ene_cyl_old))/dt - (hdot + Q_ht - Wp);

% ecuacion de estado
r(3,1) = (pre_cyl - Rgas_cyl*rho_cyl.*Tem_cyl);

%disp('r_cylinder');keyboard;pause
return
