%
%  Incompressible Navier-Stokes equations
%  using SUPG-PSPG like advdif elemset
%

type_upwind         = 1;
tau_centroide       = 1;
debug               = 0;
indx_v              = 1+(1:ndm);
indx_p              = 1;

%------------------------------------------------------------------
ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);

for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
    Ue(:,jnod)  = U(nodes,1:ndf);		 
    U0e(:,jnod) = Uold(nodes,1:ndf);   
    re(:,inod)  = xnod(nodes,1);		        
end

% UPWIND EN EL CENTROIDE 
%------------------------------------------------------------------
if nen==4,
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),0,0);
    [dshapex,N] = shape2d(X2S,0,0,nen); 
elseif nen==3,
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),1/3,1/3);
    [dshapex,N] = shape2d(X2S,1/3,1/3,nen); 
else
    error(' Bad type of element , nen ~=3, nen ~=4 ')
end

Upg          = zeros(ne,ndf);
dUpg         = zeros(ne,ndf,ndm);

for inod=1:nen,
    jnod = (inod-1)*ndf+(1:ndf);
    Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N(inod).*  U0e(:,jnod);
    for idm=1:ndm
        dUpg(:,(1:ndf),idm) = dUpg(:,(1:ndf),idm) + ...
            (dshapex(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);          
    end
end    

if tau_centroide==1,
    velpg   = Upg(:,2:3);
    hele    = comp_hele(velpg,dshapex,Jaco,nen);
    [tau,delta,bas] = supg_nsi_tet(Upg,dUpg,hele,prop,type_upwind);
    compute_P_supg;   
end


% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
    
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    
    Upg    = zeros(ne,ndf);
    U0pg    = zeros(ne,ndf);
    dUpg    = zeros(ne,ndf,ndm);
    radpg    = zeros(ne,1);
        
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
        for idm=1:ndm            
            dUpg(:,(1:ndf),idm)   = dUpg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* Ue(:,jnod);            
        end        
    end

    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
        
    if tau_centroide==0,
        velpg   = U0pg(:,2:3);
        hele    = comp_hele(velpg,dshapex,Jaco,nen);
        [tau,delta,bas] = supg_nsi_tet(U0pg,dUpg,hele,prop,type_upwind);
    end
    
    for inod=1:nen,   
        
        if tau_centroide==0,
            compute_P_supg;
        end
        
        W_supg = P_supg;      
        for idf=1:ndf,
            W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
        end

        % W_i N_j term
        
        for jnod=1:nen,        
            
            u_gN = zeros(ne,1);
            for idm=1:ndm
                u_gN = u_gN + dshapex(:,jnod,idm).*Upg(:,indx_v(idm));
            end            
                        
            ter1(:,inod,:,jnod,:) = W_supg*N(jnod)/dt; 
            
            for idm=1:ndm
                ter2(:,inod,indx_v(idm),jnod,indx_v(idm)) = rowprod(W_supg,u_gN,ndf,ndf,1); 
            end
            
        end
        
        for jnod=1:nen,
            
            % temporal terms
            if 1,
                W_Cp_N = rowprod(W_supg,Cp,ndf,ndf,ndf)*N(jnod) ;        
            else
                % DEBUG
                W_Cp_N = N(inod)*Cp*N(jnod) ;         
            end
            
            % N_A_gN or gN_A_N according to weak form
            if weak_form,
                N_A_gN = -A_gN(:,:,:,inod)*N(jnod);                 
            else          
                N_A_gN =  A_gN(:,:,:,jnod)*N(inod);              
            end
            
            
            P_A_gN = rowprod(P_supg,A_gN(:,:,:,jnod),ndf,ndf,ndf);
            
            gN_K_gN = zeros(numel,ndf,ndf);        
            for idf=1:ndf,
                for jdf=1:ndf,
                    gN_K_gN(:,idf,jdf) = ...
                        -( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                        (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                        (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                        (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
                end             
            end
            
            % reactive terms
            
            if 1,
                W_C_N = rowprod(W_supg,C_1,ndf,ndf,ndf)*N(jnod) ;        
            else
                % DEBUG
                W_C_N = C_1*N(jnod)*N(inod) ;
            end    
            
            
            % assembling
            vaux =  N_A_gN + P_A_gN - gN_K_gN + W_C_N;
            
            for idf=1:ndf,
                for jdf=1:ndf,              
                    Mele(:,inod,idf,jnod,jdf) = Mele(:,inod,idf,jnod,jdf) + ... 
                        xsj.*( W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );           
                end
            end
            
        end % jnod
        
        % temporal term
        if 1,
            W_dUdt   = rowprod(W_supg,(H-H0),ndf,ndf,1);
        else
            % DEBUG
            W_dUdt   = N(inod)*(H-H0);
        end
        
        % advective fluxes
        if weak_form,            
            gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf)).*Fa(:,1:ndf,1) ... 
                -(dshapex(:,inod,2)*ones(1,ndf)).*Fa(:,1:ndf,2);
        else
            gN_Fa = (N(inod)*ones(numel,ndf)).* AgU(:,1:ndf);
        end
        
        % diffusive fluxes
        gN_Fd = -(dshapex(:,inod,1)*ones(1,ndf)).*Fd(:,1:ndf,1) ... 
            -(dshapex(:,inod,2)*ones(1,ndf)).*Fd(:,1:ndf,2);
        
        % stabilization SUPG terms        
        P_A_gU   = rowprod(P_supg,AgU,ndf,ndf,1);
        
        % reactive and source terms        
        if 1,
            W_Fc_1   = rowprod(W_supg,Fc_1,ndf,ndf,1) ;
        else  % DEBUG
            W_Fc_1   = N(inod)*Fc_1; 
        end
        
        vaux =  gN_Fa + P_A_gU - gN_Fd + W_Fc_1;  
        
        for idf=1:ndf,
            Rele(:,inod,idf) = Rele(:,inod,idf) - (vaux(:,idf) + W_dUdt(:,idf) ./ dt ).* xsj ;
        end
        
    end % inod
    
    %   disp('bubbly_2d_ele (liquid) phase ');keyboard;pause   
    
end % ipg


% ------ Terminos de contorno
if weak_form,
    %[Mele,Rele] = bc_conv_bubbly_2d_liq(xnod,icone,Ue,ifaceb,Rele,Mele,prop,Uf_e(:,1:ndf_fields:nen*ndf_fields));   
    [Mele,Rele] = bc_conv_nsi_tet(xnod,icone,Ue,ifaceb,Rele,Mele,prop,extra_nodal_fields(:,1));      
end
