% ====================================================================
%
%                       Poiseuille flow
%
% ====================================================================

ref = 3;

read_from_file = 0;
filestate = 'Chorin_Temam_test.state';

nnwt = 1;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  4;
ndf   =  3;
tol   = 1e-6;

L = 1;                     % Depth of the fluid
Nx = 16*ref;                     % Number of elements in r dir
Ny = Nx;
xratio = 1;               % Ratio between axis / side element size
yratio = 1;                  % Ratio between axis / side element size
x0     = 0.25;
y0     = 0.5;

% PHYSICAL DATA
gravity    = -9.8*0;
visco      =  0.01;
rho        =  1.0;
axisim     = 0;
tau_factor = 0.1;

% BOUNDARY CONDITIONS
p_ref      = 0;
v_wall     = [0,0];
v_top      = [1,0];

% INITIAL VALUES
p_ini      = p_ref;
vel_ini    = v_wall;

% ====================================================================
xnod     =[];
icone    =[];
x        = x0+refh([1,1,1],Nx)*L;
y        = y0+refh([1,1,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
if read_from_file==1,
   eval(['load ' filestate '.mat']);
   U = phi;
else
   uini    = [p_ini,vel_ini];
   U       =  ones(numnp,1)*uini;   
end

indx_v = (2:ndm+1);
indx_p = 1;

%-------- fijaciones (FIXA)
nodes_bot   = (1:Nx)';
nodes_top   = Nx*(Ny-1)+(1:Nx)';
nodes_lef   = (1:Nx:Nx*Ny)';
nodes_rig   = (Nx:Nx:Nx*Ny)';
nodes_wall  = rmdblist([nodes_bot;nodes_lef;nodes_rig;nodes_top]);

fixa= zeros(numnp,ndf);
fixa(nodes_wall,indx_v) = ones(length(nodes_wall),ndm);
fixa(1,indx_p)          = 1;
%fixa(:,indx_p)          = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc                     = zeros(numnp,ndf);
forc(nodes_wall,indx_v(1)) =  -(cos(2*pi*xnod(nodes_wall,1)).*sin(2*pi*xnod(nodes_wall,2)))*exp(-8*pi^2*visco*0);    
forc(nodes_wall,indx_v(2)) =   (sin(2*pi*xnod(nodes_wall,1)).*cos(2*pi*xnod(nodes_wall,2)))*exp(-8*pi^2*visco*0);        
%forc(:,indx_p) = -0.25*(cos(4*pi*xnod(:,1))+cos(4*pi*xnod(:,2)))*exp(-16*pi^2*visco*0);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 0,
    bini   = U;
else

%    bini = 0*U;
%    bini(:,indx_p)    =  -0.25*(cos(4*pi*xnod(:,1))+cos(4*pi*xnod(:,2)))*exp(-16*pi^2*visco*0);
%    bini(:,indx_v(1)) =       -(cos(2*pi*xnod(:,1)).*sin(2*pi*xnod(:,2)))*exp(-8*pi^2*visco*0);    
%    bini(:,indx_v(2)) =        (sin(2*pi*xnod(:,1)).*cos(2*pi*xnod(:,2)))*exp(-8*pi^2*visco*0);        
    ttime=0;Chorin_Temam_test_solex;
    bini = phi_ex;
end
for k=1:ndf,
    nofree        = find(fixa(:,k)~=0);
    bini(nofree,k)   = forc(nofree,k);
end   

bc_time_flag = 1;
bc_time_t_end = 1;

[ifaceb,icone_bou]=pfbou2d(xnod,icone,1);
eles_rig = get_eles_by_nodes(nodes_rig,icone_bou,1);
bou_data_neumann = ifaceb(eles_rig,:);