%
%  Incompressible Navier-Stokes equations
%  using SUPG-PSPG like advdif elemset
%

type_upwind         = 1;
tau_centroide       = 0;
debug               = 0;

get_index;

visco = prop(2);

%------------------------------------------------------------------
ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);

for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
    Ue(:,jnod)  = U(nodes,1:ndf);		 
    U0e(:,jnod) = Uold(nodes,1:ndf);   
    re(:,inod)  = xnod(nodes,1);		        
end

% UPWIND EN EL CENTROIDE 
%------------------------------------------------------------------
if nen==4,
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),0,0);
    [dshapex,N] = shape2d(X2S,0,0,nen); 
elseif nen==3,
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),1/3,1/3);
    [dshapex,N] = shape2d(X2S,1/3,1/3,nen); 
else
    error(' Bad type of element , nen ~=3, nen ~=4 ')
end

Upg          = zeros(ne,ndf);
dUpg         = zeros(ne,ndf,ndm);

for inod=1:nen,
    jnod = (inod-1)*ndf+(1:ndf);
    Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N(inod).*  U0e(:,jnod);
    for idm=1:ndm
        dUpg(:,(1:ndf),idm) = dUpg(:,(1:ndf),idm) + ...
            (dshapex(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);          
    end
end    

if tau_centroide==1,
    velpg   = Upg(:,indx_v);
    hele    = comp_hele(velpg,dshapex,Jaco,nen);
    [tau,delta,bas] = supg_nsi_tet(Upg,dUpg,hele,prop,type_upwind);
    compute_P_supg;   
end


% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
    
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    
    Upg    = zeros(ne,ndf);
    U0pg    = zeros(ne,ndf);
    dUpg    = zeros(ne,ndf,ndm);
    radpg    = zeros(ne,1);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
        for idm=1:ndm            
            dUpg(:,(1:ndf),idm)   = dUpg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* Ue(:,jnod);            
        end        
    end
    
    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
    
    if tau_centroide==0,
        velpg   = Upg(:,indx_v);
        hele    = comp_hele(velpg,dshapex,Jaco,nen);
        [tau,delta,bas] = supg_nsi_tet(velpg,dUpg,hele,prop,type_upwind);
    end
    
    u_gu = rowprod(dUpg(:,indx_v,:),U0pg(:,indx_v),ndm,ndm,1);
    div_u = zeros(ne,1);
    for idm =1:ndm
        div_u = div_u + dUpg(:,indx_v(idm),idm) ;
    end
    strain_rate = row_transp(dUpg(:,indx_v,:));
    strain_rate = 0.5*(strain_rate + dUpg(:,indx_v,:));
    sigma = 2*visco*strain_rate;
    
    terd = rowprod(dshapex,sigma,nen,ndm,ndm);
%    terd = rowprod(dshapex,dUpg(:,indx_v,:),nen,ndm,ndm);
    dshapex_t = row_transp(dshapex);
    terd_m = visco*rowprod(dshapex,dshapex_t,nen,ndm,nen);
%    terd_m = rowprod(dshapex,dshapex_t,nen,ndm,nen);

    for inod=1:nen,   
        
        if tau_centroide==0,
            compute_P_supg;
        end
        
        W_supg = P_delta;      
        for idf=1:ndm,
            W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
        end
        
        % Residual assembling        
        R_mom = (Upg(:,indx_v)-U0pg(:,indx_v))/dt+u_gu+reshape(dUpg(:,indx_p,:),ne,ndm,1);
        R_mom_pre = reshape(dUpg(:,indx_p,:),ne,ndm,1);
        for idm=1:ndm
            Rele(:,inod,indx_v(idm)) = Rele(:,inod,indx_v(idm)) - ( ...  
                W_supg(:,idm,idm).*R_mom(:,idm) + ...                  % momentum residual ( Galerkin + SUPG )
                terd(:,inod,idm) ...
                ).*xsj;                 
            Rele(:,inod,indx_p) = Rele(:,inod,indx_p) - ( ...                  
                P_epsilon(:,1,indx_v(idm)).*R_mom(:,idm)  ... % PSPG 
                ).*xsj; 
        end
        Rele(:,inod,indx_p) = Rele(:,inod,indx_p) - ( ... 
            N(inod)*div_u ...                   % continuity Galerkin
            ).*xsj; 
        
        % Matriz assembling
        for jnod=1:nen,        
            
            u_gN = zeros(ne,ndm);
            for idm=1:ndm
                for jdm=1:ndm
                    u_gN(:,idm) = u_gN(:,idm) + dshapex(:,jnod,jdm).*U0pg(:,indx_v(jdm));
                end
            end            
            
            %keyboard;pause
            for idm=1:ndm
                Kele(:,inod,indx_v(idm),jnod,indx_v(idm)) = Kele(:,inod,indx_v(idm),jnod,indx_v(idm)) + ( ...  
                    W_supg(:,idm,idm)*N(jnod)/dt + ... % termino temporal
                    W_supg(:,idm,idm).*u_gN(:,idm)      + ... % termino convectivo                  
                    terd_m(:,inod,jnod) ... 
                    ).*xsj;  
                Kele(:,inod,indx_v(idm),jnod,indx_p) = Kele(:,inod,indx_v(idm),jnod,indx_p) + ( ...  
                    W_supg(:,idm,idm).*dshapex(:,jnod,idm) ...  % termino presion
                    ).*xsj;                     
                Kele(:,inod,indx_p,jnod,indx_v(idm)) = Kele(:,inod,indx_p,jnod,indx_v(idm)) + ( ...  
                    N(inod)*dshapex(:,jnod,idm) + ... % termino continuidad Galerkin
                    P_epsilon(:,1,indx_v(idm))*N(jnod)/dt + ... % estabilizacion continuidad temporal
                    P_epsilon(:,1,indx_v(idm)).*u_gN(:,idm) ... % estabilizacion continuidad convectivo
                    ).*xsj; 
                Kele(:,inod,indx_p,jnod,indx_p) = Kele(:,inod,indx_p,jnod,indx_p) + ( ...
                    P_epsilon(:,1,indx_v(idm)).*dshapex(:,jnod,idm) ...
                    ).*xsj; 
                
                for jdm=1:ndm
                    Kele(:,inod,indx_v(idm),jnod,indx_v(jdm)) = ... 
                        Kele(:,inod,indx_v(idm),jnod,indx_v(jdm)) + ( ...                      
                        visco*dshapex(:,inod,jdm).*dshapex(:,jnod,idm) ...
                    ).*xsj; 
                end
            end
            
        end % jnod
        
    end % inod
    
    %   disp('bubbly_2d_ele (liquid) phase ');keyboard;pause   
    
end % ipg

