% ====================================================================
%
%                       Poiseuille flow
%
% ====================================================================

ref = 5;
ref = 2;

read_from_file = 0;
filestate = 'sqcav.state-Re_400';

nnwt = 5;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  4;
ndf   =  3;
tol   = 1e-6;

L = 1;                     % Depth of the fluid
Nx = 21*ref;                     % Number of elements in r dir
Ny = Nx;
xratio = 1;               % Ratio between axis / side element size
yratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity    = -9.8*0;
visco      =  1/400;
visco      =  1;
rho        =  1.0;
axisim     = 0;
tau_factor = 1;

% BOUNDARY CONDITIONS
p_ref      = 0;
v_wall     = [0,0];
v_top      = [1,0];

% INITIAL VALUES
p_ini      = p_ref;
vel_ini    = v_wall;

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([1,5,1],Nx)*L;
y        = refh([1,5,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
if read_from_file==1,
   eval(['load ' filestate '.mat']);
   U = phi;
else
   uini    = [p_ini,vel_ini];
   U       =  ones(numnp,1)*uini;   
end

indx_v = (2:ndm+1);
indx_p = 1;

%-------- fijaciones (FIXA)
nodes_bot   = (1:Nx)';
nodes_top   = Nx*(Ny-1)+(1:Nx)';
nodes_lef   = (1:Nx:Nx*Ny)';
nodes_rig   = (Nx:Nx:Nx*Ny)';
nodes_wall  = rmdblist([nodes_bot;nodes_lef;nodes_rig]);

fixa= zeros(numnp,ndf);

fixa(nodes_wall,indx_v) = ones(length(nodes_wall),ndm);
fixa(nodes_top,indx_v)  = ones(length(nodes_top),ndm);
fixa(1,indx_p)          = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc                     = zeros(numnp,ndf);
forc(1,indx_p)           = p_ref;
forc(nodes_wall,indx_v)  = ones(length(nodes_wall),1)*v_wall;
forc(nodes_top,indx_v)   = ones(length(nodes_top),1)*v_top;

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;