function [xnod,icone]=cc2d(ri,re,Nr,Nc)
%
%   Obtener una malla 2D sobre una seccion plana
%   limitada por una circumferencia exterior y en
%   el interior puede tener otra (corona circular)
%   o ser llena (circulo)
%   NOTA: es una parte de un generador 2D y 1/2
%   
%         [xnod,icone]=cc2d(ri,re,Nr,Nc)
%
%   ri : radio interior ( ri puede ser 0 ) 
%	re : radio externo
%   Nr : nro de nodos en el radio
%   Nc : nro de nodos en la direccion circumferencial
%

tol=1e-6;
%Nc = 8*(fix((Nc-1)/8)+1);
Nc = 8*(round(Nc/8));
Nx = fix(Nc/4);
Ny = Nx;

is_corona = 1;
if ri<=0, is_corona=0; ri = re/4; end
if is_corona==0, 
    if Nr<Nx, Nr=Nx;end
end

x_1=[]; ico_1=[];

if is_corona==0,
psi = (0:Nx)'/Nx; psi=2*psi; psi=psi-1; x = psi*ri; y = x;
Nx=length(x); Ny=Nx;

for k=1:Ny,
  x_1=[x_1;[x ones(Nx,1)*y(k)]];
end

ico = [(1:Nx-1)' (2:Nx)' (2:Nx)'+Nx (1:Nx-1)'+Nx];
for k=1:Ny-1,
  ip = (k-1)*Nx;
  ico_1=[ico_1; ico+ip];
end


% nodos entre el cuadrado interior y la circumferencia externa
xmax=max(x);xmin=min(x);ymax=xmax;ymin=xmin;
n1 = find(abs(x_1(:,2)-ymin)<tol);
n2 = find(abs(x_1(:,1)-xmax)<tol);
n3 = find(abs(x_1(:,2)-ymax)<tol);
n4 = find(abs(x_1(:,1)-xmin)<tol);
nbou = [n1;n2(2:length(n2));n3(length(n3)-1:-1:1);n4(length(n4)-1:-1:2)];
xint = x_1(nbou,:);
theta = atan2(xint(:,2),xint(:,1));
xext = [re*cos(theta), re*sin(theta)];
nbou = [nbou;n1(1)];
else

% nodos entre la circumferencia interior y la circumferencia externa
theta = ((0:Nc-1)'/Nc)*2*pi-(3/4*pi);
xint = [ri*cos(theta), ri*sin(theta)];
xext = [re*cos(theta), re*sin(theta)];
nbou = [(1:length(xint))';1]; 
x_1  = xint;

end

nx_1 = length(x_1(:,1));
if is_corona==0, Nr   = Nr-fix(Nx/2); end

x_2 = [];
dx = (xext-xint)/Nr;
for k=1:Nr,
  x   = xint+k*dx;
  x_2 = [x_2 ; x ];
end

ico_2 = []; 
nint = length(nbou);

ico_old = [ nbou(2:nint) nbou(1:nint-1)];
nbou    = nx_1+[(1:nint-1)';1];
ico_new = [ nbou(1:nint-1) nbou(2:nint) ];
ico = [ ico_old ico_new ];

for k=1:Nr,
  ico_2   = [ico_2 ; ico];
  nbou    = nx_1+k*(nint-1)+[(1:nint-1)';1];
  ico_old = [ico_new(:,2) ico_new(:,1)];
  ico_new = [nbou(1:nint-1) nbou(2:nint) ];
  ico = [ ico_old ico_new ];

end

xnod = [x_1;x_2];
icone = [ico_1;ico_2];