function [gUn_ele] = grad_phi_normal_bou(xnod,icone,state,ifaceb)
%
%     [gUn_ele] = grad_phi_normal_bou(xnod,icone,state,[ifaceb])
%
%  calcula el grad U normal de cada ndf por cada elemento 
%
%  Aquellos elementos que no miran al contorno deben tener valor nulo
%
%  ifaceb argumento opcional si uno quiere calcular el grad_U normal en 
%  un subconjunto del contorno
%

tol          = 1e-12;

[nnod,ndm]=size(xnod);
[nele,nen]=size(icone);
[bas,ndf]=size(state);

if nen==4, 
    quad_to_tri;     
end

if nargin <4, 
    [ifaceb,nodes_bou] = pfbou2d(xnod,icone,1); 
else
    if nen==4, error('The mesh should be converted in triangles and the boundary info will be lost ');end
end

[nele,nen]=size(icone);
[neleb,bas]=size(ifaceb);

U = state;

ceros    = zeros(nele,nen*ndf);
  
Rg   = zeros(nnod,ndf);
Mg   = zeros(nnod,1);

Rele = zeros(nele,ndf*nen);
Mele = zeros(nele,1*nen);
  
npg = 2;
xpg = sqrt(3)/3;
xpg = [-xpg xpg];
wpg = [1 1];

eles = ifaceb(:,1);
faces = ifaceb(:,2);

n1 = faces;
n2 = n1+1;
n2 = n2.*(n2<=nen)+(n2>nen);
n1 = getcol(icone(eles,:),n1);
n2 = getcol(icone(eles,:),n2);
dd = xnod(n2,:)-xnod(n1,:);
dl = sqrt(sum(dd'.^2))';
dd = dd./(dl*[1,1]);
nx = dd(:,2); ny = -dd(:,1);
nor = [nx,ny];

% Localizacion
%------------------------------------------------------------------
Ue    = ceros;
for inod=1:nen,
   for idf=1:ndf,
     nodes = icone(eles,inod);
     Ue(eles,(inod-1)*ndf+idf) = U(nodes,idf);
   end 
end

% Loop sobre los puntos de Gauss
for ipg=1:npg,

  Nb(1)  = 0.5*(1-xpg(ipg));
  Nb(2)  = 0.5*(1+xpg(ipg));
  xp(:,1) = xnod(n1,1)*Nb(1)+xnod(n2,1)*Nb(2);
  xp(:,2) = xnod(n1,2)*Nb(1)+xnod(n2,2)*Nb(2);
  [psi] = x2psi(xnod,icone(eles,:),xp);
  mask = (psi<tol);
  psi = psi.*(1-mask)+tol*mask;
  
  [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psi(:,1),psi(:,2));
  %  [dNdx,dNdy,N]  = shapefun(X2S,psi(:,1),psi(:,2),nen); 
  [dNdx,dNdy,N]  = shape2d(X2S,psi(:,1),psi(:,2),nen); 
  Jaco           = dl/2;
  xsj            = wpg(ipg)*Jaco;
  
  Upg    = zeros(length(eles),ndf);
  dUdxpg = zeros(length(eles),ndf);
  dUdypg = zeros(length(eles),ndf);

  for inod=1:nen,
      for idf=1:ndf,
          Upg(:,idf)    =    Upg(:,idf) +    N(:,inod).* Ue(eles,(inod-1)*ndf+idf);
          dUdxpg(:,idf) = dUdxpg(:,idf) + dNdx(:,inod).* Ue(eles,(inod-1)*ndf+idf);
          dUdypg(:,idf) = dUdypg(:,idf) + dNdy(:,inod).* Ue(eles,(inod-1)*ndf+idf);            
      end   
      Mele(eles,inod) = Mele(eles,inod) + N(:,inod).*xsj;      
  end

  if 0,
  for inod=1:nen,      
      for idf=1:ndf,
          dUnor = dUdxpg(:,idf).*nx+dUdypg(:,idf).*ny;
          Rele(eles,(inod-1)*ndf+idf) = Rele(eles,(inod-1)*ndf+idf) + N(:,inod).*dUnor.*xsj;
      end      
  end % inod
  else
      inod=1;
      for idf=1:ndf,
          dUnor = dUdxpg(:,idf).*nx+dUdypg(:,idf).*ny;
          Rele(eles,(inod-1)*ndf+idf) = Rele(eles,(inod-1)*ndf+idf) + dUnor.*xsj;
      end      
  end
  
 end % ipg

 gUn_ele = Rele;
 
return

 for inod=1:nen,
     nodes = icone(:,inod);
     for idf=1:ndf,
         Rg(nodes,idf) = Rg(nodes,idf) + Rele(:,(inod-1)*ndf+idf);      
     end
     Mg(nodes,1) = Mg(nodes,1) + Mele(:,inod);          
 end
 
 
 mask = (Mg(:,1)<tol);
 Mg(:,1)=Mg(:,1).*(1-mask)+mask;
 for idf=1:ndf,
     gUn_ele(:,idf) = Rg(:,idf)./Mg(:,1);
 end
