function [Kg,Rg] = advdif_2d(xnod,icone,ifaceb,fixa,forc,teco,lin_con,ther_contact,prop,...
    veloc,reac,source,phi,phiold,ijob)                       
%------------------------------------------------------------------
%
%     [Kg,Rg] = advdif_2d(xnod,icone,fixa,forc,teco,peri,prop,
%                      veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion adveccion difusion reaccion no estacionaria c/fuente
%        #.- elemento cuadrangular y triangular
%        #.- plano y axisimetrico
%
%        ijob = 1 ---> matriz tangente y residuo
%        ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

[numel,nen]   = size(icone);

[numnp,ndf]   = size(phi);
[numnp,ndm]   = size(xnod);

if ndf~=1, error(' NDF debe ser 1');end

ijob = 'get';
handle_prop_elem;

if 0
    [bas,ibas] = size(prop_elem);
    if ibas<nummat, error(' prop_elem array should have nummat columns '); end
    
    % global properties definition (default)
    rho      = prop(1) ;
    Cp       = prop(2) ;
    kappa    = prop(3) ;  
    npg      = prop(4) ;
    tau_fac  = prop(6) ;
    shocap   = prop(7) ;
    axisim   = prop(8) ;
    type_upw = prop(9) ;
    dtfic    = prop(10);
    lumped   = prop(11);
    tol      = prop(12);
    dt       = prop(13);
    alpha    = prop(14);
    dt       = alpha*dt;
end

T = alpha*phi(:,1)+(1-alpha)*phiold;

if ndm==1,
    pgauss1d;
elseif ndm==2, 
    pgauss2d; 
elseif ndm==3,
    pgauss3d; 
end

Told = phiold(:,1);

Rg   = 0*forc;
Kele =zeros(numel,nen,ndf,nen,ndf);
Rele =zeros(numel,nen,ndf);

if ~isempty(ther_contact), 
    numel_th = length(ther_contact(:,1));
    Kele_th =zeros(numel_th,4,ndf,4,ndf);
    Rele_th =zeros(numel_th,4,ndf);     
end  

if length(eles)>0,    
    advdif_2d_ele;
end


% ------ Terminos de contorno
eles = (1:numel)';  
ceros = zeros(length(eles),nen);
Te = ceros;   
vele = zeros(length(eles),nen,ndm);  
for inod=1:nen,
    nodes = icone(eles,inod);
    Te(:,inod) = T(nodes);
    vele(:,inod,1:ndm) = veloc(nodes,1:ndm);       
end

% Convective heat flux at boundary
%------------------------------------------------------------------
if ~isempty(teco), 
    [Kele,Rele] = bc_teco_advdif_2d(xnod,icone,Te,teco,Kele,Rele,ijob,axisim);
end

if weak_form,
    [Rele,Kele] = bcconv_advdif_2d(xnod,icone,vele,Te,ifaceb,Rele,Kele,axisim);
end

% Thermal contact 
%------------------------------------------------------------------
if ~isempty(ther_contact), 
    [Kele_th,Rele_th] = bc_thermal_contact(xnod,icone,T,ther_contact,Kele_th,Rele_th,ijob,axisim);
end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
if ~isempty(ther_contact), 
    icone_th = ther_contact(:,1:4);
    [Rg,Kg] = gather([Rele;Rele_th],[Kele;Kele_th],[icone;icone_th],nen,ndf);
else
    [Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);
end


if 0,
    Rg = Rg+forc;
end

% Linear restrictions
%------------------------------------------------------------------
if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end

% disp('fem001');keyboard;pause