% ====================================================================
%
%                       Poiseuille flow
%
% ====================================================================

ref = 1;

read_from_file = 0;
filestate_to_read = 'sqcav-Re_400-5_phi';

filestate = 'sqcav-Re_400-5-tmp';

nnwt = 3;

shocap                = 0;
weak_form             = 1; 
reverse_normal        = 0;

Cou   =  1;
npg   =  4;
ndf   =  4;
tol   = 1e-6;

L = 1;                     % Depth of the fluid
Nx = 21*ref;                     % Number of elements in r dir
Ny = Nx;
xratio = 1;               % Ratio between axis / side element size
yratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity    = -9.8*0;
Rey        =  400;
rho        =  1000.0;
visco      =  rho/Rey
axisim     = 0;
tau_factor = 1;

nphases    = 1;
g_dir      = 2;
vslip      = 0;
Sct_vf     = 1;
nu_vf      = 1e-4;
rho_g      = 1;

% BOUNDARY CONDITIONS
p_ref      = 0;
v_wall     = [0,0];
v_top      = [1,0];
vf_ref     = 1e-2;

% INITIAL VALUES
p_ini      = p_ref;
vel_ini    = v_wall;
vf_ini     = vf_ref;

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([1,5,1],Nx)*L;
y        = refh([1,5,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);
xmax = max(xnod);
xmin = min(xnod);

indx_v = (2:ndm+1);
indx_p = 1;
indx_vf = ndm+2;

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate_to_read '.mat']);
    U = phi;
else
    uini    = [p_ini,vel_ini,0*vf_ref];
    U       =  ones(numnp,1)*uini;   
    nodes_gas = find(xnod(:,2)>(xmin(2)+xmax(2))/2);
    U(nodes_gas,indx_vf) = 2*vf_ini;
end


%-------- fijaciones (FIXA)
nodes_bot   = (1:Nx)';
nodes_top   = Nx*(Ny-1)+(1:Nx)';
nodes_lef   = (1:Nx:Nx*Ny)';
nodes_rig   = (Nx:Nx:Nx*Ny)';
nodes_wall  = rmdblist([nodes_bot;nodes_lef;nodes_rig]);

fixa= zeros(numnp,ndf);

fixa(nodes_wall,indx_v) = ones(length(nodes_wall),ndm);
fixa(nodes_top,indx_v)  = ones(length(nodes_top),ndm);
fixa(1,indx_p)          = 1;

if 0
if 1
    fixa(nodes_bot,indx_vf) = 1;
else
    fixa(1,indx_vf) = 1;    
end
end

%fixa(:,indx_vf) = 1;
%fixa(:,indx_v) = 1;
%fixa(:,indx_p) = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc                     = zeros(numnp,ndf);
forc(1,indx_p)           = p_ref;

if 1
    forc(nodes_top,indx_v)   = ones(length(nodes_top),1)*v_top;
elseif 0
    % v_top(t) como una funcion del tiempo
    forc(nodes_top,indx_v)   = ones(length(nodes_top),1)*[0,0];
else
    fx = 0.5*(tanh((xnod(nodes_top,1)-0.25)/0.05)-tanh((xnod(nodes_top,1)-0.75)/0.05));
    forc(nodes_top,indx_v)   = fx*[1,0];
end

forc(nodes_wall,indx_v)  = ones(length(nodes_wall),1)*v_wall;

if 0
if 1
    forc(nodes_bot,indx_vf)  = 0;
else
    forc(1,indx_vf)  = vf_ref;
end
end

%forc(:,indx_vf) = vf_ref;
%forc(:,indx_v) = U(:,indx_v);
%forc(:,indx_p) = U(:,indx_p);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 100000;

hc_gas = [];