% ====================================================================
%
%                       pipe bubbly
%
%  two phase flow in a pipe
%
% ====================================================================

ref = 1;

vslip_variable = 0;

read_from_file = 1;
filestate_to_read = 'pipe_bubbly_phi_2';

filestate = '';

nnwt = 6;

shocap                = 0;
weak_form             = 1; 
reverse_normal        = 0;

Cou   =  1;
npg   =  4;
ndf   =  6;
tol   = 1e-6;

Lx = 5;                     % Depth of the fluid
Ly = 1;
Nx = 51*ref;                     % Number of elements in r dir
Ny = 11*ref;
xratio = 1;               % Ratio between axis / side element size
yratio = 1;                  % Ratio between axis / side element size

% BOUNDARY CONDITIONS
p_ref      = 0;
v_wall     = [0,0];
vf_ref     = 0.1;
v_inlet    = 1;

% INITIAL VALUES
p_ini      = p_ref;
vel_ini    = [v_inlet,0];
vf_ini     = vf_ref;

% PHYSICAL DATA
gravity    = -9.8*0;
Rey        =  100;
rho        =  10.0;
visco      =  rho*v_inlet*Ly/Rey
axisim     = 0;
tau_factor = 1;

nphases    = 1;
g_dir      = 2;
vslip      = 0.0;
Sct_vf     = 1;
visco_g    = 1e-4;
rho_g      = 1;
dp         = 1e-3;

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([1,2,1],Nx)*Lx;
y        = refh([1,1,1],Ny)*Ly;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);
xmax = max(xnod);
xmin = min(xnod);

get_index

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate_to_read '.mat']);
    U = phi;
else
    uini    = [p_ini,vel_ini,vf_ini,vel_ini];
    U       =  ones(numnp,1)*uini;
    if 0
        v1 = vf_ref*(1-0.5*(1+tanh((xnod(:,1)-0.25)/0.1)));    
        U(:,indx_vf) = v1;    
    end
end

%-------- fijaciones (FIXA)
nodes_bot   = (1:Nx)';
nodes_top   = Nx*(Ny-1)+(1:Nx)';
nodes_lef   = (1:Nx:Nx*Ny)';
nodes_rig   = (Nx:Nx:Nx*Ny)';

nodes_wall  = rmdblist([nodes_bot;nodes_top]);
nodes_in = nodes_lef;
nodes_out = nodes_rig;

fixa                    = zeros(numnp,ndf);
fixa(nodes_out,indx_p)  = 1;
fixa(nodes_out,indx_v(2))  = 1;
fixa(nodes_wall,indx_v) = ones(length(nodes_wall),ndm);
fixa(nodes_in,indx_v)   = 1;
fixa(nodes_in,indx_vf)  = 1;
fixa(nodes_wall,indx_v_d) = ones(length(nodes_wall),ndm);
fixa(nodes_in,indx_v_d) = 1;

%fixa(:,indx_v_d) = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc                       = zeros(numnp,ndf);
forc(nodes_out,indx_p)     = p_ref;
forc(nodes_in,indx_v(1))   = v_inlet;
forc(nodes_wall,indx_v)    = ones(length(nodes_wall),1)*v_wall;
forc(nodes_in,indx_vf)     = vf_ref;
forc(nodes_in,indx_v_d(1)) = v_inlet;
forc(nodes_wall,indx_v_d)  = ones(length(nodes_wall),1)*v_wall;

%forc(:,indx_v_d) = 0;

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 100000;

hc_gas = [];