% ====================================================================
%
%                      water faucet
%
% ====================================================================

get_index

ref = 1;

vslip_variable = 0;

read_from_file = 0;
filestate = 'water_faucet_0';

slip_wall_d = 1;

filestate = '';

nnwt = 3;

p_atm = 1.0133e5;

shocap                = 0;
weak_form             = 1; 
reverse_normal        = 0;
LES                   = 0;

Cou   =  1;
npg   =  4;
ndf   =  6;
tol   = 1e-6;

H = 12;                     % Depth of the fluid
R = 1;                   % Radius of the container
Nr = 2;                     % Number of elements in r dir
Nz = 51;
Nx = Nr;
Ny = Nz;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

x        = (0:Nr-1)'/(Nr-1)*R;
y        = refh([1,1,1],Nz)*H;

% genera malla
%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
   xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
icone    =[];
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
   icone=[icone; (k-1)*Nx+ico ];
end

% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

% PHYSICAL DATA
gravity    =  9.8;
vinlet     =  1;
rho        =  1000;
visco      =  0.8e-3;          % liquid phase viscosity
axisim     = 0;
tau_factor = 1;

nphases    = 1;
g_dir      = 2;
vslip      = 0.0;
Sct_vf     = 1;
visco_g  =  1.000e-5;        % gas phase viscosity
rho_g      = 1;
dp         = 1e-3;

% INITIAL VALUES
p_ini       = p_atm;
alpha_g_in  = 0.2;
alpha_g_ini = 0.2;
vel_l_ini   = [0,10];
vel_l_in    = [0,10]
vel_g_ini   = [0,0];
vel_g_in    = [0,0];

xmin = min(xnod);
xmax = max(xnod);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate_to_read '.mat']);
    U = phi;
elseif 0
    uini    = [p_ini,alpha_g_ini,vel_l_ini,vel_g_ini];
    U       =  ones(numnp,1)*uini;          
    Ampl   = 0.1;
    sigma  = 0.2;
    sigma  = 0.1;      
    xc     = H/2;
    z      = xnod(:,2)-xc;
    U(:,indx_gas(1)) = alpha_g_in+alpha_g_ini*exp(-z.^2/sigma);  
    U(:,indx_liq(1)) = p_ini+rho_l*abs(gravity)*(H-xnod(:,2));       
else
    uini    = [p_ini,vel_l_ini,alpha_g_ini,vel_g_ini];
    U       =  ones(numnp,1)*uini;              
    rho_l = rho;    
    rho_m     = compute_rho_m(U(:,indx_vf),rho_l,rho_g);        
end

% BOUNDARY CONDITIONS

%-------- fijaciones (FIXA)
nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_wall= find(abs(xnod(:,1)-xmax(1))<tol);
nodes_axis= find(abs(xnod(:,1)-xmin(1))<tol);

%-------- fijaciones (FIXA)
fixa = zeros(numnp,ndf);
forc = zeros(numnp,ndf);
    
% fijacion a la salida
fixa([nodes_top],indx_p)=1;
%fixa([nodes_top],indx_vf)=1;

forc([nodes_top],indx_p) = p_ini;
%forc([nodes_top],indx_vf)   = alpha_g_in;

% velocidad lateral nula en todos los nodos
fixa(:,indx_v(1))=1;
fixa(:,indx_v_d(1))=1;

% a la entrada
fixa(nodes_bot,indx_v(2))   = 1;
fixa(nodes_bot,indx_vf)     = 1;
fixa(nodes_bot,indx_v_d(2)) = 1;
forc(nodes_bot,indx_v(2))   = vel_l_in(2);
forc(nodes_bot,indx_vf)     = alpha_g_in;
forc(nodes_bot,indx_v_d(2)) = vel_g_in(2);

if 0
fixa(:,indx_p) = 1;
forc(:,indx_p) = p_ini;

fixa(:,indx_vf) = 1;
forc(:,indx_vf) = U(:,indx_vf);

fixa(:,indx_v_d) = 1;
forc(:,indx_v_d) = U(:,indx_v_d);
end

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];

for icompo = 1:ndf
   lin_con = [lin_con ; ... 
         [ones(length(nodes_axis),1) nodes_axis icompo*ones(length(nodes_axis),1) ...
            -ones(length(nodes_wall),1) nodes_wall icompo*ones(length(nodes_wall),1)]];      
end


% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 100000;

hc_gas = [];