%
%  Incompressible Navier-Stokes equations
%

type_upwind         = 1;
tau_centroide       = 0;
debug               = 0;
indx_v              = 1+(1:ndm);
indx_p              = 1;
indx_vf             = ndm+1+1;

visco = prop(2);

%------------------------------------------------------------------
ne  = length(eles);

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);

for inod=1:nen,
    nodes        = icone(eles,inod);
    jnod         = (inod-1)*ndf+(1:ndf);
    U1e(:,jnod)  = U(nodes,1:ndf);		 
    Ue(:,jnod)   = alpha*U(nodes,1:ndf)+(1-alpha)*Uold(nodes,1:ndf); 
    U0e(:,jnod)  = Uold(nodes,1:ndf);   
    re(:,inod)   = xnod(nodes,1);		        
end

if tau_centroide==1,
    % UPWIND EN EL CENTROIDE 
    %------------------------------------------------------------------
    if nen==4,
        [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),0,0);
        [dshapex,N] = shape2d(X2S,0,0,nen); 
    elseif nen==3,
        [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),1/3,1/3);
        [dshapex,N] = shape2d(X2S,1/3,1/3,nen); 
    else
        error(' Bad type of element , nen ~=3, nen ~=4 ')
    end
    
    Upg          = zeros(ne,ndf);
    dUpg         = zeros(ne,ndf,ndm);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N(inod).*  U0e(:,jnod);
        for idm=1:ndm
            dUpg(:,(1:ndf),idm) = dUpg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);          
        end
    end    
    
    velpg   = Upg(:,indx_v);
    hele    = comp_hele(velpg,dshapex,Jaco,nen);
    [tau,delta,bas] = supg_nsi_tet(velpg,dUpg,hele,prop,type_upwind);
    velpg_vf = velpg;
    velpg_vf(:,g_dir) = velpg_vf(:,g_dir) + vslip;        
    hele_vf  = comp_hele(velpg_vf,dshapex,Jaco,nen);        
    [tau_vf,delta_vf,bas] = supg_nsi_tet(velpg_vf,dUpg,hele_vf,prop,type_upwind);        
    
    compute_P_supg;           
end

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
    
    %tfs_0=cputime;
    
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 

    %tfs_1=cputime;
    %disp([' dt assemble - nsi_fs_ele (0) = ' num2str(tfs_1-tfs_0)])   
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    
    Upg     = zeros(ne,ndf);
    U0pg    = Upg;
    U1pg    = Upg;    
    dUpg    = zeros(ne,ndf,ndm);
    dU0pg   = dUpg;
    dU1pg   = dUpg;    
    radpg   = zeros(ne,1);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
        U1pg(:,(1:ndf)) = U1pg(:,(1:ndf)) + N(inod).*  U1e(:,jnod);        
        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
        for idm=1:ndm            
            dUpg(:,(1:ndf),idm)   = dUpg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* Ue(:,jnod);            
            dU0pg(:,(1:ndf),idm)   = dU0pg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);                        
            dU1pg(:,(1:ndf),idm)   = dU1pg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* U1e(:,jnod);                                    
        end        
    end
    
    %tfs_2=cputime;
    %disp([' dt assemble - nsi_fs_ele (1) = ' num2str(tfs_2-tfs_1)])   
    
    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
    
    if tau_centroide==0,        
        velpg   = Upg(:,indx_v);
        hele    = comp_hele(velpg,dshapex,Jaco,nen);
        [tau,delta,bas] = supg_nsi_tet(velpg,dUpg,hele,prop,type_upwind);
        velpg_vf = velpg;
        velpg_vf(:,g_dir) = velpg_vf(:,g_dir) + vslip;        
        hele_vf  = comp_hele(velpg_vf,dshapex,Jaco,nen);        
        [tau_vf,delta_vf,bas] = supg_nsi_tet(velpg_vf,dUpg,hele_vf,prop,type_upwind);        
    end

    % disp(['  nsi_fs_ele  ' ]);keyboard;pause    
    % mixture density computation
    rho_l = rho;
    vf_ctf = U0pg(:,indx_vf);
    vf_ctf = min([vf_ctf';ones(1,length(vf_ctf))])';
    vf_ctf = max([vf_ctf';1e-6*ones(1,length(vf_ctf))])';    
    rho_m_old = compute_rho_m(vf_ctf,rho_l,rho_g);
    vf_ctf = Upg(:,indx_vf);    
    vf_ctf = min([vf_ctf';ones(1,length(vf_ctf))])';
    vf_ctf = max([vf_ctf';1e-6*ones(1,length(vf_ctf))])';    
    rho_m     = compute_rho_m(vf_ctf,rho_l,rho_g);

    if fs_step==1
        % predictor matrix and residue and poisson matrix and void fraction
        % matrix
        KyR_1y4;
    elseif fs_step==2,
        % poisson residue 
        R_2;
    elseif fs_step==3,
        % projection residue 
        R_3;
    elseif fs_step==0,        
        % projection matrix
        K_0;
    elseif fs_step==4,        
        % phase k mass conservation (ASM) 
        R_4;
    else
        error(' fs_step not defined ')
    end

    %tfs_3=cputime;
    %disp([' dt assemble - nsi_fs_ele (2) = ' num2str(tfs_3-tfs_2)])   
    
end % ipg

