function [Fd,Kxx,Kxy,Kyx,Kyy] = Dflux(Up,dUp,prop);
%--------------------------------------------------------------------
%
%         [Fd,Kxx,Kxy,Kyx,Kyy] = Dflux(Up,dUp,prop);
%
%  Diffusive fluxes and jacobians for the gas phase in the two phase flow eqs 
%
%--------------------------------------------------------------------

ijob = 'get';
handle_prop_elem;

[N,ndf]  = size(Up);
ndm = (ndf-2)/2;

get_index;
alpha_g = Up(:,indx_g_vf);
alpha_l     = 1-alpha_g;

Fd     = zeros(N,ndf,2);
Kxx    = zeros(N,ndf,ndf);

if ndm==2,
    Kxy    = zeros(N,ndf,ndf);
    Kyx    = zeros(N,ndf,ndf);
    Kyy    = zeros(N,ndf,ndf);
end


grad_u      = reshape(dUp(:,indx_l_vel(1),1:2),N,2);
grad_v      = reshape(dUp(:,indx_l_vel(2),1:2),N,2);
defo(:,1,1) = grad_u(:,1)+grad_u(:,1);
defo(:,1,2) = grad_u(:,2)+grad_v(:,1);
defo(:,2,1) = grad_v(:,1)+grad_u(:,2);
defo(:,2,2) = grad_v(:,2)+grad_v(:,2);

defo_l      = defo/2;
tau_l       = 2*defo_l;

grad_u      = reshape(dUp(:,indx_g_vel(1),1:2),N,2);
grad_v      = reshape(dUp(:,indx_g_vel(2),1:2),N,2);
defo(:,1,1) = grad_u(:,1)+grad_u(:,1);
defo(:,1,2) = grad_u(:,2)+grad_v(:,1);
defo(:,2,1) = grad_v(:,1)+grad_u(:,2);
defo(:,2,2) = grad_v(:,2)+grad_v(:,2);

defo_g      = defo/2;
tau_g       = 2*defo_g;

if 0
    if Sato_model_coef>0,
        v_rel = vel_l-vel_g;
        v_slip = sqrt(sum(v_rel'.^2)');
        mu_Sato = Sato_model_coef*d_bubble*rho_l*alpha_g.*v_slip;
        visco_l = visco_l + mu_Sato;
        visco_g = visco_l*rho_g/rho_l;
    end
end

model_visco_std = 1;

if model_visco_std==1,
    visco_mix_l = alpha_l*visco_l;   
    visco_mix_g = alpha_g*visco_g;       
    %visco_mix   = visco_l;   
elseif model_visco_std==2,
    alpha_avg   = 0.5*(max(alpha_l)+min(alpha_l));
    visco_mix_l = alpha_avg*visco_l;
    alpha_avg   = 0.5*(max(alpha_g)+min(alpha_g));
    visco_mix_g = alpha_avg*visco_g;    
elseif model_visco_std==0,
    visco_mix_l   = visco_l;    
    visco_mix_g   = visco_g;        
else
    error(' Bad model_visco_std choice')
end

for k=1:ndm,
    for m=1:ndm,
        Fd(:,indx_l_vel(k),m) = visco_mix_l.*tau_l(:,k,m);       
        Fd(:,indx_g_vel(k),m) = visco_mix_g.*tau_g(:,k,m);             
    end
end

vaux_l      = visco_mix_l;
vaux_g      = visco_mix_g;
delta       = eye(2);
uno_1       = [1;0];
uno_2       = [0;1];

tmp_xx = (delta(1,1)*eye(2)+uno_1*uno_1');
tmp_xy = (delta(1,2)*eye(2)+uno_2*uno_1');
tmp_yx = (delta(2,1)*eye(2)+uno_1*uno_2');
tmp_yy = (delta(2,2)*eye(2)+uno_2*uno_2');

for j=1:ndm
    for k=1:ndm,
        % LIQ       
        Kxx(:,indx_l_vel(j),indx_l_vel(k))  = vaux_l*tmp_xx(j,k);
        Kxy(:,indx_l_vel(j),indx_l_vel(k))  = vaux_l*tmp_xy(j,k);
        Kyx(:,indx_l_vel(j),indx_l_vel(k))  = vaux_l*tmp_yx(j,k);
        Kyy(:,indx_l_vel(j),indx_l_vel(k))  = vaux_l*tmp_yy(j,k);      
        % GAS
        Kxx(:,indx_g_vel(j),indx_g_vel(k))  = vaux_g*tmp_xx(j,k);
        Kxy(:,indx_g_vel(j),indx_g_vel(k))  = vaux_g*tmp_xy(j,k);
        Kyx(:,indx_g_vel(j),indx_g_vel(k))  = vaux_g*tmp_yx(j,k);
        Kyy(:,indx_g_vel(j),indx_g_vel(k))  = vaux_g*tmp_yy(j,k);            
    end
end   
