% ====================================================================
%
%                       Two phase Poiseuille flow
%
% ====================================================================

read_from_file = 0;
filestate = 'last_state';

nnwt = 3;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  4;
ndf   =  6;
tol   = 1e-6;


L = 20;                     % Depth of the fluid
R = 0.5;                   % Radius of the container
Nr = 21;                     % Number of elements in r dir
Nz = 41;
Nx = Nr;
Ny = Nz;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8;
visco_l  =  0.8e-3;          % liquid phase viscosity
visco_l  =  0.01;
visco_g  =  1.000e-5;        % gas phase viscosity
rho_l    =  1000.0;
rho_l    =  10;
rho_g    =  1.0;

Temp    = 300;
gamma_g = 1.4;
gamma_l = 2.8;   
Cp_g    = 1000;
Cp_l    = 4000;       
p_infty = 8.5e8;

d_bubble              = 0.002;
comp_interphase_terms = 1;
axisim                = 0;
drag_factor           = 1;

% sigma : coeficiente para presion interfacial
coef_sigma                 = 1;

% BOUNDARY CONDITIONS
p_ref      = 1e5;
p_bot      = p_ref+rho_l*abs(gravity)*L;
alpha_g_in = 0.1;
vel_g_in   = [0,1];
vel_l_wall = [0,0];
vel_g_wall = [0,0];
vel_l_in   = [0,1];

% INITIAL VALUES
p_ini       = p_ref;
alpha_g_ini = alpha_g_in;
vel_l_ini   = vel_l_in;
vel_g_ini   = vel_g_in;

% ESCALEO
if 0,
   visco_l=visco_l/sqrt(rho_l);
   visco_g=visco_g/sqrt(rho_l);
   rho_g=rho_g/sqrt(rho_l);
   p_bot=p_bot/sqrt(rho_l);    
   p_ini=p_ini/sqrt(rho_l);    
   p_ref=p_ref/sqrt(rho_l);    
   rho_l=rho_l/sqrt(rho_l);
end

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([2,2,1],Nx)*R;
y        = refh([1,1,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

get_index;

%-------- Solucion inicial (PHI)
if read_from_file==1,
   eval(['load ' filestate]);
   U = phi;
else
   uini    = [alpha_g_ini,vel_g_ini,vel_l_ini,p_ini];
   U       =  ones(numnp,1)*uini;   
   U(:,indx_l_p) = (p_bot-rho_l*abs(gravity)*xnod(:,2));
end

%-------- fijaciones (FIXA)
nodes_bot   = (1:Nx)';
nodes_top   = Nx*(Ny-1)+(1:Nx)';
nodes_lef   = (1:Nx:Nx*Ny)';
nodes_rig   = (Nx:Nx:Nx*Ny)';

fixa= zeros(numnp,ndf);

fixa(nodes_bot,indx_l_vel)    = ones(length(nodes_bot),2);
fixa(nodes_top,indx_l_p)      = ones(length(nodes_top),1);
fixa(nodes_top,indx_l_vel(1)) = ones(length(nodes_top),1);
fixa(nodes_lef,indx_l_vel(1)) = ones(length(nodes_lef),1);
fixa(nodes_rig,indx_l_vel)    = ones(length(nodes_rig),2);

fixa(nodes_bot,indx_g_vf) = ones(length(nodes_bot),1);
fixa(nodes_bot,indx_g_vel) = ones(length(nodes_bot),2);
fixa(nodes_lef,indx_g_vel(1)) = ones(length(nodes_lef),1);
fixa(nodes_rig,indx_g_vel) = ones(length(nodes_rig),2);

%-------- Cargas/Desplazamientos nodales(FORC)
forc= zeros(numnp,ndf);
forc(nodes_top,indx_l_p)  = p_ref;
forc(nodes_bot,indx_l_vel)  = ones(length(nodes_bot),1)*vel_l_in;
forc(nodes_bot,indx_g_vf)  = alpha_g_in;
forc(nodes_bot,indx_g_vel)  = ones(length(nodes_bot),1)*vel_g_in;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;