%
%                     analisis del modelo de Garcia Cascales & Corberan
%
%  calculo autovalores SIMBOLICO
%
%  variables conservativas ---> W = [alpha_g*rho_g, alpha_l*rho_l,
%                                    alpha_g*rho_g*ug , alpha_l*rho_l*ul ]
%  variables primitivas    ---> V = [alpha_g, ug, ul, p]
%

syms rho_g ug alpha_g ul rho_l p gamma_g gamma_l T Cp_g Cp_l cc_g cc_l p_infty Lambda sigma p_i p_i_0

alpha_l = 1-alpha_g;

% ecuaciones de estado
%gamma_g = 1.4;
%gamma_l = 2.8;
%Cp_g = 1008;
%Cp_l = 4186;
%T = 300;
h_g = Cp_g*T;
h_l = Cp_l*T;
%p_infty = 8.5e8;
rho_g = gamma_g*p/cc_g;
rho_l = gamma_l*(p_infty-p)/cc_l;
%rho_l = gamma_l*p/cc_l;

arho_l = rho_l*alpha_l;
arho_g = rho_g*alpha_g;

if 0
    drho_g_dp = diff(rho_g,p);
    drho_l_dp = diff(rho_l,p);
elseif 0
    drho_g_dp =  gamma_g/((gamma_g-1)*h_g);
    drho_l_dp = -gamma_l/((gamma_l-1)*h_l);    
else
    drho_g_dp =  gamma_g/cc_g;
    drho_l_dp = -gamma_l/cc_l; 
end

% matriz temporal A
A = [rho_g,0,0,alpha_g*drho_g_dp;
    -rho_l,0,0,alpha_l*drho_l_dp;
    rho_g*ug,arho_g, 0 , alpha_g*ug*drho_g_dp;
    -rho_l*ul, 0 , arho_l , alpha_l*ul*drho_l_dp ];

% su inversa
Ai = inv(A);

% jacobiano advectivo en base conservativa
dalpha_g_dW1 = Ai(1,1);
dalpha_g_dW2 = Ai(1,2);
dalpha_g_dW3 = Ai(1,3);
dalpha_g_dW4 = Ai(1,4);
dp_dW1 = Ai(4,1);
dp_dW2 = Ai(4,2);
dp_dW3 = Ai(4,3);
dp_dW4 = Ai(4,4);

J_W = [ 0 , 0 , 1 , 0 ;
    0 , 0 , 0 , 1 ;
    -ug^2+p*dalpha_g_dW1+alpha_g*dp_dW1 , p*dalpha_g_dW2+alpha_g*dp_dW2 , 2*ug+p*dalpha_g_dW3+alpha_g*dp_dW3 , p*dalpha_g_dW4+alpha_g*dp_dW4 ;
    -p*dalpha_g_dW1+alpha_l*dp_dW1 , -ul^2-p*dalpha_g_dW2+alpha_l*dp_dW2, -p*dalpha_g_dW3+alpha_l*dp_dW3 , 2*ul-p*dalpha_g_dW4+alpha_l*dp_dW4;  
];

detJJ = simple(det(J_W-Lambda*eye(4)));

detJ = collect(detJJ,Lambda);

if check    
    c4 = 1;
    c3 = -2*ug-2*ul;
    c2 = ug^2 + ul^2 + 4*ug*ul - cc_g/gamma_g + p/(p-alpha_g*p_infty)*cc_l/gamma_l;    
    c1 =  (cc_g/gamma_g - ug^2)*2*ul - 2*ug*ul^2 - 2*ug* p / (p-alpha_g*p_infty)*cc_l/gamma_l;
    c0 = ( ug^2-cc_g/gamma_g )* ( ul^2 + p / (p-alpha_g*p_infty)*cc_l/gamma_l );
    simple(detJ-(c4*Lambda^4+c3*Lambda^3+c2*Lambda^2+c1*Lambda+c0))
end

[V,D] = eig(J_W);

return

% termino fuente
% parte que tiene en cuenta la diferencia de presion entre la presion prop dicha y la presion interfacial

%p_i = sigma*alpha_g*alpha_l*rho_g*rho_l/(alpha_g*rho_l+alpha_l*rho_g)*(ug-ul)^2;
p_i = p_i_0*(ug-ul)^2;

JS_V = [0,0,0,0; 0,0,0,0; p_i , 0,0,0; -p_i,0,0,0];

%la transformo a la base conservativa
JS_W = JS_V*Ai;

% calculo los autovalores de J_W - JS_W
detJJ = simple(det((J_W-JS_W)-Lambda*eye(4)));
detJS = collect(detJJ,Lambda);

return

