%
%                     analisis del modelo de Garcia Cascales & Corberan
%
%  calculo autovalores NUMERICO
%
%  variables conservativas ---> W = [alpha_g*rho_g, alpha_l*rho_l,
%                                    alpha_g*rho_g*ug , alpha_l*rho_l*ul ]
%  variables primitivas    ---> V = [alpha_g, ug, ul, p]
%

sigma = 3;

% ecuaciones de estado
gamma_g = 1.4;
gamma_l = 2.8;
Cp_g = 1008;
Cp_l = 4186;
T = 300;
h_g = Cp_g*T;
h_l = Cp_l*T;
p_infty = 8.5e8;
cc_g = ((gamma_g-1)*Cp_g*T);
cc_l = ((gamma_l-1)*Cp_l*T);


alpha_g_v = (0.05:0.05:0.99);
%alpha_g_v = 0.1;
ug_v = exp(-5:0.25:5);
%ug_v = [1.1,2,5];
ul_v = exp(-5:0.25:5);
%ul_v = 1;
p_v  = 10.^(4:0.5:6);
p_v  = 1e5;
ul = 1;
ug = 1;
p = 1e5;
figure(1);clf;hold on
bfh = [];bfh2 = [];

for alpha_g=alpha_g_v
    for ug = ug_v;
        for p=p_v
            
            rho_g = gamma_g*p/cc_g;
            rho_l = gamma_l*(p_infty-p)/cc_l;
            
            alpha_l = 1-alpha_g;
            arho_l = rho_l*alpha_l;
            arho_g = rho_g*alpha_g;
            
            drho_g_dp =  gamma_g/cc_g;
            drho_l_dp = -gamma_l/cc_l; 
            
            % matriz temporal A
            A = [rho_g,0,0,alpha_g*drho_g_dp;
                -rho_l,0,0,alpha_l*drho_l_dp;
                rho_g*ug,arho_g, 0 , alpha_g*ug*drho_g_dp;
                -rho_l*ul, 0 , arho_l , alpha_l*ul*drho_l_dp ];
            
            % su inversa
            Ai = inv(A);
            
            % jacobiano advectivo en base conservativa
            dalpha_g_dW1 = Ai(1,1);
            dalpha_g_dW2 = Ai(1,2);
            dalpha_g_dW3 = Ai(1,3);
            dalpha_g_dW4 = Ai(1,4);
            dp_dW1 = Ai(4,1);
            dp_dW2 = Ai(4,2);
            dp_dW3 = Ai(4,3);
            dp_dW4 = Ai(4,4);
            
            J_W = [ 0 , 0 , 1 , 0 ;
                0 , 0 , 0 , 1 ;
                -ug^2+p*dalpha_g_dW1+alpha_g*dp_dW1 , p*dalpha_g_dW2+alpha_g*dp_dW2 , 2*ug+p*dalpha_g_dW3+alpha_g*dp_dW3 , p*dalpha_g_dW4+alpha_g*dp_dW4 ;
                -p*dalpha_g_dW1+alpha_l*dp_dW1 , -ul^2-p*dalpha_g_dW2+alpha_l*dp_dW2, -p*dalpha_g_dW3+alpha_l*dp_dW3 , 2*ul-p*dalpha_g_dW4+alpha_l*dp_dW4;  
            ];
            
            [V,D]=eig(J_W);
            
            if isreal(D)==0, error( ' Complex Eigenvalues in J_W ' ); end
            Lambda(1,1) = ul + sqrt(cc_l/gamma_l * p/(alpha_g*p_infty - p ));
            Lambda(2,1) = ul - sqrt(cc_l/gamma_l * p/(alpha_g*p_infty - p ));    
            Lambda(3,1) = ug + sqrt(cc_g/gamma_g);
            Lambda(4,1) = ug - sqrt(cc_g/gamma_g);    
            
            p_i = sigma*alpha_g*alpha_l*rho_g*rho_l/(alpha_g*rho_l+alpha_l*rho_g)*(ug-ul)^2;
            
            JS_V = [0,0,0,0; 0,0,0,0; p_i , 0,0,0; -p_i,0,0,0];
            
            %la transformo a la base conservativa
            JS_W = JS_V*Ai;
            
            [V2,D2]=eig(J_W-JS_W);
            
            if 0
                plot(sort(diag(D)),'bo');hold on;plot(sort(Lambda),'r*')
                pause
            else
%                bfh = [bfh, abs(sort(diag(D))-sort(Lambda))];
                bfh = [bfh, sort(diag(D))];                
%                bfh2 = [bfh2, [sort(diag(D)) , sort(diag(D2))]];
%                bfh2 = [bfh2, [(sort(diag(D))-sort(diag(D2)))]];  
                bfh2 = [bfh2, [(max(abs(diag(D)))-max(abs(diag(D2))))]];  
            end
            
            if isreal(D2)==0, 
                disp( ' Complex Eigenvalues in J_W - JS_W ' );
                keyboard;pause
            end
            
            
        end
    end
end


return

detJJ = simple(det(J_W-Lambda*eye(4)));

detJ = collect(detJJ,Lambda);

if check    
    c4 = 1;
    c3 = -2*ug-2*ul;
    c2 = ug^2 + ul^2 + 4*ug*ul - cc_g/gamma_g + p/(p-alpha_g*p_infty)*cc_l/gamma_l;    
    c1 =  (cc_g/gamma_g - ug^2)*2*ul - 2*ug*ul^2 - 2*ug* p / (p-alpha_g*p_infty)*cc_l/gamma_l;
    c0 = ( ug^2-cc_g/gamma_g )* ( ul^2 + p / (p-alpha_g*p_infty)*cc_l/gamma_l );
    simple(detJ-(c4*Lambda^4+c3*Lambda^3+c2*Lambda^2+c1*Lambda+c0))
end

% termino fuente
% parte que tiene en cuenta la diferencia de presion entre la presion prop dicha y la presion interfacial

p_i = sigma*alpha_g*alpha_l*rho_g*rho_l/(alpha_g*rho_l+alpha_l*rho_g)*(ug-ul)^2;

JS_V = [0,0,0,0; 0,0,0,0; p_i , 0,0,0; -p_i,0,0,0];

%la transformo a la base conservativa
JS_W = JS_V*Ai;

% calculo los autovalores de J_W - JS_W
detJJ = simple(det((J_W-JS_W)-Lambda*eye(4)));
detJS = collect(detJJ,Lambda);

return

