function [H,Cp,Cpi] = enthalpy_function(Up,prop,rho_g,rho_l,drho_g_dp,drho_l_dp)
%
%      [H,Cp,Cpi] = enthalpy_function(Up,prop,rho_g,rho_l,drho_g_dp,drho_l_dp)
%
%       Computes enthalpy function to handle primitive variables in 2D 
%

ijob = 'get';
handle_prop_elem;

[N,ndf]  = size(Up);
ndm = (ndf-2)/2;

get_index;

alpha_g = Up(:,indx_g_vf);
alpha_l = 1-alpha_g;

p = Up(:,indx_l_p);
vel_g = Up(:,indx_g_vel);
vel_l = Up(:,indx_l_vel);

Cp                            =  zeros(N,ndf,ndf);
Cp(:,indx_g_con,indx_g_vf)    =  rho_g;
Cp(:,indx_g_con,indx_l_p)     =  alpha_g.*drho_g_dp;

Cp(:,indx_l_con,indx_g_vf)    = -rho_l;
Cp(:,indx_l_con,indx_l_p)     =  (1-alpha_g).*drho_l_dp;

for idm=1:ndm    
    Cp(:,indx_g_mom(idm),indx_g_vf)    =  rho_g.*vel_g(:,idm);
    Cp(:,indx_l_mom(idm),indx_g_vf)    = -rho_l.*vel_l(:,idm);
    
    Cp(:,indx_g_mom(idm),indx_g_vel(idm))   =  alpha_g.*rho_g;
    Cp(:,indx_l_mom(idm),indx_l_vel(idm))   =  (1-alpha_g).*rho_l;    
    Cp(:,indx_g_mom(idm),indx_l_p)          =  alpha_g.*vel_g(:,idm).*drho_g_dp;
    Cp(:,indx_l_mom(idm),indx_l_p)          =  (1-alpha_g).*vel_l(:,idm).*drho_l_dp;    
end

H(:,indx_g_con)   = rho_g.*alpha_g;
H(:,indx_l_con)   = rho_l.*(1-alpha_g);

for idm=1:ndm
    H(:,indx_g_mom(idm))   = rho_g.*    alpha_g.*vel_g(:,idm);
    H(:,indx_l_mom(idm))   = rho_l.*(1-alpha_g).*vel_l(:,idm);
end

ag2 = (gamma_g-1)*Cp_g*Temp;
al2 = (gamma_l-1)*Cp_l*Temp;
Den = p-alpha_g*p_infty;
Cpi = zeros(N,ndf,ndf);

if 1
    for kk=1:N,
        Cpi(kk,:,:) = inv(reshape(Cp(kk,:,:),ndf,ndf));
    end
else    
    Cpi(:,indx_g_vf,indx_g_con)    =  ag2.*alpha_l./(gamma_g*Den);
    Cpi(:,indx_g_vf,indx_l_con)    =  al2.*alpha_g./(gamma_l*Den);
    Cpi(:,indx_g_vel(1),indx_g_con)    = -ag2.*vel_g(:,1)./(gamma_g*alpha_g.*p);
    Cpi(:,indx_g_vel(1),indx_g_mom(1))    =  ag2./(gamma_g*alpha_g.*p);
    Cpi(:,indx_g_vel(2),indx_g_con) = -ag2.*vel_g(:,2)./(gamma_g*alpha_g.*p);
    Cpi(:,indx_g_vel(2),indx_g_mom(2)) =  ag2./(gamma_g*alpha_g.*p);
    Cpi(:,indx_l_vel(1),indx_l_con) =  al2.*vel_l(:,1)./(gamma_l*alpha_l.*(p-p_infty));
    Cpi(:,indx_l_vel(1),indx_l_mom(1)) =  al2./(gamma_l*alpha_l.*(p_infty-p));
    Cpi(:,indx_l_vel(2),indx_l_con) =  al2.*vel_l(:,2)./(gamma_l*alpha_l.*(p-p_infty));
    Cpi(:,indx_l_vel(2),indx_l_vel(2)) =  al2./(gamma_l*alpha_l.*(p_infty-p));
    Cpi(:,indx_l_p,indx_g_con) =  ag2.*(p-p_infty)./(gamma_g*Den);
    Cpi(:,indx_l_p,indx_l_con) = -al2.*p./(gamma_l*Den);
end