% ====================================================================
%
%                       RIEMANN Test (high relative velocity)
%
% ====================================================================

read_from_file = 0;
filestate = 'last_state';

nnwt =2;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  2;
ndf   =  6;
tol   = 1e-6;

L = 10;                     % Depth of the fluid
Nx = 401;
Nx = 101;
Ny = 2;
xratio = 1;               % Ratio between axis / side element size
yratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8*0;
visco_l  =  1e-10;
visco_g  =  1e-10;        % gas phase viscosity

p_ref      = 1e5;

Temp    = 308.15;
gamma_g = 1.4;
gamma_l = 2.8;   
Cp_g    = 1000;
Cp_l    = 4000;       
p_infty = 8.5e8;
ag2 = (gamma_g-1)*Cp_g*Temp;
al2 = (gamma_l-1)*Cp_l*Temp;

rho_l    =  gamma_l*(p_infty-p_ref)/al2;
rho_g    =  gamma_g*p_ref/ag2;

rho_l_user = rho_l;
rho_g_user = rho_g;

tau_factor           = 1;

d_bubble              = 0.002;
comp_interphase_terms = 1;
axisim                = 0;

% sigma : coeficiente para presion interfacial
coef_sigma    = 3;
coef_Cd       = 0.0;

stabi_with_nonconservative_terms = 0;
interface_pressure_analytical = 0;
model_get_density = 0;

% BOUNDARY CONDITIONS
p_L        = 2.65*p_ref;
p_R        = 2.65*p_ref;
alpha_g_L  = 0.29;
alpha_g_R  = 0.30;
vel_g_L    = [65,0];
vel_l_L    = [ 1,0];
vel_g_R    = [50,0];
vel_l_R    = [ 1,0];

state_L = [alpha_g_L,p_L,vel_g_L,vel_l_L];
state_R = [alpha_g_R,p_R,vel_g_R,vel_l_R];

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([xratio,1,xratio],Nx)*L;
y        = refh([1,1,1],Ny)*L/Nx;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

get_index;

xmin = min(xnod);
xmax = max(xnod);
x_half = 0.5*(xmin+xmax);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate]);
    U = phi;
else
    uini    = ones(numnp,1)*state_L;
    nn = find(xnod(:,1)>x_half(1));
    uini(nn,:) = ones(length(nn),1)*state_R;
    U = uini;    
%    U=0.5*(1+tanh((xnod(:,1)-x_half(1))/(0.01*L)))*(state_R-state_L)+ones(numnp,1)*state_L;    
    U=0.5*(1+tanh((xnod(:,1)-x_half(1))/(0.005*L)))*(state_R-state_L)+ones(numnp,1)*state_L;        
end

%-------- fijaciones (FIXA)
nodes_bot   = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top   = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_lef   = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig   = find(abs(xnod(:,1)-xmax(1))<tol);

fixa= zeros(numnp,ndf);
forc= zeros(numnp,ndf);

fixa(nodes_lef,:)   = ones(length(nodes_lef),ndf);
fixa(nodes_rig,:)   = ones(length(nodes_rig),ndf);

forc(nodes_lef,:)   = ones(length(nodes_lef),1)*state_L;
forc(nodes_rig,:)   = ones(length(nodes_rig),1)*state_R;

fixa(:,indx_g_vel(2)) = 1;
fixa(:,indx_l_vel(2)) = 1;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 1
    for icompo=1:ndf
        if 0
            lin_con = [lin_con ; ... 
                    [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                        -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];    
        end
        lin_con = [lin_con ; ...
                [ones(length(nodes_bot),1) nodes_bot icompo*ones(length(nodes_bot),1) ...
                    -ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1)]];                        
    end
end


% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;