% ====================================================================
%
%                       square cavity for bubbly flow
%
% ====================================================================

ref = 1;

read_from_file = 0;
filestate_to_read = 'sqcav-Re_400-single_phi';

filestate = 'sqcav';

nnwt =3;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  4;
ndf   =  6;
tol   = 1e-6;

L = 1;                     % Depth of the fluid
Nx = 20;
Ny = 20;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8*0;
visco_l  =  1e+3;
visco_g  =  1e-1;        % gas phase viscosity

p_ref      = 1e5;

Temp    = 273+35;
gamma_g = 1.4;
gamma_l = 2.8;   
gamma_l = 1.8;   
Cp_g    = 1000;
Cp_l    = 4000;       
p_infty = 8.5e8;

ag2 = (gamma_g-1)*Cp_g*Temp;
al2 = (gamma_l-1)*Cp_l*Temp;
rho_l    =  gamma_l*(p_infty-p_ref)/al2;
rho_l_user = 1000;
rho_g    =  gamma_g*p_ref/ag2;
rho_g_user = rho_g;

tau_factor           = 1;

d_bubble              = 0.002;
comp_interphase_terms = 1;
axisim                = 0;

% sigma : coeficiente para presion interfacial
coef_sigma    = 0*1.01;
coef_Cd       = 0.0;

% BOUNDARY CONDITIONS
p_ini       = p_ref;
alpha_g_ini = 0.99;
alpha_g_in  = 0.99;
alpha_g_ini_low = 0.01;
vel_g_ini   = 1e-6*[1,1];
vel_g_in    = 1e-6*[1,1];
vel_l_in    = 1e-4*[1,1];
vel_l_ini   = 1e-4*[1,1];
v_top      = [1,0];
v_wall     = [0,0];

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([1,2,1],Nx)*L;
y        = refh([1,2,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);
xmax = max(xnod);
xmin = min(xnod);
xhalf = 0.5*(xmin+xmax);
ijob = 'set';
handle_prop_elem;

get_index;

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate]);
    U = phi;
else
    uini    = ones(numnp,1)*[alpha_g_ini,p_ini,vel_g_ini,vel_l_ini];
    if 0
        nn = find(xnod(:,2) < xhalf(2));
        uini(nn,indx_g_vf) = alpha_g_ini_low; 
    end
    U = uini;
end

nodes_bot   = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top   = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_lef   = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig   = find(abs(xnod(:,1)-xmax(1))<tol);
nodes_wall  = [nodes_lef;nodes_bot;nodes_rig];
nodes_int   = complement((1:numnp)',[nodes_wall;nodes_top]);

fixa= zeros(numnp,ndf);
forc= zeros(numnp,ndf);

% gas phase variables all fixed
fixa(:,indx_g_vf)     = 1;
fixa(:,indx_g_vel(1)) = 1;
fixa(:,indx_g_vel(2)) = 1;
forc(:,indx_g_vf)     = alpha_g_ini;
forc(:,indx_g_vf)     = U(:,indx_g_vf);
forc(:,indx_g_vel(1)) = vel_g_ini(1);
forc(:,indx_g_vel(2)) = vel_g_ini(2);

% velocity fixed at top boundary
fixa(nodes_top,indx_l_vel)   = ones(length(nodes_top),2);
forc(nodes_top,indx_l_vel)   = ones(length(nodes_top),1)*v_top;

% non slip at wall 
fixa(nodes_wall,indx_l_vel)   = ones(length(nodes_wall),2);
forc(nodes_wall,indx_l_vel)   = ones(length(nodes_wall),1)*v_wall;

% pressure fixed at an internal node
fixa(nodes_int(1),indx_l_p)   = 1;
forc(nodes_int(1),indx_l_p)   = p_ref;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0
    for icompo=1:ndf
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];    
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;