% ====================================================================
%
%                       Two phase flow in a quasi 1D problem
%
% ====================================================================

read_from_file = 0;
filestate = 'last_state';

nnwt =3;

shocap                = 0;
weak_form             = 0; 

Cou   =  1;
npg   =  4;
ndf   =  6;
tol   = 1e-6;

L = 2;                     % Depth of the fluid
Nx = 2;
Ny = 41;
rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = -9.8*0;
visco_l  =  1e-10;
visco_g  =  1.000e-10;        % gas phase viscosity

p_ref      = 1e5;
p_ref      = 1e2;

Temp    = 300;
gamma_g = 1.4;
gamma_l = 2.8;   
Cp_g    = 1000;
Cp_l    = 4000;       
p_infty = 8.5e8;
ag2 = (gamma_g-1)*Cp_g*Temp;
al2 = (gamma_l-1)*Cp_l*Temp;

rho_l    =  gamma_l*(p_infty-p_ref)/al2;
rho_g    =  gamma_g*p_ref/ag2;

rho_l_user = rho_l;
rho_g_user = rho_g;

tau_factor           = 1;

d_bubble              = 0.002;
comp_interphase_terms = 1;
axisim                = 0;

% sigma : coeficiente para presion interfacial
coef_sigma     = 0;
coef_Cd        = 0;

% BOUNDARY CONDITIONS
p_bot      = p_ref; %+rho_l*abs(gravity)*L;
p_top      = p_ref;
alpha_g_in = 0.1;
vel_g_in   = [0,1];
vel_l_in   = [0,1e-4];

% INITIAL VALUES
p_ini       = p_ref;
alpha_g_ini = alpha_g_in;
vel_l_ini   = vel_l_in;
vel_g_ini   = [0,0.0001];
vel_g_ini   = vel_g_in;

% ====================================================================
xnod     =[];
icone    =[];
x        = refh([1,1,1],Nx)*L/20;
y        = refh([1,1,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

get_index;

xmin = min(xnod);
xmax = max(xnod);
xhalf = 0.5*(xmin+xmax);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate]);
    U = phi;
else
    uini    = [alpha_g_ini,p_ini,vel_g_ini,vel_l_ini];
    U       =  ones(numnp,1)*uini;   
    U(:,indx_l_p) = rho_l*abs(gravity)*(L-xnod(:,2))+p_ref;
    U(:,indx_g_vf) = alpha_g_ini*(1+0.1*sin(2*pi*xnod(:,2)/L));
    % U(:,indx_g_vf) = alpha_g_ini*(1+0.01*exp(-(xnod(:,2)-xhalf(2)).^2/0.1));
    %nn = find(xnod(:,2)>xhalf(2));
    %U(nn,indx_g_vf) = alpha_g_ini*1.1;    
end

%-------- fijaciones (FIXA)

nodes_bot   = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top   = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_lef   = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig   = find(abs(xnod(:,1)-xmax(1))<tol);

fixa= zeros(numnp,ndf);
forc= zeros(numnp,ndf);

fixa(:,indx_l_vel) = 1;
forc(:,indx_l_vel(2)) = U(:,indx_l_vel(2));

fixa(:,indx_l_p) = 1;
forc(:,indx_l_p) = U(:,indx_l_p);

fixa(:,indx_g_vel(1)) = 1;

fixa([nodes_bot],indx_g_vf) = 1;
forc([nodes_bot],indx_g_vf) = alpha_g_in;

fixa([nodes_bot],indx_g_vel(2)) = 1;
forc([nodes_bot],indx_g_vel(2)) = vel_g_in(2);

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 1
    for icompo=1:ndf
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];    
        
    end
end


% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;