%
%  script para chequear que el calculo simbolico hecho en try1 para
%  determinar los autovalores de A partido por bloques (liquido/gas)
%  sea equivalente al que obtendriamos si lo hacemos en forma numerica
%

global data

if 1

    alpha_g = 0.1*(1+0.1*randn);
    p       = 1e5*(1+0.1*randn);
    
    ug      =  -1*(1+0.1*randn);
    vg      = 0.2*(1+0.1*randn);

    ug      =   0*(1+0.1*randn);
    vg      =-0.2*(1+0.1*randn);
    
    ul      =   2*(1+0.1*randn);
    vl      =-0.1*(1+0.1*randn);

    ul      =   0*(1+0.1*randn);
    vl      =-0.1*(1+0.1*randn);

    rho_g = 1;    
    rho_l = 1000;    
    gamma_g = 1.4;
    gamma_l = 2.8;
    Cp_g = 1008;
    Cp_l = 4186;
    T = 300;
    h_g = Cp_g*T;
    h_l = Cp_l*T;
    p_infty = 8.5e8;
    cc_g = ((gamma_g-1)*Cp_g*T);
    cc_l = ((gamma_l-1)*Cp_l*T);
    sigma = 1;    
else
    node=6;
    rho_g = rho_g(node);
    ug    = Upg(node,indx_g_vel(1));
    alpha_g = Upg(node,indx_g_vf);
    ul  = Upg(node,indx_l_vel(1));
    rho_l = rho_l(node);
    p = Upg(node,indx_l_p);
    T = Temp;
    cc_g = ((gamma_g-1)*Cp_g*T);
    cc_l = ((gamma_l-1)*Cp_l*T);
    vl = Upg(node,indx_l_vel(2));
    vg = Upg(node,indx_g_vel(2));        
end

data.gamma_l = gamma_l;
data.gamma_g = gamma_g;
data.cg = sqrt(cc_g);
data.cl = sqrt(cc_l);
data.p_infty = p_infty;

vel_g = [ug,vg];
vel_l = [ul,vl];

indx_g_vf = 1;
indx_l_p = 2;
indx_g_vel = (3:4);
indx_l_vel = (5:6);

data.indx_g_vf  = indx_g_vf;
data.indx_l_p   = indx_l_p;
data.indx_g_vel = indx_g_vel;
data.indx_l_vel = indx_l_vel;

alpha_l = 1-alpha_g;

% ecuaciones de estado
h_g = Cp_g*T;
h_l = Cp_l*T;

rho_g = gamma_g*p/cc_g;
rho_l = gamma_l*(p_infty-p)/cc_l;

arho_l = rho_l*alpha_l;
arho_g = rho_g*alpha_g;

drho_g_dp =  gamma_g/cc_g;
drho_l_dp = -gamma_l/cc_l; 

% matriz temporal A
A = [rho_g,alpha_g*drho_g_dp,0,0,0,0;
    -rho_l,alpha_l*drho_l_dp,0,0,0,0;    
    rho_g*ug, alpha_g*ug*drho_g_dp,arho_g, 0 , 0, 0;
    rho_g*vg, alpha_g*vg*drho_g_dp , 0 ,arho_g, 0 , 0;        
    -rho_l*ul, alpha_l*ul*drho_l_dp , 0 , 0, arho_l , 0 ;
    -rho_l*vl, alpha_l*vl*drho_l_dp , 0 , 0, 0, arho_l 
];

% su inversa
Ai = inv(A);

dalpha_g_dW1 = Ai(1,1);
dalpha_g_dW2 = Ai(1,2);
dp_dW1       = Ai(2,1);
dp_dW2       = Ai(2,2);    
dapdw1 = p*dalpha_g_dW1+alpha_g*dp_dW1;
dapdw2 = p*dalpha_g_dW2+alpha_g*dp_dW2;
dalpdw1 = -p*dalpha_g_dW1+(1-alpha_g)*dp_dW1;
dalpdw2 = -p*dalpha_g_dW2+(1-alpha_g)*dp_dW2;

Bx = [0,0,1,0,0,0;
    0,0,0,0,1,0;
    -ug^2+dapdw1 , dapdw2, 2*ug, 0 , 0 , 0 ;
    -ug*vg , 0, vg , ug , 0 , 0 ;
    dalpdw1 , -ul^2+dalpdw2 , 0 , 0, 2*ul , 0;
    0 , -ul*vl , 0 , 0 , vl, ul 
];

By = [0,0,0,1,0,0;
    0,0,0,0,0,1;
    -ug*vg ,0 , vg , ug , 0 , 0 ;
    -vg^2+dapdw1 ,  dapdw2 , 0 , 2*vg , 0 , 0 ;
    0 , -ul*vl , 0 , 0 , vl, ul ;
    dalpdw1 , -vl^2+dalpdw2 , 0 , 0, 0 , 2*vl ];    

Up(indx_g_vf) = alpha_g;
Up(indx_g_vel) = vel_g;
Up(indx_l_p) = p;
Up(indx_l_vel) = vel_l;

% bloques GAS
nx = ug/sqrt(ug^2+vg^2); ny = vg/sqrt(ug^2+vg^2);
Bx_g=Bx([indx_g_vf,indx_g_vel],[indx_g_vf,indx_g_vel]);
By_g=By([indx_g_vf,indx_g_vel],[indx_g_vf,indx_g_vel]);
Bn_g = Bx_g*nx+By_g*ny;
[Vg,Dg] = (eig(Bn_g));
tau_g = (Vg*inv(abs(Dg))*inv(Vg));

[Vg_symb,Dg_symb,Bn_g_symb,Vgi_symb] = eig_symb(Up,[nx,ny],'G');

disp(' GAS - V*D*inv(V)-Bg')
disp(' ===================================== ')
%Vg_symb=[Vg_symb(:,2),Vg_symb(:,1),Vg_symb(:,3)];
erro = Vg_symb*Dg_symb*inv(Vg_symb)-Bn_g
disp([' Max error = ' num2str(abs(max(max(erro))))])
if abs(max(max(erro)))>1e-8, error(' excede tolerancia'); end

erro = inv(Vg_symb)-Vgi_symb;
disp([' Max error en la inversa = ' num2str(abs(max(max(erro))))])
if abs(max(max(erro)))>1e-8, error(' excede tolerancia'); end

% bloque LIQUIDO
nx = ul/sqrt(ul^2+vl^2); ny = vl/sqrt(ul^2+vl^2);
Bx_l=Bx([indx_l_p,indx_l_vel],[indx_l_p,indx_l_vel]);
By_l=By([indx_l_p,indx_l_vel],[indx_l_p,indx_l_vel]);
Bn_l = Bx_l*nx+By_l*ny;
[Vl,Dl] = (eig(Bn_l));
tau_l = (Vl*inv(abs(Dl))*inv(Vl));            

[Vl_symb,Dl_symb,Bn_l_symb,Vli_symb] = eig_symb(Up,[nx,ny],'L');

disp(' LIQUID - V*D*inv(V)-Bl')
disp(' ===================================== ')
%Vl_symb=[Vl_symb(:,2),Vl_symb(:,1),Vl_symb(:,3)];
erro = Vl_symb*Dl_symb*inv(Vl_symb)-Bn_l
disp([' Max error en autovectores = ' num2str(abs(max(max(erro))))])
if abs(max(max(erro)))>1e-8, error(' excede tolerancia'); end

erro = inv(Vl_symb)-Vli_symb;
disp([' Max error en la inversa = ' num2str(abs(max(max(erro))))])
if abs(max(max(erro)))>1e-8, error(' excede tolerancia'); end

return


nor = [nx,ny];
load tempo99
Vl2 = ...
subs(subs(subs(subs(subs(subs(subs(subs(subs(Vl,p_infty,data.p_infty),gamma_l,data.gamma_l),p,Up(indx_l_p)), ... 
    ul,Up(indx_l_vel(1))),vl,Up(indx_l_vel(2))),alpha_g,Up(indx_g_vf)),nx,nor(1)),ny,nor(2)),cc_l,(data.cl)^2);

Dl2 = ...
subs(subs(subs(subs(subs(subs(subs(subs(subs(Dl,p_infty,data.p_infty),gamma_l,data.gamma_l),p,Up(indx_l_p)), ... 
    ul,Up(indx_l_vel(1))),vl,Up(indx_l_vel(2))),alpha_g,Up(indx_g_vf)),nx,nor(1)),ny,nor(2)),cc_l,(data.cl)^2);

if 0
[NN,DD] = numden(Vl(1,1));

NNN = subs(subs(subs(subs(subs(subs(subs(subs(subs(NN,p_infty,data.p_infty),gamma_l,data.gamma_l),p,Up(indx_l_p)), ... 
    ul,Up(indx_l_vel(1))),vl,Up(indx_l_vel(2))),alpha_g,Up(indx_g_vf)),nx,nor(1)),ny,nor(2)),cc_l,data.cl);

DDD = subs(subs(subs(subs(subs(subs(subs(subs(subs(DD,p_infty,data.p_infty),gamma_l,data.gamma_l),p,Up(indx_l_p)), ... 
    ul,Up(indx_l_vel(1))),vl,Up(indx_l_vel(2))),alpha_g,Up(indx_g_vf)),nx,nor(1)),ny,nor(2)),cc_l,data.cl);
end

disp(' Vl2 = ')
disp(' ========================================================================= ')
Vl2

%keyboard;pause
%[Vl_symb,Dl_symb,Bn_l_symb] = eig_symb(Up,nor,'L');

return

vaux = ...
    gamma_l*(p*nx^2*vl+nx*ul*p*ny-1/2/(gamma_l*p-gamma_l*alpha_g*p_infty)*(2*gamma_l*nx*ul*p-2*gamma_l*nx*ul*alpha_g*p_infty+2*gamma_l*ny*vl*p-2*gamma_l*ny*vl*alpha_g*p_infty+2*(-gamma_l*p^2*nx^2*cc_l-gamma_l*p^2*ny^2*cc_l+gamma_l*alpha_g*p_infty*nx^2*p*cc_l+gamma_l*alpha_g*p_infty*ny^2*p*cc_l)^(1/2))*p*ny+2*ny^2*vl*p-nx^2*vl*alpha_g*p_infty-nx*ul*alpha_g*p_infty*ny-2*ny^2*vl*alpha_g*p_infty+1/2/(gamma_l*p-gamma_l*alpha_g*p_infty)*(2*gamma_l*nx*ul*p-2*gamma_l*nx*ul*alpha_g*p_infty+2*gamma_l*ny*vl*p-2*gamma_l*ny*vl*alpha_g*p_infty+2*(-gamma_l*p^2*nx^2*cc_l-gamma_l*p^2*ny^2*cc_l+gamma_l*alpha_g*p_infty*nx^2*p*cc_l+gamma_l*alpha_g*p_infty*ny^2*p*cc_l)^(1/2))*alpha_g*p_infty*ny)/(p*gamma_l*vl^2*nx^2+gamma_l*ny^2*vl^2*p+ny^2*p*cc_l-gamma_l*vl^2*alpha_g*p_infty*nx^2-gamma_l*ny^2*vl^2*alpha_g*p_infty);
subs(subs(subs(subs(subs(subs(subs(subs(subs(vaux,p_infty,data.p_infty),gamma_l,data.gamma_l),p,Up(indx_l_p)), ... 
    ul,Up(indx_l_vel(1))),vl,Up(indx_l_vel(2))),alpha_g,Up(indx_g_vf)),nx,nor(1)),ny,nor(2)),cc_l,data.cl);

 