% ====================================================================
%
%                       WATER FAUCET Test
%
% ====================================================================

read_from_file = 0;
filestate = 'last_state';

periodic_for_gas = 0;

nnwt =2;

shocap                = 0;
weak_form             = 1; 

Cou   =  1;
npg   =  2;
ndf   =  6;
tol   = 1e-6;

L = 12;                     % Depth of the fluid

Nx = 2;
Ny = 100;

rratio = 1;               % Ratio between axis / side element size
zratio = 1;                  % Ratio between axis / side element size

% PHYSICAL DATA
gravity  = 9.81*1;
gravity  = 9.81*0.1;
visco_l  =  1e-10;
visco_g  =  1e-10;        % gas phase viscosity

p_ref      = 1e5;

Temp    = 273+35;
gamma_g = 1.4;
gamma_l = 2.8;   
Cp_g    = 1000;
Cp_l    = 4000;       
p_infty = 8.5e8;
ag2 = (gamma_g-1)*Cp_g*Temp;
al2 = (gamma_l-1)*Cp_l*Temp;

rho_l    =  gamma_l*(p_infty-p_ref)/al2;
rho_g    =  gamma_g*p_ref/ag2;

rho_l      = 1000;
rho_g      = 1;

rho_l_user = rho_l;
rho_g_user = rho_g;

tau_factor           = 1;

d_bubble              = 0.002;
comp_interphase_terms = 1;
axisim                = 0;

% sigma : coeficiente para presion interfacial
coef_sigma    = 1;
coef_Cd       = 0.0;

stabi_with_nonconservative_terms = 0;

interface_pressure_analytical = 1;

% BOUNDARY CONDITIONS
p_ini       = p_ref;
alpha_g_ini = 0.20;
alpha_g_in  = 0.20;
vel_g_ini   = 1e-9*[0,1];
vel_l_ini   = 10*[0,1];
vel_g_in    = [0,0];
vel_l_in    = 10*[0,1];

% ====================================================================
xnod     =[];
icone    =[];
y        = refh([1,1,1],Ny)*L;
x        = refh([1,1,1],Nx)*L/Ny;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

%-------- conectividades (ICONE)
ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
for k=1:Ny-1,
    icone=[icone; (k-1)*Nx+ico ];
end
% convertimos cuadrangulos en triangulos
quad2tri=0;
if quad2tri, quad_to_tri; end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

get_index;

xmin = min(xnod);
xmax = max(xnod);
x_half = 0.5*(xmin+xmax);

%-------- fijaciones (FIXA)
nodes_bot   = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top   = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_lef   = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig   = find(abs(xnod(:,1)-xmax(1))<tol);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate]);
    U = phi;
elseif 0
    uini    = ones(numnp,1)*[alpha_g_ini,p_ini,vel_g_ini,vel_l_ini];
    if 0
        nn = find((xnod(:,2)>x_half(2)/2)&(xnod(:,2)<3*x_half(2)/4));
        uini(nn,indx_g_vf) = 1.2*alpha_g_ini;
    end
    U = uini;    
elseif 0
    try4;  % resuelve analiticamente el estado estacionario del water faucet para inicializar con eso !!!
    U = uini;        
elseif 0
    try5;  % resuelve analiticamente el estado estacionario del water faucet para inicializar con eso !!!
    U = uini;            
elseif 0
    try6;  % resuelve analiticamente el estado estacionario del water faucet para inicializar con eso !!!
    U = uini;                
else
    try7;  % resuelve analiticamente el estado estacionario del water faucet para inicializar con eso !!!
    U = uini;                
end

fixa= zeros(numnp,ndf);
forc= zeros(numnp,ndf);

% void fraction at bot
fixa(nodes_bot,indx_g_vf)   = ones(length(nodes_bot),1);
forc(nodes_bot,indx_g_vf)   = U(nodes_bot,indx_g_vf);
% void fraction at top
fixa(nodes_top,indx_g_vf)   = ones(length(nodes_top),1);
forc(nodes_top,indx_g_vf)   = U(nodes_top,indx_g_vf);
% void fraction fixed 
if 0
    fixa(:,indx_g_vf)   = 1;
    forc(:,indx_g_vf)   = U(:,indx_g_vf);        
end

% horizontal velocity fixed
fixa(:,indx_g_vel(1)) = 1;
fixa(:,indx_l_vel(1)) = 1;

% liquid velocity at bot
fixa(nodes_bot,indx_l_vel)   = ones(length(nodes_bot),2);
forc(nodes_bot,indx_l_vel(2))   = U(nodes_bot,indx_l_vel(2));
% liquid velocity at top
fixa(nodes_top,indx_l_vel)   = ones(length(nodes_top),2);
forc(nodes_top,indx_l_vel(2))   = U(nodes_top,indx_l_vel(2));
% vertical liquid velocity fixed
if 0
    fixa(:,indx_l_vel(2)) = 1;
    forc(:,indx_l_vel(2)) = U(:,indx_l_vel(2));
end

% gas velocity at bot
fixa(nodes_bot,indx_g_vel)   = ones(length(nodes_bot),2);
forc(nodes_bot,indx_g_vel(2))   = U(nodes_bot,indx_g_vel(2));        
% gas velocity at top
fixa(nodes_top,indx_g_vel)   = ones(length(nodes_top),2);
forc(nodes_top,indx_g_vel(2))   = U(nodes_top,indx_g_vel(2));        
% vertical gas velocity fixed
if 0
    fixa(:,indx_g_vel(2)) = 1;
    forc(:,indx_g_vel(2)) = U(:,indx_g_vel(2));    
end

% pressure fixed at ends
fixa(nodes_bot,indx_l_p)   = ones(length(nodes_bot),1);
forc(nodes_bot,indx_l_p)   = U(nodes_bot,indx_l_p);    
fixa(nodes_top,indx_l_p)   = ones(length(nodes_top),1);
forc(nodes_top,indx_l_p)   = U(nodes_top,indx_l_p);    
% pressure fixed 
if 0
    fixa(:,indx_l_p)   = 1;
    forc(:,indx_l_p)   = U(:,indx_l_p);    
end

%keyboard;pause

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 1
    for icompo=1:ndf
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];    
    end
    if (periodic_for_gas==1),
        for icompo=[indx_g_vf,indx_g_vel]
            lin_con = [lin_con ; ... 
                    [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                        -ones(length(nodes_bot),1) nodes_bot icompo*ones(length(nodes_bot),1) ]];
        end
    end
end

% condicion inicial en primitivas
%-----------------------------------------------
if 1,
    bini   = U;
    for k=1:ndf,
        nofree        = find(fixa(:,k)~=0);
        bini(nofree,k)   = forc(nofree,k);
    end   
end

bc_time_flag = 0;
bc_time_t_end = 1;