%
%  Incompressible Navier-Stokes equations
%  using SUPG-PSPG like advdif elemset
%

type_upwind         = 1;
tau_centroide       = 0;
debug               = 0;

get_index;

idf_gas = [indx_vf,indx_vg];
ndf_g = ndf/2;

visco = prop(2);

%------------------------------------------------------------------
ne  = length(eles);
[bas,ndf_fields] = size(extra_nodal_fields);

% Localizacion de las variables
%------------------------------------------------------------------
ceros = zeros(ne,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(ne,nen);

for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    knod  = (inod-1)*ndf_fields+(1:ndf_fields);            
    Ue(:,jnod)  = U(nodes,1:ndf);		 
    U0e(:,jnod) = Uold(nodes,1:ndf);   
    re(:,inod)  = xnod(nodes,1);		        
end

% UPWIND EN EL CENTROIDE 
%------------------------------------------------------------------
if nen==4,
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),0,0);
    [dshapex,N] = shape2d(X2S,0,0,nen); 
elseif nen==3,
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),1/3,1/3);
    [dshapex,N] = shape2d(X2S,1/3,1/3,nen); 
else
    error(' Bad type of element , nen ~=3, nen ~=4 ')
end

Upg          = zeros(ne,ndf);
dUpg         = zeros(ne,ndf,ndm);

for inod=1:nen,
    jnod = (inod-1)*ndf+(1:ndf);
    Upg(:,(1:ndf))    =  Upg(:,(1:ndf)) + N(inod).*  U0e(:,jnod);
    for idm=1:ndm
        dUpg(:,(1:ndf),idm) = dUpg(:,(1:ndf),idm) + ...
            (dshapex(:,inod,idm)*ones(1,ndf)).* U0e(:,jnod);          
    end
end    

if tau_centroide==1,
    velpg   = Upg(:,indx_v);
    hele    = comp_hele(velpg,dshapex,Jaco,nen);
    [tau,delta,bas] = supg_nsi_tet(Upg,dUpg,hele,prop,type_upwind);
    compute_P_supg;   
end


% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
    
    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    
    Upg    = zeros(ne,ndf);
    U0pg    = zeros(ne,ndf);
    dUpg    = zeros(ne,ndf,ndm);
    radpg    = zeros(ne,1);
    
    for inod=1:nen,
        jnod = (inod-1)*ndf+(1:ndf);
        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
        for idm=1:ndm            
            dUpg(:,(1:ndf),idm)   = dUpg(:,(1:ndf),idm) + ...
                (dshapex(:,inod,idm)*ones(1,ndf)).* Ue(:,jnod);            
        end        
    end
    
    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
    
    if tau_centroide==0,
        velpg   = Upg(:,indx_v);
        hele    = comp_hele(velpg,dshapex,Jaco,nen);
        [tau,delta,bas] = supg_nsi_tet(velpg,dUpg,hele,prop,type_upwind);
        velpg_vf = velpg;
        velpg_vf(:,g_dir) = velpg_vf(:,g_dir) + vslip;                
    end
    
    rho_l = rho;
    rho_m_old = compute_rho_m(U0pg(:,indx_vf),rho_l,rho_g);
    rho_m     = compute_rho_m(Upg(:,indx_vf),rho_l,rho_g);
    
    u_gu = rowprod(dUpg(:,indx_v,:),U0pg(:,indx_v),ndm,ndm,1);
    div_u = zeros(ne,1);
    for idm =1:ndm
        div_u = div_u + dUpg(:,indx_v(idm),idm) ;
    end
    
    u_gvf = zeros(ne,1);
    for idm=1:ndm
        u_gvf = u_gvf + dUpg(:,indx_vf,idm).*velpg_vf(:,idm);
    end
    
    %    disp(' *** NSI_TET_ELE.M ***');keyboard;pause
    
    strain_rate = row_transp(dUpg(:,indx_v,:));
    strain_rate = 0.5*(strain_rate + dUpg(:,indx_v,:));
    sigma = 2*visco*strain_rate;
    
    terd = rowprod(dshapex,sigma,nen,ndm,ndm);
    %    terd = rowprod(dshapex,dUpg(:,indx_v,:),nen,ndm,ndm);
    dshapex_t = row_transp(dshapex);
    terd_m = visco*rowprod(dshapex,dshapex_t,nen,ndm,nen);
    %    terd_m = rowprod(dshapex,dshapex_t,nen,ndm,nen);
    
    for inod=1:nen,   
        
        if tau_centroide==0,
            compute_P_supg_continuous;
            compute_P_supg_disperse;
        end
        
        W_supg = P_delta;      
        for idf=1:ndm,
            W_supg(:,idf,idf)=W_supg(:,idf,idf)+N(inod);
        end
        
        % Residual assembling        
        % flow equations
        R_mom = (rho_m*[1,1]).*((Upg(:,indx_v)-U0pg(:,indx_v))/dt+u_gu) + ...
            reshape(dUpg(:,indx_p,:),ne,ndm,1);
        for idm=1:ndm
            Rele(:,inod,indx_v(idm)) = Rele(:,inod,indx_v(idm)) - ( ...  
                W_supg(:,idm,idm).*R_mom(:,idm) + ...                  % momentum residual ( Galerkin + SUPG )
                terd(:,inod,idm) ...
                ).*xsj;                 
            Rele(:,inod,indx_p) = Rele(:,inod,indx_p) - ( ...                  
                P_epsilon(:,1,indx_v(idm)).*R_mom(:,idm)  ... % PSPG 
                ).*xsj; 
        end
        Rele(:,inod,indx_p) = Rele(:,inod,indx_p) - ( ... 
            N(inod)*rho_m.*div_u ...                   % continuity Galerkin
            ).*xsj; 
        
        % disperse phases ( mass & momentum eqs )

        W_supg_gas = P_supg_gas;      
        for idf=1:ndm,
            W_supg_gas(:,idf,idf) = W_supg_gas(:,idf,idf) + N(inod);
        end
        
        % temporal term
        if lump_time_term==0,
            W_dUdt   = rowprod(W_supg_gas,dH,ndf,ndf,1);
        else
            W_dUdt   = N(inod)*dH;         
        end
        
        % advective fluxes
        if weak_form,    
            gN_Fa = -(dshapex(:,inod,1)*ones(1,ndf_g)).*Fa(:,:,1) -(dshapex(:,inod,2)*ones(1,ndf_g)).*Fa(:,:,2);                    
        else
            gN_Fa = (N(inod)*ones(numel,ndf_g)).* AgU;
        end
        
        if inviscid==0
            % diffusive fluxes            
            gN_Fd = (dshapex(:,inod,1)*ones(1,ndf_g)).*Fd(:,:,1) +(dshapex(:,inod,2)*ones(1,ndf_g)).*Fd(:,:,2);                
        end
        
        % stabilization SUPG terms        
        P_A_gU   = rowprod(P_supg_gas,AgU,ndf_g,ndf_g,1);
        
        gN_D_gU = zeros(ne,ndf_g);
        
        if(abs(shocap)>0)                
            for idm=1:ndm              
                gN_D_gU(:,indx_g_vf) = gN_D_gU(:,indx_g_vf) + ...
                    dshapex(:,inod,idm).*(shocap*rho_g.*delta_g).*dUpg(:,indx_g_vf,idm);
            end
        end        
        
        % reactive and source terms        
        if lump_reac_term==0,
            W_Fc  = rowprod(W_supg_gas,Fc,ndf_g,ndf_g,1) ;
        else  % DEBUG
            W_Fc   = N(inod)*Fc; 
        end
        
        vaux =  (gN_Fa + P_A_gU - W_Fc)  + W_dUdt/ dt;  
        if abs(shocap), vaux = vaux + gN_D_gU; end
        if inviscid==0, vaux= vaux +  gN_Fd; end
        
        for idf=idf_gas,
            Rele(:,inod,idf) = Rele(:,inod,idf) - vaux .* xsj ;                   
        end
                        
        % Matriz assembling
        for jnod=1:nen,        
            
            u_gN = zeros(ne,ndm);
            for idm=1:ndm
                for jdm=1:ndm
                    u_gN(:,idm) = u_gN(:,idm) + dshapex(:,jnod,jdm).*U0pg(:,indx_v(jdm));
                end
            end            
            
            for idm=1:ndm
                Kele(:,inod,indx_v(idm),jnod,indx_v(idm)) = Kele(:,inod,indx_v(idm),jnod,indx_v(idm)) + ( ...  
                    W_supg(:,idm,idm).*rho_m.*N(jnod)/dt + ... % termino temporal
                    W_supg(:,idm,idm).*rho_m.*u_gN(:,idm)      + ... % termino convectivo                  
                    terd_m(:,inod,jnod) ... 
                    ).*xsj;  
                Kele(:,inod,indx_v(idm),jnod,indx_p) = Kele(:,inod,indx_v(idm),jnod,indx_p) + ( ...  
                    W_supg(:,idm,idm).*dshapex(:,jnod,idm) ...  % termino presion
                    ).*xsj;                     
                Kele(:,inod,indx_p,jnod,indx_v(idm)) = Kele(:,inod,indx_p,jnod,indx_v(idm)) + ( ...  
                    N(inod)*dshapex(:,jnod,idm).*rho_m + ... % termino continuidad Galerkin
                    P_epsilon(:,1,indx_v(idm)).*rho_m.*N(jnod)/dt + ... % estabilizacion continuidad temporal
                    P_epsilon(:,1,indx_v(idm)).*rho_m.*u_gN(:,idm) ... % estabilizacion continuidad convectivo
                    ).*xsj; 
                Kele(:,inod,indx_p,jnod,indx_p) = Kele(:,inod,indx_p,jnod,indx_p) + ( ...
                    P_epsilon(:,1,indx_v(idm)).*dshapex(:,jnod,idm) ...
                    ).*xsj; 
                
                for jdm=1:ndm
                    Kele(:,inod,indx_v(idm),jnod,indx_v(jdm)) = ... 
                        Kele(:,inod,indx_v(idm),jnod,indx_v(jdm)) + ( ...                      
                        visco*dshapex(:,inod,jdm).*dshapex(:,jnod,idm) ...
                        ).*xsj; 
                end
            end

            % disperse phases            
            % temporal terms
            if lump_time_term == 0,
                W_Cp_N = rowprod(W_supg_gas,Cp,ndf_g,ndf_g,ndf_g)*N(jnod) ;        
            else
                % DEBUG
                W_Cp_N = N(inod)*Cp*N(jnod) ;         
            end
            
            % N_A_gN or gN_A_N according to weak form
            if weak_form,
                N_A_gN = -A_gN(:,:,:,inod)*N(jnod);                 
            else          
                N_A_gN =  A_gN(:,:,:,jnod)*N(inod);              
            end
            
            P_A_gN = rowprod(P_supg_gas,A_gN(:,:,:,jnod),ndf_g,ndf_g,ndf_g);  
                
            % shock capturing term
            if (abs(shocap)>0)
                gN_D_gN = zeros(numel,ndf,ndf);                        
                for idm=1:ndm
                    gN_D_gN(:,indx_g_vf,indx_g_vf) = gN_D_gN(:,indx_g_vf,indx_g_vf) + ...
                        dshapex(:,inod,idm).*(shocap*rho_g.*delta_g).*dshapex(:,jnod,idm);
                    gN_D_gN(:,indx_l_p,indx_l_p) = gN_D_gN(:,indx_l_p,indx_l_p) + ...
                        dshapex(:,inod,idm).*(shocap*rho_l.*delta_l).*dshapex(:,jnod,idm);                    
                end
            end
            
            if inviscid==0            
                gN_K_gN = zeros(numel,ndf,ndf);                        
                for idf=1:ndf,
                    for jdf=1:ndf,
                        gN_K_gN(:,idf,jdf) = ...
                            ( (dshapex(:,inod,1).*Kxx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                            (dshapex(:,inod,1).*Kxy(:,idf,jdf).*dshapex(:,jnod,2))+ ...
                            (dshapex(:,inod,2).*Kyx(:,idf,jdf).*dshapex(:,jnod,1))+ ...
                            (dshapex(:,inod,2).*Kyy(:,idf,jdf).*dshapex(:,jnod,2)) );
                    end             
                end                
            end

            % reactive terms
            if lump_reac_term == 0;,
                W_C_N = rowprod(W_supg_gas,Cjac,ndf_g,ndf_g,ndf_g)*N(jnod) ;        
            else
                % DEBUG
                W_C_N = Cjac*N(jnod)*N(inod) ;
            end    

            vaux =  (N_A_gN + P_A_gN - W_C_N ) ;
            if abs(shocap), vaux = vaux + gN_D_gN; end
            if inviscid==0, vaux = vaux +  gN_K_gN; end
            
            for idf=1:ndf_g,
                idf_g = idf_gas(idf);
                for jdf=1:ndf_g,     
                    jdf_g = jdf_gas(jdf);
                    Kele(:,inod,idf_g,jnod,jdf_g) = Kele(:,inod,idf_g,jnod,jdf_g) + xsj.*( W_Cp_N(:,idf,jdf) ./ dt + vaux(:,idf,jdf) );
                end
            end
            
        end % jnod
        
    end % inod
    
    %   disp('bubbly_2d_ele (liquid) phase ');keyboard;pause   
    
end % ipg

