%
%  postprocesing to compute some rheological properties of solid model
%

tol = 1e-6;

%alpha_s = (1-alpha_g);
alpha_s = max([alpha_s';tol*ones(1,length(alpha_s))])';
alpha_s = min([alpha_s';ones(1,length(alpha_s))])';
alpha_g = 1-alpha_s;

% solid pressure
% without granular temperature
alpha_g_crit = 0.376;
G0 = 1;
c = 600;
G = G0*exp(-c*(alpha_g-alpha_g_crit));

% gas density according to a state equation
T_gas = 600+273;
p_atm = 1.0133e5;
rho_gas = (p_atm+p_gas) / (287*T_gas);

% radial distribution function (g0)
alpha_s_max = 1-alpha_g_crit;
g0 = (1-(alpha_s/alpha_s_max).^(1/3)).^(-1);
g0 = g0.*(alpha_s<alpha_s_max)+tol*(alpha_s>=alpha_s_max);

% granular temperature
rho_p = rho_g;
e = 0.99;
K1 = 2*(1+e)*rho_p*g0;
K3 = rho_p*dp/2*(sqrt(pi)/(3*(3-e))*(1+2/5*(1+e)*(3*e-1)*alpha_s.*g0) + 8*alpha_s ./ (5*sqrt(pi)).*g0*(1+e));
K2 = 4/(3*sqrt(pi))*dp*rho_p*(1+e)*alpha_s.*g0-2/3*K3;
K4 = 12*(1-e^2)*rho_p*g0/(dp*sqrt(pi));

X  = div_U; 
X2 = div_U2;
X2 = sqrt(X2.^2);
disc = (K1.*alpha_s+rho_p).^2.*X.^2+4*K4.*alpha_s.*(2*K3.*X2+K2.*X.^2);
Tg = -(K1.*alpha_s+rho_p).*X+sqrt(disc);
Tg = Tg ./ (2*alpha_s.*K4);
Tg = Tg.^2;

% solid pressure 
% KTGF
p_KTGF = rho_p*alpha_s.*Tg.*(1+2*(1+e)*g0.*alpha_s);
% frictional
cF = 0.05;
cn = 2;
cq = 3;
alpha_s_min = 0.5;
tol = 1e-3;

mask = abs(alpha_s-alpha_s_min)>tol;
alpha_s = alpha_s.*mask+(alpha_s_min-tol).*(1-mask);

mask = abs(alpha_s-alpha_s_max)>tol;
alpha_s = alpha_s.*mask+(alpha_s_max-tol).*(1-mask);

p_F = cF*(alpha_s-alpha_s_min).^cn./(alpha_s_max-alpha_s).^cq;

dpfdvf = p_F.*(cn*(alpha_s-alpha_s_max)-cq*(alpha_s-alpha_s_min))./((alpha_s_max-alpha_s).*(alpha_s_min-alpha_s));

G0 = 1;
cc = 200;
alpha_g_crit = 0.422;
Gfun = G0*exp(-cc*(alpha_g-alpha_g_crit));
