function [Kg,Rg] = elliptic_grid(xnod,icone,ifaceb,fixa,forc,teco,lin_con,ther_contact,prop,...
    prop_elem,veloc,reac,source,phi,phiold,ijob)                       
%------------------------------------------------------------------
%
%     [Kg,Rg] = advdif_2d(xnod,icone,fixa,forc,teco,peri,prop,
%                      veloc,reac,source,phi,phiold,ijob)
%
%       Caracteristicas del elemento:
%
%        #.- ecuacion adveccion difusion reaccion no estacionaria c/fuente
%        #.- elemento cuadrangular y triangular
%        #.- plano y axisimetrico
%
%        ijob = 1 ---> matriz tangente y residuo
%        ijob = 0 ---> solo residuo
%
%------------------------------------------------------------------

[numel,nen]   = size(icone);
if min(icone(:,nen))<=0,
    id_material = abs(icone(:,nen));
    nummat = max(id_material)-min(id_material)+1;
    icone = icone(:,1:nen-1);
else
    nummat = 1;
    id_material = ones(numel,1);
    
    if 0,
        [nr,nc]=size(prop);
        if nr==1, prop=prop'; end
        [nr,nc]=size(prop);   
        prop = prop(:,1);
    end
    
end

if nummat~=1, error(' ELLIPTIC_GRID.m : nummat should be 1'); end

[numel,nen]   = size(icone);
[numnp,ndf]   = size(phi);
[numnp,ndm]   = size(xnod);

if ndf~=2, error(' ELLIPTIC_GRID.m NDF debe ser 2');end

[bas,ibas] = size(prop_elem);
if ibas<nummat, error(' prop_elem array should have nummat columns '); end

% global properties definition (default)
if 0,
    rho      = prop(1) ;
    Cp       = prop(2) ;
    kappa    = prop(3) ;  
    npg      = prop(4) ;
    tau_fac  = prop(6) ;
    shocap   = prop(7) ;
    axisim   = prop(8) ;
    type_upw = prop(9) ;
    dtfic    = prop(10);
    lumped   = prop(11);
    tol      = prop(12);
    dt       = prop(13);
    alpha    = prop(14);
else
    npg      = prop(4) ;
    initial_guess = prop(7);
    axisim   = prop(8) ;
    coef     = prop(9);
    explicit = prop(10);
    tol      = prop(12);
    dt       = prop(13);
    alpha    = prop(14);
end

U = phi;
Uold = phiold;

if ndm==1,
    pgauss1d;
elseif ndm==2, 
    pgauss2d; 
elseif ndm==3,
    pgauss3d; 
end

Kele =zeros(numel,nen,ndf,nen,ndf);
Rele =zeros(numel,nen,ndf);

for k=1:nummat,  
    eles  = find(id_material==k);  
    if length(eles)>0,    
        if 0
            rho      = prop_elem(1,k) ;
            Cp       = prop_elem(2,k) ;
            kappa    = prop_elem(3,k) ;     
        end
        elliptic_grid_ele;
    end
end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
[Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);

%disp('elliptic_grid.m routine');keyboard;pause