%====================================================================================
%
%  Write BC files from a surface mesh with an extra field for the material code
%  This extra field identifies each surface and a list of nodes and connectivities
%  for each boundary region is written in files.
%
%  USER should define the fileroot and the Region identifier. Also check the cols
%  to be removed in the volume and surface mesh and which column correspond to the
%  material identificator.
%
%  REMARK : To generate the volume and surface mesh simultaneously with GID use
%           Meshing/Mesh criteria/Mesh/Surface after assigning to each part of the
%           boundary the corresponding material identificator.
% 
%====================================================================================

fileroot = 'output';
Region = struct('Id',{'inout','wall'});
Region(1).code = 3;
Region(2).code = 4;

ijob=1;  % read also boundary mesh connectivities
[xnod,icone,icone_surf] = read_mesh('prueba',ijob);

cols = (2:4);
xnod = xnod(:,cols);

cols = (2:5);
icone= icone(:,cols);

cols = (5);
mate= icone_surf(:,cols);

cols = (2:4);
icone_surf= icone_surf(:,cols);

kmin = min(mate);
kmax = max(mate);

if kmin ~= min(Region.code), 
   error(' Error *** RegionCode is not compatible with Mate [kmin]'); end
if kmax ~= max(Region.code), 
   error(' Error *** RegionCode is not compatible with Mate [kmax]'); end

for k=kmin:kmax,
   nn=find(mate==k);
   if isempty(nn)==0,
      nodes = rmdblist(m2v(icone_surf(nn,:)));
      eles  = icone_surf(nn,:);
      [numel,nen]=size(eles);
      flag=0;
%      for m=min(Region.code):max(Region.code)
      for m=1:length(Region)         
         if(Region(m).code==k)
            if flag==1, error('2 codes with the same material '); end
            flag=1;
            filesufix = Region(m).Id;
            filename = [fileroot '.fixa_' filesufix '.nod'];
            my_fprintf(filename,'%7i \n',nodes);
            filename = [fileroot '.fixa_' filesufix '.con'];
            formato = [];
            for ii=1:nen,
               formato = [formato,'%7i '];
            end         
            formato = [formato,' \n '];
            my_fprintf(filename,formato,eles);                     
         end         
      end      
   end   
end
