function [dux,duy] = grad(xnod,icone,u)
%
%      [dux,duy] = grad(xnod,icone,u)
%
%  para elementos cuadrangulares EXCLUSIVAMENTE
%
%   El gradiente es calculado en cada nodo como
%   promedio del gradiente en cada elemento
%   que contiene al nodo
%

[nele,nen]=size(icone);

% Calculo jacobiano transformacion al elemento master
%--------------------------------------------------------
x1 = xnod(icone(:,1),1);
x2 = xnod(icone(:,2),1);
x3 = xnod(icone(:,3),1);
x4 = xnod(icone(:,4),1);
y1 = xnod(icone(:,1),2);
y2 = xnod(icone(:,2),2);
y3 = xnod(icone(:,3),2);
y4 = xnod(icone(:,4),2);


psi = [-1 1 1 -1 ];
eta = [-1 -1 1 1 ];
psipg = 0;
etapg = 0;

% dN_j/d(psi_k)

dN1d1 = 0.25*psi(1)*(1+eta(1)*etapg);
dN1d2 = 0.25*eta(1)*(1+psi(1)*psipg);

dN2d1 = 0.25*psi(2)*(1+eta(2)*etapg);
dN2d2 = 0.25*eta(2)*(1+psi(2)*psipg);

dN3d1 = 0.25*psi(3)*(1+eta(3)*etapg);
dN3d2 = 0.25*eta(3)*(1+psi(3)*psipg);

dN4d1 = 0.25*psi(4)*(1+eta(4)*etapg);
dN4d2 = 0.25*eta(4)*(1+psi(4)*psipg);

dxdpsi = x1*dN1d1+x2*dN2d1+x3*dN3d1+x4*dN4d1;
dxdeta = x1*dN1d2+x2*dN2d2+x3*dN3d2+x4*dN4d2;

dydpsi = y1*dN1d1+y2*dN2d1+y3*dN3d1+y4*dN4d1;
dydeta = y1*dN1d2+y2*dN2d2+y3*dN3d2+y4*dN4d2;

jaco = dxdpsi.*dydeta-dxdeta.*dydpsi;

dpsidx = dydeta ./ jaco ;
detady = dxdpsi ./ jaco ;
dpsidy = -dxdeta ./ jaco ;
detadx = -dydpsi ./ jaco ;

u1 = u(icone(:,1),1);
u2 = u(icone(:,2),1);
u3 = u(icone(:,3),1);
u4 = u(icone(:,4),1);

dudx = u1.*(dN1d1.*dpsidx+dN1d2.*detadx) + ... 
       u2.*(dN2d1.*dpsidx+dN2d2.*detadx) + ... 
       u3.*(dN3d1.*dpsidx+dN3d2.*detadx) + ... 
       u4.*(dN4d1.*dpsidx+dN4d2.*detadx) ;

dudy = u1.*(dN1d1.*dpsidy+dN1d2.*detady) + ... 
       u2.*(dN2d1.*dpsidy+dN2d2.*detady) + ... 
       u3.*(dN3d1.*dpsidy+dN3d2.*detady) + ... 
       u4.*(dN4d1.*dpsidy+dN4d2.*detady) ;

dux = zeros(length(xnod(:,1)),1);

dux(icone(:,1)) = dudx;
dux(icone(:,2)) = dux(icone(:,2))+dudx;
dux(icone(:,3)) = dux(icone(:,3))+dudx;
dux(icone(:,4)) = dux(icone(:,4))+dudx;

duy = zeros(length(xnod(:,1)),1);

duy(icone(:,1)) = dudy;
duy(icone(:,2)) = duy(icone(:,2))+dudy;
duy(icone(:,3)) = duy(icone(:,3))+dudy;
duy(icone(:,4)) = duy(icone(:,4))+dudy;

factor = zeros(length(xnod(:,1)),1);
factor(icone(:,1)) = ones(nele,1);
factor(icone(:,2)) = factor(icone(:,2))+ones(nele,1);
factor(icone(:,3)) = factor(icone(:,3))+ones(nele,1);
factor(icone(:,4)) = factor(icone(:,4))+ones(nele,1);

dux = dux ./ factor;
duy = duy ./ factor;