function [grad_U] = grad_phi_by_ele(xnod,icone,vec,npg)
%
%      [grad_phi] = grad_vec(xnod,icone,phi,[...npg])
%
%   Computes grad U with U the state vector composed by ndf fields
%
%   dU_i/dx_j = field "i" , direction "j"
%

if nargin <4, npg=1; end

[nnod,ndm] = size(xnod);
[nele,nen] = size(icone);
[bas,ndf]  = size(vec);

if ndm==2,
   pgauss2d;
else
   pgauss3d;  
end

vec_ele = zeros(nele,nen,ndf);

for inod=1:nen,
    for idf=1:ndf
        nodes = icone(:,inod);
        vec_ele(:,inod,idf) = vec(nodes,idf);
    end   
end   

grad_U = zeros(nele,npg,ndf,ndm);
   
for ipg=1:npg
   
   if ndm==2,
      [X2S,S2X,Jaco] = xy2st(xnod,icone,psipg(ipg),etapg(ipg));  
      [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen);        
   else            
      [X2S,S2X,Jaco] = xyz2rst(xnod,icone,psipg(ipg),etapg(ipg),zetapg(ipg));  
      [dNdx,dNdy,dNdz,N] = shape3d(X2S,psipg(ipg),etapg(ipg),zetapg(ipg),nen);  
   end
   
   xsj =  wpg(ipg) .* Jaco;

   for inod=1:nen,
       for idf=1:ndf,
           for jdm=1:ndm
               grad_U(:,ipg,idf,jdm)=grad_U(:,ipg,idf,jdm)+dshapex(:,inod,jdm).*vec_ele(:,inod,idf);
           end
       end
   end
   
end
