function [xnod,icone] = mblock_2d(filename)
%
%   [xnod,icone] = mblock_2d(filename)
%
%   2D multiblock mesh generation
%
%   filename : input file name (*.m) with the following data
%
%      surfaces.edges (structure)  edges by each surface 
%      edges.points   (structure)  points by each edge
%      xnod           (array)      coordinates of the original control points to
%                                  define the edges
%      Np             (array)      number of subdivisions of each group of edges.
%             

%
% rutina basada en resolver un problema multiblock 2D 
% compuesto de un dominio con un conjunto de superficies
% de 4 lados que lo tapizan y cada lado o arista es una
% curva definida por un conjunto de puntos debiendo satisfacer que
% 
% [1] la particion sea conforme 
% [2] de un tamao aproximado ingresado como dato por el usuario
% [3] respete los puntos ingresados como datos para definir las curvas
%

% Estrategia de calculo
%
% [1] dividir el dominio en superficies o bloques, curvas y puntos.
% [2] obtener los grupos de aristas que deben tener la misma particion.
% [3] hacer la particion de los distintos grupos.
% [4] mallar cada superficie.
%

% [1] dividir el dominio en superficies, curvas y puntos.
%datos_example_1;
%datos_example_2;
%datos_example_3;
%datos_example_4;
%draw_geo
eval(filename)

% [2] obtener los grupos de aristas que deben tener la misma particion.
[grupos]=split_edges_by_topology(superficie);

% [3] hacer la particion de los distintos grupos.
% aqui se debe tener en cuenta que cada grupo consta de un numero arbitrario de aristas
% 
% Estrategia
%
% Dado un h_ele calcular N[i] = fixa(L[i]/h_ele)+1;   i=1,number_aristas_of_group
% Tomar como N[i] = max(N[i],number_of_points_of_arista[i]) sobre i=1,number_aristas_of_group
% Ubicar los number_of_points_of_arista[i] definidos por el usuario eld puntos de la particion
%        de forma que todas las aristas del grupo tengan N[i] nodos.
%


number_of_groups=length(grupos);

if exist('Ng','var')==0
   Ng = -100;
   for k=1:number_of_groups,
      arista_of_grupo = grupos(k).aristas;
      for kk=1:length(arista_of_grupo);
         punto_of_arista = arista(arista_of_grupo(kk)).puntos;
         Ng = max(Ng,length(punto_of_arista));
      end
   end   
   Ng = 3*Ng*ones(1,number_of_groups)
end


bfh  = [];
for k=1:number_of_groups,
   arista_of_grupo = grupos(k).aristas;
   for kk=1:length(arista_of_grupo);
      n1=length(bfh)+1;
      punto_of_arista = arista(arista_of_grupo(kk)).puntos;
      [xa]=split_curve_N_points(xnod(punto_of_arista,:),Ng(k));      
      bfh=[bfh;xa];
      n2=length(bfh);      
      arista(arista_of_grupo(kk))=setfield(arista(arista_of_grupo(kk)),'puntos',(n1:n2));
   end
end

xnod = bfh;


% [4] mallar cada superficie.
icone=[];
for ks=1:length(superficie),
   aristas = superficie(ks).aristas;
   for k=1:length(aristas),      
      pp=getfield(arista(aristas(k)),'puntos');      
      eval(['nodes_' num2str(k) '= pp ;']);      
   end   
   [xnod,icosurf]=split_surface(xnod,nodes_1,nodes_2,nodes_3,nodes_4);    
   icone= [icone;icosurf];
%   keyboard;pause

end


[numnp,ndm]=size(xnod);
[map] = rmdbnode(xnod);
nmap = length(map(:,1));


indx = (1:numnp)';
indx(map(:,1))=indx(map(:,2));
indx(numnp-nmap+(1:nmap))=map(:,1);

xnod(indx,:)=xnod;

if 0,
   [bas,ibas]=sort(indx(1:numnp-nmap));   
end

xnod(numnp-nmap+(1:nmap),:)=[];

[numel,nen]=size(icone);
for k=1:nen
   icone(:,k)=indx(icone(:,k));
end

figure(1);clf;pltmsh(xnod,icone);hold on;plot(bfh(:,1),bfh(:,2),'y.')

%keyboard;pause

return
